/*
 * @(#)SaoReader.java
 *
 * Copyright (C) 1997-2007 Seiichi Yoshida
 * All rights reserved.
 */

package net.aerith.misao.catalog.io;
import java.io.*;
import java.net.*;
import java.util.Vector;
import net.aerith.misao.util.*;
import net.aerith.misao.util.star.*;
import net.aerith.misao.catalog.*;
import net.aerith.misao.catalog.star.*;

/**
 * The <code>SaoReader</code> is a class to read the SAO Star Catalog.
 * <p>
 * The (x,y) position is also set properly so that (0,0) represents
 * the specified R.A. and Decl. to <tt>open</tt> method and (1,1) 
 * represents the position 1 deg to the west and 1 deg to the north.
 *
 * @author  Seiichi Yoshida (comet@aerith.net)
 * @version 2004 May 8
 */

public class SaoReader extends FileReader {
	/**
	 * The smallest R.A. in degree of the block.
	 */
	public static int[] ra_start;

	/**
	 * The largest R.A. in degree of the block.
	 */
	public static int[] ra_end;

	/**
	 * The smallest Decl. in degree of the block.
	 */
	public static int[] decl_start;

	/**
	 * The largest Decl. in degree of the block.
	 */
	public static int[] decl_end;

	/**
	 * Constructs an empty <code>SaoReader</code>.
	 */
	public SaoReader ( ) {
		super();
	}

	/**
	 * Constructs a <code>SaoReader</code> with URL of the catalog
	 * file.
	 * @param url the URL of the catalog file.
	 */
	public SaoReader ( URL url ) {
		super();

		addURL(url);
	}

	/**
	 * Gets the catalog name. It must be unique among all subclasses.
	 * @return the catalog name.
	 */
	public String getName ( ) {
		return "SAO Star Catalog";
	}

	/**
	 * Checks if the catalog supports the use in PIXY examination.
	 * @return true if the catalog can be used in PIXY examination.
	 */
	public boolean supportsExamination ( ) {
		return true;
	}

	/**
	 * Gets the maximum error of position in arcsec. It is the search
	 * area size to identify with other stars.
	 * @return the maximum error of position in arcsec.
	 */
	public double getMaximumPositionErrorInArcsec ( ) {
		return 5.0;
	}

	/**
	 * Creates a <code>CatalogStar</code> object from the specified
	 * one line record in the file. If some more records are required
	 * to create a star object, it returns null. This method must be
	 * overrided in the subclasses.
	 * @param record the one line record in the file.
	 * @return the star object.
	 */
	public CatalogStar createStar ( String record ) {
		Coor coor = Coor.create(record.substring(150, 152) + " " + record.substring(152, 154) + " " + record.substring(154, 160) + " " + record.substring(167, 170) + " " + record.substring(170, 172) + " " + record.substring(172, 177));

		if (circum_area == null  ||  circum_area.inArea(coor)) {
			double p_mag = 999.9;
			double v_mag = 999.9;
			if (record.substring(76, 80).trim().length() > 0)
				p_mag = Format.doubleValueOf(record.substring(76, 80).trim());
			if (record.substring(80, 84).trim().length() > 0)
				v_mag = Format.doubleValueOf(record.substring(80, 84).trim());

			double mag = p_mag;
			if (mag > v_mag)
				mag = v_mag;

			if (mag <= limiting_mag) {
				int number = Integer.parseInt(record.substring(0, 6).trim());

				SaoStar star = new SaoStar(number, coor);

				if (p_mag < 99.9)
					star.setPMagnitude(p_mag);
				if (v_mag < 99.9)
					star.setVMagnitude(v_mag);

				if (record.substring(160, 167).trim().length() > 0)
					star.setProperMotionInRA(Format.doubleValueOf(record.substring(160, 167).trim()));
				if (record.substring(177, 183).trim().length() > 0)
					star.setProperMotionInDecl(Format.doubleValueOf(record.substring(177, 183).trim()));

				if (record.substring(84, 87).trim().length() > 0)
					star.setSpectrum(record.substring(84, 87).trim());

				if (center_coor != null) {
					ChartMapFunction cmf = new ChartMapFunction(center_coor, 1.0, 0.0);
					Position position = cmf.mapCoordinatesToXY(coor);
					star.setPosition(position);
				}

				return star;
			}
		}

		return null;
	}

	/**
	 * Gets the help message.
	 * @return the help message.
	 */
	public String getHelpMessage ( ) {
		String html = "<html><body>";
		html += "<p>";
		html += "SAO Star Catalog J2000<br>";
		html += "Smithsonian Astrophysical Observatory Star Catalog<br>";
		html += "Astronomical Data Center catalog No. 1131A<br>";
		html += "ADC CD-ROM Vol. 4<br>";
		html += "</p><p>";
		html += "Download:";
		html += "<blockquote>";
		html += "<u><font color=\"#0000ff\">ftp://dbc.nao.ac.jp/DBC/NASAADC/catalogs/1/1131A/sao.dat.gz</font></u>";
		html += "</blockquote>";
		html += "</p>";
		html += "</body></html>";
		return html;
	}

	/**
	 * Gets the number of blocks in a file. If 0, it means the file is
	 * not separated into blocks. It must be overrided in the 
	 * subclasses if neccessary.
	 * @return the size of a block.
	 */
	public long getBlockCount ( ) {
		return 518;
	}

	/**
	 * Gets the number of records in a block. If 0, it means the file
	 * is not separated into blocks. It must be overrided in the 
	 * subclasses if neccessary.
	 * @return the size of a block.
	 */
	public long getBlockSize ( ) {
		if (current_block >= getBlockCount())
			return 0;

		if (current_block == getBlockCount() -1)
			return 497;

		return 500;
	}

	/**
	 * Gets the characters of a record. If 0, it means the record size
	 * is not constant, so the reader skips the specified lines while
	 * reading one line by one. Otherwise, the reader skips the 
	 * specified characters at one time. It must be overrided in the 
	 * subclasses if neccessary.
	 * @return the size of a record.
	 */
	public long getRecordSize ( ) {
		return 205;
	}

	/**
	 * Checks if the current block is overlapping on the specified 
	 * circum area.
	 * @return true if the current block is overlapping on the 
	 * specified circum area.
	 */
	public boolean overlapsBlock ( ) {
		if (current_block >= getBlockCount())
			return true;

		if (circum_area == null)
			return true;

		Coor start_coor = new Coor((double)ra_start[(int)current_block], (double)decl_start[(int)current_block]);
		Coor end_coor = new Coor((double)ra_end[(int)current_block], (double)decl_end[(int)current_block]);
		if (circum_area.overlapsArea(start_coor, end_coor))
			return true;

		return false;
	}

	/**
	 * Sets the R.A. range of each block.
	 */
	static {
		ra_start = new int[518];
		ra_end = new int[518];
		decl_start = new int[518];
		decl_end = new int[518];

		ra_start[0] = 0;
		ra_end[0] = 71;
		decl_start[0] = 80;
		decl_end[0] = 90;
		ra_start[1] = 48;
		ra_end[1] = 135;
		decl_start[1] = 79;
		decl_end[1] = 90;
		ra_start[2] = 96;
		ra_end[2] = 147;
		decl_start[2] = 79;
		decl_end[2] = 90;
		ra_start[3] = 140;
		ra_end[3] = 184;
		decl_start[3] = 79;
		decl_end[3] = 90;
		ra_start[4] = 183;
		ra_end[4] = 227;
		decl_start[4] = 79;
		decl_end[4] = 90;
		ra_start[5] = 215;
		ra_end[5] = 272;
		decl_start[5] = 79;
		decl_end[5] = 90;
		ra_start[6] = 259;
		ra_end[6] = 315;
		decl_start[6] = 80;
		decl_end[6] = 90;
		ra_start[7] = 309;
		ra_end[7] = 360;
		decl_start[7] = 80;
		decl_end[7] = 90;
		ra_start[8] = 0;
		ra_end[8] = 360;
		decl_start[8] = 70;
		decl_end[8] = 88;
		ra_start[9] = 27;
		ra_end[9] = 58;
		decl_start[9] = 70;
		decl_end[9] = 81;
		ra_start[10] = 57;
		ra_end[10] = 81;
		decl_start[10] = 70;
		decl_end[10] = 81;
		ra_start[11] = 80;
		ra_end[11] = 104;
		decl_start[11] = 69;
		decl_end[11] = 80;
		ra_start[12] = 103;
		ra_end[12] = 126;
		decl_start[12] = 69;
		decl_end[12] = 80;
		ra_start[13] = 125;
		ra_end[13] = 150;
		decl_start[13] = 69;
		decl_end[13] = 80;
		ra_start[14] = 149;
		ra_end[14] = 182;
		decl_start[14] = 69;
		decl_end[14] = 80;
		ra_start[15] = 181;
		ra_end[15] = 216;
		decl_start[15] = 69;
		decl_end[15] = 80;
		ra_start[16] = 215;
		ra_end[16] = 246;
		decl_start[16] = 69;
		decl_end[16] = 80;
		ra_start[17] = 245;
		ra_end[17] = 271;
		decl_start[17] = 69;
		decl_end[17] = 81;
		ra_start[18] = 270;
		ra_end[18] = 296;
		decl_start[18] = 70;
		decl_end[18] = 81;
		ra_start[19] = 294;
		ra_end[19] = 319;
		decl_start[19] = 70;
		decl_end[19] = 81;
		ra_start[20] = 318;
		ra_end[20] = 342;
		decl_start[20] = 70;
		decl_end[20] = 81;
		ra_start[21] = 0;
		ra_end[21] = 360;
		decl_start[21] = 60;
		decl_end[21] = 81;
		ra_start[22] = 2;
		ra_end[22] = 15;
		decl_start[22] = 60;
		decl_end[22] = 71;
		ra_start[23] = 14;
		ra_end[23] = 28;
		decl_start[23] = 60;
		decl_end[23] = 71;
		ra_start[24] = 27;
		ra_end[24] = 44;
		decl_start[24] = 60;
		decl_end[24] = 71;
		ra_start[25] = 43;
		ra_end[25] = 61;
		decl_start[25] = 60;
		decl_end[25] = 71;
		ra_start[26] = 60;
		ra_end[26] = 83;
		decl_start[26] = 60;
		decl_end[26] = 71;
		ra_start[27] = 81;
		ra_end[27] = 104;
		decl_start[27] = 59;
		decl_end[27] = 70;
		ra_start[28] = 103;
		ra_end[28] = 125;
		decl_start[28] = 59;
		decl_end[28] = 70;
		ra_start[29] = 124;
		ra_end[29] = 148;
		decl_start[29] = 59;
		decl_end[29] = 70;
		ra_start[30] = 147;
		ra_end[30] = 172;
		decl_start[30] = 59;
		decl_end[30] = 70;
		ra_start[31] = 171;
		ra_end[31] = 198;
		decl_start[31] = 59;
		decl_end[31] = 70;
		ra_start[32] = 197;
		ra_end[32] = 222;
		decl_start[32] = 59;
		decl_end[32] = 70;
		ra_start[33] = 221;
		ra_end[33] = 244;
		decl_start[33] = 59;
		decl_end[33] = 70;
		ra_start[34] = 243;
		ra_end[34] = 263;
		decl_start[34] = 59;
		decl_end[34] = 70;
		ra_start[35] = 262;
		ra_end[35] = 282;
		decl_start[35] = 60;
		decl_end[35] = 71;
		ra_start[36] = 281;
		ra_end[36] = 296;
		decl_start[36] = 60;
		decl_end[36] = 71;
		ra_start[37] = 296;
		ra_end[37] = 311;
		decl_start[37] = 60;
		decl_end[37] = 71;
		ra_start[38] = 310;
		ra_end[38] = 324;
		decl_start[38] = 60;
		decl_end[38] = 71;
		ra_start[39] = 323;
		ra_end[39] = 335;
		decl_start[39] = 60;
		decl_end[39] = 71;
		ra_start[40] = 334;
		ra_end[40] = 349;
		decl_start[40] = 60;
		decl_end[40] = 71;
		ra_start[41] = 0;
		ra_end[41] = 360;
		decl_start[41] = 60;
		decl_end[41] = 71;
		ra_start[42] = 0;
		ra_end[42] = 9;
		decl_start[42] = 50;
		decl_end[42] = 70;
		ra_start[43] = 8;
		ra_end[43] = 17;
		decl_start[43] = 50;
		decl_end[43] = 61;
		ra_start[44] = 16;
		ra_end[44] = 26;
		decl_start[44] = 50;
		decl_end[44] = 61;
		ra_start[45] = 25;
		ra_end[45] = 33;
		decl_start[45] = 50;
		decl_end[45] = 61;
		ra_start[46] = 32;
		ra_end[46] = 40;
		decl_start[46] = 50;
		decl_end[46] = 61;
		ra_start[47] = 39;
		ra_end[47] = 51;
		decl_start[47] = 50;
		decl_end[47] = 61;
		ra_start[48] = 50;
		ra_end[48] = 64;
		decl_start[48] = 50;
		decl_end[48] = 61;
		ra_start[49] = 63;
		ra_end[49] = 77;
		decl_start[49] = 50;
		decl_end[49] = 61;
		ra_start[50] = 76;
		ra_end[50] = 90;
		decl_start[50] = 50;
		decl_end[50] = 60;
		ra_start[51] = 89;
		ra_end[51] = 104;
		decl_start[51] = 49;
		decl_end[51] = 60;
		ra_start[52] = 102;
		ra_end[52] = 117;
		decl_start[52] = 49;
		decl_end[52] = 60;
		ra_start[53] = 116;
		ra_end[53] = 133;
		decl_start[53] = 49;
		decl_end[53] = 60;
		ra_start[54] = 132;
		ra_end[54] = 152;
		decl_start[54] = 49;
		decl_end[54] = 60;
		ra_start[55] = 151;
		ra_end[55] = 171;
		decl_start[55] = 49;
		decl_end[55] = 60;
		ra_start[56] = 170;
		ra_end[56] = 192;
		decl_start[56] = 49;
		decl_end[56] = 60;
		ra_start[57] = 191;
		ra_end[57] = 212;
		decl_start[57] = 49;
		decl_end[57] = 60;
		ra_start[58] = 211;
		ra_end[58] = 231;
		decl_start[58] = 49;
		decl_end[58] = 60;
		ra_start[59] = 230;
		ra_end[59] = 249;
		decl_start[59] = 49;
		decl_end[59] = 60;
		ra_start[60] = 248;
		ra_end[60] = 265;
		decl_start[60] = 49;
		decl_end[60] = 60;
		ra_start[61] = 264;
		ra_end[61] = 278;
		decl_start[61] = 49;
		decl_end[61] = 60;
		ra_start[62] = 277;
		ra_end[62] = 289;
		decl_start[62] = 50;
		decl_end[62] = 61;
		ra_start[63] = 288;
		ra_end[63] = 297;
		decl_start[63] = 50;
		decl_end[63] = 61;
		ra_start[64] = 296;
		ra_end[64] = 306;
		decl_start[64] = 50;
		decl_end[64] = 61;
		ra_start[65] = 305;
		ra_end[65] = 314;
		decl_start[65] = 50;
		decl_end[65] = 61;
		ra_start[66] = 313;
		ra_end[66] = 323;
		decl_start[66] = 50;
		decl_end[66] = 61;
		ra_start[67] = 322;
		ra_end[67] = 331;
		decl_start[67] = 50;
		decl_end[67] = 61;
		ra_start[68] = 330;
		ra_end[68] = 338;
		decl_start[68] = 50;
		decl_end[68] = 61;
		ra_start[69] = 337;
		ra_end[69] = 345;
		decl_start[69] = 50;
		decl_end[69] = 61;
		ra_start[70] = 344;
		ra_end[70] = 353;
		decl_start[70] = 50;
		decl_end[70] = 61;
		ra_start[71] = 0;
		ra_end[71] = 360;
		decl_start[71] = 50;
		decl_end[71] = 61;
		ra_start[72] = 0;
		ra_end[72] = 9;
		decl_start[72] = 40;
		decl_end[72] = 57;
		ra_start[73] = 8;
		ra_end[73] = 18;
		decl_start[73] = 40;
		decl_end[73] = 51;
		ra_start[74] = 17;
		ra_end[74] = 27;
		decl_start[74] = 40;
		decl_end[74] = 51;
		ra_start[75] = 26;
		ra_end[75] = 36;
		decl_start[75] = 40;
		decl_end[75] = 51;
		ra_start[76] = 36;
		ra_end[76] = 46;
		decl_start[76] = 40;
		decl_end[76] = 51;
		ra_start[77] = 45;
		ra_end[77] = 55;
		decl_start[77] = 40;
		decl_end[77] = 51;
		ra_start[78] = 54;
		ra_end[78] = 66;
		decl_start[78] = 40;
		decl_end[78] = 51;
		ra_start[79] = 65;
		ra_end[79] = 77;
		decl_start[79] = 40;
		decl_end[79] = 51;
		ra_start[80] = 76;
		ra_end[80] = 85;
		decl_start[80] = 40;
		decl_end[80] = 51;
		ra_start[81] = 84;
		ra_end[81] = 95;
		decl_start[81] = 40;
		decl_end[81] = 51;
		ra_start[82] = 94;
		ra_end[82] = 106;
		decl_start[82] = 39;
		decl_end[82] = 50;
		ra_start[83] = 105;
		ra_end[83] = 117;
		decl_start[83] = 39;
		decl_end[83] = 50;
		ra_start[84] = 116;
		ra_end[84] = 131;
		decl_start[84] = 39;
		decl_end[84] = 50;
		ra_start[85] = 130;
		ra_end[85] = 146;
		decl_start[85] = 39;
		decl_end[85] = 50;
		ra_start[86] = 145;
		ra_end[86] = 164;
		decl_start[86] = 39;
		decl_end[86] = 50;
		ra_start[87] = 163;
		ra_end[87] = 181;
		decl_start[87] = 39;
		decl_end[87] = 50;
		ra_start[88] = 180;
		ra_end[88] = 198;
		decl_start[88] = 39;
		decl_end[88] = 50;
		ra_start[89] = 197;
		ra_end[89] = 216;
		decl_start[89] = 39;
		decl_end[89] = 50;
		ra_start[90] = 215;
		ra_end[90] = 231;
		decl_start[90] = 39;
		decl_end[90] = 50;
		ra_start[91] = 230;
		ra_end[91] = 245;
		decl_start[91] = 39;
		decl_end[91] = 50;
		ra_start[92] = 244;
		ra_end[92] = 257;
		decl_start[92] = 39;
		decl_end[92] = 50;
		ra_start[93] = 256;
		ra_end[93] = 268;
		decl_start[93] = 39;
		decl_end[93] = 50;
		ra_start[94] = 267;
		ra_end[94] = 277;
		decl_start[94] = 40;
		decl_end[94] = 51;
		ra_start[95] = 276;
		ra_end[95] = 286;
		decl_start[95] = 40;
		decl_end[95] = 51;
		ra_start[96] = 285;
		ra_end[96] = 293;
		decl_start[96] = 40;
		decl_end[96] = 51;
		ra_start[97] = 292;
		ra_end[97] = 300;
		decl_start[97] = 40;
		decl_end[97] = 51;
		ra_start[98] = 299;
		ra_end[98] = 306;
		decl_start[98] = 40;
		decl_end[98] = 51;
		ra_start[99] = 305;
		ra_end[99] = 311;
		decl_start[99] = 40;
		decl_end[99] = 51;
		ra_start[100] = 310;
		ra_end[100] = 318;
		decl_start[100] = 40;
		decl_end[100] = 51;
		ra_start[101] = 317;
		ra_end[101] = 324;
		decl_start[101] = 40;
		decl_end[101] = 51;
		ra_start[102] = 323;
		ra_end[102] = 330;
		decl_start[102] = 40;
		decl_end[102] = 51;
		ra_start[103] = 329;
		ra_end[103] = 337;
		decl_start[103] = 40;
		decl_end[103] = 51;
		ra_start[104] = 336;
		ra_end[104] = 344;
		decl_start[104] = 40;
		decl_end[104] = 51;
		ra_start[105] = 343;
		ra_end[105] = 352;
		decl_start[105] = 40;
		decl_end[105] = 51;
		ra_start[106] = 351;
		ra_end[106] = 360;
		decl_start[106] = 40;
		decl_end[106] = 51;
		ra_start[107] = 0;
		ra_end[107] = 360;
		decl_start[107] = 30;
		decl_end[107] = 51;
		ra_start[108] = 8;
		ra_end[108] = 18;
		decl_start[108] = 30;
		decl_end[108] = 41;
		ra_start[109] = 17;
		ra_end[109] = 28;
		decl_start[109] = 30;
		decl_end[109] = 41;
		ra_start[110] = 27;
		ra_end[110] = 36;
		decl_start[110] = 30;
		decl_end[110] = 41;
		ra_start[111] = 35;
		ra_end[111] = 44;
		decl_start[111] = 30;
		decl_end[111] = 41;
		ra_start[112] = 43;
		ra_end[112] = 53;
		decl_start[112] = 30;
		decl_end[112] = 41;
		ra_start[113] = 52;
		ra_end[113] = 63;
		decl_start[113] = 30;
		decl_end[113] = 41;
		ra_start[114] = 62;
		ra_end[114] = 74;
		decl_start[114] = 30;
		decl_end[114] = 41;
		ra_start[115] = 73;
		ra_end[115] = 82;
		decl_start[115] = 30;
		decl_end[115] = 41;
		ra_start[116] = 81;
		ra_end[116] = 88;
		decl_start[116] = 30;
		decl_end[116] = 41;
		ra_start[117] = 87;
		ra_end[117] = 96;
		decl_start[117] = 30;
		decl_end[117] = 40;
		ra_start[118] = 95;
		ra_end[118] = 103;
		decl_start[118] = 29;
		decl_end[118] = 40;
		ra_start[119] = 102;
		ra_end[119] = 111;
		decl_start[119] = 29;
		decl_end[119] = 40;
		ra_start[120] = 110;
		ra_end[120] = 120;
		decl_start[120] = 29;
		decl_end[120] = 40;
		ra_start[121] = 119;
		ra_end[121] = 131;
		decl_start[121] = 29;
		decl_end[121] = 40;
		ra_start[122] = 130;
		ra_end[122] = 142;
		decl_start[122] = 29;
		decl_end[122] = 40;
		ra_start[123] = 141;
		ra_end[123] = 156;
		decl_start[123] = 29;
		decl_end[123] = 40;
		ra_start[124] = 155;
		ra_end[124] = 171;
		decl_start[124] = 29;
		decl_end[124] = 40;
		ra_start[125] = 170;
		ra_end[125] = 187;
		decl_start[125] = 29;
		decl_end[125] = 40;
		ra_start[126] = 186;
		ra_end[126] = 201;
		decl_start[126] = 29;
		decl_end[126] = 40;
		ra_start[127] = 200;
		ra_end[127] = 214;
		decl_start[127] = 29;
		decl_end[127] = 40;
		ra_start[128] = 213;
		ra_end[128] = 227;
		decl_start[128] = 29;
		decl_end[128] = 40;
		ra_start[129] = 226;
		ra_end[129] = 240;
		decl_start[129] = 29;
		decl_end[129] = 40;
		ra_start[130] = 239;
		ra_end[130] = 251;
		decl_start[130] = 29;
		decl_end[130] = 40;
		ra_start[131] = 250;
		ra_end[131] = 261;
		decl_start[131] = 29;
		decl_end[131] = 40;
		ra_start[132] = 260;
		ra_end[132] = 270;
		decl_start[132] = 29;
		decl_end[132] = 40;
		ra_start[133] = 269;
		ra_end[133] = 278;
		decl_start[133] = 30;
		decl_end[133] = 40;
		ra_start[134] = 277;
		ra_end[134] = 284;
		decl_start[134] = 30;
		decl_end[134] = 41;
		ra_start[135] = 283;
		ra_end[135] = 289;
		decl_start[135] = 30;
		decl_end[135] = 41;
		ra_start[136] = 288;
		ra_end[136] = 294;
		decl_start[136] = 30;
		decl_end[136] = 41;
		ra_start[137] = 293;
		ra_end[137] = 299;
		decl_start[137] = 30;
		decl_end[137] = 41;
		ra_start[138] = 298;
		ra_end[138] = 303;
		decl_start[138] = 30;
		decl_end[138] = 41;
		ra_start[139] = 302;
		ra_end[139] = 307;
		decl_start[139] = 30;
		decl_end[139] = 41;
		ra_start[140] = 306;
		ra_end[140] = 312;
		decl_start[140] = 30;
		decl_end[140] = 41;
		ra_start[141] = 311;
		ra_end[141] = 318;
		decl_start[141] = 30;
		decl_end[141] = 41;
		ra_start[142] = 317;
		ra_end[142] = 324;
		decl_start[142] = 30;
		decl_end[142] = 41;
		ra_start[143] = 323;
		ra_end[143] = 332;
		decl_start[143] = 30;
		decl_end[143] = 41;
		ra_start[144] = 331;
		ra_end[144] = 339;
		decl_start[144] = 30;
		decl_end[144] = 41;
		ra_start[145] = 338;
		ra_end[145] = 347;
		decl_start[145] = 30;
		decl_end[145] = 41;
		ra_start[146] = 346;
		ra_end[146] = 357;
		decl_start[146] = 30;
		decl_end[146] = 41;
		ra_start[147] = 0;
		ra_end[147] = 360;
		decl_start[147] = 20;
		decl_end[147] = 41;
		ra_start[148] = 6;
		ra_end[148] = 17;
		decl_start[148] = 20;
		decl_end[148] = 31;
		ra_start[149] = 16;
		ra_end[149] = 29;
		decl_start[149] = 20;
		decl_end[149] = 31;
		ra_start[150] = 28;
		ra_end[150] = 40;
		decl_start[150] = 20;
		decl_end[150] = 31;
		ra_start[151] = 39;
		ra_end[151] = 54;
		decl_start[151] = 20;
		decl_end[151] = 31;
		ra_start[152] = 53;
		ra_end[152] = 64;
		decl_start[152] = 20;
		decl_end[152] = 31;
		ra_start[153] = 63;
		ra_end[153] = 78;
		decl_start[153] = 20;
		decl_end[153] = 31;
		ra_start[154] = 77;
		ra_end[154] = 87;
		decl_start[154] = 20;
		decl_end[154] = 31;
		ra_start[155] = 86;
		ra_end[155] = 92;
		decl_start[155] = 20;
		decl_end[155] = 31;
		ra_start[156] = 91;
		ra_end[156] = 99;
		decl_start[156] = 19;
		decl_end[156] = 30;
		ra_start[157] = 98;
		ra_end[157] = 106;
		decl_start[157] = 19;
		decl_end[157] = 30;
		ra_start[158] = 105;
		ra_end[158] = 114;
		decl_start[158] = 19;
		decl_end[158] = 30;
		ra_start[159] = 113;
		ra_end[159] = 123;
		decl_start[159] = 19;
		decl_end[159] = 30;
		ra_start[160] = 122;
		ra_end[160] = 134;
		decl_start[160] = 19;
		decl_end[160] = 30;
		ra_start[161] = 133;
		ra_end[161] = 147;
		decl_start[161] = 19;
		decl_end[161] = 30;
		ra_start[162] = 146;
		ra_end[162] = 162;
		decl_start[162] = 19;
		decl_end[162] = 30;
		ra_start[163] = 161;
		ra_end[163] = 178;
		decl_start[163] = 19;
		decl_end[163] = 30;
		ra_start[164] = 177;
		ra_end[164] = 192;
		decl_start[164] = 19;
		decl_end[164] = 30;
		ra_start[165] = 192;
		ra_end[165] = 208;
		decl_start[165] = 19;
		decl_end[165] = 30;
		ra_start[166] = 207;
		ra_end[166] = 222;
		decl_start[166] = 19;
		decl_end[166] = 30;
		ra_start[167] = 221;
		ra_end[167] = 237;
		decl_start[167] = 19;
		decl_end[167] = 30;
		ra_start[168] = 236;
		ra_end[168] = 250;
		decl_start[168] = 19;
		decl_end[168] = 30;
		ra_start[169] = 249;
		ra_end[169] = 260;
		decl_start[169] = 19;
		decl_end[169] = 30;
		ra_start[170] = 259;
		ra_end[170] = 269;
		decl_start[170] = 19;
		decl_end[170] = 30;
		ra_start[171] = 268;
		ra_end[171] = 276;
		decl_start[171] = 20;
		decl_end[171] = 30;
		ra_start[172] = 275;
		ra_end[172] = 283;
		decl_start[172] = 20;
		decl_end[172] = 31;
		ra_start[173] = 282;
		ra_end[173] = 289;
		decl_start[173] = 20;
		decl_end[173] = 31;
		ra_start[174] = 288;
		ra_end[174] = 295;
		decl_start[174] = 20;
		decl_end[174] = 31;
		ra_start[175] = 294;
		ra_end[175] = 300;
		decl_start[175] = 20;
		decl_end[175] = 31;
		ra_start[176] = 299;
		ra_end[176] = 305;
		decl_start[176] = 20;
		decl_end[176] = 31;
		ra_start[177] = 304;
		ra_end[177] = 311;
		decl_start[177] = 20;
		decl_end[177] = 31;
		ra_start[178] = 310;
		ra_end[178] = 318;
		decl_start[178] = 20;
		decl_end[178] = 31;
		ra_start[179] = 317;
		ra_end[179] = 327;
		decl_start[179] = 20;
		decl_end[179] = 31;
		ra_start[180] = 326;
		ra_end[180] = 337;
		decl_start[180] = 20;
		decl_end[180] = 31;
		ra_start[181] = 336;
		ra_end[181] = 347;
		decl_start[181] = 20;
		decl_end[181] = 31;
		ra_start[182] = 346;
		ra_end[182] = 358;
		decl_start[182] = 20;
		decl_end[182] = 31;
		ra_start[183] = 0;
		ra_end[183] = 360;
		decl_start[183] = 10;
		decl_end[183] = 31;
		ra_start[184] = 9;
		ra_end[184] = 24;
		decl_start[184] = 10;
		decl_end[184] = 21;
		ra_start[185] = 23;
		ra_end[185] = 39;
		decl_start[185] = 10;
		decl_end[185] = 21;
		ra_start[186] = 38;
		ra_end[186] = 54;
		decl_start[186] = 10;
		decl_end[186] = 21;
		ra_start[187] = 53;
		ra_end[187] = 69;
		decl_start[187] = 10;
		decl_end[187] = 21;
		ra_start[188] = 68;
		ra_end[188] = 81;
		decl_start[188] = 10;
		decl_end[188] = 21;
		ra_start[189] = 80;
		ra_end[189] = 89;
		decl_start[189] = 10;
		decl_end[189] = 20;
		ra_start[190] = 88;
		ra_end[190] = 95;
		decl_start[190] = 10;
		decl_end[190] = 20;
		ra_start[191] = 94;
		ra_end[191] = 101;
		decl_start[191] = 9;
		decl_end[191] = 20;
		ra_start[192] = 100;
		ra_end[192] = 107;
		decl_start[192] = 9;
		decl_end[192] = 20;
		ra_start[193] = 106;
		ra_end[193] = 114;
		decl_start[193] = 9;
		decl_end[193] = 20;
		ra_start[194] = 113;
		ra_end[194] = 121;
		decl_start[194] = 9;
		decl_end[194] = 20;
		ra_start[195] = 120;
		ra_end[195] = 130;
		decl_start[195] = 9;
		decl_end[195] = 20;
		ra_start[196] = 129;
		ra_end[196] = 140;
		decl_start[196] = 9;
		decl_end[196] = 20;
		ra_start[197] = 139;
		ra_end[197] = 154;
		decl_start[197] = 9;
		decl_end[197] = 20;
		ra_start[198] = 153;
		ra_end[198] = 169;
		decl_start[198] = 9;
		decl_end[198] = 20;
		ra_start[199] = 168;
		ra_end[199] = 184;
		decl_start[199] = 9;
		decl_end[199] = 20;
		ra_start[200] = 183;
		ra_end[200] = 200;
		decl_start[200] = 9;
		decl_end[200] = 20;
		ra_start[201] = 199;
		ra_end[201] = 216;
		decl_start[201] = 9;
		decl_end[201] = 20;
		ra_start[202] = 215;
		ra_end[202] = 231;
		decl_start[202] = 9;
		decl_end[202] = 20;
		ra_start[203] = 230;
		ra_end[203] = 244;
		decl_start[203] = 9;
		decl_end[203] = 20;
		ra_start[204] = 243;
		ra_end[204] = 255;
		decl_start[204] = 9;
		decl_end[204] = 20;
		ra_start[205] = 254;
		ra_end[205] = 265;
		decl_start[205] = 9;
		decl_end[205] = 20;
		ra_start[206] = 264;
		ra_end[206] = 274;
		decl_start[206] = 10;
		decl_end[206] = 20;
		ra_start[207] = 273;
		ra_end[207] = 281;
		decl_start[207] = 10;
		decl_end[207] = 21;
		ra_start[208] = 280;
		ra_end[208] = 287;
		decl_start[208] = 10;
		decl_end[208] = 21;
		ra_start[209] = 286;
		ra_end[209] = 294;
		decl_start[209] = 10;
		decl_end[209] = 21;
		ra_start[210] = 293;
		ra_end[210] = 300;
		decl_start[210] = 10;
		decl_end[210] = 21;
		ra_start[211] = 299;
		ra_end[211] = 306;
		decl_start[211] = 10;
		decl_end[211] = 21;
		ra_start[212] = 305;
		ra_end[212] = 312;
		decl_start[212] = 10;
		decl_end[212] = 21;
		ra_start[213] = 311;
		ra_end[213] = 320;
		decl_start[213] = 10;
		decl_end[213] = 21;
		ra_start[214] = 319;
		ra_end[214] = 329;
		decl_start[214] = 10;
		decl_end[214] = 21;
		ra_start[215] = 328;
		ra_end[215] = 339;
		decl_start[215] = 10;
		decl_end[215] = 21;
		ra_start[216] = 338;
		ra_end[216] = 350;
		decl_start[216] = 10;
		decl_end[216] = 21;
		ra_start[217] = 0;
		ra_end[217] = 360;
		decl_start[217] = 1;
		decl_end[217] = 21;
		ra_start[218] = 1;
		ra_end[218] = 13;
		decl_start[218] = 0;
		decl_end[218] = 11;
		ra_start[219] = 12;
		ra_end[219] = 24;
		decl_start[219] = 0;
		decl_end[219] = 11;
		ra_start[220] = 24;
		ra_end[220] = 36;
		decl_start[220] = 0;
		decl_end[220] = 11;
		ra_start[221] = 35;
		ra_end[221] = 48;
		decl_start[221] = 0;
		decl_end[221] = 11;
		ra_start[222] = 47;
		ra_end[222] = 59;
		decl_start[222] = 0;
		decl_end[222] = 11;
		ra_start[223] = 58;
		ra_end[223] = 71;
		decl_start[223] = 0;
		decl_end[223] = 11;
		ra_start[224] = 70;
		ra_end[224] = 78;
		decl_start[224] = 0;
		decl_end[224] = 11;
		ra_start[225] = 77;
		ra_end[225] = 85;
		decl_start[225] = 0;
		decl_end[225] = 11;
		ra_start[226] = 84;
		ra_end[226] = 93;
		decl_start[226] = 0;
		decl_end[226] = 11;
		ra_start[227] = 92;
		ra_end[227] = 98;
		decl_start[227] = -1;
		decl_end[227] = 10;
		ra_start[228] = 97;
		ra_end[228] = 103;
		decl_start[228] = -1;
		decl_end[228] = 10;
		ra_start[229] = 102;
		ra_end[229] = 108;
		decl_start[229] = -1;
		decl_end[229] = 10;
		ra_start[230] = 107;
		ra_end[230] = 113;
		decl_start[230] = -1;
		decl_end[230] = 10;
		ra_start[231] = 112;
		ra_end[231] = 118;
		decl_start[231] = -1;
		decl_end[231] = 10;
		ra_start[232] = 117;
		ra_end[232] = 124;
		decl_start[232] = -1;
		decl_end[232] = 10;
		ra_start[233] = 123;
		ra_end[233] = 130;
		decl_start[233] = -1;
		decl_end[233] = 10;
		ra_start[234] = 129;
		ra_end[234] = 139;
		decl_start[234] = -1;
		decl_end[234] = 10;
		ra_start[235] = 138;
		ra_end[235] = 150;
		decl_start[235] = -1;
		decl_end[235] = 10;
		ra_start[236] = 149;
		ra_end[236] = 162;
		decl_start[236] = -1;
		decl_end[236] = 10;
		ra_start[237] = 162;
		ra_end[237] = 176;
		decl_start[237] = -1;
		decl_end[237] = 10;
		ra_start[238] = 175;
		ra_end[238] = 190;
		decl_start[238] = -1;
		decl_end[238] = 10;
		ra_start[239] = 189;
		ra_end[239] = 204;
		decl_start[239] = -1;
		decl_end[239] = 10;
		ra_start[240] = 203;
		ra_end[240] = 218;
		decl_start[240] = -1;
		decl_end[240] = 10;
		ra_start[241] = 217;
		ra_end[241] = 232;
		decl_start[241] = -1;
		decl_end[241] = 10;
		ra_start[242] = 231;
		ra_end[242] = 245;
		decl_start[242] = -1;
		decl_end[242] = 10;
		ra_start[243] = 244;
		ra_end[243] = 256;
		decl_start[243] = -1;
		decl_end[243] = 10;
		ra_start[244] = 255;
		ra_end[244] = 264;
		decl_start[244] = -1;
		decl_end[244] = 10;
		ra_start[245] = 263;
		ra_end[245] = 271;
		decl_start[245] = -1;
		decl_end[245] = 10;
		ra_start[246] = 270;
		ra_end[246] = 277;
		decl_start[246] = 0;
		decl_end[246] = 11;
		ra_start[247] = 276;
		ra_end[247] = 284;
		decl_start[247] = 0;
		decl_end[247] = 11;
		ra_start[248] = 283;
		ra_end[248] = 290;
		decl_start[248] = 0;
		decl_end[248] = 11;
		ra_start[249] = 289;
		ra_end[249] = 297;
		decl_start[249] = 0;
		decl_end[249] = 11;
		ra_start[250] = 296;
		ra_end[250] = 303;
		decl_start[250] = 0;
		decl_end[250] = 11;
		ra_start[251] = 302;
		ra_end[251] = 309;
		decl_start[251] = 0;
		decl_end[251] = 11;
		ra_start[252] = 308;
		ra_end[252] = 316;
		decl_start[252] = 0;
		decl_end[252] = 11;
		ra_start[253] = 315;
		ra_end[253] = 326;
		decl_start[253] = 0;
		decl_end[253] = 11;
		ra_start[254] = 325;
		ra_end[254] = 336;
		decl_start[254] = 0;
		decl_end[254] = 11;
		ra_start[255] = 335;
		ra_end[255] = 348;
		decl_start[255] = 0;
		decl_end[255] = 11;
		ra_start[256] = 347;
		ra_end[256] = 360;
		decl_start[256] = 0;
		decl_end[256] = 11;
		ra_start[257] = 0;
		ra_end[257] = 360;
		decl_start[257] = -10;
		decl_end[257] = 10;
		ra_start[258] = 13;
		ra_end[258] = 27;
		decl_start[258] = -10;
		decl_end[258] = 1;
		ra_start[259] = 26;
		ra_end[259] = 39;
		decl_start[259] = -10;
		decl_end[259] = 1;
		ra_start[260] = 38;
		ra_end[260] = 52;
		decl_start[260] = -10;
		decl_end[260] = 1;
		ra_start[261] = 51;
		ra_end[261] = 63;
		decl_start[261] = -10;
		decl_end[261] = 1;
		ra_start[262] = 62;
		ra_end[262] = 73;
		decl_start[262] = -10;
		decl_end[262] = 1;
		ra_start[263] = 72;
		ra_end[263] = 81;
		decl_start[263] = -10;
		decl_end[263] = 1;
		ra_start[264] = 80;
		ra_end[264] = 87;
		decl_start[264] = -10;
		decl_end[264] = 1;
		ra_start[265] = 86;
		ra_end[265] = 94;
		decl_start[265] = -10;
		decl_end[265] = 0;
		ra_start[266] = 93;
		ra_end[266] = 100;
		decl_start[266] = -10;
		decl_end[266] = 0;
		ra_start[267] = 99;
		ra_end[267] = 105;
		decl_start[267] = -11;
		decl_end[267] = 0;
		ra_start[268] = 104;
		ra_end[268] = 110;
		decl_start[268] = -11;
		decl_end[268] = 0;
		ra_start[269] = 109;
		ra_end[269] = 116;
		decl_start[269] = -11;
		decl_end[269] = 0;
		ra_start[270] = 115;
		ra_end[270] = 122;
		decl_start[270] = -11;
		decl_end[270] = 0;
		ra_start[271] = 121;
		ra_end[271] = 128;
		decl_start[271] = -11;
		decl_end[271] = 0;
		ra_start[272] = 127;
		ra_end[272] = 136;
		decl_start[272] = -11;
		decl_end[272] = 0;
		ra_start[273] = 135;
		ra_end[273] = 145;
		decl_start[273] = -11;
		decl_end[273] = 0;
		ra_start[274] = 144;
		ra_end[274] = 156;
		decl_start[274] = -11;
		decl_end[274] = 0;
		ra_start[275] = 155;
		ra_end[275] = 167;
		decl_start[275] = -11;
		decl_end[275] = 0;
		ra_start[276] = 167;
		ra_end[276] = 180;
		decl_start[276] = -11;
		decl_end[276] = 0;
		ra_start[277] = 179;
		ra_end[277] = 193;
		decl_start[277] = -11;
		decl_end[277] = 0;
		ra_start[278] = 192;
		ra_end[278] = 206;
		decl_start[278] = -11;
		decl_end[278] = 0;
		ra_start[279] = 205;
		ra_end[279] = 219;
		decl_start[279] = -11;
		decl_end[279] = 0;
		ra_start[280] = 218;
		ra_end[280] = 231;
		decl_start[280] = -11;
		decl_end[280] = 0;
		ra_start[281] = 230;
		ra_end[281] = 243;
		decl_start[281] = -11;
		decl_end[281] = 0;
		ra_start[282] = 242;
		ra_end[282] = 256;
		decl_start[282] = -11;
		decl_end[282] = 0;
		ra_start[283] = 255;
		ra_end[283] = 270;
		decl_start[283] = -10;
		decl_end[283] = 0;
		ra_start[284] = 269;
		ra_end[284] = 281;
		decl_start[284] = -10;
		decl_end[284] = 1;
		ra_start[285] = 280;
		ra_end[285] = 287;
		decl_start[285] = -10;
		decl_end[285] = 1;
		ra_start[286] = 286;
		ra_end[286] = 293;
		decl_start[286] = -10;
		decl_end[286] = 1;
		ra_start[287] = 292;
		ra_end[287] = 301;
		decl_start[287] = -10;
		decl_end[287] = 1;
		ra_start[288] = 300;
		ra_end[288] = 308;
		decl_start[288] = -10;
		decl_end[288] = 1;
		ra_start[289] = 307;
		ra_end[289] = 315;
		decl_start[289] = -10;
		decl_end[289] = 1;
		ra_start[290] = 314;
		ra_end[290] = 324;
		decl_start[290] = -10;
		decl_end[290] = 1;
		ra_start[291] = 323;
		ra_end[291] = 335;
		decl_start[291] = -10;
		decl_end[291] = 1;
		ra_start[292] = 334;
		ra_end[292] = 347;
		decl_start[292] = -10;
		decl_end[292] = 1;
		ra_start[293] = 346;
		ra_end[293] = 360;
		decl_start[293] = -10;
		decl_end[293] = 1;
		ra_start[294] = 0;
		ra_end[294] = 360;
		decl_start[294] = -20;
		decl_end[294] = 1;
		ra_start[295] = 13;
		ra_end[295] = 27;
		decl_start[295] = -20;
		decl_end[295] = -9;
		ra_start[296] = 26;
		ra_end[296] = 40;
		decl_start[296] = -20;
		decl_end[296] = -9;
		ra_start[297] = 39;
		ra_end[297] = 53;
		decl_start[297] = -20;
		decl_end[297] = -9;
		ra_start[298] = 52;
		ra_end[298] = 65;
		decl_start[298] = -20;
		decl_end[298] = -9;
		ra_start[299] = 64;
		ra_end[299] = 74;
		decl_start[299] = -20;
		decl_end[299] = -9;
		ra_start[300] = 73;
		ra_end[300] = 83;
		decl_start[300] = -20;
		decl_end[300] = -9;
		ra_start[301] = 82;
		ra_end[301] = 90;
		decl_start[301] = -20;
		decl_end[301] = -9;
		ra_start[302] = 89;
		ra_end[302] = 97;
		decl_start[302] = -21;
		decl_end[302] = -10;
		ra_start[303] = 96;
		ra_end[303] = 103;
		decl_start[303] = -21;
		decl_end[303] = -10;
		ra_start[304] = 102;
		ra_end[304] = 108;
		decl_start[304] = -21;
		decl_end[304] = -10;
		ra_start[305] = 107;
		ra_end[305] = 113;
		decl_start[305] = -21;
		decl_end[305] = -10;
		ra_start[306] = 112;
		ra_end[306] = 118;
		decl_start[306] = -21;
		decl_end[306] = -10;
		ra_start[307] = 117;
		ra_end[307] = 124;
		decl_start[307] = -21;
		decl_end[307] = -10;
		ra_start[308] = 123;
		ra_end[308] = 130;
		decl_start[308] = -21;
		decl_end[308] = -10;
		ra_start[309] = 129;
		ra_end[309] = 139;
		decl_start[309] = -21;
		decl_end[309] = -10;
		ra_start[310] = 138;
		ra_end[310] = 148;
		decl_start[310] = -21;
		decl_end[310] = -10;
		ra_start[311] = 147;
		ra_end[311] = 157;
		decl_start[311] = -21;
		decl_end[311] = -10;
		ra_start[312] = 156;
		ra_end[312] = 168;
		decl_start[312] = -21;
		decl_end[312] = -10;
		ra_start[313] = 167;
		ra_end[313] = 180;
		decl_start[313] = -21;
		decl_end[313] = -10;
		ra_start[314] = 179;
		ra_end[314] = 192;
		decl_start[314] = -21;
		decl_end[314] = -10;
		ra_start[315] = 191;
		ra_end[315] = 204;
		decl_start[315] = -21;
		decl_end[315] = -10;
		ra_start[316] = 203;
		ra_end[316] = 216;
		decl_start[316] = -21;
		decl_end[316] = -10;
		ra_start[317] = 215;
		ra_end[317] = 227;
		decl_start[317] = -21;
		decl_end[317] = -10;
		ra_start[318] = 226;
		ra_end[318] = 237;
		decl_start[318] = -21;
		decl_end[318] = -10;
		ra_start[319] = 236;
		ra_end[319] = 250;
		decl_start[319] = -21;
		decl_end[319] = -10;
		ra_start[320] = 249;
		ra_end[320] = 261;
		decl_start[320] = -21;
		decl_end[320] = -10;
		ra_start[321] = 260;
		ra_end[321] = 271;
		decl_start[321] = -20;
		decl_end[321] = -10;
		ra_start[322] = 270;
		ra_end[322] = 278;
		decl_start[322] = -20;
		decl_end[322] = -10;
		ra_start[323] = 277;
		ra_end[323] = 285;
		decl_start[323] = -20;
		decl_end[323] = -9;
		ra_start[324] = 284;
		ra_end[324] = 291;
		decl_start[324] = -20;
		decl_end[324] = -9;
		ra_start[325] = 290;
		ra_end[325] = 298;
		decl_start[325] = -20;
		decl_end[325] = -9;
		ra_start[326] = 297;
		ra_end[326] = 306;
		decl_start[326] = -20;
		decl_end[326] = -9;
		ra_start[327] = 305;
		ra_end[327] = 315;
		decl_start[327] = -20;
		decl_end[327] = -9;
		ra_start[328] = 314;
		ra_end[328] = 324;
		decl_start[328] = -20;
		decl_end[328] = -9;
		ra_start[329] = 323;
		ra_end[329] = 335;
		decl_start[329] = -20;
		decl_end[329] = -9;
		ra_start[330] = 334;
		ra_end[330] = 347;
		decl_start[330] = -20;
		decl_end[330] = -9;
		ra_start[331] = 0;
		ra_end[331] = 360;
		decl_start[331] = -20;
		decl_end[331] = -9;
		ra_start[332] = 0;
		ra_end[332] = 11;
		decl_start[332] = -30;
		decl_end[332] = -11;
		ra_start[333] = 10;
		ra_end[333] = 21;
		decl_start[333] = -30;
		decl_end[333] = -19;
		ra_start[334] = 20;
		ra_end[334] = 31;
		decl_start[334] = -30;
		decl_end[334] = -19;
		ra_start[335] = 30;
		ra_end[335] = 41;
		decl_start[335] = -30;
		decl_end[335] = -19;
		ra_start[336] = 40;
		ra_end[336] = 51;
		decl_start[336] = -30;
		decl_end[336] = -19;
		ra_start[337] = 50;
		ra_end[337] = 60;
		decl_start[337] = -30;
		decl_end[337] = -19;
		ra_start[338] = 59;
		ra_end[338] = 68;
		decl_start[338] = -30;
		decl_end[338] = -19;
		ra_start[339] = 67;
		ra_end[339] = 76;
		decl_start[339] = -30;
		decl_end[339] = -19;
		ra_start[340] = 75;
		ra_end[340] = 83;
		decl_start[340] = -30;
		decl_end[340] = -20;
		ra_start[341] = 82;
		ra_end[341] = 90;
		decl_start[341] = -30;
		decl_end[341] = -20;
		ra_start[342] = 88;
		ra_end[342] = 95;
		decl_start[342] = -30;
		decl_end[342] = -20;
		ra_start[343] = 94;
		ra_end[343] = 100;
		decl_start[343] = -31;
		decl_end[343] = -20;
		ra_start[344] = 99;
		ra_end[344] = 104;
		decl_start[344] = -31;
		decl_end[344] = -20;
		ra_start[345] = 103;
		ra_end[345] = 107;
		decl_start[345] = -31;
		decl_end[345] = -20;
		ra_start[346] = 106;
		ra_end[346] = 111;
		decl_start[346] = -31;
		decl_end[346] = -20;
		ra_start[347] = 110;
		ra_end[347] = 114;
		decl_start[347] = -31;
		decl_end[347] = -20;
		ra_start[348] = 113;
		ra_end[348] = 117;
		decl_start[348] = -31;
		decl_end[348] = -20;
		ra_start[349] = 116;
		ra_end[349] = 121;
		decl_start[349] = -31;
		decl_end[349] = -20;
		ra_start[350] = 120;
		ra_end[350] = 125;
		decl_start[350] = -31;
		decl_end[350] = -20;
		ra_start[351] = 124;
		ra_end[351] = 128;
		decl_start[351] = -31;
		decl_end[351] = -20;
		ra_start[352] = 127;
		ra_end[352] = 133;
		decl_start[352] = -31;
		decl_end[352] = -20;
		ra_start[353] = 132;
		ra_end[353] = 138;
		decl_start[353] = -31;
		decl_end[353] = -20;
		ra_start[354] = 137;
		ra_end[354] = 143;
		decl_start[354] = -31;
		decl_end[354] = -20;
		ra_start[355] = 142;
		ra_end[355] = 147;
		decl_start[355] = -31;
		decl_end[355] = -20;
		ra_start[356] = 146;
		ra_end[356] = 153;
		decl_start[356] = -31;
		decl_end[356] = -20;
		ra_start[357] = 152;
		ra_end[357] = 159;
		decl_start[357] = -31;
		decl_end[357] = -20;
		ra_start[358] = 158;
		ra_end[358] = 167;
		decl_start[358] = -31;
		decl_end[358] = -20;
		ra_start[359] = 166;
		ra_end[359] = 174;
		decl_start[359] = -31;
		decl_end[359] = -20;
		ra_start[360] = 173;
		ra_end[360] = 183;
		decl_start[360] = -31;
		decl_end[360] = -20;
		ra_start[361] = 182;
		ra_end[361] = 190;
		decl_start[361] = -31;
		decl_end[361] = -20;
		ra_start[362] = 190;
		ra_end[362] = 200;
		decl_start[362] = -31;
		decl_end[362] = -20;
		ra_start[363] = 199;
		ra_end[363] = 208;
		decl_start[363] = -31;
		decl_end[363] = -20;
		ra_start[364] = 207;
		ra_end[364] = 216;
		decl_start[364] = -31;
		decl_end[364] = -20;
		ra_start[365] = 215;
		ra_end[365] = 224;
		decl_start[365] = -31;
		decl_end[365] = -20;
		ra_start[366] = 223;
		ra_end[366] = 233;
		decl_start[366] = -31;
		decl_end[366] = -20;
		ra_start[367] = 232;
		ra_end[367] = 240;
		decl_start[367] = -31;
		decl_end[367] = -20;
		ra_start[368] = 239;
		ra_end[368] = 250;
		decl_start[368] = -31;
		decl_end[368] = -20;
		ra_start[369] = 249;
		ra_end[369] = 257;
		decl_start[369] = -31;
		decl_end[369] = -20;
		ra_start[370] = 256;
		ra_end[370] = 264;
		decl_start[370] = -31;
		decl_end[370] = -20;
		ra_start[371] = 263;
		ra_end[371] = 270;
		decl_start[371] = -31;
		decl_end[371] = -20;
		ra_start[372] = 269;
		ra_end[372] = 274;
		decl_start[372] = -30;
		decl_end[372] = -20;
		ra_start[373] = 273;
		ra_end[373] = 279;
		decl_start[373] = -30;
		decl_end[373] = -20;
		ra_start[374] = 278;
		ra_end[374] = 285;
		decl_start[374] = -30;
		decl_end[374] = -19;
		ra_start[375] = 284;
		ra_end[375] = 291;
		decl_start[375] = -30;
		decl_end[375] = -19;
		ra_start[376] = 290;
		ra_end[376] = 297;
		decl_start[376] = -30;
		decl_end[376] = -19;
		ra_start[377] = 296;
		ra_end[377] = 303;
		decl_start[377] = -30;
		decl_end[377] = -19;
		ra_start[378] = 302;
		ra_end[378] = 310;
		decl_start[378] = -30;
		decl_end[378] = -19;
		ra_start[379] = 309;
		ra_end[379] = 317;
		decl_start[379] = -30;
		decl_end[379] = -19;
		ra_start[380] = 316;
		ra_end[380] = 325;
		decl_start[380] = -30;
		decl_end[380] = -19;
		ra_start[381] = 324;
		ra_end[381] = 334;
		decl_start[381] = -30;
		decl_end[381] = -19;
		ra_start[382] = 333;
		ra_end[382] = 344;
		decl_start[382] = -30;
		decl_end[382] = -19;
		ra_start[383] = 343;
		ra_end[383] = 354;
		decl_start[383] = -30;
		decl_end[383] = -19;
		ra_start[384] = 0;
		ra_end[384] = 360;
		decl_start[384] = -40;
		decl_end[384] = -19;
		ra_start[385] = 5;
		ra_end[385] = 19;
		decl_start[385] = -40;
		decl_end[385] = -29;
		ra_start[386] = 18;
		ra_end[386] = 32;
		decl_start[386] = -40;
		decl_end[386] = -29;
		ra_start[387] = 31;
		ra_end[387] = 45;
		decl_start[387] = -40;
		decl_end[387] = -29;
		ra_start[388] = 44;
		ra_end[388] = 57;
		decl_start[388] = -40;
		decl_end[388] = -29;
		ra_start[389] = 56;
		ra_end[389] = 67;
		decl_start[389] = -40;
		decl_end[389] = -29;
		ra_start[390] = 66;
		ra_end[390] = 76;
		decl_start[390] = -40;
		decl_end[390] = -29;
		ra_start[391] = 75;
		ra_end[391] = 84;
		decl_start[391] = -40;
		decl_end[391] = -29;
		ra_start[392] = 83;
		ra_end[392] = 92;
		decl_start[392] = -40;
		decl_end[392] = -29;
		ra_start[393] = 91;
		ra_end[393] = 100;
		decl_start[393] = -40;
		decl_end[393] = -30;
		ra_start[394] = 99;
		ra_end[394] = 106;
		decl_start[394] = -40;
		decl_end[394] = -30;
		ra_start[395] = 105;
		ra_end[395] = 112;
		decl_start[395] = -41;
		decl_end[395] = -30;
		ra_start[396] = 111;
		ra_end[396] = 118;
		decl_start[396] = -41;
		decl_end[396] = -30;
		ra_start[397] = 117;
		ra_end[397] = 124;
		decl_start[397] = -41;
		decl_end[397] = -30;
		ra_start[398] = 123;
		ra_end[398] = 131;
		decl_start[398] = -41;
		decl_end[398] = -30;
		ra_start[399] = 130;
		ra_end[399] = 137;
		decl_start[399] = -41;
		decl_end[399] = -30;
		ra_start[400] = 136;
		ra_end[400] = 144;
		decl_start[400] = -41;
		decl_end[400] = -30;
		ra_start[401] = 143;
		ra_end[401] = 151;
		decl_start[401] = -41;
		decl_end[401] = -30;
		ra_start[402] = 150;
		ra_end[402] = 159;
		decl_start[402] = -41;
		decl_end[402] = -30;
		ra_start[403] = 158;
		ra_end[403] = 166;
		decl_start[403] = -41;
		decl_end[403] = -30;
		ra_start[404] = 165;
		ra_end[404] = 173;
		decl_start[404] = -41;
		decl_end[404] = -30;
		ra_start[405] = 172;
		ra_end[405] = 180;
		decl_start[405] = -41;
		decl_end[405] = -30;
		ra_start[406] = 179;
		ra_end[406] = 188;
		decl_start[406] = -41;
		decl_end[406] = -30;
		ra_start[407] = 187;
		ra_end[407] = 195;
		decl_start[407] = -41;
		decl_end[407] = -30;
		ra_start[408] = 194;
		ra_end[408] = 203;
		decl_start[408] = -41;
		decl_end[408] = -30;
		ra_start[409] = 202;
		ra_end[409] = 209;
		decl_start[409] = -41;
		decl_end[409] = -30;
		ra_start[410] = 208;
		ra_end[410] = 216;
		decl_start[410] = -41;
		decl_end[410] = -30;
		ra_start[411] = 215;
		ra_end[411] = 223;
		decl_start[411] = -41;
		decl_end[411] = -30;
		ra_start[412] = 222;
		ra_end[412] = 230;
		decl_start[412] = -41;
		decl_end[412] = -30;
		ra_start[413] = 229;
		ra_end[413] = 238;
		decl_start[413] = -41;
		decl_end[413] = -30;
		ra_start[414] = 237;
		ra_end[414] = 245;
		decl_start[414] = -41;
		decl_end[414] = -30;
		ra_start[415] = 244;
		ra_end[415] = 252;
		decl_start[415] = -41;
		decl_end[415] = -30;
		ra_start[416] = 251;
		ra_end[416] = 258;
		decl_start[416] = -41;
		decl_end[416] = -30;
		ra_start[417] = 257;
		ra_end[417] = 264;
		decl_start[417] = -41;
		decl_end[417] = -30;
		ra_start[418] = 263;
		ra_end[418] = 270;
		decl_start[418] = -41;
		decl_end[418] = -30;
		ra_start[419] = 269;
		ra_end[419] = 276;
		decl_start[419] = -40;
		decl_end[419] = -29;
		ra_start[420] = 275;
		ra_end[420] = 281;
		decl_start[420] = -40;
		decl_end[420] = -29;
		ra_start[421] = 280;
		ra_end[421] = 288;
		decl_start[421] = -40;
		decl_end[421] = -29;
		ra_start[422] = 287;
		ra_end[422] = 296;
		decl_start[422] = -40;
		decl_end[422] = -29;
		ra_start[423] = 295;
		ra_end[423] = 305;
		decl_start[423] = -40;
		decl_end[423] = -29;
		ra_start[424] = 304;
		ra_end[424] = 314;
		decl_start[424] = -40;
		decl_end[424] = -29;
		ra_start[425] = 313;
		ra_end[425] = 322;
		decl_start[425] = -40;
		decl_end[425] = -29;
		ra_start[426] = 321;
		ra_end[426] = 331;
		decl_start[426] = -40;
		decl_end[426] = -29;
		ra_start[427] = 330;
		ra_end[427] = 341;
		decl_start[427] = -40;
		decl_end[427] = -29;
		ra_start[428] = 340;
		ra_end[428] = 351;
		decl_start[428] = -40;
		decl_end[428] = -29;
		ra_start[429] = 0;
		ra_end[429] = 360;
		decl_start[429] = -50;
		decl_end[429] = -29;
		ra_start[430] = 3;
		ra_end[430] = 22;
		decl_start[430] = -50;
		decl_end[430] = -39;
		ra_start[431] = 21;
		ra_end[431] = 41;
		decl_start[431] = -50;
		decl_end[431] = -39;
		ra_start[432] = 40;
		ra_end[432] = 58;
		decl_start[432] = -50;
		decl_end[432] = -39;
		ra_start[433] = 57;
		ra_end[433] = 73;
		decl_start[433] = -50;
		decl_end[433] = -39;
		ra_start[434] = 72;
		ra_end[434] = 86;
		decl_start[434] = -50;
		decl_end[434] = -39;
		ra_start[435] = 85;
		ra_end[435] = 99;
		decl_start[435] = -50;
		decl_end[435] = -39;
		ra_start[436] = 98;
		ra_end[436] = 108;
		decl_start[436] = -51;
		decl_end[436] = -40;
		ra_start[437] = 107;
		ra_end[437] = 117;
		decl_start[437] = -51;
		decl_end[437] = -40;
		ra_start[438] = 116;
		ra_end[438] = 123;
		decl_start[438] = -51;
		decl_end[438] = -40;
		ra_start[439] = 122;
		ra_end[439] = 128;
		decl_start[439] = -51;
		decl_end[439] = -40;
		ra_start[440] = 127;
		ra_end[440] = 133;
		decl_start[440] = -51;
		decl_end[440] = -40;
		ra_start[441] = 132;
		ra_end[441] = 139;
		decl_start[441] = -51;
		decl_end[441] = -40;
		ra_start[442] = 138;
		ra_end[442] = 148;
		decl_start[442] = -51;
		decl_end[442] = -40;
		ra_start[443] = 147;
		ra_end[443] = 156;
		decl_start[443] = -51;
		decl_end[443] = -40;
		ra_start[444] = 155;
		ra_end[444] = 166;
		decl_start[444] = -51;
		decl_end[444] = -40;
		ra_start[445] = 165;
		ra_end[445] = 177;
		decl_start[445] = -51;
		decl_end[445] = -40;
		ra_start[446] = 176;
		ra_end[446] = 189;
		decl_start[446] = -51;
		decl_end[446] = -40;
		ra_start[447] = 188;
		ra_end[447] = 199;
		decl_start[447] = -51;
		decl_end[447] = -40;
		ra_start[448] = 198;
		ra_end[448] = 209;
		decl_start[448] = -51;
		decl_end[448] = -40;
		ra_start[449] = 208;
		ra_end[449] = 219;
		decl_start[449] = -51;
		decl_end[449] = -40;
		ra_start[450] = 218;
		ra_end[450] = 228;
		decl_start[450] = -51;
		decl_end[450] = -40;
		ra_start[451] = 227;
		ra_end[451] = 235;
		decl_start[451] = -51;
		decl_end[451] = -40;
		ra_start[452] = 234;
		ra_end[452] = 242;
		decl_start[452] = -51;
		decl_end[452] = -40;
		ra_start[453] = 241;
		ra_end[453] = 250;
		decl_start[453] = -51;
		decl_end[453] = -40;
		ra_start[454] = 249;
		ra_end[454] = 255;
		decl_start[454] = -51;
		decl_end[454] = -40;
		ra_start[455] = 254;
		ra_end[455] = 263;
		decl_start[455] = -51;
		decl_end[455] = -40;
		ra_start[456] = 262;
		ra_end[456] = 269;
		decl_start[456] = -51;
		decl_end[456] = -40;
		ra_start[457] = 268;
		ra_end[457] = 277;
		decl_start[457] = -50;
		decl_end[457] = -39;
		ra_start[458] = 276;
		ra_end[458] = 287;
		decl_start[458] = -50;
		decl_end[458] = -39;
		ra_start[459] = 286;
		ra_end[459] = 301;
		decl_start[459] = -50;
		decl_end[459] = -39;
		ra_start[460] = 300;
		ra_end[460] = 316;
		decl_start[460] = -50;
		decl_end[460] = -39;
		ra_start[461] = 316;
		ra_end[461] = 333;
		decl_start[461] = -50;
		decl_end[461] = -39;
		ra_start[462] = 332;
		ra_end[462] = 349;
		decl_start[462] = -50;
		decl_end[462] = -39;
		ra_start[463] = 0;
		ra_end[463] = 360;
		decl_start[463] = -60;
		decl_end[463] = -39;
		ra_start[464] = 4;
		ra_end[464] = 26;
		decl_start[464] = -60;
		decl_end[464] = -49;
		ra_start[465] = 25;
		ra_end[465] = 47;
		decl_start[465] = -60;
		decl_end[465] = -49;
		ra_start[466] = 46;
		ra_end[466] = 66;
		decl_start[466] = -60;
		decl_end[466] = -49;
		ra_start[467] = 65;
		ra_end[467] = 83;
		decl_start[467] = -60;
		decl_end[467] = -49;
		ra_start[468] = 82;
		ra_end[468] = 97;
		decl_start[468] = -61;
		decl_end[468] = -50;
		ra_start[469] = 96;
		ra_end[469] = 108;
		decl_start[469] = -61;
		decl_end[469] = -50;
		ra_start[470] = 107;
		ra_end[470] = 117;
		decl_start[470] = -61;
		decl_end[470] = -50;
		ra_start[471] = 116;
		ra_end[471] = 127;
		decl_start[471] = -61;
		decl_end[471] = -50;
		ra_start[472] = 126;
		ra_end[472] = 136;
		decl_start[472] = -61;
		decl_end[472] = -50;
		ra_start[473] = 135;
		ra_end[473] = 142;
		decl_start[473] = -61;
		decl_end[473] = -50;
		ra_start[474] = 141;
		ra_end[474] = 149;
		decl_start[474] = -61;
		decl_end[474] = -50;
		ra_start[475] = 148;
		ra_end[475] = 156;
		decl_start[475] = -61;
		decl_end[475] = -50;
		ra_start[476] = 155;
		ra_end[476] = 163;
		decl_start[476] = -61;
		decl_end[476] = -50;
		ra_start[477] = 162;
		ra_end[477] = 171;
		decl_start[477] = -61;
		decl_end[477] = -50;
		ra_start[478] = 170;
		ra_end[478] = 179;
		decl_start[478] = -61;
		decl_end[478] = -50;
		ra_start[479] = 178;
		ra_end[479] = 188;
		decl_start[479] = -61;
		decl_end[479] = -50;
		ra_start[480] = 187;
		ra_end[480] = 196;
		decl_start[480] = -61;
		decl_end[480] = -50;
		ra_start[481] = 195;
		ra_end[481] = 205;
		decl_start[481] = -61;
		decl_end[481] = -50;
		ra_start[482] = 204;
		ra_end[482] = 213;
		decl_start[482] = -61;
		decl_end[482] = -50;
		ra_start[483] = 212;
		ra_end[483] = 222;
		decl_start[483] = -61;
		decl_end[483] = -50;
		ra_start[484] = 221;
		ra_end[484] = 232;
		decl_start[484] = -61;
		decl_end[484] = -50;
		ra_start[485] = 231;
		ra_end[485] = 238;
		decl_start[485] = -61;
		decl_end[485] = -50;
		ra_start[486] = 237;
		ra_end[486] = 244;
		decl_start[486] = -61;
		decl_end[486] = -50;
		ra_start[487] = 243;
		ra_end[487] = 250;
		decl_start[487] = -61;
		decl_end[487] = -50;
		ra_start[488] = 249;
		ra_end[488] = 258;
		decl_start[488] = -61;
		decl_end[488] = -50;
		ra_start[489] = 257;
		ra_end[489] = 267;
		decl_start[489] = -61;
		decl_end[489] = -50;
		ra_start[490] = 266;
		ra_end[490] = 278;
		decl_start[490] = -60;
		decl_end[490] = -49;
		ra_start[491] = 277;
		ra_end[491] = 290;
		decl_start[491] = -60;
		decl_end[491] = -49;
		ra_start[492] = 289;
		ra_end[492] = 304;
		decl_start[492] = -60;
		decl_end[492] = -49;
		ra_start[493] = 303;
		ra_end[493] = 321;
		decl_start[493] = -60;
		decl_end[493] = -49;
		ra_start[494] = 320;
		ra_end[494] = 339;
		decl_start[494] = -60;
		decl_end[494] = -49;
		ra_start[495] = 338;
		ra_end[495] = 357;
		decl_start[495] = -60;
		decl_end[495] = -49;
		ra_start[496] = 0;
		ra_end[496] = 360;
		decl_start[496] = -70;
		decl_end[496] = -49;
		ra_start[497] = 32;
		ra_end[497] = 66;
		decl_start[497] = -70;
		decl_end[497] = -59;
		ra_start[498] = 65;
		ra_end[498] = 95;
		decl_start[498] = -70;
		decl_end[498] = -59;
		ra_start[499] = 94;
		ra_end[499] = 119;
		decl_start[499] = -71;
		decl_end[499] = -60;
		ra_start[500] = 118;
		ra_end[500] = 139;
		decl_start[500] = -71;
		decl_end[500] = -60;
		ra_start[501] = 138;
		ra_end[501] = 158;
		decl_start[501] = -71;
		decl_end[501] = -60;
		ra_start[502] = 157;
		ra_end[502] = 175;
		decl_start[502] = -71;
		decl_end[502] = -60;
		ra_start[503] = 174;
		ra_end[503] = 191;
		decl_start[503] = -71;
		decl_end[503] = -60;
		ra_start[504] = 190;
		ra_end[504] = 209;
		decl_start[504] = -71;
		decl_end[504] = -60;
		ra_start[505] = 208;
		ra_end[505] = 226;
		decl_start[505] = -71;
		decl_end[505] = -60;
		ra_start[506] = 225;
		ra_end[506] = 246;
		decl_start[506] = -71;
		decl_end[506] = -60;
		ra_start[507] = 244;
		ra_end[507] = 266;
		decl_start[507] = -71;
		decl_end[507] = -60;
		ra_start[508] = 265;
		ra_end[508] = 289;
		decl_start[508] = -70;
		decl_end[508] = -59;
		ra_start[509] = 288;
		ra_end[509] = 322;
		decl_start[509] = -70;
		decl_end[509] = -59;
		ra_start[510] = 321;
		ra_end[510] = 353;
		decl_start[510] = -70;
		decl_end[510] = -59;
		ra_start[511] = 0;
		ra_end[511] = 360;
		decl_start[511] = -80;
		decl_end[511] = -59;
		ra_start[512] = 53;
		ra_end[512] = 126;
		decl_start[512] = -80;
		decl_end[512] = -69;
		ra_start[513] = 124;
		ra_end[513] = 196;
		decl_start[513] = -81;
		decl_end[513] = -70;
		ra_start[514] = 195;
		ra_end[514] = 263;
		decl_start[514] = -81;
		decl_end[514] = -70;
		ra_start[515] = 262;
		ra_end[515] = 332;
		decl_start[515] = -80;
		decl_end[515] = -69;
		ra_start[516] = 0;
		ra_end[516] = 360;
		decl_start[516] = -89;
		decl_end[516] = -69;
		ra_start[517] = 0;
		ra_end[517] = 360;
		decl_start[517] = -90;
		decl_end[517] = -79;
	}
}
