/*
 * @(#)Ac2000Reader.java
 *
 * Copyright (C) 1997-2007 Seiichi Yoshida
 * All rights reserved.
 */

package net.aerith.misao.catalog.io;
import java.io.*;
import java.net.*;
import java.util.Vector;
import net.aerith.misao.util.*;
import net.aerith.misao.util.star.*;
import net.aerith.misao.catalog.*;
import net.aerith.misao.catalog.star.*;
import net.aerith.misao.io.Decoder;
import net.aerith.misao.io.CdromNotFoundException;

/**
 * The <code>Ac2000Reader</code> is a class to read the AC 2000.2 
 * Catalogue.
 * <p>
 * The (x,y) position is also set properly so that (0,0) represents
 * the specified R.A. and Decl. to <tt>open</tt> method and (1,1) 
 * represents the position 1 deg to the west and 1 deg to the north.
 *
 * @author  Seiichi Yoshida (comet@aerith.net)
 * @version 2004 May 8
 */

public class Ac2000Reader extends CatalogReader {
	/**
	 * The number of files.
	 */
	protected final static int file_N = 14;

	/**
	 * The file names.
	 */
	protected final static String[] filenames = {
		"ac_p60.dat", "ac_p50.dat", "ac_p40.dat", "ac_p30.dat",
		"ac_p20.dat", "ac_p10.dat", "ac_p00.dat", "ac_n00.dat",
		"ac_n10.dat", "ac_n20.dat", "ac_n30.dat", "ac_n40.dat",
		"ac_n50.dat", "ac_n60.dat"
	};

	/**
	 * The start declinations.
	 */
	protected final static int[] decl_start = {
		60, 50, 40, 30, 20, 10, 0,
		-10, -20, -30, -40, -50, -60, -90
	};

	/**
	 * The end declinations.
	 */
	protected final static int[] decl_end = {
		90, 60, 50, 40, 30, 20, 10,
		0, -10, -20, -30, -40, -50, -60
	};

	/**
	 * The index of stars per 1 deg in R.A. for each files.
	 */
	protected final static int[][] index;

	/**
	 * The current_input stream reader.
	 */
	protected BufferedReader current_reader;

	/**
	 * The index of current file in <code>file_list</code>.
	 */
	protected int current_file_index_in_list;

	/**
	 * The index of current file (0-13).
	 */
	protected int current_file_index;

	/**
	 * The index of current R.A. block (0-359).
	 */
	protected int current_ra;

	/**
	 * The index of current star.
	 */
	protected int current_star_index;

	/**
	 * The circum area to read stars.
	 */
	protected CircumArea circum_area;

	/**
	 * The list of file index to be read.
	 */
	protected Vector file_list;

	/**
	 * Constructs an empty <code>Ac2000Reader</code>.
	 */
	public Ac2000Reader ( ) {
		url_set.ignoreCases(true);
		url_set.acceptGzipped(true);

		setDefaultURL();
	}

	/**
	 * Constructs a <code>Ac2000Reader</code> with URL of directory 
	 * containing the AC 2000.2 Catalogue.
	 * @param url the URL of directory containing CD-ROMs data.
	 */
	public Ac2000Reader ( URL url ) {
		this();

		addURL(url);
	}

	/**
	 * Gets the catalog name. It must be unique among all subclasses.
	 * @return the catalog name.
	 */
	public String getName ( ) {
		return "AC 2000.2 Catalogue";
	}

	/**
	 * Checks if the catalog data is in a directory.
	 * @return true if the catalog data is in a directory.
	 */
	public boolean isInDirectory ( ) {
		return true;
	}

	/**
	 * Checks if the catalog supports the use in PIXY examination.
	 * @return true if the catalog can be used in PIXY examination.
	 */
	public boolean supportsExamination ( ) {
		return true;
	}

	/**
	 * Gets the maximum error of position in arcsec. It is the search
	 * area size to identify with other stars.
	 * @return the maximum error of position in arcsec.
	 */
	public double getMaximumPositionErrorInArcsec ( ) {
		return 5.0;
	}

	/**
	 * Gets the help message.
	 * @return the help message.
	 */
	public String getHelpMessage ( ) {
		String html = "<html><body>";
		html += "<p>";
		html += "AC 2000.2: The Astrographic Catalogue on the Hipparcos System<br>";
		html += "Astronomical Data Center catalog No. 1275<br>";
		html += "</p><p>";
		html += "Download:";
		html += "<blockquote>";
		html += "<u><font color=\"#0000ff\">ftp://dbc.nao.ac.jp/DBC/NASAADC/catalogs/1/1275/</font></u>";
		html += "</blockquote>";
		html += "</p>";
		html += "</body></html>";
		return html;
	}

	/**
	 * Opens a catalog. This method must be invoked at first.
	 * @param coor the R.A. and Decl. of the center.
	 * @param fov  the field of view to read in degree.
	 * @exception IOException if a file cannot be accessed.
	 * @exception FileNotFoundException if a file does not exists in
	 * any URL, and the file is in any CD-ROMs.
	 * @exception CdromNotFoundException if a file does not exists in
	 * any URL, and the file is in a CD-ROM.
	 */
	public void open ( Coor coor, double fov )
		throws IOException, FileNotFoundException, CdromNotFoundException
	{
		center_coor = coor;
		circum_area = null;
		if (coor != null)
			circum_area = new CircumArea(coor, fov / 2.0);
		file_list = new Vector();

		// Select files to read.
		for (int i = 0 ; i < file_N ; i++) {
			Coor start_coor = Coor.create("0 0 0 " + decl_start[i] + " 0 0");
			Coor end_coor = Coor.create("12 0 0 " + decl_end[i] + " 0 0");

			if (circum_area == null  ||  circum_area.overlapsArea(start_coor, end_coor)) {
				file_list.addElement(new Integer(i));
			} else {
				start_coor = Coor.create("12 0 0 " + decl_start[i] + " 0 0");
				end_coor = Coor.create("0 0 0 " + decl_end[i] + " 0 0");

				if (circum_area == null  ||  circum_area.overlapsArea(start_coor, end_coor))
					file_list.addElement(new Integer(i));
			}
		}

		// Initializes to read data one by one.
		current_reader = null;
	}

	/**
	 * Reads one data from the opened catalog. After this method is 
	 * invoked, the cursor is promoted to tne next star. When every 
	 * data is read, it returns null. Note that data with the same ID
	 * are also returned one by one.
	 * @return a star data.
	 * @exception IOException if a file cannot be accessed.
	 * @exception FileNotFoundException if a file does not exists in
	 * any URL, and the file is in any CD-ROMs.
	 * @exception CdromNotFoundException if a file does not exists in
	 * any URL, and the file is in a CD-ROM.
	 * @exception QueryFailException if the query to the server is 
	 * failed.
	 */
	public CatalogStar readNext ( )
		throws IOException, FileNotFoundException, CdromNotFoundException, QueryFailException
	{
		while (true) {
			try {
				// In the case of the very first reading.
				if (current_reader == null) {
					current_file_index_in_list = -1;
					throw new EOFException();
				}

				current_star_index++;

				// In the case it reaches to the end of a R.A. block.
				if (current_ra < 0  ||  current_star_index > index[current_file_index][current_ra]) {
					current_ra++;

					// In the case it reaches to the last R.A. block.
					if (current_ra == 360)
						throw new EOFException();

					Coor start_coor = new Coor((double)current_ra, (double)decl_start[current_file_index]);
					Coor end_coor = new Coor((double)(current_ra + 1), (double)decl_end[current_file_index]);
					if (circum_area == null  ||  circum_area.overlapsArea(start_coor, end_coor)) {
						// In the case within the field.
					} else {
						// In the case not within the field.

						// In the case it reaches to the last R.A. block.
						if (current_ra + 1 == 360)
							throw new EOFException();

						// Skips to the next block.
						long skip_records = index[current_file_index][current_ra] - current_star_index + 1;
						StreamSkipTimer skip_timer = new StreamSkipTimer(current_reader, 10000);
						skip_timer.skip(skip_records * 94);

						current_star_index = index[current_file_index][current_ra];

						continue;
					}
				}

				String record = current_reader.readLine();
				if (record == null)
					throw new EOFException();

				Coor coor = Coor.create(record.substring(0, 25));
				double b_mag = Format.doubleValueOf(record.substring(26, 31).trim());

				if ((circum_area == null  ||  circum_area.inArea(coor))  &&  (double)b_mag <= limiting_mag) {
					int number = Format.intValueOf(record.substring(56, 64).trim());

					Ac2000Star star = new Ac2000Star(number, coor, b_mag);

					if (record.substring(85, 91).trim().length() > 0) {
						double v_mag = Format.doubleValueOf(record.substring(85, 91).trim());
						star.setVMagnitude(v_mag);
					}

					if (center_coor != null) {
						ChartMapFunction cmf = new ChartMapFunction(center_coor, 1.0, 0.0);
						Position position = cmf.mapCoordinatesToXY(coor);
						star.setPosition(position);
					}

					return star;
				}
			} catch (EOFException e) {
				// All files have been read.
				if (current_file_index_in_list + 1 ==  file_list.size())
					return null;

				current_file_index = ((Integer)file_list.elementAt(current_file_index_in_list + 1)).intValue();

				URL url = url_set.existsFile(filenames[current_file_index]);
				BufferedReader reader = Decoder.newReader(url);

				// Closes the current file after confirming the next file can be open properly.
				if (current_reader != null)
					current_reader.close();
				current_file_index_in_list++;

				current_reader = reader;

				current_ra = -1;
				current_star_index = -1;
			}
		}
	}

	/**
	 * Closes a catalog. This method must be invoked finally.
	 * @exception IOException if a file cannot be accessed.
	 */
	public void close ( )
		throws IOException
	{
		current_reader.close();
		current_reader = null;
	}

	/**
	 * Initializes the index in <tt>ac_p60.dat</tt>.
	 */
	private static void initialize_p60 ( ) {
		index[0][0] = 1316;
		index[0][1] = 2518;
		index[0][2] = 3721;
		index[0][3] = 4936;
		index[0][4] = 6151;
		index[0][5] = 7383;
		index[0][6] = 8614;
		index[0][7] = 10062;
		index[0][8] = 11310;
		index[0][9] = 12524;
		index[0][10] = 13609;
		index[0][11] = 14651;
		index[0][12] = 15749;
		index[0][13] = 16905;
		index[0][14] = 18014;
		index[0][15] = 19200;
		index[0][16] = 20190;
		index[0][17] = 21195;
		index[0][18] = 22097;
		index[0][19] = 22957;
		index[0][20] = 23811;
		index[0][21] = 24747;
		index[0][22] = 25863;
		index[0][23] = 27062;
		index[0][24] = 28343;
		index[0][25] = 29619;
		index[0][26] = 31071;
		index[0][27] = 32181;
		index[0][28] = 33217;
		index[0][29] = 34268;
		index[0][30] = 35287;
		index[0][31] = 36058;
		index[0][32] = 36872;
		index[0][33] = 37688;
		index[0][34] = 38503;
		index[0][35] = 39290;
		index[0][36] = 40051;
		index[0][37] = 40925;
		index[0][38] = 41887;
		index[0][39] = 42792;
		index[0][40] = 43759;
		index[0][41] = 44688;
		index[0][42] = 45659;
		index[0][43] = 46671;
		index[0][44] = 47620;
		index[0][45] = 48534;
		index[0][46] = 49278;
		index[0][47] = 50021;
		index[0][48] = 50851;
		index[0][49] = 51691;
		index[0][50] = 52492;
		index[0][51] = 53195;
		index[0][52] = 53995;
		index[0][53] = 54764;
		index[0][54] = 55507;
		index[0][55] = 56247;
		index[0][56] = 56963;
		index[0][57] = 57735;
		index[0][58] = 58484;
		index[0][59] = 59190;
		index[0][60] = 59903;
		index[0][61] = 60688;
		index[0][62] = 61419;
		index[0][63] = 62038;
		index[0][64] = 62680;
		index[0][65] = 63338;
		index[0][66] = 63970;
		index[0][67] = 64548;
		index[0][68] = 65197;
		index[0][69] = 65894;
		index[0][70] = 66604;
		index[0][71] = 67288;
		index[0][72] = 68087;
		index[0][73] = 68846;
		index[0][74] = 69589;
		index[0][75] = 70389;
		index[0][76] = 71204;
		index[0][77] = 71987;
		index[0][78] = 72782;
		index[0][79] = 73581;
		index[0][80] = 74384;
		index[0][81] = 75148;
		index[0][82] = 75977;
		index[0][83] = 76747;
		index[0][84] = 77563;
		index[0][85] = 78381;
		index[0][86] = 79201;
		index[0][87] = 79979;
		index[0][88] = 80738;
		index[0][89] = 81422;
		index[0][90] = 82108;
		index[0][91] = 82830;
		index[0][92] = 83553;
		index[0][93] = 84267;
		index[0][94] = 84945;
		index[0][95] = 85573;
		index[0][96] = 86224;
		index[0][97] = 86890;
		index[0][98] = 87486;
		index[0][99] = 88117;
		index[0][100] = 88722;
		index[0][101] = 89381;
		index[0][102] = 90099;
		index[0][103] = 90803;
		index[0][104] = 91544;
		index[0][105] = 92241;
		index[0][106] = 92951;
		index[0][107] = 93626;
		index[0][108] = 94347;
		index[0][109] = 95098;
		index[0][110] = 95841;
		index[0][111] = 96576;
		index[0][112] = 97264;
		index[0][113] = 97972;
		index[0][114] = 98627;
		index[0][115] = 99291;
		index[0][116] = 99883;
		index[0][117] = 100490;
		index[0][118] = 101130;
		index[0][119] = 101787;
		index[0][120] = 102353;
		index[0][121] = 102964;
		index[0][122] = 103588;
		index[0][123] = 104185;
		index[0][124] = 104742;
		index[0][125] = 105340;
		index[0][126] = 105944;
		index[0][127] = 106556;
		index[0][128] = 107129;
		index[0][129] = 107690;
		index[0][130] = 108226;
		index[0][131] = 108773;
		index[0][132] = 109366;
		index[0][133] = 109869;
		index[0][134] = 110391;
		index[0][135] = 110904;
		index[0][136] = 111459;
		index[0][137] = 111943;
		index[0][138] = 112413;
		index[0][139] = 112860;
		index[0][140] = 113310;
		index[0][141] = 113751;
		index[0][142] = 114213;
		index[0][143] = 114695;
		index[0][144] = 115145;
		index[0][145] = 115541;
		index[0][146] = 115930;
		index[0][147] = 116309;
		index[0][148] = 116729;
		index[0][149] = 117205;
		index[0][150] = 117674;
		index[0][151] = 118128;
		index[0][152] = 118547;
		index[0][153] = 118942;
		index[0][154] = 119333;
		index[0][155] = 119700;
		index[0][156] = 120109;
		index[0][157] = 120569;
		index[0][158] = 121010;
		index[0][159] = 121484;
		index[0][160] = 121938;
		index[0][161] = 122314;
		index[0][162] = 122691;
		index[0][163] = 123100;
		index[0][164] = 123592;
		index[0][165] = 124105;
		index[0][166] = 124544;
		index[0][167] = 125020;
		index[0][168] = 125470;
		index[0][169] = 125925;
		index[0][170] = 126345;
		index[0][171] = 126771;
		index[0][172] = 127202;
		index[0][173] = 127655;
		index[0][174] = 128093;
		index[0][175] = 128535;
		index[0][176] = 129007;
		index[0][177] = 129436;
		index[0][178] = 129878;
		index[0][179] = 130332;
		index[0][180] = 130743;
		index[0][181] = 131125;
		index[0][182] = 131522;
		index[0][183] = 131888;
		index[0][184] = 132224;
		index[0][185] = 132623;
		index[0][186] = 132999;
		index[0][187] = 133351;
		index[0][188] = 133699;
		index[0][189] = 134105;
		index[0][190] = 134532;
		index[0][191] = 134962;
		index[0][192] = 135427;
		index[0][193] = 135851;
		index[0][194] = 136311;
		index[0][195] = 136770;
		index[0][196] = 137252;
		index[0][197] = 137717;
		index[0][198] = 138226;
		index[0][199] = 138700;
		index[0][200] = 139166;
		index[0][201] = 139599;
		index[0][202] = 140090;
		index[0][203] = 140544;
		index[0][204] = 140995;
		index[0][205] = 141446;
		index[0][206] = 141914;
		index[0][207] = 142356;
		index[0][208] = 142829;
		index[0][209] = 143288;
		index[0][210] = 143754;
		index[0][211] = 144303;
		index[0][212] = 144757;
		index[0][213] = 145222;
		index[0][214] = 145672;
		index[0][215] = 146117;
		index[0][216] = 146557;
		index[0][217] = 147052;
		index[0][218] = 147562;
		index[0][219] = 148066;
		index[0][220] = 148559;
		index[0][221] = 149072;
		index[0][222] = 149557;
		index[0][223] = 150050;
		index[0][224] = 150561;
		index[0][225] = 151072;
		index[0][226] = 151634;
		index[0][227] = 152175;
		index[0][228] = 152727;
		index[0][229] = 153292;
		index[0][230] = 153856;
		index[0][231] = 154397;
		index[0][232] = 154970;
		index[0][233] = 155541;
		index[0][234] = 156115;
		index[0][235] = 156696;
		index[0][236] = 157294;
		index[0][237] = 157906;
		index[0][238] = 158497;
		index[0][239] = 159102;
		index[0][240] = 159693;
		index[0][241] = 160236;
		index[0][242] = 160782;
		index[0][243] = 161399;
		index[0][244] = 162030;
		index[0][245] = 162633;
		index[0][246] = 163288;
		index[0][247] = 163890;
		index[0][248] = 164499;
		index[0][249] = 165053;
		index[0][250] = 165633;
		index[0][251] = 166195;
		index[0][252] = 166803;
		index[0][253] = 167488;
		index[0][254] = 168148;
		index[0][255] = 168771;
		index[0][256] = 169449;
		index[0][257] = 170059;
		index[0][258] = 170733;
		index[0][259] = 171401;
		index[0][260] = 172057;
		index[0][261] = 172711;
		index[0][262] = 173356;
		index[0][263] = 174001;
		index[0][264] = 174590;
		index[0][265] = 175220;
		index[0][266] = 175837;
		index[0][267] = 176477;
		index[0][268] = 177199;
		index[0][269] = 177918;
		index[0][270] = 178671;
		index[0][271] = 179485;
		index[0][272] = 180269;
		index[0][273] = 181063;
		index[0][274] = 181914;
		index[0][275] = 182697;
		index[0][276] = 183532;
		index[0][277] = 184344;
		index[0][278] = 185134;
		index[0][279] = 185895;
		index[0][280] = 186735;
		index[0][281] = 187539;
		index[0][282] = 188360;
		index[0][283] = 189207;
		index[0][284] = 190021;
		index[0][285] = 190914;
		index[0][286] = 191829;
		index[0][287] = 192697;
		index[0][288] = 193649;
		index[0][289] = 194590;
		index[0][290] = 195511;
		index[0][291] = 196453;
		index[0][292] = 197406;
		index[0][293] = 198312;
		index[0][294] = 199415;
		index[0][295] = 200355;
		index[0][296] = 201348;
		index[0][297] = 202374;
		index[0][298] = 203256;
		index[0][299] = 204133;
		index[0][300] = 204958;
		index[0][301] = 205809;
		index[0][302] = 206611;
		index[0][303] = 207488;
		index[0][304] = 208423;
		index[0][305] = 209263;
		index[0][306] = 210063;
		index[0][307] = 210813;
		index[0][308] = 211491;
		index[0][309] = 212239;
		index[0][310] = 213013;
		index[0][311] = 213755;
		index[0][312] = 214545;
		index[0][313] = 215352;
		index[0][314] = 216059;
		index[0][315] = 216661;
		index[0][316] = 217425;
		index[0][317] = 218156;
		index[0][318] = 218860;
		index[0][319] = 219597;
		index[0][320] = 220420;
		index[0][321] = 221221;
		index[0][322] = 222061;
		index[0][323] = 222911;
		index[0][324] = 223766;
		index[0][325] = 224615;
		index[0][326] = 225495;
		index[0][327] = 226584;
		index[0][328] = 227709;
		index[0][329] = 228758;
		index[0][330] = 229922;
		index[0][331] = 231137;
		index[0][332] = 232231;
		index[0][333] = 233187;
		index[0][334] = 234186;
		index[0][335] = 234962;
		index[0][336] = 235754;
		index[0][337] = 236593;
		index[0][338] = 237429;
		index[0][339] = 238267;
		index[0][340] = 239087;
		index[0][341] = 239993;
		index[0][342] = 240836;
		index[0][343] = 241773;
		index[0][344] = 242733;
		index[0][345] = 243714;
		index[0][346] = 244766;
		index[0][347] = 245874;
		index[0][348] = 246950;
		index[0][349] = 247932;
		index[0][350] = 248909;
		index[0][351] = 249936;
		index[0][352] = 250796;
		index[0][353] = 251704;
		index[0][354] = 252699;
		index[0][355] = 253796;
		index[0][356] = 254935;
		index[0][357] = 256206;
		index[0][358] = 257454;
		index[0][359] = 258695;
	}

	/**
	 * Initializes the index in <tt>ac_p50.dat</tt>.
	 */
	private static void initialize_p50 ( ) {
		index[1][0] = 1413;
		index[1][1] = 2873;
		index[1][2] = 4356;
		index[1][3] = 5840;
		index[1][4] = 7351;
		index[1][5] = 8896;
		index[1][6] = 10451;
		index[1][7] = 12119;
		index[1][8] = 13629;
		index[1][9] = 15062;
		index[1][10] = 16590;
		index[1][11] = 18204;
		index[1][12] = 20046;
		index[1][13] = 22035;
		index[1][14] = 23806;
		index[1][15] = 25311;
		index[1][16] = 26533;
		index[1][17] = 27656;
		index[1][18] = 28968;
		index[1][19] = 30390;
		index[1][20] = 31713;
		index[1][21] = 33029;
		index[1][22] = 34344;
		index[1][23] = 35637;
		index[1][24] = 37134;
		index[1][25] = 38715;
		index[1][26] = 40584;
		index[1][27] = 42322;
		index[1][28] = 44030;
		index[1][29] = 45742;
		index[1][30] = 47383;
		index[1][31] = 49021;
		index[1][32] = 50526;
		index[1][33] = 52147;
		index[1][34] = 54040;
		index[1][35] = 55997;
		index[1][36] = 57831;
		index[1][37] = 59387;
		index[1][38] = 60648;
		index[1][39] = 61906;
		index[1][40] = 63122;
		index[1][41] = 64313;
		index[1][42] = 65212;
		index[1][43] = 65961;
		index[1][44] = 66609;
		index[1][45] = 67179;
		index[1][46] = 67710;
		index[1][47] = 68220;
		index[1][48] = 68764;
		index[1][49] = 69345;
		index[1][50] = 70016;
		index[1][51] = 70654;
		index[1][52] = 71303;
		index[1][53] = 71963;
		index[1][54] = 72606;
		index[1][55] = 73202;
		index[1][56] = 73822;
		index[1][57] = 74402;
		index[1][58] = 74966;
		index[1][59] = 75710;
		index[1][60] = 76533;
		index[1][61] = 77308;
		index[1][62] = 77946;
		index[1][63] = 78503;
		index[1][64] = 79013;
		index[1][65] = 79624;
		index[1][66] = 80157;
		index[1][67] = 80798;
		index[1][68] = 81399;
		index[1][69] = 82018;
		index[1][70] = 82675;
		index[1][71] = 83288;
		index[1][72] = 83837;
		index[1][73] = 84290;
		index[1][74] = 84768;
		index[1][75] = 85339;
		index[1][76] = 85923;
		index[1][77] = 86389;
		index[1][78] = 86895;
		index[1][79] = 87331;
		index[1][80] = 87786;
		index[1][81] = 88293;
		index[1][82] = 88741;
		index[1][83] = 89225;
		index[1][84] = 89761;
		index[1][85] = 90322;
		index[1][86] = 91004;
		index[1][87] = 91699;
		index[1][88] = 92315;
		index[1][89] = 93059;
		index[1][90] = 93734;
		index[1][91] = 94443;
		index[1][92] = 95160;
		index[1][93] = 95786;
		index[1][94] = 96481;
		index[1][95] = 97077;
		index[1][96] = 97679;
		index[1][97] = 98308;
		index[1][98] = 99086;
		index[1][99] = 99800;
		index[1][100] = 100513;
		index[1][101] = 101097;
		index[1][102] = 101740;
		index[1][103] = 102361;
		index[1][104] = 102996;
		index[1][105] = 103600;
		index[1][106] = 104176;
		index[1][107] = 104844;
		index[1][108] = 105532;
		index[1][109] = 106072;
		index[1][110] = 106585;
		index[1][111] = 107090;
		index[1][112] = 107636;
		index[1][113] = 108073;
		index[1][114] = 108533;
		index[1][115] = 109007;
		index[1][116] = 109489;
		index[1][117] = 109962;
		index[1][118] = 110442;
		index[1][119] = 110849;
		index[1][120] = 111268;
		index[1][121] = 111724;
		index[1][122] = 112222;
		index[1][123] = 112669;
		index[1][124] = 113093;
		index[1][125] = 113473;
		index[1][126] = 113841;
		index[1][127] = 114246;
		index[1][128] = 114592;
		index[1][129] = 114959;
		index[1][130] = 115294;
		index[1][131] = 115632;
		index[1][132] = 115905;
		index[1][133] = 116198;
		index[1][134] = 116520;
		index[1][135] = 116897;
		index[1][136] = 117260;
		index[1][137] = 117634;
		index[1][138] = 117960;
		index[1][139] = 118310;
		index[1][140] = 118621;
		index[1][141] = 118879;
		index[1][142] = 119170;
		index[1][143] = 119406;
		index[1][144] = 119674;
		index[1][145] = 119976;
		index[1][146] = 120256;
		index[1][147] = 120538;
		index[1][148] = 120855;
		index[1][149] = 121150;
		index[1][150] = 121471;
		index[1][151] = 121811;
		index[1][152] = 122144;
		index[1][153] = 122435;
		index[1][154] = 122684;
		index[1][155] = 122948;
		index[1][156] = 123231;
		index[1][157] = 123455;
		index[1][158] = 123675;
		index[1][159] = 123886;
		index[1][160] = 124108;
		index[1][161] = 124367;
		index[1][162] = 124645;
		index[1][163] = 124909;
		index[1][164] = 125192;
		index[1][165] = 125455;
		index[1][166] = 125704;
		index[1][167] = 125949;
		index[1][168] = 126185;
		index[1][169] = 126436;
		index[1][170] = 126718;
		index[1][171] = 126998;
		index[1][172] = 127257;
		index[1][173] = 127528;
		index[1][174] = 127788;
		index[1][175] = 128021;
		index[1][176] = 128246;
		index[1][177] = 128445;
		index[1][178] = 128663;
		index[1][179] = 128906;
		index[1][180] = 129116;
		index[1][181] = 129363;
		index[1][182] = 129604;
		index[1][183] = 129835;
		index[1][184] = 130039;
		index[1][185] = 130246;
		index[1][186] = 130449;
		index[1][187] = 130645;
		index[1][188] = 130861;
		index[1][189] = 131096;
		index[1][190] = 131302;
		index[1][191] = 131505;
		index[1][192] = 131712;
		index[1][193] = 131927;
		index[1][194] = 132160;
		index[1][195] = 132376;
		index[1][196] = 132595;
		index[1][197] = 132802;
		index[1][198] = 133026;
		index[1][199] = 133232;
		index[1][200] = 133422;
		index[1][201] = 133633;
		index[1][202] = 133848;
		index[1][203] = 134084;
		index[1][204] = 134333;
		index[1][205] = 134623;
		index[1][206] = 134865;
		index[1][207] = 135073;
		index[1][208] = 135298;
		index[1][209] = 135538;
		index[1][210] = 135781;
		index[1][211] = 136006;
		index[1][212] = 136219;
		index[1][213] = 136444;
		index[1][214] = 136728;
		index[1][215] = 137004;
		index[1][216] = 137281;
		index[1][217] = 137521;
		index[1][218] = 137818;
		index[1][219] = 138087;
		index[1][220] = 138330;
		index[1][221] = 138584;
		index[1][222] = 138861;
		index[1][223] = 139075;
		index[1][224] = 139294;
		index[1][225] = 139555;
		index[1][226] = 139834;
		index[1][227] = 140122;
		index[1][228] = 140392;
		index[1][229] = 140658;
		index[1][230] = 140910;
		index[1][231] = 141165;
		index[1][232] = 141456;
		index[1][233] = 141705;
		index[1][234] = 141994;
		index[1][235] = 142266;
		index[1][236] = 142516;
		index[1][237] = 142762;
		index[1][238] = 143059;
		index[1][239] = 143350;
		index[1][240] = 143651;
		index[1][241] = 143969;
		index[1][242] = 144262;
		index[1][243] = 144569;
		index[1][244] = 144893;
		index[1][245] = 145187;
		index[1][246] = 145534;
		index[1][247] = 145846;
		index[1][248] = 146130;
		index[1][249] = 146461;
		index[1][250] = 146764;
		index[1][251] = 147041;
		index[1][252] = 147339;
		index[1][253] = 147643;
		index[1][254] = 147994;
		index[1][255] = 148335;
		index[1][256] = 148665;
		index[1][257] = 148972;
		index[1][258] = 149272;
		index[1][259] = 149559;
		index[1][260] = 149819;
		index[1][261] = 150119;
		index[1][262] = 150411;
		index[1][263] = 150736;
		index[1][264] = 151133;
		index[1][265] = 151565;
		index[1][266] = 152049;
		index[1][267] = 152557;
		index[1][268] = 152952;
		index[1][269] = 153360;
		index[1][270] = 153815;
		index[1][271] = 154292;
		index[1][272] = 154792;
		index[1][273] = 155334;
		index[1][274] = 155870;
		index[1][275] = 156427;
		index[1][276] = 157000;
		index[1][277] = 157613;
		index[1][278] = 158173;
		index[1][279] = 158800;
		index[1][280] = 159441;
		index[1][281] = 160173;
		index[1][282] = 160978;
		index[1][283] = 161798;
		index[1][284] = 162500;
		index[1][285] = 163284;
		index[1][286] = 164059;
		index[1][287] = 164902;
		index[1][288] = 165713;
		index[1][289] = 166507;
		index[1][290] = 167158;
		index[1][291] = 167781;
		index[1][292] = 168543;
		index[1][293] = 169279;
		index[1][294] = 170184;
		index[1][295] = 171213;
		index[1][296] = 172282;
		index[1][297] = 173442;
		index[1][298] = 174552;
		index[1][299] = 175557;
		index[1][300] = 176453;
		index[1][301] = 177215;
		index[1][302] = 178090;
		index[1][303] = 179132;
		index[1][304] = 180115;
		index[1][305] = 180955;
		index[1][306] = 181891;
		index[1][307] = 182718;
		index[1][308] = 183560;
		index[1][309] = 184427;
		index[1][310] = 185153;
		index[1][311] = 185701;
		index[1][312] = 186149;
		index[1][313] = 186570;
		index[1][314] = 186973;
		index[1][315] = 187353;
		index[1][316] = 187809;
		index[1][317] = 188295;
		index[1][318] = 188843;
		index[1][319] = 189522;
		index[1][320] = 190264;
		index[1][321] = 190871;
		index[1][322] = 191585;
		index[1][323] = 192431;
		index[1][324] = 193648;
		index[1][325] = 194686;
		index[1][326] = 195652;
		index[1][327] = 196403;
		index[1][328] = 197437;
		index[1][329] = 198552;
		index[1][330] = 199758;
		index[1][331] = 201195;
		index[1][332] = 202940;
		index[1][333] = 204894;
		index[1][334] = 207232;
		index[1][335] = 209867;
		index[1][336] = 212205;
		index[1][337] = 213931;
		index[1][338] = 215469;
		index[1][339] = 216694;
		index[1][340] = 217972;
		index[1][341] = 219732;
		index[1][342] = 221437;
		index[1][343] = 223232;
		index[1][344] = 225083;
		index[1][345] = 226839;
		index[1][346] = 228423;
		index[1][347] = 229821;
		index[1][348] = 231045;
		index[1][349] = 232251;
		index[1][350] = 233419;
		index[1][351] = 234660;
		index[1][352] = 235837;
		index[1][353] = 237108;
		index[1][354] = 238204;
		index[1][355] = 239417;
		index[1][356] = 240660;
		index[1][357] = 242037;
		index[1][358] = 243339;
		index[1][359] = 244781;
	}

	/**
	 * Initializes the index in <tt>ac_p40.dat</tt>.
	 */
	private static void initialize_p40 ( ) {
		index[2][0] = 729;
		index[2][1] = 1438;
		index[2][2] = 2217;
		index[2][3] = 2982;
		index[2][4] = 3924;
		index[2][5] = 4877;
		index[2][6] = 5686;
		index[2][7] = 6403;
		index[2][8] = 7114;
		index[2][9] = 7863;
		index[2][10] = 8573;
		index[2][11] = 9312;
		index[2][12] = 9996;
		index[2][13] = 10724;
		index[2][14] = 11511;
		index[2][15] = 12256;
		index[2][16] = 13051;
		index[2][17] = 13955;
		index[2][18] = 14819;
		index[2][19] = 15593;
		index[2][20] = 16416;
		index[2][21] = 17405;
		index[2][22] = 18361;
		index[2][23] = 19287;
		index[2][24] = 20256;
		index[2][25] = 21119;
		index[2][26] = 22046;
		index[2][27] = 22872;
		index[2][28] = 23731;
		index[2][29] = 24573;
		index[2][30] = 25365;
		index[2][31] = 26235;
		index[2][32] = 27129;
		index[2][33] = 28022;
		index[2][34] = 28856;
		index[2][35] = 29677;
		index[2][36] = 30375;
		index[2][37] = 31175;
		index[2][38] = 32115;
		index[2][39] = 33063;
		index[2][40] = 34091;
		index[2][41] = 34970;
		index[2][42] = 35920;
		index[2][43] = 36789;
		index[2][44] = 37552;
		index[2][45] = 38208;
		index[2][46] = 38829;
		index[2][47] = 39409;
		index[2][48] = 40030;
		index[2][49] = 40674;
		index[2][50] = 41405;
		index[2][51] = 42131;
		index[2][52] = 42802;
		index[2][53] = 43462;
		index[2][54] = 43998;
		index[2][55] = 44565;
		index[2][56] = 45076;
		index[2][57] = 45545;
		index[2][58] = 46000;
		index[2][59] = 46464;
		index[2][60] = 46885;
		index[2][61] = 47321;
		index[2][62] = 47782;
		index[2][63] = 48196;
		index[2][64] = 48643;
		index[2][65] = 49134;
		index[2][66] = 49636;
		index[2][67] = 50075;
		index[2][68] = 50564;
		index[2][69] = 50999;
		index[2][70] = 51320;
		index[2][71] = 51815;
		index[2][72] = 52513;
		index[2][73] = 53181;
		index[2][74] = 53985;
		index[2][75] = 54766;
		index[2][76] = 55561;
		index[2][77] = 56285;
		index[2][78] = 56945;
		index[2][79] = 57582;
		index[2][80] = 58162;
		index[2][81] = 58676;
		index[2][82] = 59205;
		index[2][83] = 59829;
		index[2][84] = 60347;
		index[2][85] = 60755;
		index[2][86] = 61172;
		index[2][87] = 61588;
		index[2][88] = 62064;
		index[2][89] = 62558;
		index[2][90] = 63047;
		index[2][91] = 63621;
		index[2][92] = 64180;
		index[2][93] = 64814;
		index[2][94] = 65478;
		index[2][95] = 66058;
		index[2][96] = 66566;
		index[2][97] = 67016;
		index[2][98] = 67439;
		index[2][99] = 67893;
		index[2][100] = 68331;
		index[2][101] = 68833;
		index[2][102] = 69308;
		index[2][103] = 69648;
		index[2][104] = 70032;
		index[2][105] = 70413;
		index[2][106] = 70872;
		index[2][107] = 71302;
		index[2][108] = 71675;
		index[2][109] = 72007;
		index[2][110] = 72316;
		index[2][111] = 72639;
		index[2][112] = 73004;
		index[2][113] = 73316;
		index[2][114] = 73597;
		index[2][115] = 73904;
		index[2][116] = 74216;
		index[2][117] = 74530;
		index[2][118] = 74855;
		index[2][119] = 75117;
		index[2][120] = 75402;
		index[2][121] = 75666;
		index[2][122] = 75912;
		index[2][123] = 76159;
		index[2][124] = 76391;
		index[2][125] = 76660;
		index[2][126] = 76898;
		index[2][127] = 77146;
		index[2][128] = 77372;
		index[2][129] = 77632;
		index[2][130] = 77904;
		index[2][131] = 78202;
		index[2][132] = 78408;
		index[2][133] = 78652;
		index[2][134] = 78891;
		index[2][135] = 79151;
		index[2][136] = 79398;
		index[2][137] = 79673;
		index[2][138] = 79952;
		index[2][139] = 80172;
		index[2][140] = 80395;
		index[2][141] = 80617;
		index[2][142] = 80867;
		index[2][143] = 81102;
		index[2][144] = 81350;
		index[2][145] = 81559;
		index[2][146] = 81771;
		index[2][147] = 81945;
		index[2][148] = 82147;
		index[2][149] = 82337;
		index[2][150] = 82543;
		index[2][151] = 82735;
		index[2][152] = 82934;
		index[2][153] = 83127;
		index[2][154] = 83328;
		index[2][155] = 83532;
		index[2][156] = 83748;
		index[2][157] = 84005;
		index[2][158] = 84261;
		index[2][159] = 84463;
		index[2][160] = 84626;
		index[2][161] = 84802;
		index[2][162] = 84987;
		index[2][163] = 85207;
		index[2][164] = 85409;
		index[2][165] = 85605;
		index[2][166] = 85774;
		index[2][167] = 85924;
		index[2][168] = 86062;
		index[2][169] = 86214;
		index[2][170] = 86366;
		index[2][171] = 86517;
		index[2][172] = 86725;
		index[2][173] = 86864;
		index[2][174] = 87015;
		index[2][175] = 87154;
		index[2][176] = 87295;
		index[2][177] = 87453;
		index[2][178] = 87618;
		index[2][179] = 87785;
		index[2][180] = 87946;
		index[2][181] = 88094;
		index[2][182] = 88251;
		index[2][183] = 88393;
		index[2][184] = 88535;
		index[2][185] = 88668;
		index[2][186] = 88814;
		index[2][187] = 88975;
		index[2][188] = 89137;
		index[2][189] = 89303;
		index[2][190] = 89442;
		index[2][191] = 89592;
		index[2][192] = 89739;
		index[2][193] = 89913;
		index[2][194] = 90102;
		index[2][195] = 90271;
		index[2][196] = 90459;
		index[2][197] = 90646;
		index[2][198] = 90808;
		index[2][199] = 90955;
		index[2][200] = 91109;
		index[2][201] = 91275;
		index[2][202] = 91441;
		index[2][203] = 91624;
		index[2][204] = 91804;
		index[2][205] = 91991;
		index[2][206] = 92167;
		index[2][207] = 92360;
		index[2][208] = 92550;
		index[2][209] = 92762;
		index[2][210] = 92996;
		index[2][211] = 93226;
		index[2][212] = 93463;
		index[2][213] = 93735;
		index[2][214] = 93977;
		index[2][215] = 94257;
		index[2][216] = 94538;
		index[2][217] = 94821;
		index[2][218] = 95075;
		index[2][219] = 95315;
		index[2][220] = 95578;
		index[2][221] = 95820;
		index[2][222] = 96035;
		index[2][223] = 96255;
		index[2][224] = 96505;
		index[2][225] = 96733;
		index[2][226] = 96985;
		index[2][227] = 97232;
		index[2][228] = 97445;
		index[2][229] = 97679;
		index[2][230] = 97915;
		index[2][231] = 98136;
		index[2][232] = 98388;
		index[2][233] = 98624;
		index[2][234] = 98876;
		index[2][235] = 99082;
		index[2][236] = 99344;
		index[2][237] = 99564;
		index[2][238] = 99789;
		index[2][239] = 100036;
		index[2][240] = 100325;
		index[2][241] = 100604;
		index[2][242] = 100905;
		index[2][243] = 101206;
		index[2][244] = 101474;
		index[2][245] = 101762;
		index[2][246] = 102091;
		index[2][247] = 102439;
		index[2][248] = 102737;
		index[2][249] = 103060;
		index[2][250] = 103387;
		index[2][251] = 103726;
		index[2][252] = 104044;
		index[2][253] = 104387;
		index[2][254] = 104792;
		index[2][255] = 105197;
		index[2][256] = 105580;
		index[2][257] = 105966;
		index[2][258] = 106314;
		index[2][259] = 106720;
		index[2][260] = 107135;
		index[2][261] = 107612;
		index[2][262] = 108088;
		index[2][263] = 108515;
		index[2][264] = 108983;
		index[2][265] = 109419;
		index[2][266] = 109901;
		index[2][267] = 110375;
		index[2][268] = 110837;
		index[2][269] = 111343;
		index[2][270] = 111863;
		index[2][271] = 112354;
		index[2][272] = 112809;
		index[2][273] = 113293;
		index[2][274] = 113852;
		index[2][275] = 114354;
		index[2][276] = 114958;
		index[2][277] = 115596;
		index[2][278] = 116245;
		index[2][279] = 116886;
		index[2][280] = 117611;
		index[2][281] = 118307;
		index[2][282] = 118970;
		index[2][283] = 119617;
		index[2][284] = 120365;
		index[2][285] = 121153;
		index[2][286] = 121897;
		index[2][287] = 122741;
		index[2][288] = 123623;
		index[2][289] = 124598;
		index[2][290] = 125499;
		index[2][291] = 126515;
		index[2][292] = 127740;
		index[2][293] = 129134;
		index[2][294] = 130511;
		index[2][295] = 131873;
		index[2][296] = 133279;
		index[2][297] = 134735;
		index[2][298] = 136368;
		index[2][299] = 138008;
		index[2][300] = 139651;
		index[2][301] = 141116;
		index[2][302] = 142614;
		index[2][303] = 144084;
		index[2][304] = 145659;
		index[2][305] = 147141;
		index[2][306] = 148549;
		index[2][307] = 149987;
		index[2][308] = 151301;
		index[2][309] = 152454;
		index[2][310] = 153797;
		index[2][311] = 155233;
		index[2][312] = 156591;
		index[2][313] = 157925;
		index[2][314] = 159347;
		index[2][315] = 160458;
		index[2][316] = 162093;
		index[2][317] = 163742;
		index[2][318] = 164995;
		index[2][319] = 166063;
		index[2][320] = 167033;
		index[2][321] = 168142;
		index[2][322] = 169585;
		index[2][323] = 170963;
		index[2][324] = 172215;
		index[2][325] = 173289;
		index[2][326] = 174327;
		index[2][327] = 175410;
		index[2][328] = 176560;
		index[2][329] = 177705;
		index[2][330] = 178839;
		index[2][331] = 179845;
		index[2][332] = 180878;
		index[2][333] = 181947;
		index[2][334] = 182839;
		index[2][335] = 183736;
		index[2][336] = 184621;
		index[2][337] = 185506;
		index[2][338] = 186357;
		index[2][339] = 187181;
		index[2][340] = 187980;
		index[2][341] = 188812;
		index[2][342] = 189699;
		index[2][343] = 190573;
		index[2][344] = 191438;
		index[2][345] = 192181;
		index[2][346] = 192907;
		index[2][347] = 193589;
		index[2][348] = 194404;
		index[2][349] = 195270;
		index[2][350] = 196115;
		index[2][351] = 196953;
		index[2][352] = 197692;
		index[2][353] = 198445;
		index[2][354] = 199197;
		index[2][355] = 199980;
		index[2][356] = 200894;
		index[2][357] = 201816;
		index[2][358] = 202615;
		index[2][359] = 203383;
	}

	/**
	 * Initializes the index in <tt>ac_p30.dat</tt>.
	 */
	private static void initialize_p30 ( ) {
		index[3][0] = 937;
		index[3][1] = 1774;
		index[3][2] = 2637;
		index[3][3] = 3549;
		index[3][4] = 4499;
		index[3][5] = 5518;
		index[3][6] = 6496;
		index[3][7] = 7519;
		index[3][8] = 8522;
		index[3][9] = 9498;
		index[3][10] = 10465;
		index[3][11] = 11399;
		index[3][12] = 12367;
		index[3][13] = 13295;
		index[3][14] = 14312;
		index[3][15] = 15344;
		index[3][16] = 16400;
		index[3][17] = 17517;
		index[3][18] = 18542;
		index[3][19] = 19592;
		index[3][20] = 20542;
		index[3][21] = 21527;
		index[3][22] = 22510;
		index[3][23] = 23493;
		index[3][24] = 24451;
		index[3][25] = 25368;
		index[3][26] = 26302;
		index[3][27] = 27285;
		index[3][28] = 28332;
		index[3][29] = 29436;
		index[3][30] = 30446;
		index[3][31] = 31404;
		index[3][32] = 32357;
		index[3][33] = 33245;
		index[3][34] = 34159;
		index[3][35] = 35132;
		index[3][36] = 36092;
		index[3][37] = 37245;
		index[3][38] = 38274;
		index[3][39] = 39299;
		index[3][40] = 40292;
		index[3][41] = 41328;
		index[3][42] = 42386;
		index[3][43] = 43576;
		index[3][44] = 44841;
		index[3][45] = 45952;
		index[3][46] = 46913;
		index[3][47] = 47927;
		index[3][48] = 48903;
		index[3][49] = 49776;
		index[3][50] = 50788;
		index[3][51] = 51712;
		index[3][52] = 52549;
		index[3][53] = 53202;
		index[3][54] = 53853;
		index[3][55] = 54521;
		index[3][56] = 55233;
		index[3][57] = 55993;
		index[3][58] = 56752;
		index[3][59] = 57635;
		index[3][60] = 58784;
		index[3][61] = 60025;
		index[3][62] = 61032;
		index[3][63] = 62052;
		index[3][64] = 63038;
		index[3][65] = 64089;
		index[3][66] = 65065;
		index[3][67] = 65989;
		index[3][68] = 67066;
		index[3][69] = 67875;
		index[3][70] = 68665;
		index[3][71] = 69277;
		index[3][72] = 69975;
		index[3][73] = 70751;
		index[3][74] = 71666;
		index[3][75] = 72868;
		index[3][76] = 74435;
		index[3][77] = 76508;
		index[3][78] = 78920;
		index[3][79] = 81309;
		index[3][80] = 83775;
		index[3][81] = 85902;
		index[3][82] = 88248;
		index[3][83] = 90156;
		index[3][84] = 92205;
		index[3][85] = 94114;
		index[3][86] = 96375;
		index[3][87] = 98457;
		index[3][88] = 100648;
		index[3][89] = 102231;
		index[3][90] = 103741;
		index[3][91] = 105465;
		index[3][92] = 107437;
		index[3][93] = 109072;
		index[3][94] = 110556;
		index[3][95] = 111976;
		index[3][96] = 113614;
		index[3][97] = 115181;
		index[3][98] = 116780;
		index[3][99] = 118356;
		index[3][100] = 119863;
		index[3][101] = 121223;
		index[3][102] = 122579;
		index[3][103] = 124011;
		index[3][104] = 125320;
		index[3][105] = 126655;
		index[3][106] = 127807;
		index[3][107] = 129090;
		index[3][108] = 130330;
		index[3][109] = 131506;
		index[3][110] = 132703;
		index[3][111] = 133833;
		index[3][112] = 134992;
		index[3][113] = 136068;
		index[3][114] = 137128;
		index[3][115] = 138105;
		index[3][116] = 139077;
		index[3][117] = 140067;
		index[3][118] = 140989;
		index[3][119] = 141857;
		index[3][120] = 142635;
		index[3][121] = 143370;
		index[3][122] = 144003;
		index[3][123] = 144635;
		index[3][124] = 145283;
		index[3][125] = 145928;
		index[3][126] = 146604;
		index[3][127] = 147217;
		index[3][128] = 147874;
		index[3][129] = 148566;
		index[3][130] = 149205;
		index[3][131] = 149809;
		index[3][132] = 150387;
		index[3][133] = 150900;
		index[3][134] = 151438;
		index[3][135] = 151960;
		index[3][136] = 152429;
		index[3][137] = 152892;
		index[3][138] = 153387;
		index[3][139] = 153868;
		index[3][140] = 154364;
		index[3][141] = 154898;
		index[3][142] = 155341;
		index[3][143] = 155794;
		index[3][144] = 156156;
		index[3][145] = 156592;
		index[3][146] = 157063;
		index[3][147] = 157552;
		index[3][148] = 158066;
		index[3][149] = 158579;
		index[3][150] = 159040;
		index[3][151] = 159454;
		index[3][152] = 159873;
		index[3][153] = 160306;
		index[3][154] = 160664;
		index[3][155] = 161001;
		index[3][156] = 161407;
		index[3][157] = 161832;
		index[3][158] = 162238;
		index[3][159] = 162536;
		index[3][160] = 162848;
		index[3][161] = 163219;
		index[3][162] = 163560;
		index[3][163] = 163939;
		index[3][164] = 164313;
		index[3][165] = 164658;
		index[3][166] = 165007;
		index[3][167] = 165333;
		index[3][168] = 165671;
		index[3][169] = 166002;
		index[3][170] = 166362;
		index[3][171] = 166685;
		index[3][172] = 167048;
		index[3][173] = 167435;
		index[3][174] = 167776;
		index[3][175] = 168196;
		index[3][176] = 168545;
		index[3][177] = 168842;
		index[3][178] = 169164;
		index[3][179] = 169548;
		index[3][180] = 169885;
		index[3][181] = 170202;
		index[3][182] = 170536;
		index[3][183] = 170850;
		index[3][184] = 171225;
		index[3][185] = 171576;
		index[3][186] = 171983;
		index[3][187] = 172361;
		index[3][188] = 172749;
		index[3][189] = 173081;
		index[3][190] = 173400;
		index[3][191] = 173726;
		index[3][192] = 174041;
		index[3][193] = 174375;
		index[3][194] = 174724;
		index[3][195] = 175096;
		index[3][196] = 175437;
		index[3][197] = 175753;
		index[3][198] = 176050;
		index[3][199] = 176383;
		index[3][200] = 176713;
		index[3][201] = 177018;
		index[3][202] = 177320;
		index[3][203] = 177662;
		index[3][204] = 178017;
		index[3][205] = 178346;
		index[3][206] = 178703;
		index[3][207] = 179076;
		index[3][208] = 179476;
		index[3][209] = 179840;
		index[3][210] = 180202;
		index[3][211] = 180549;
		index[3][212] = 180891;
		index[3][213] = 181294;
		index[3][214] = 181646;
		index[3][215] = 182012;
		index[3][216] = 182356;
		index[3][217] = 182719;
		index[3][218] = 183118;
		index[3][219] = 183486;
		index[3][220] = 183852;
		index[3][221] = 184261;
		index[3][222] = 184708;
		index[3][223] = 185063;
		index[3][224] = 185439;
		index[3][225] = 185810;
		index[3][226] = 186204;
		index[3][227] = 186586;
		index[3][228] = 186986;
		index[3][229] = 187372;
		index[3][230] = 187774;
		index[3][231] = 188169;
		index[3][232] = 188596;
		index[3][233] = 188993;
		index[3][234] = 189446;
		index[3][235] = 189916;
		index[3][236] = 190389;
		index[3][237] = 190911;
		index[3][238] = 191356;
		index[3][239] = 191865;
		index[3][240] = 192381;
		index[3][241] = 192857;
		index[3][242] = 193386;
		index[3][243] = 193909;
		index[3][244] = 194424;
		index[3][245] = 194973;
		index[3][246] = 195628;
		index[3][247] = 196217;
		index[3][248] = 196941;
		index[3][249] = 197714;
		index[3][250] = 198421;
		index[3][251] = 199078;
		index[3][252] = 199770;
		index[3][253] = 200484;
		index[3][254] = 201247;
		index[3][255] = 202004;
		index[3][256] = 202730;
		index[3][257] = 203560;
		index[3][258] = 204355;
		index[3][259] = 205095;
		index[3][260] = 205982;
		index[3][261] = 206907;
		index[3][262] = 207928;
		index[3][263] = 208883;
		index[3][264] = 209863;
		index[3][265] = 210968;
		index[3][266] = 212066;
		index[3][267] = 213175;
		index[3][268] = 214288;
		index[3][269] = 215476;
		index[3][270] = 216659;
		index[3][271] = 217807;
		index[3][272] = 219024;
		index[3][273] = 220379;
		index[3][274] = 221656;
		index[3][275] = 223052;
		index[3][276] = 224358;
		index[3][277] = 225622;
		index[3][278] = 227000;
		index[3][279] = 228362;
		index[3][280] = 229677;
		index[3][281] = 230928;
		index[3][282] = 232430;
		index[3][283] = 234068;
		index[3][284] = 235807;
		index[3][285] = 237813;
		index[3][286] = 239766;
		index[3][287] = 241707;
		index[3][288] = 243602;
		index[3][289] = 246171;
		index[3][290] = 248902;
		index[3][291] = 251597;
		index[3][292] = 254671;
		index[3][293] = 257939;
		index[3][294] = 261700;
		index[3][295] = 265181;
		index[3][296] = 268921;
		index[3][297] = 272587;
		index[3][298] = 276209;
		index[3][299] = 279637;
		index[3][300] = 282743;
		index[3][301] = 285950;
		index[3][302] = 289008;
		index[3][303] = 291682;
		index[3][304] = 293876;
		index[3][305] = 296143;
		index[3][306] = 298326;
		index[3][307] = 300581;
		index[3][308] = 302759;
		index[3][309] = 304595;
		index[3][310] = 306784;
		index[3][311] = 309325;
		index[3][312] = 312062;
		index[3][313] = 315043;
		index[3][314] = 317666;
		index[3][315] = 319916;
		index[3][316] = 322277;
		index[3][317] = 324995;
		index[3][318] = 327631;
		index[3][319] = 329974;
		index[3][320] = 331852;
		index[3][321] = 333561;
		index[3][322] = 335097;
		index[3][323] = 336575;
		index[3][324] = 338165;
		index[3][325] = 339844;
		index[3][326] = 341223;
		index[3][327] = 342537;
		index[3][328] = 343823;
		index[3][329] = 345281;
		index[3][330] = 346724;
		index[3][331] = 348392;
		index[3][332] = 349741;
		index[3][333] = 351114;
		index[3][334] = 352393;
		index[3][335] = 353804;
		index[3][336] = 355266;
		index[3][337] = 356668;
		index[3][338] = 358124;
		index[3][339] = 359487;
		index[3][340] = 360872;
		index[3][341] = 362129;
		index[3][342] = 363393;
		index[3][343] = 364630;
		index[3][344] = 365822;
		index[3][345] = 366981;
		index[3][346] = 368119;
		index[3][347] = 369197;
		index[3][348] = 370182;
		index[3][349] = 371204;
		index[3][350] = 372289;
		index[3][351] = 373259;
		index[3][352] = 374144;
		index[3][353] = 375021;
		index[3][354] = 375937;
		index[3][355] = 376842;
		index[3][356] = 377774;
		index[3][357] = 378830;
		index[3][358] = 379865;
		index[3][359] = 380755;
	}

	/**
	 * Initializes the index in <tt>ac_p20.dat</tt>.
	 */
	private static void initialize_p20 ( ) {
		index[4][0] = 633;
		index[4][1] = 1212;
		index[4][2] = 1811;
		index[4][3] = 2561;
		index[4][4] = 3337;
		index[4][5] = 4135;
		index[4][6] = 4831;
		index[4][7] = 5450;
		index[4][8] = 6073;
		index[4][9] = 6821;
		index[4][10] = 7659;
		index[4][11] = 8368;
		index[4][12] = 9042;
		index[4][13] = 9761;
		index[4][14] = 10424;
		index[4][15] = 11210;
		index[4][16] = 11976;
		index[4][17] = 12797;
		index[4][18] = 13599;
		index[4][19] = 14279;
		index[4][20] = 14932;
		index[4][21] = 15569;
		index[4][22] = 16148;
		index[4][23] = 16720;
		index[4][24] = 17208;
		index[4][25] = 17724;
		index[4][26] = 18266;
		index[4][27] = 18835;
		index[4][28] = 19402;
		index[4][29] = 20028;
		index[4][30] = 20672;
		index[4][31] = 21308;
		index[4][32] = 21929;
		index[4][33] = 22500;
		index[4][34] = 23107;
		index[4][35] = 23781;
		index[4][36] = 24443;
		index[4][37] = 25029;
		index[4][38] = 25611;
		index[4][39] = 26211;
		index[4][40] = 26713;
		index[4][41] = 27173;
		index[4][42] = 27721;
		index[4][43] = 28317;
		index[4][44] = 28875;
		index[4][45] = 29339;
		index[4][46] = 29887;
		index[4][47] = 30432;
		index[4][48] = 30915;
		index[4][49] = 31333;
		index[4][50] = 31780;
		index[4][51] = 32283;
		index[4][52] = 32850;
		index[4][53] = 33460;
		index[4][54] = 34136;
		index[4][55] = 34823;
		index[4][56] = 35720;
		index[4][57] = 36576;
		index[4][58] = 37268;
		index[4][59] = 37965;
		index[4][60] = 38723;
		index[4][61] = 39449;
		index[4][62] = 40008;
		index[4][63] = 40521;
		index[4][64] = 41002;
		index[4][65] = 41572;
		index[4][66] = 42050;
		index[4][67] = 42473;
		index[4][68] = 42804;
		index[4][69] = 43213;
		index[4][70] = 43791;
		index[4][71] = 44318;
		index[4][72] = 44777;
		index[4][73] = 45425;
		index[4][74] = 46129;
		index[4][75] = 46962;
		index[4][76] = 47757;
		index[4][77] = 48539;
		index[4][78] = 49265;
		index[4][79] = 49995;
		index[4][80] = 50806;
		index[4][81] = 51590;
		index[4][82] = 52469;
		index[4][83] = 53927;
		index[4][84] = 55666;
		index[4][85] = 57434;
		index[4][86] = 59617;
		index[4][87] = 61922;
		index[4][88] = 64013;
		index[4][89] = 66421;
		index[4][90] = 69368;
		index[4][91] = 72562;
		index[4][92] = 75100;
		index[4][93] = 76732;
		index[4][94] = 78704;
		index[4][95] = 80820;
		index[4][96] = 82898;
		index[4][97] = 84956;
		index[4][98] = 86995;
		index[4][99] = 88928;
		index[4][100] = 90899;
		index[4][101] = 92968;
		index[4][102] = 94890;
		index[4][103] = 96914;
		index[4][104] = 99080;
		index[4][105] = 101142;
		index[4][106] = 102919;
		index[4][107] = 104531;
		index[4][108] = 105890;
		index[4][109] = 107190;
		index[4][110] = 108810;
		index[4][111] = 110193;
		index[4][112] = 111521;
		index[4][113] = 112912;
		index[4][114] = 114249;
		index[4][115] = 115389;
		index[4][116] = 116454;
		index[4][117] = 117527;
		index[4][118] = 118552;
		index[4][119] = 119620;
		index[4][120] = 120749;
		index[4][121] = 121960;
		index[4][122] = 123044;
		index[4][123] = 123973;
		index[4][124] = 125004;
		index[4][125] = 125998;
		index[4][126] = 126800;
		index[4][127] = 127413;
		index[4][128] = 128052;
		index[4][129] = 128736;
		index[4][130] = 129497;
		index[4][131] = 130184;
		index[4][132] = 130856;
		index[4][133] = 131553;
		index[4][134] = 132259;
		index[4][135] = 132858;
		index[4][136] = 133393;
		index[4][137] = 133899;
		index[4][138] = 134506;
		index[4][139] = 135111;
		index[4][140] = 135713;
		index[4][141] = 136274;
		index[4][142] = 136784;
		index[4][143] = 137311;
		index[4][144] = 137903;
		index[4][145] = 138519;
		index[4][146] = 139115;
		index[4][147] = 139692;
		index[4][148] = 140242;
		index[4][149] = 140863;
		index[4][150] = 141524;
		index[4][151] = 142136;
		index[4][152] = 142698;
		index[4][153] = 143225;
		index[4][154] = 143772;
		index[4][155] = 144260;
		index[4][156] = 144807;
		index[4][157] = 145359;
		index[4][158] = 145826;
		index[4][159] = 146287;
		index[4][160] = 146699;
		index[4][161] = 147109;
		index[4][162] = 147547;
		index[4][163] = 147981;
		index[4][164] = 148387;
		index[4][165] = 148770;
		index[4][166] = 149192;
		index[4][167] = 149492;
		index[4][168] = 149857;
		index[4][169] = 150262;
		index[4][170] = 150687;
		index[4][171] = 151079;
		index[4][172] = 151430;
		index[4][173] = 151721;
		index[4][174] = 152035;
		index[4][175] = 152367;
		index[4][176] = 152795;
		index[4][177] = 153201;
		index[4][178] = 153597;
		index[4][179] = 153965;
		index[4][180] = 154442;
		index[4][181] = 154906;
		index[4][182] = 155318;
		index[4][183] = 155658;
		index[4][184] = 156091;
		index[4][185] = 156506;
		index[4][186] = 156934;
		index[4][187] = 157330;
		index[4][188] = 157750;
		index[4][189] = 158111;
		index[4][190] = 158574;
		index[4][191] = 159085;
		index[4][192] = 159548;
		index[4][193] = 159986;
		index[4][194] = 160382;
		index[4][195] = 160781;
		index[4][196] = 161155;
		index[4][197] = 161537;
		index[4][198] = 161917;
		index[4][199] = 162257;
		index[4][200] = 162643;
		index[4][201] = 163025;
		index[4][202] = 163467;
		index[4][203] = 163961;
		index[4][204] = 164481;
		index[4][205] = 164973;
		index[4][206] = 165420;
		index[4][207] = 165925;
		index[4][208] = 166432;
		index[4][209] = 166884;
		index[4][210] = 167322;
		index[4][211] = 167750;
		index[4][212] = 168155;
		index[4][213] = 168588;
		index[4][214] = 168978;
		index[4][215] = 169431;
		index[4][216] = 169902;
		index[4][217] = 170335;
		index[4][218] = 170741;
		index[4][219] = 171141;
		index[4][220] = 171521;
		index[4][221] = 171900;
		index[4][222] = 172297;
		index[4][223] = 172719;
		index[4][224] = 173130;
		index[4][225] = 173514;
		index[4][226] = 173975;
		index[4][227] = 174447;
		index[4][228] = 174825;
		index[4][229] = 175206;
		index[4][230] = 175669;
		index[4][231] = 176157;
		index[4][232] = 176649;
		index[4][233] = 177083;
		index[4][234] = 177586;
		index[4][235] = 178110;
		index[4][236] = 178656;
		index[4][237] = 179180;
		index[4][238] = 179730;
		index[4][239] = 180306;
		index[4][240] = 180872;
		index[4][241] = 181406;
		index[4][242] = 181922;
		index[4][243] = 182433;
		index[4][244] = 183024;
		index[4][245] = 183672;
		index[4][246] = 184280;
		index[4][247] = 184844;
		index[4][248] = 185525;
		index[4][249] = 186182;
		index[4][250] = 186848;
		index[4][251] = 187622;
		index[4][252] = 188246;
		index[4][253] = 189065;
		index[4][254] = 190063;
		index[4][255] = 190997;
		index[4][256] = 191947;
		index[4][257] = 192946;
		index[4][258] = 194015;
		index[4][259] = 195057;
		index[4][260] = 196183;
		index[4][261] = 197336;
		index[4][262] = 198498;
		index[4][263] = 199732;
		index[4][264] = 201432;
		index[4][265] = 203125;
		index[4][266] = 204737;
		index[4][267] = 206589;
		index[4][268] = 208237;
		index[4][269] = 209751;
		index[4][270] = 211087;
		index[4][271] = 212570;
		index[4][272] = 214164;
		index[4][273] = 215629;
		index[4][274] = 216982;
		index[4][275] = 218395;
		index[4][276] = 220111;
		index[4][277] = 222029;
		index[4][278] = 223841;
		index[4][279] = 225529;
		index[4][280] = 227452;
		index[4][281] = 229569;
		index[4][282] = 231528;
		index[4][283] = 233376;
		index[4][284] = 235201;
		index[4][285] = 236934;
		index[4][286] = 238681;
		index[4][287] = 240688;
		index[4][288] = 242575;
		index[4][289] = 244132;
		index[4][290] = 245939;
		index[4][291] = 247727;
		index[4][292] = 249214;
		index[4][293] = 251227;
		index[4][294] = 253723;
		index[4][295] = 256329;
		index[4][296] = 259153;
		index[4][297] = 261857;
		index[4][298] = 264145;
		index[4][299] = 266660;
		index[4][300] = 269659;
		index[4][301] = 272603;
		index[4][302] = 275509;
		index[4][303] = 278266;
		index[4][304] = 280883;
		index[4][305] = 283653;
		index[4][306] = 286155;
		index[4][307] = 288347;
		index[4][308] = 290789;
		index[4][309] = 293267;
		index[4][310] = 296017;
		index[4][311] = 298649;
		index[4][312] = 300801;
		index[4][313] = 302637;
		index[4][314] = 304451;
		index[4][315] = 306245;
		index[4][316] = 308083;
		index[4][317] = 309873;
		index[4][318] = 311563;
		index[4][319] = 313091;
		index[4][320] = 314826;
		index[4][321] = 316412;
		index[4][322] = 317943;
		index[4][323] = 319386;
		index[4][324] = 320734;
		index[4][325] = 322043;
		index[4][326] = 323378;
		index[4][327] = 324639;
		index[4][328] = 326088;
		index[4][329] = 327454;
		index[4][330] = 328727;
		index[4][331] = 329886;
		index[4][332] = 331098;
		index[4][333] = 332409;
		index[4][334] = 333719;
		index[4][335] = 334937;
		index[4][336] = 335942;
		index[4][337] = 337015;
		index[4][338] = 338242;
		index[4][339] = 339339;
		index[4][340] = 340389;
		index[4][341] = 341379;
		index[4][342] = 342240;
		index[4][343] = 343041;
		index[4][344] = 343822;
		index[4][345] = 344581;
		index[4][346] = 345243;
		index[4][347] = 345924;
		index[4][348] = 346633;
		index[4][349] = 347372;
		index[4][350] = 348101;
		index[4][351] = 348814;
		index[4][352] = 349531;
		index[4][353] = 350158;
		index[4][354] = 350723;
		index[4][355] = 351288;
		index[4][356] = 351969;
		index[4][357] = 352685;
		index[4][358] = 353389;
		index[4][359] = 354026;
	}

	/**
	 * Initializes the index in <tt>ac_p10.dat</tt>.
	 */
	private static void initialize_p10 ( ) {
		index[5][0] = 549;
		index[5][1] = 1065;
		index[5][2] = 1604;
		index[5][3] = 2131;
		index[5][4] = 2647;
		index[5][5] = 3154;
		index[5][6] = 3717;
		index[5][7] = 4304;
		index[5][8] = 4793;
		index[5][9] = 5245;
		index[5][10] = 5762;
		index[5][11] = 6247;
		index[5][12] = 6647;
		index[5][13] = 7130;
		index[5][14] = 7667;
		index[5][15] = 8128;
		index[5][16] = 8609;
		index[5][17] = 9068;
		index[5][18] = 9613;
		index[5][19] = 10199;
		index[5][20] = 10750;
		index[5][21] = 11193;
		index[5][22] = 11682;
		index[5][23] = 12144;
		index[5][24] = 12667;
		index[5][25] = 13194;
		index[5][26] = 13688;
		index[5][27] = 14157;
		index[5][28] = 14610;
		index[5][29] = 15066;
		index[5][30] = 15613;
		index[5][31] = 16150;
		index[5][32] = 16634;
		index[5][33] = 17071;
		index[5][34] = 17543;
		index[5][35] = 18072;
		index[5][36] = 18571;
		index[5][37] = 19007;
		index[5][38] = 19477;
		index[5][39] = 19921;
		index[5][40] = 20399;
		index[5][41] = 20869;
		index[5][42] = 21463;
		index[5][43] = 22008;
		index[5][44] = 22463;
		index[5][45] = 22902;
		index[5][46] = 23323;
		index[5][47] = 23745;
		index[5][48] = 24154;
		index[5][49] = 24534;
		index[5][50] = 24848;
		index[5][51] = 25269;
		index[5][52] = 25777;
		index[5][53] = 26149;
		index[5][54] = 26584;
		index[5][55] = 27063;
		index[5][56] = 27532;
		index[5][57] = 28023;
		index[5][58] = 28515;
		index[5][59] = 29035;
		index[5][60] = 29494;
		index[5][61] = 29882;
		index[5][62] = 30290;
		index[5][63] = 30610;
		index[5][64] = 30925;
		index[5][65] = 31282;
		index[5][66] = 31732;
		index[5][67] = 32133;
		index[5][68] = 32597;
		index[5][69] = 33096;
		index[5][70] = 33512;
		index[5][71] = 33997;
		index[5][72] = 34513;
		index[5][73] = 35025;
		index[5][74] = 35676;
		index[5][75] = 36306;
		index[5][76] = 36977;
		index[5][77] = 37750;
		index[5][78] = 38502;
		index[5][79] = 39163;
		index[5][80] = 39980;
		index[5][81] = 40942;
		index[5][82] = 41746;
		index[5][83] = 42964;
		index[5][84] = 44333;
		index[5][85] = 45719;
		index[5][86] = 47287;
		index[5][87] = 49240;
		index[5][88] = 51451;
		index[5][89] = 54187;
		index[5][90] = 56987;
		index[5][91] = 59217;
		index[5][92] = 61515;
		index[5][93] = 64710;
		index[5][94] = 67489;
		index[5][95] = 70523;
		index[5][96] = 73610;
		index[5][97] = 75621;
		index[5][98] = 77616;
		index[5][99] = 79787;
		index[5][100] = 82548;
		index[5][101] = 86154;
		index[5][102] = 89187;
		index[5][103] = 91922;
		index[5][104] = 94774;
		index[5][105] = 97724;
		index[5][106] = 100568;
		index[5][107] = 103316;
		index[5][108] = 105684;
		index[5][109] = 107599;
		index[5][110] = 109098;
		index[5][111] = 110721;
		index[5][112] = 112268;
		index[5][113] = 113937;
		index[5][114] = 115484;
		index[5][115] = 116714;
		index[5][116] = 117858;
		index[5][117] = 118950;
		index[5][118] = 120131;
		index[5][119] = 121191;
		index[5][120] = 122149;
		index[5][121] = 123054;
		index[5][122] = 123924;
		index[5][123] = 124821;
		index[5][124] = 125736;
		index[5][125] = 126675;
		index[5][126] = 127505;
		index[5][127] = 128254;
		index[5][128] = 128998;
		index[5][129] = 129856;
		index[5][130] = 130691;
		index[5][131] = 131505;
		index[5][132] = 132455;
		index[5][133] = 133163;
		index[5][134] = 133791;
		index[5][135] = 134377;
		index[5][136] = 135012;
		index[5][137] = 135604;
		index[5][138] = 136200;
		index[5][139] = 136820;
		index[5][140] = 137326;
		index[5][141] = 137796;
		index[5][142] = 138329;
		index[5][143] = 138885;
		index[5][144] = 139401;
		index[5][145] = 139898;
		index[5][146] = 140358;
		index[5][147] = 140849;
		index[5][148] = 141332;
		index[5][149] = 141786;
		index[5][150] = 142261;
		index[5][151] = 142680;
		index[5][152] = 143126;
		index[5][153] = 143533;
		index[5][154] = 143944;
		index[5][155] = 144311;
		index[5][156] = 144670;
		index[5][157] = 145006;
		index[5][158] = 145379;
		index[5][159] = 145779;
		index[5][160] = 146184;
		index[5][161] = 146577;
		index[5][162] = 147049;
		index[5][163] = 147494;
		index[5][164] = 147922;
		index[5][165] = 148357;
		index[5][166] = 148848;
		index[5][167] = 149319;
		index[5][168] = 149757;
		index[5][169] = 150149;
		index[5][170] = 150485;
		index[5][171] = 150878;
		index[5][172] = 151267;
		index[5][173] = 151653;
		index[5][174] = 152056;
		index[5][175] = 152383;
		index[5][176] = 152728;
		index[5][177] = 153050;
		index[5][178] = 153431;
		index[5][179] = 153784;
		index[5][180] = 154112;
		index[5][181] = 154409;
		index[5][182] = 154744;
		index[5][183] = 155105;
		index[5][184] = 155513;
		index[5][185] = 155907;
		index[5][186] = 156270;
		index[5][187] = 156620;
		index[5][188] = 156949;
		index[5][189] = 157310;
		index[5][190] = 157753;
		index[5][191] = 158191;
		index[5][192] = 158614;
		index[5][193] = 158982;
		index[5][194] = 159299;
		index[5][195] = 159652;
		index[5][196] = 160036;
		index[5][197] = 160404;
		index[5][198] = 160739;
		index[5][199] = 161104;
		index[5][200] = 161410;
		index[5][201] = 161780;
		index[5][202] = 162144;
		index[5][203] = 162447;
		index[5][204] = 162752;
		index[5][205] = 163076;
		index[5][206] = 163486;
		index[5][207] = 163869;
		index[5][208] = 164188;
		index[5][209] = 164489;
		index[5][210] = 164835;
		index[5][211] = 165202;
		index[5][212] = 165622;
		index[5][213] = 166075;
		index[5][214] = 166471;
		index[5][215] = 166802;
		index[5][216] = 167101;
		index[5][217] = 167419;
		index[5][218] = 167785;
		index[5][219] = 168191;
		index[5][220] = 168603;
		index[5][221] = 168992;
		index[5][222] = 169379;
		index[5][223] = 169821;
		index[5][224] = 170206;
		index[5][225] = 170559;
		index[5][226] = 170952;
		index[5][227] = 171311;
		index[5][228] = 171808;
		index[5][229] = 172348;
		index[5][230] = 172884;
		index[5][231] = 173373;
		index[5][232] = 173836;
		index[5][233] = 174268;
		index[5][234] = 174741;
		index[5][235] = 175404;
		index[5][236] = 176042;
		index[5][237] = 176551;
		index[5][238] = 177158;
		index[5][239] = 177781;
		index[5][240] = 178351;
		index[5][241] = 178994;
		index[5][242] = 179624;
		index[5][243] = 180078;
		index[5][244] = 180563;
		index[5][245] = 181125;
		index[5][246] = 181715;
		index[5][247] = 182243;
		index[5][248] = 182746;
		index[5][249] = 183273;
		index[5][250] = 183818;
		index[5][251] = 184347;
		index[5][252] = 184983;
		index[5][253] = 185869;
		index[5][254] = 186717;
		index[5][255] = 187482;
		index[5][256] = 188282;
		index[5][257] = 188919;
		index[5][258] = 189481;
		index[5][259] = 190195;
		index[5][260] = 191050;
		index[5][261] = 191915;
		index[5][262] = 192855;
		index[5][263] = 193923;
		index[5][264] = 195172;
		index[5][265] = 196391;
		index[5][266] = 197603;
		index[5][267] = 198755;
		index[5][268] = 200133;
		index[5][269] = 201428;
		index[5][270] = 202628;
		index[5][271] = 203978;
		index[5][272] = 205538;
		index[5][273] = 206890;
		index[5][274] = 208231;
		index[5][275] = 209651;
		index[5][276] = 211261;
		index[5][277] = 212884;
		index[5][278] = 214596;
		index[5][279] = 216157;
		index[5][280] = 217742;
		index[5][281] = 219405;
		index[5][282] = 221545;
		index[5][283] = 223514;
		index[5][284] = 225105;
		index[5][285] = 226555;
		index[5][286] = 228173;
		index[5][287] = 229338;
		index[5][288] = 230397;
		index[5][289] = 231294;
		index[5][290] = 231957;
		index[5][291] = 232940;
		index[5][292] = 234782;
		index[5][293] = 236928;
		index[5][294] = 239144;
		index[5][295] = 241800;
		index[5][296] = 245117;
		index[5][297] = 248320;
		index[5][298] = 250739;
		index[5][299] = 253150;
		index[5][300] = 255961;
		index[5][301] = 258796;
		index[5][302] = 260891;
		index[5][303] = 263073;
		index[5][304] = 265094;
		index[5][305] = 266823;
		index[5][306] = 268527;
		index[5][307] = 270324;
		index[5][308] = 271981;
		index[5][309] = 273452;
		index[5][310] = 274853;
		index[5][311] = 276151;
		index[5][312] = 277542;
		index[5][313] = 279118;
		index[5][314] = 280564;
		index[5][315] = 281722;
		index[5][316] = 282901;
		index[5][317] = 284090;
		index[5][318] = 285157;
		index[5][319] = 286240;
		index[5][320] = 287380;
		index[5][321] = 288388;
		index[5][322] = 289367;
		index[5][323] = 290372;
		index[5][324] = 291339;
		index[5][325] = 292236;
		index[5][326] = 293302;
		index[5][327] = 294285;
		index[5][328] = 295204;
		index[5][329] = 296040;
		index[5][330] = 296805;
		index[5][331] = 297553;
		index[5][332] = 298462;
		index[5][333] = 299366;
		index[5][334] = 300338;
		index[5][335] = 301183;
		index[5][336] = 301902;
		index[5][337] = 302509;
		index[5][338] = 303159;
		index[5][339] = 303755;
		index[5][340] = 304386;
		index[5][341] = 305056;
		index[5][342] = 305778;
		index[5][343] = 306575;
		index[5][344] = 307266;
		index[5][345] = 307873;
		index[5][346] = 308481;
		index[5][347] = 309008;
		index[5][348] = 309653;
		index[5][349] = 310427;
		index[5][350] = 311060;
		index[5][351] = 311515;
		index[5][352] = 312093;
		index[5][353] = 312667;
		index[5][354] = 313297;
		index[5][355] = 313986;
		index[5][356] = 314626;
		index[5][357] = 315187;
		index[5][358] = 315709;
		index[5][359] = 316294;
	}

	/**
	 * Initializes the index in <tt>ac_p00.dat</tt>.
	 */
	private static void initialize_p00 ( ) {
		index[6][0] = 624;
		index[6][1] = 1181;
		index[6][2] = 1694;
		index[6][3] = 2200;
		index[6][4] = 2738;
		index[6][5] = 3284;
		index[6][6] = 3926;
		index[6][7] = 4576;
		index[6][8] = 5204;
		index[6][9] = 5751;
		index[6][10] = 6316;
		index[6][11] = 6821;
		index[6][12] = 7361;
		index[6][13] = 7890;
		index[6][14] = 8492;
		index[6][15] = 9140;
		index[6][16] = 9654;
		index[6][17] = 10097;
		index[6][18] = 10634;
		index[6][19] = 11200;
		index[6][20] = 11867;
		index[6][21] = 12544;
		index[6][22] = 13129;
		index[6][23] = 13574;
		index[6][24] = 14076;
		index[6][25] = 14629;
		index[6][26] = 15180;
		index[6][27] = 15715;
		index[6][28] = 16416;
		index[6][29] = 17141;
		index[6][30] = 17775;
		index[6][31] = 18258;
		index[6][32] = 18846;
		index[6][33] = 19432;
		index[6][34] = 20037;
		index[6][35] = 20625;
		index[6][36] = 21207;
		index[6][37] = 21782;
		index[6][38] = 22366;
		index[6][39] = 22881;
		index[6][40] = 23422;
		index[6][41] = 24022;
		index[6][42] = 24673;
		index[6][43] = 25268;
		index[6][44] = 25925;
		index[6][45] = 26598;
		index[6][46] = 27229;
		index[6][47] = 27771;
		index[6][48] = 28296;
		index[6][49] = 28884;
		index[6][50] = 29472;
		index[6][51] = 30135;
		index[6][52] = 30709;
		index[6][53] = 31214;
		index[6][54] = 31796;
		index[6][55] = 32426;
		index[6][56] = 32967;
		index[6][57] = 33411;
		index[6][58] = 33831;
		index[6][59] = 34231;
		index[6][60] = 34656;
		index[6][61] = 35055;
		index[6][62] = 35478;
		index[6][63] = 35930;
		index[6][64] = 36478;
		index[6][65] = 36999;
		index[6][66] = 37556;
		index[6][67] = 38074;
		index[6][68] = 38698;
		index[6][69] = 39447;
		index[6][70] = 40226;
		index[6][71] = 41121;
		index[6][72] = 42097;
		index[6][73] = 43039;
		index[6][74] = 43968;
		index[6][75] = 44929;
		index[6][76] = 46043;
		index[6][77] = 47132;
		index[6][78] = 48193;
		index[6][79] = 49091;
		index[6][80] = 50211;
		index[6][81] = 51401;
		index[6][82] = 52523;
		index[6][83] = 53575;
		index[6][84] = 54559;
		index[6][85] = 55370;
		index[6][86] = 56713;
		index[6][87] = 58072;
		index[6][88] = 59271;
		index[6][89] = 60598;
		index[6][90] = 62686;
		index[6][91] = 64929;
		index[6][92] = 67071;
		index[6][93] = 68920;
		index[6][94] = 70820;
		index[6][95] = 72507;
		index[6][96] = 74440;
		index[6][97] = 76319;
		index[6][98] = 78661;
		index[6][99] = 81007;
		index[6][100] = 83856;
		index[6][101] = 86465;
		index[6][102] = 89254;
		index[6][103] = 91379;
		index[6][104] = 93977;
		index[6][105] = 96525;
		index[6][106] = 99272;
		index[6][107] = 101444;
		index[6][108] = 103443;
		index[6][109] = 105264;
		index[6][110] = 107428;
		index[6][111] = 109307;
		index[6][112] = 111182;
		index[6][113] = 112719;
		index[6][114] = 114132;
		index[6][115] = 115643;
		index[6][116] = 117236;
		index[6][117] = 118523;
		index[6][118] = 119910;
		index[6][119] = 121103;
		index[6][120] = 122258;
		index[6][121] = 123435;
		index[6][122] = 124603;
		index[6][123] = 125615;
		index[6][124] = 126670;
		index[6][125] = 127688;
		index[6][126] = 128769;
		index[6][127] = 129678;
		index[6][128] = 130572;
		index[6][129] = 131469;
		index[6][130] = 132323;
		index[6][131] = 133047;
		index[6][132] = 133799;
		index[6][133] = 134505;
		index[6][134] = 135198;
		index[6][135] = 135848;
		index[6][136] = 136490;
		index[6][137] = 137051;
		index[6][138] = 137632;
		index[6][139] = 138201;
		index[6][140] = 138759;
		index[6][141] = 139279;
		index[6][142] = 139883;
		index[6][143] = 140435;
		index[6][144] = 141013;
		index[6][145] = 141559;
		index[6][146] = 142092;
		index[6][147] = 142609;
		index[6][148] = 143153;
		index[6][149] = 143673;
		index[6][150] = 144162;
		index[6][151] = 144675;
		index[6][152] = 145157;
		index[6][153] = 145676;
		index[6][154] = 146166;
		index[6][155] = 146604;
		index[6][156] = 147061;
		index[6][157] = 147468;
		index[6][158] = 147978;
		index[6][159] = 148452;
		index[6][160] = 148950;
		index[6][161] = 149364;
		index[6][162] = 149798;
		index[6][163] = 150133;
		index[6][164] = 150520;
		index[6][165] = 150895;
		index[6][166] = 151334;
		index[6][167] = 151727;
		index[6][168] = 152149;
		index[6][169] = 152480;
		index[6][170] = 152883;
		index[6][171] = 153276;
		index[6][172] = 153680;
		index[6][173] = 154026;
		index[6][174] = 154349;
		index[6][175] = 154668;
		index[6][176] = 155008;
		index[6][177] = 155327;
		index[6][178] = 155695;
		index[6][179] = 155997;
		index[6][180] = 156333;
		index[6][181] = 156694;
		index[6][182] = 157058;
		index[6][183] = 157426;
		index[6][184] = 157809;
		index[6][185] = 158121;
		index[6][186] = 158519;
		index[6][187] = 158934;
		index[6][188] = 159289;
		index[6][189] = 159624;
		index[6][190] = 160025;
		index[6][191] = 160375;
		index[6][192] = 160788;
		index[6][193] = 161105;
		index[6][194] = 161435;
		index[6][195] = 161760;
		index[6][196] = 162096;
		index[6][197] = 162383;
		index[6][198] = 162714;
		index[6][199] = 163033;
		index[6][200] = 163334;
		index[6][201] = 163633;
		index[6][202] = 163985;
		index[6][203] = 164315;
		index[6][204] = 164736;
		index[6][205] = 165136;
		index[6][206] = 165477;
		index[6][207] = 165776;
		index[6][208] = 166094;
		index[6][209] = 166385;
		index[6][210] = 166730;
		index[6][211] = 167120;
		index[6][212] = 167517;
		index[6][213] = 167876;
		index[6][214] = 168247;
		index[6][215] = 168575;
		index[6][216] = 168933;
		index[6][217] = 169290;
		index[6][218] = 169705;
		index[6][219] = 170060;
		index[6][220] = 170449;
		index[6][221] = 170815;
		index[6][222] = 171235;
		index[6][223] = 171675;
		index[6][224] = 172105;
		index[6][225] = 172531;
		index[6][226] = 172976;
		index[6][227] = 173382;
		index[6][228] = 173781;
		index[6][229] = 174171;
		index[6][230] = 174577;
		index[6][231] = 175015;
		index[6][232] = 175467;
		index[6][233] = 175889;
		index[6][234] = 176389;
		index[6][235] = 176886;
		index[6][236] = 177436;
		index[6][237] = 177853;
		index[6][238] = 178349;
		index[6][239] = 178864;
		index[6][240] = 179390;
		index[6][241] = 179940;
		index[6][242] = 180559;
		index[6][243] = 181145;
		index[6][244] = 181778;
		index[6][245] = 182449;
		index[6][246] = 183199;
		index[6][247] = 183827;
		index[6][248] = 184570;
		index[6][249] = 185225;
		index[6][250] = 185963;
		index[6][251] = 186729;
		index[6][252] = 187540;
		index[6][253] = 188206;
		index[6][254] = 188934;
		index[6][255] = 189809;
		index[6][256] = 190764;
		index[6][257] = 191412;
		index[6][258] = 192142;
		index[6][259] = 192906;
		index[6][260] = 193913;
		index[6][261] = 194904;
		index[6][262] = 196079;
		index[6][263] = 197271;
		index[6][264] = 198446;
		index[6][265] = 199590;
		index[6][266] = 200719;
		index[6][267] = 201836;
		index[6][268] = 203016;
		index[6][269] = 204182;
		index[6][270] = 205626;
		index[6][271] = 206921;
		index[6][272] = 208515;
		index[6][273] = 210220;
		index[6][274] = 211959;
		index[6][275] = 213764;
		index[6][276] = 215461;
		index[6][277] = 216850;
		index[6][278] = 218188;
		index[6][279] = 219915;
		index[6][280] = 221621;
		index[6][281] = 222696;
		index[6][282] = 223839;
		index[6][283] = 224871;
		index[6][284] = 225719;
		index[6][285] = 226571;
		index[6][286] = 227578;
		index[6][287] = 228561;
		index[6][288] = 230059;
		index[6][289] = 231448;
		index[6][290] = 232766;
		index[6][291] = 234420;
		index[6][292] = 236191;
		index[6][293] = 237936;
		index[6][294] = 239502;
		index[6][295] = 240845;
		index[6][296] = 242428;
		index[6][297] = 244203;
		index[6][298] = 246025;
		index[6][299] = 247791;
		index[6][300] = 249583;
		index[6][301] = 251159;
		index[6][302] = 253021;
		index[6][303] = 254464;
		index[6][304] = 255747;
		index[6][305] = 257015;
		index[6][306] = 258408;
		index[6][307] = 259770;
		index[6][308] = 261196;
		index[6][309] = 262536;
		index[6][310] = 263810;
		index[6][311] = 264958;
		index[6][312] = 266211;
		index[6][313] = 267401;
		index[6][314] = 268607;
		index[6][315] = 269657;
		index[6][316] = 270645;
		index[6][317] = 271545;
		index[6][318] = 272513;
		index[6][319] = 273370;
		index[6][320] = 274322;
		index[6][321] = 275222;
		index[6][322] = 276045;
		index[6][323] = 276901;
		index[6][324] = 277779;
		index[6][325] = 278526;
		index[6][326] = 279224;
		index[6][327] = 280004;
		index[6][328] = 280889;
		index[6][329] = 281729;
		index[6][330] = 282468;
		index[6][331] = 283127;
		index[6][332] = 283757;
		index[6][333] = 284305;
		index[6][334] = 284916;
		index[6][335] = 285527;
		index[6][336] = 286075;
		index[6][337] = 286565;
		index[6][338] = 287101;
		index[6][339] = 287649;
		index[6][340] = 288259;
		index[6][341] = 288856;
		index[6][342] = 289371;
		index[6][343] = 289825;
		index[6][344] = 290347;
		index[6][345] = 290880;
		index[6][346] = 291428;
		index[6][347] = 292005;
		index[6][348] = 292498;
		index[6][349] = 292981;
		index[6][350] = 293473;
		index[6][351] = 293926;
		index[6][352] = 294405;
		index[6][353] = 294841;
		index[6][354] = 295250;
		index[6][355] = 295660;
		index[6][356] = 296054;
		index[6][357] = 296522;
		index[6][358] = 297049;
		index[6][359] = 297603;
	}

	/**
	 * Initializes the index in <tt>ac_n00.dat</tt>.
	 */
	private static void initialize_n00 ( ) {
		index[7][0] = 404;
		index[7][1] = 860;
		index[7][2] = 1355;
		index[7][3] = 1778;
		index[7][4] = 2166;
		index[7][5] = 2578;
		index[7][6] = 2981;
		index[7][7] = 3409;
		index[7][8] = 3723;
		index[7][9] = 4070;
		index[7][10] = 4458;
		index[7][11] = 4816;
		index[7][12] = 5245;
		index[7][13] = 5685;
		index[7][14] = 6027;
		index[7][15] = 6410;
		index[7][16] = 6775;
		index[7][17] = 7143;
		index[7][18] = 7519;
		index[7][19] = 7896;
		index[7][20] = 8276;
		index[7][21] = 8640;
		index[7][22] = 8982;
		index[7][23] = 9375;
		index[7][24] = 9748;
		index[7][25] = 10172;
		index[7][26] = 10569;
		index[7][27] = 11042;
		index[7][28] = 11441;
		index[7][29] = 11893;
		index[7][30] = 12398;
		index[7][31] = 12875;
		index[7][32] = 13329;
		index[7][33] = 13801;
		index[7][34] = 14170;
		index[7][35] = 14586;
		index[7][36] = 15016;
		index[7][37] = 15443;
		index[7][38] = 15880;
		index[7][39] = 16365;
		index[7][40] = 16860;
		index[7][41] = 17344;
		index[7][42] = 17874;
		index[7][43] = 18411;
		index[7][44] = 18809;
		index[7][45] = 19255;
		index[7][46] = 19767;
		index[7][47] = 20279;
		index[7][48] = 20741;
		index[7][49] = 21186;
		index[7][50] = 21682;
		index[7][51] = 22209;
		index[7][52] = 22852;
		index[7][53] = 23380;
		index[7][54] = 23879;
		index[7][55] = 24379;
		index[7][56] = 24762;
		index[7][57] = 25213;
		index[7][58] = 25740;
		index[7][59] = 26190;
		index[7][60] = 26641;
		index[7][61] = 27165;
		index[7][62] = 27679;
		index[7][63] = 28239;
		index[7][64] = 28843;
		index[7][65] = 29487;
		index[7][66] = 30032;
		index[7][67] = 30669;
		index[7][68] = 31323;
		index[7][69] = 32006;
		index[7][70] = 32732;
		index[7][71] = 33329;
		index[7][72] = 34036;
		index[7][73] = 34829;
		index[7][74] = 35661;
		index[7][75] = 36521;
		index[7][76] = 37587;
		index[7][77] = 38729;
		index[7][78] = 39703;
		index[7][79] = 40541;
		index[7][80] = 41485;
		index[7][81] = 42402;
		index[7][82] = 43330;
		index[7][83] = 44247;
		index[7][84] = 44883;
		index[7][85] = 45368;
		index[7][86] = 45897;
		index[7][87] = 46466;
		index[7][88] = 47185;
		index[7][89] = 48020;
		index[7][90] = 48967;
		index[7][91] = 50077;
		index[7][92] = 51272;
		index[7][93] = 52377;
		index[7][94] = 53654;
		index[7][95] = 55060;
		index[7][96] = 56440;
		index[7][97] = 57646;
		index[7][98] = 59116;
		index[7][99] = 60807;
		index[7][100] = 63603;
		index[7][101] = 66766;
		index[7][102] = 69395;
		index[7][103] = 72233;
		index[7][104] = 75384;
		index[7][105] = 78731;
		index[7][106] = 82635;
		index[7][107] = 86866;
		index[7][108] = 91026;
		index[7][109] = 94197;
		index[7][110] = 97737;
		index[7][111] = 101419;
		index[7][112] = 103860;
		index[7][113] = 106158;
		index[7][114] = 108586;
		index[7][115] = 110438;
		index[7][116] = 112672;
		index[7][117] = 114749;
		index[7][118] = 116716;
		index[7][119] = 118561;
		index[7][120] = 120167;
		index[7][121] = 121672;
		index[7][122] = 122923;
		index[7][123] = 124325;
		index[7][124] = 125653;
		index[7][125] = 126752;
		index[7][126] = 128092;
		index[7][127] = 129235;
		index[7][128] = 130514;
		index[7][129] = 131889;
		index[7][130] = 133116;
		index[7][131] = 134349;
		index[7][132] = 135331;
		index[7][133] = 136299;
		index[7][134] = 137383;
		index[7][135] = 138238;
		index[7][136] = 138926;
		index[7][137] = 139522;
		index[7][138] = 140122;
		index[7][139] = 140731;
		index[7][140] = 141454;
		index[7][141] = 142203;
		index[7][142] = 142839;
		index[7][143] = 143457;
		index[7][144] = 144130;
		index[7][145] = 144779;
		index[7][146] = 145416;
		index[7][147] = 145857;
		index[7][148] = 146297;
		index[7][149] = 146845;
		index[7][150] = 147407;
		index[7][151] = 147911;
		index[7][152] = 148459;
		index[7][153] = 148951;
		index[7][154] = 149468;
		index[7][155] = 150032;
		index[7][156] = 150675;
		index[7][157] = 151252;
		index[7][158] = 151812;
		index[7][159] = 152325;
		index[7][160] = 152775;
		index[7][161] = 153269;
		index[7][162] = 153827;
		index[7][163] = 154321;
		index[7][164] = 154785;
		index[7][165] = 155251;
		index[7][166] = 155717;
		index[7][167] = 156249;
		index[7][168] = 156725;
		index[7][169] = 157206;
		index[7][170] = 157688;
		index[7][171] = 158150;
		index[7][172] = 158547;
		index[7][173] = 159058;
		index[7][174] = 159592;
		index[7][175] = 160049;
		index[7][176] = 160467;
		index[7][177] = 160827;
		index[7][178] = 161198;
		index[7][179] = 161691;
		index[7][180] = 162145;
		index[7][181] = 162556;
		index[7][182] = 162935;
		index[7][183] = 163298;
		index[7][184] = 163677;
		index[7][185] = 164083;
		index[7][186] = 164478;
		index[7][187] = 164804;
		index[7][188] = 165228;
		index[7][189] = 165708;
		index[7][190] = 166228;
		index[7][191] = 166685;
		index[7][192] = 167145;
		index[7][193] = 167607;
		index[7][194] = 168076;
		index[7][195] = 168574;
		index[7][196] = 169143;
		index[7][197] = 169664;
		index[7][198] = 170148;
		index[7][199] = 170739;
		index[7][200] = 171252;
		index[7][201] = 171687;
		index[7][202] = 172001;
		index[7][203] = 172411;
		index[7][204] = 172916;
		index[7][205] = 173425;
		index[7][206] = 173908;
		index[7][207] = 174429;
		index[7][208] = 174907;
		index[7][209] = 175323;
		index[7][210] = 175774;
		index[7][211] = 176170;
		index[7][212] = 176595;
		index[7][213] = 177002;
		index[7][214] = 177553;
		index[7][215] = 178150;
		index[7][216] = 178678;
		index[7][217] = 179175;
		index[7][218] = 179608;
		index[7][219] = 179962;
		index[7][220] = 180344;
		index[7][221] = 180782;
		index[7][222] = 181240;
		index[7][223] = 181664;
		index[7][224] = 182183;
		index[7][225] = 182655;
		index[7][226] = 183266;
		index[7][227] = 183837;
		index[7][228] = 184383;
		index[7][229] = 184900;
		index[7][230] = 185409;
		index[7][231] = 185880;
		index[7][232] = 186357;
		index[7][233] = 186851;
		index[7][234] = 187344;
		index[7][235] = 187796;
		index[7][236] = 188273;
		index[7][237] = 188763;
		index[7][238] = 189286;
		index[7][239] = 189685;
		index[7][240] = 190169;
		index[7][241] = 190678;
		index[7][242] = 191091;
		index[7][243] = 191595;
		index[7][244] = 192207;
		index[7][245] = 192937;
		index[7][246] = 193750;
		index[7][247] = 194286;
		index[7][248] = 194894;
		index[7][249] = 195399;
		index[7][250] = 196037;
		index[7][251] = 196532;
		index[7][252] = 197039;
		index[7][253] = 197495;
		index[7][254] = 197954;
		index[7][255] = 198402;
		index[7][256] = 198879;
		index[7][257] = 199315;
		index[7][258] = 199693;
		index[7][259] = 200045;
		index[7][260] = 200429;
		index[7][261] = 200869;
		index[7][262] = 201449;
		index[7][263] = 201962;
		index[7][264] = 202379;
		index[7][265] = 202898;
		index[7][266] = 203571;
		index[7][267] = 204075;
		index[7][268] = 204467;
		index[7][269] = 204822;
		index[7][270] = 205142;
		index[7][271] = 205476;
		index[7][272] = 205768;
		index[7][273] = 206046;
		index[7][274] = 206374;
		index[7][275] = 206879;
		index[7][276] = 207356;
		index[7][277] = 207783;
		index[7][278] = 208322;
		index[7][279] = 209348;
		index[7][280] = 211867;
		index[7][281] = 213783;
		index[7][282] = 215954;
		index[7][283] = 217824;
		index[7][284] = 219905;
		index[7][285] = 221703;
		index[7][286] = 223341;
		index[7][287] = 224437;
		index[7][288] = 225735;
		index[7][289] = 227531;
		index[7][290] = 229538;
		index[7][291] = 230829;
		index[7][292] = 232024;
		index[7][293] = 233030;
		index[7][294] = 234185;
		index[7][295] = 235192;
		index[7][296] = 236133;
		index[7][297] = 237115;
		index[7][298] = 238196;
		index[7][299] = 239280;
		index[7][300] = 240311;
		index[7][301] = 241275;
		index[7][302] = 242295;
		index[7][303] = 243603;
		index[7][304] = 245338;
		index[7][305] = 246659;
		index[7][306] = 248072;
		index[7][307] = 249276;
		index[7][308] = 250620;
		index[7][309] = 251831;
		index[7][310] = 253238;
		index[7][311] = 254333;
		index[7][312] = 255379;
		index[7][313] = 256428;
		index[7][314] = 257485;
		index[7][315] = 258396;
		index[7][316] = 259638;
		index[7][317] = 260670;
		index[7][318] = 261577;
		index[7][319] = 262284;
		index[7][320] = 263123;
		index[7][321] = 263879;
		index[7][322] = 264596;
		index[7][323] = 265410;
		index[7][324] = 266176;
		index[7][325] = 266820;
		index[7][326] = 267547;
		index[7][327] = 268178;
		index[7][328] = 268674;
		index[7][329] = 269128;
		index[7][330] = 269612;
		index[7][331] = 270116;
		index[7][332] = 270699;
		index[7][333] = 271215;
		index[7][334] = 271644;
		index[7][335] = 272043;
		index[7][336] = 272565;
		index[7][337] = 273065;
		index[7][338] = 273659;
		index[7][339] = 274141;
		index[7][340] = 274662;
		index[7][341] = 275230;
		index[7][342] = 275784;
		index[7][343] = 276257;
		index[7][344] = 276785;
		index[7][345] = 277341;
		index[7][346] = 277856;
		index[7][347] = 278375;
		index[7][348] = 278943;
		index[7][349] = 279539;
		index[7][350] = 279979;
		index[7][351] = 280447;
		index[7][352] = 280931;
		index[7][353] = 281418;
		index[7][354] = 281905;
		index[7][355] = 282305;
		index[7][356] = 282736;
		index[7][357] = 283153;
		index[7][358] = 283668;
		index[7][359] = 284058;
	}

	/**
	 * Initializes the index in <tt>ac_n10.dat</tt>.
	 */
	private static void initialize_n10 ( ) {
		index[8][0] = 468;
		index[8][1] = 941;
		index[8][2] = 1495;
		index[8][3] = 2039;
		index[8][4] = 2511;
		index[8][5] = 2912;
		index[8][6] = 3375;
		index[8][7] = 3771;
		index[8][8] = 4200;
		index[8][9] = 4660;
		index[8][10] = 5123;
		index[8][11] = 5586;
		index[8][12] = 6021;
		index[8][13] = 6417;
		index[8][14] = 6818;
		index[8][15] = 7255;
		index[8][16] = 7658;
		index[8][17] = 8114;
		index[8][18] = 8507;
		index[8][19] = 8945;
		index[8][20] = 9426;
		index[8][21] = 9879;
		index[8][22] = 10356;
		index[8][23] = 10822;
		index[8][24] = 11241;
		index[8][25] = 11657;
		index[8][26] = 12065;
		index[8][27] = 12505;
		index[8][28] = 12965;
		index[8][29] = 13439;
		index[8][30] = 13907;
		index[8][31] = 14410;
		index[8][32] = 14877;
		index[8][33] = 15311;
		index[8][34] = 15769;
		index[8][35] = 16181;
		index[8][36] = 16631;
		index[8][37] = 17061;
		index[8][38] = 17536;
		index[8][39] = 18027;
		index[8][40] = 18496;
		index[8][41] = 18990;
		index[8][42] = 19530;
		index[8][43] = 20114;
		index[8][44] = 20654;
		index[8][45] = 21174;
		index[8][46] = 21632;
		index[8][47] = 22071;
		index[8][48] = 22522;
		index[8][49] = 23025;
		index[8][50] = 23507;
		index[8][51] = 24035;
		index[8][52] = 24530;
		index[8][53] = 24999;
		index[8][54] = 25495;
		index[8][55] = 26020;
		index[8][56] = 26547;
		index[8][57] = 27115;
		index[8][58] = 27675;
		index[8][59] = 28242;
		index[8][60] = 28833;
		index[8][61] = 29494;
		index[8][62] = 30152;
		index[8][63] = 30830;
		index[8][64] = 31408;
		index[8][65] = 32093;
		index[8][66] = 32863;
		index[8][67] = 33621;
		index[8][68] = 34290;
		index[8][69] = 34928;
		index[8][70] = 35522;
		index[8][71] = 36227;
		index[8][72] = 36984;
		index[8][73] = 37668;
		index[8][74] = 38442;
		index[8][75] = 39199;
		index[8][76] = 40064;
		index[8][77] = 40967;
		index[8][78] = 41944;
		index[8][79] = 42918;
		index[8][80] = 44020;
		index[8][81] = 45110;
		index[8][82] = 46013;
		index[8][83] = 46954;
		index[8][84] = 48078;
		index[8][85] = 49226;
		index[8][86] = 50458;
		index[8][87] = 51657;
		index[8][88] = 52645;
		index[8][89] = 53481;
		index[8][90] = 54495;
		index[8][91] = 55713;
		index[8][92] = 57194;
		index[8][93] = 59335;
		index[8][94] = 61623;
		index[8][95] = 63531;
		index[8][96] = 65156;
		index[8][97] = 66962;
		index[8][98] = 69001;
		index[8][99] = 71031;
		index[8][100] = 73831;
		index[8][101] = 76392;
		index[8][102] = 78705;
		index[8][103] = 81356;
		index[8][104] = 84227;
		index[8][105] = 87111;
		index[8][106] = 90678;
		index[8][107] = 94208;
		index[8][108] = 97317;
		index[8][109] = 101182;
		index[8][110] = 105595;
		index[8][111] = 110175;
		index[8][112] = 115081;
		index[8][113] = 120122;
		index[8][114] = 124845;
		index[8][115] = 129456;
		index[8][116] = 133801;
		index[8][117] = 137968;
		index[8][118] = 141992;
		index[8][119] = 145667;
		index[8][120] = 148891;
		index[8][121] = 151841;
		index[8][122] = 154523;
		index[8][123] = 156947;
		index[8][124] = 159228;
		index[8][125] = 161375;
		index[8][126] = 163440;
		index[8][127] = 165562;
		index[8][128] = 167496;
		index[8][129] = 169189;
		index[8][130] = 170923;
		index[8][131] = 172502;
		index[8][132] = 174060;
		index[8][133] = 175546;
		index[8][134] = 176883;
		index[8][135] = 178037;
		index[8][136] = 179269;
		index[8][137] = 180588;
		index[8][138] = 181926;
		index[8][139] = 183207;
		index[8][140] = 184395;
		index[8][141] = 185536;
		index[8][142] = 186746;
		index[8][143] = 187846;
		index[8][144] = 188876;
		index[8][145] = 189821;
		index[8][146] = 190702;
		index[8][147] = 191612;
		index[8][148] = 192556;
		index[8][149] = 193463;
		index[8][150] = 194344;
		index[8][151] = 195238;
		index[8][152] = 196216;
		index[8][153] = 197088;
		index[8][154] = 197788;
		index[8][155] = 198448;
		index[8][156] = 199186;
		index[8][157] = 199892;
		index[8][158] = 200649;
		index[8][159] = 201440;
		index[8][160] = 202199;
		index[8][161] = 202851;
		index[8][162] = 203547;
		index[8][163] = 204278;
		index[8][164] = 204906;
		index[8][165] = 205602;
		index[8][166] = 206328;
		index[8][167] = 207049;
		index[8][168] = 207791;
		index[8][169] = 208470;
		index[8][170] = 209117;
		index[8][171] = 209714;
		index[8][172] = 210325;
		index[8][173] = 210949;
		index[8][174] = 211440;
		index[8][175] = 211914;
		index[8][176] = 212448;
		index[8][177] = 213084;
		index[8][178] = 213698;
		index[8][179] = 214256;
		index[8][180] = 214856;
		index[8][181] = 215370;
		index[8][182] = 215874;
		index[8][183] = 216317;
		index[8][184] = 216816;
		index[8][185] = 217308;
		index[8][186] = 217760;
		index[8][187] = 218217;
		index[8][188] = 218757;
		index[8][189] = 219307;
		index[8][190] = 219789;
		index[8][191] = 220287;
		index[8][192] = 220871;
		index[8][193] = 221429;
		index[8][194] = 221985;
		index[8][195] = 222506;
		index[8][196] = 223038;
		index[8][197] = 223576;
		index[8][198] = 224126;
		index[8][199] = 224658;
		index[8][200] = 225235;
		index[8][201] = 225723;
		index[8][202] = 226272;
		index[8][203] = 226914;
		index[8][204] = 227541;
		index[8][205] = 228133;
		index[8][206] = 228807;
		index[8][207] = 229415;
		index[8][208] = 230058;
		index[8][209] = 230619;
		index[8][210] = 231267;
		index[8][211] = 231937;
		index[8][212] = 232697;
		index[8][213] = 233367;
		index[8][214] = 233953;
		index[8][215] = 234511;
		index[8][216] = 235195;
		index[8][217] = 235783;
		index[8][218] = 236384;
		index[8][219] = 236924;
		index[8][220] = 237546;
		index[8][221] = 238298;
		index[8][222] = 239030;
		index[8][223] = 239706;
		index[8][224] = 240469;
		index[8][225] = 241167;
		index[8][226] = 241854;
		index[8][227] = 242553;
		index[8][228] = 243255;
		index[8][229] = 243960;
		index[8][230] = 244652;
		index[8][231] = 245342;
		index[8][232] = 246143;
		index[8][233] = 246981;
		index[8][234] = 247872;
		index[8][235] = 248761;
		index[8][236] = 249532;
		index[8][237] = 250221;
		index[8][238] = 250940;
		index[8][239] = 251576;
		index[8][240] = 252206;
		index[8][241] = 252939;
		index[8][242] = 253590;
		index[8][243] = 254305;
		index[8][244] = 254971;
		index[8][245] = 255690;
		index[8][246] = 256272;
		index[8][247] = 256838;
		index[8][248] = 257248;
		index[8][249] = 257761;
		index[8][250] = 258354;
		index[8][251] = 258805;
		index[8][252] = 259145;
		index[8][253] = 259650;
		index[8][254] = 260239;
		index[8][255] = 260976;
		index[8][256] = 261588;
		index[8][257] = 262230;
		index[8][258] = 262935;
		index[8][259] = 263683;
		index[8][260] = 264519;
		index[8][261] = 265343;
		index[8][262] = 266104;
		index[8][263] = 267182;
		index[8][264] = 268375;
		index[8][265] = 269568;
		index[8][266] = 270443;
		index[8][267] = 271299;
		index[8][268] = 272340;
		index[8][269] = 273728;
		index[8][270] = 275252;
		index[8][271] = 276619;
		index[8][272] = 278870;
		index[8][273] = 281924;
		index[8][274] = 285822;
		index[8][275] = 289414;
		index[8][276] = 292938;
		index[8][277] = 296174;
		index[8][278] = 298939;
		index[8][279] = 300908;
		index[8][280] = 302801;
		index[8][281] = 304846;
		index[8][282] = 306742;
		index[8][283] = 308680;
		index[8][284] = 310194;
		index[8][285] = 312172;
		index[8][286] = 314209;
		index[8][287] = 315910;
		index[8][288] = 318197;
		index[8][289] = 320704;
		index[8][290] = 322625;
		index[8][291] = 324414;
		index[8][292] = 325898;
		index[8][293] = 327782;
		index[8][294] = 329631;
		index[8][295] = 331288;
		index[8][296] = 332533;
		index[8][297] = 333612;
		index[8][298] = 334740;
		index[8][299] = 335847;
		index[8][300] = 336826;
		index[8][301] = 337822;
		index[8][302] = 338834;
		index[8][303] = 340113;
		index[8][304] = 341323;
		index[8][305] = 342974;
		index[8][306] = 344607;
		index[8][307] = 346208;
		index[8][308] = 347510;
		index[8][309] = 348791;
		index[8][310] = 350112;
		index[8][311] = 351506;
		index[8][312] = 352614;
		index[8][313] = 353715;
		index[8][314] = 354755;
		index[8][315] = 355844;
		index[8][316] = 356761;
		index[8][317] = 357553;
		index[8][318] = 358305;
		index[8][319] = 359132;
		index[8][320] = 359935;
		index[8][321] = 360718;
		index[8][322] = 361513;
		index[8][323] = 362358;
		index[8][324] = 363082;
		index[8][325] = 363883;
		index[8][326] = 364628;
		index[8][327] = 365398;
		index[8][328] = 366079;
		index[8][329] = 366801;
		index[8][330] = 367579;
		index[8][331] = 368313;
		index[8][332] = 368944;
		index[8][333] = 369534;
		index[8][334] = 370138;
		index[8][335] = 370729;
		index[8][336] = 371389;
		index[8][337] = 372025;
		index[8][338] = 372578;
		index[8][339] = 373133;
		index[8][340] = 373658;
		index[8][341] = 374206;
		index[8][342] = 374693;
		index[8][343] = 375197;
		index[8][344] = 375748;
		index[8][345] = 376393;
		index[8][346] = 376861;
		index[8][347] = 377378;
		index[8][348] = 377908;
		index[8][349] = 378435;
		index[8][350] = 378970;
		index[8][351] = 379534;
		index[8][352] = 380066;
		index[8][353] = 380617;
		index[8][354] = 381157;
		index[8][355] = 381668;
		index[8][356] = 382153;
		index[8][357] = 382637;
		index[8][358] = 383161;
		index[8][359] = 383609;
	}

	/**
	 * Initializes the index in <tt>ac_n20.dat</tt>.
	 */
	private static void initialize_n20 ( ) {
		index[9][0] = 383;
		index[9][1] = 778;
		index[9][2] = 1159;
		index[9][3] = 1586;
		index[9][4] = 1979;
		index[9][5] = 2338;
		index[9][6] = 2714;
		index[9][7] = 3034;
		index[9][8] = 3380;
		index[9][9] = 3804;
		index[9][10] = 4200;
		index[9][11] = 4576;
		index[9][12] = 4938;
		index[9][13] = 5334;
		index[9][14] = 5662;
		index[9][15] = 6048;
		index[9][16] = 6451;
		index[9][17] = 6843;
		index[9][18] = 7227;
		index[9][19] = 7600;
		index[9][20] = 7998;
		index[9][21] = 8412;
		index[9][22] = 8847;
		index[9][23] = 9231;
		index[9][24] = 9580;
		index[9][25] = 9947;
		index[9][26] = 10323;
		index[9][27] = 10691;
		index[9][28] = 11079;
		index[9][29] = 11462;
		index[9][30] = 11865;
		index[9][31] = 12243;
		index[9][32] = 12615;
		index[9][33] = 13064;
		index[9][34] = 13474;
		index[9][35] = 13879;
		index[9][36] = 14285;
		index[9][37] = 14674;
		index[9][38] = 15089;
		index[9][39] = 15538;
		index[9][40] = 15962;
		index[9][41] = 16371;
		index[9][42] = 16810;
		index[9][43] = 17224;
		index[9][44] = 17680;
		index[9][45] = 18103;
		index[9][46] = 18561;
		index[9][47] = 18979;
		index[9][48] = 19418;
		index[9][49] = 19844;
		index[9][50] = 20324;
		index[9][51] = 20772;
		index[9][52] = 21173;
		index[9][53] = 21544;
		index[9][54] = 21951;
		index[9][55] = 22388;
		index[9][56] = 22869;
		index[9][57] = 23372;
		index[9][58] = 23930;
		index[9][59] = 24478;
		index[9][60] = 25043;
		index[9][61] = 25600;
		index[9][62] = 26101;
		index[9][63] = 26616;
		index[9][64] = 27160;
		index[9][65] = 27781;
		index[9][66] = 28324;
		index[9][67] = 28970;
		index[9][68] = 29631;
		index[9][69] = 30341;
		index[9][70] = 31028;
		index[9][71] = 31764;
		index[9][72] = 32461;
		index[9][73] = 33111;
		index[9][74] = 33789;
		index[9][75] = 34530;
		index[9][76] = 35295;
		index[9][77] = 36059;
		index[9][78] = 36868;
		index[9][79] = 37733;
		index[9][80] = 38717;
		index[9][81] = 39734;
		index[9][82] = 40730;
		index[9][83] = 41759;
		index[9][84] = 42830;
		index[9][85] = 43952;
		index[9][86] = 45104;
		index[9][87] = 46297;
		index[9][88] = 47349;
		index[9][89] = 48610;
		index[9][90] = 49856;
		index[9][91] = 51243;
		index[9][92] = 52642;
		index[9][93] = 54147;
		index[9][94] = 55901;
		index[9][95] = 57766;
		index[9][96] = 59527;
		index[9][97] = 60962;
		index[9][98] = 62495;
		index[9][99] = 64333;
		index[9][100] = 66354;
		index[9][101] = 68908;
		index[9][102] = 71235;
		index[9][103] = 73742;
		index[9][104] = 76395;
		index[9][105] = 79059;
		index[9][106] = 81939;
		index[9][107] = 84742;
		index[9][108] = 87552;
		index[9][109] = 90782;
		index[9][110] = 93995;
		index[9][111] = 97133;
		index[9][112] = 100726;
		index[9][113] = 105121;
		index[9][114] = 109509;
		index[9][115] = 114475;
		index[9][116] = 118745;
		index[9][117] = 122143;
		index[9][118] = 126098;
		index[9][119] = 129674;
		index[9][120] = 132856;
		index[9][121] = 135480;
		index[9][122] = 137889;
		index[9][123] = 140773;
		index[9][124] = 143922;
		index[9][125] = 147022;
		index[9][126] = 149625;
		index[9][127] = 152814;
		index[9][128] = 155964;
		index[9][129] = 158334;
		index[9][130] = 160236;
		index[9][131] = 161968;
		index[9][132] = 163423;
		index[9][133] = 164819;
		index[9][134] = 166028;
		index[9][135] = 166972;
		index[9][136] = 168026;
		index[9][137] = 169394;
		index[9][138] = 170868;
		index[9][139] = 172328;
		index[9][140] = 173754;
		index[9][141] = 175163;
		index[9][142] = 176451;
		index[9][143] = 177725;
		index[9][144] = 178964;
		index[9][145] = 180234;
		index[9][146] = 181562;
		index[9][147] = 182765;
		index[9][148] = 183748;
		index[9][149] = 184692;
		index[9][150] = 185654;
		index[9][151] = 186611;
		index[9][152] = 187478;
		index[9][153] = 188471;
		index[9][154] = 189491;
		index[9][155] = 190595;
		index[9][156] = 191575;
		index[9][157] = 192555;
		index[9][158] = 193490;
		index[9][159] = 194399;
		index[9][160] = 195367;
		index[9][161] = 196234;
		index[9][162] = 196970;
		index[9][163] = 197726;
		index[9][164] = 198375;
		index[9][165] = 199088;
		index[9][166] = 199806;
		index[9][167] = 200548;
		index[9][168] = 201203;
		index[9][169] = 201912;
		index[9][170] = 202652;
		index[9][171] = 203374;
		index[9][172] = 204137;
		index[9][173] = 204842;
		index[9][174] = 205494;
		index[9][175] = 206067;
		index[9][176] = 206676;
		index[9][177] = 207416;
		index[9][178] = 208031;
		index[9][179] = 208630;
		index[9][180] = 209249;
		index[9][181] = 209870;
		index[9][182] = 210444;
		index[9][183] = 211099;
		index[9][184] = 211687;
		index[9][185] = 212274;
		index[9][186] = 212809;
		index[9][187] = 213294;
		index[9][188] = 213878;
		index[9][189] = 214616;
		index[9][190] = 215398;
		index[9][191] = 216099;
		index[9][192] = 216806;
		index[9][193] = 217459;
		index[9][194] = 218046;
		index[9][195] = 218685;
		index[9][196] = 219253;
		index[9][197] = 219923;
		index[9][198] = 220577;
		index[9][199] = 221130;
		index[9][200] = 221718;
		index[9][201] = 222356;
		index[9][202] = 222992;
		index[9][203] = 223613;
		index[9][204] = 224126;
		index[9][205] = 224706;
		index[9][206] = 225336;
		index[9][207] = 225967;
		index[9][208] = 226645;
		index[9][209] = 227332;
		index[9][210] = 228032;
		index[9][211] = 228738;
		index[9][212] = 229385;
		index[9][213] = 230059;
		index[9][214] = 230811;
		index[9][215] = 231594;
		index[9][216] = 232399;
		index[9][217] = 233230;
		index[9][218] = 234040;
		index[9][219] = 234809;
		index[9][220] = 235548;
		index[9][221] = 236273;
		index[9][222] = 236919;
		index[9][223] = 237592;
		index[9][224] = 238274;
		index[9][225] = 239046;
		index[9][226] = 239794;
		index[9][227] = 240485;
		index[9][228] = 241147;
		index[9][229] = 241686;
		index[9][230] = 242348;
		index[9][231] = 243092;
		index[9][232] = 243852;
		index[9][233] = 244537;
		index[9][234] = 245329;
		index[9][235] = 246108;
		index[9][236] = 246857;
		index[9][237] = 247743;
		index[9][238] = 248629;
		index[9][239] = 249462;
		index[9][240] = 250344;
		index[9][241] = 251199;
		index[9][242] = 252129;
		index[9][243] = 252844;
		index[9][244] = 253353;
		index[9][245] = 253726;
		index[9][246] = 254099;
		index[9][247] = 254555;
		index[9][248] = 255139;
		index[9][249] = 255719;
		index[9][250] = 256635;
		index[9][251] = 257815;
		index[9][252] = 259065;
		index[9][253] = 260456;
		index[9][254] = 261903;
		index[9][255] = 263349;
		index[9][256] = 264688;
		index[9][257] = 265745;
		index[9][258] = 266653;
		index[9][259] = 267393;
		index[9][260] = 268129;
		index[9][261] = 268824;
		index[9][262] = 269557;
		index[9][263] = 270506;
		index[9][264] = 271677;
		index[9][265] = 272836;
		index[9][266] = 274217;
		index[9][267] = 276322;
		index[9][268] = 279173;
		index[9][269] = 281893;
		index[9][270] = 285197;
		index[9][271] = 289059;
		index[9][272] = 292781;
		index[9][273] = 296280;
		index[9][274] = 299297;
		index[9][275] = 302063;
		index[9][276] = 304390;
		index[9][277] = 306398;
		index[9][278] = 308590;
		index[9][279] = 310566;
		index[9][280] = 312236;
		index[9][281] = 313725;
		index[9][282] = 315074;
		index[9][283] = 316732;
		index[9][284] = 318349;
		index[9][285] = 319911;
		index[9][286] = 321661;
		index[9][287] = 323372;
		index[9][288] = 325250;
		index[9][289] = 327029;
		index[9][290] = 329127;
		index[9][291] = 331119;
		index[9][292] = 332680;
		index[9][293] = 334088;
		index[9][294] = 335558;
		index[9][295] = 336771;
		index[9][296] = 337917;
		index[9][297] = 339109;
		index[9][298] = 340381;
		index[9][299] = 341530;
		index[9][300] = 342537;
		index[9][301] = 343623;
		index[9][302] = 344695;
		index[9][303] = 345698;
		index[9][304] = 346770;
		index[9][305] = 347969;
		index[9][306] = 348999;
		index[9][307] = 349992;
		index[9][308] = 351118;
		index[9][309] = 352161;
		index[9][310] = 353136;
		index[9][311] = 354017;
		index[9][312] = 354799;
		index[9][313] = 355517;
		index[9][314] = 356215;
		index[9][315] = 356969;
		index[9][316] = 357742;
		index[9][317] = 358512;
		index[9][318] = 359271;
		index[9][319] = 359965;
		index[9][320] = 360658;
		index[9][321] = 361304;
		index[9][322] = 361889;
		index[9][323] = 362515;
		index[9][324] = 363106;
		index[9][325] = 363775;
		index[9][326] = 364338;
		index[9][327] = 364955;
		index[9][328] = 365585;
		index[9][329] = 366166;
		index[9][330] = 366764;
		index[9][331] = 367342;
		index[9][332] = 367850;
		index[9][333] = 368340;
		index[9][334] = 368819;
		index[9][335] = 369356;
		index[9][336] = 369913;
		index[9][337] = 370504;
		index[9][338] = 371083;
		index[9][339] = 371623;
		index[9][340] = 372157;
		index[9][341] = 372585;
		index[9][342] = 373008;
		index[9][343] = 373472;
		index[9][344] = 373921;
		index[9][345] = 374373;
		index[9][346] = 374848;
		index[9][347] = 375273;
		index[9][348] = 375712;
		index[9][349] = 376169;
		index[9][350] = 376612;
		index[9][351] = 377019;
		index[9][352] = 377476;
		index[9][353] = 377886;
		index[9][354] = 378296;
		index[9][355] = 378664;
		index[9][356] = 379049;
		index[9][357] = 379456;
		index[9][358] = 379840;
		index[9][359] = 380217;
	}

	/**
	 * Initializes the index in <tt>ac_n30.dat</tt>.
	 */
	private static void initialize_n30 ( ) {
		index[10][0] = 364;
		index[10][1] = 698;
		index[10][2] = 1013;
		index[10][3] = 1329;
		index[10][4] = 1629;
		index[10][5] = 1930;
		index[10][6] = 2220;
		index[10][7] = 2508;
		index[10][8] = 2854;
		index[10][9] = 3186;
		index[10][10] = 3467;
		index[10][11] = 3787;
		index[10][12] = 4122;
		index[10][13] = 4520;
		index[10][14] = 4830;
		index[10][15] = 5158;
		index[10][16] = 5527;
		index[10][17] = 5870;
		index[10][18] = 6203;
		index[10][19] = 6556;
		index[10][20] = 6894;
		index[10][21] = 7205;
		index[10][22] = 7528;
		index[10][23] = 7845;
		index[10][24] = 8174;
		index[10][25] = 8490;
		index[10][26] = 8829;
		index[10][27] = 9154;
		index[10][28] = 9517;
		index[10][29] = 9837;
		index[10][30] = 10108;
		index[10][31] = 10377;
		index[10][32] = 10649;
		index[10][33] = 10975;
		index[10][34] = 11251;
		index[10][35] = 11610;
		index[10][36] = 11912;
		index[10][37] = 12247;
		index[10][38] = 12560;
		index[10][39] = 12883;
		index[10][40] = 13184;
		index[10][41] = 13507;
		index[10][42] = 13852;
		index[10][43] = 14199;
		index[10][44] = 14578;
		index[10][45] = 14902;
		index[10][46] = 15243;
		index[10][47] = 15585;
		index[10][48] = 15939;
		index[10][49] = 16261;
		index[10][50] = 16584;
		index[10][51] = 16949;
		index[10][52] = 17301;
		index[10][53] = 17608;
		index[10][54] = 17978;
		index[10][55] = 18341;
		index[10][56] = 18730;
		index[10][57] = 19093;
		index[10][58] = 19401;
		index[10][59] = 19751;
		index[10][60] = 20120;
		index[10][61] = 20503;
		index[10][62] = 20873;
		index[10][63] = 21272;
		index[10][64] = 21653;
		index[10][65] = 22007;
		index[10][66] = 22454;
		index[10][67] = 22962;
		index[10][68] = 23440;
		index[10][69] = 23953;
		index[10][70] = 24384;
		index[10][71] = 24819;
		index[10][72] = 25276;
		index[10][73] = 25767;
		index[10][74] = 26273;
		index[10][75] = 26798;
		index[10][76] = 27323;
		index[10][77] = 27894;
		index[10][78] = 28486;
		index[10][79] = 29083;
		index[10][80] = 29662;
		index[10][81] = 30205;
		index[10][82] = 30828;
		index[10][83] = 31465;
		index[10][84] = 32096;
		index[10][85] = 32746;
		index[10][86] = 33424;
		index[10][87] = 34120;
		index[10][88] = 34820;
		index[10][89] = 35575;
		index[10][90] = 36309;
		index[10][91] = 37200;
		index[10][92] = 38084;
		index[10][93] = 38916;
		index[10][94] = 39734;
		index[10][95] = 40601;
		index[10][96] = 41466;
		index[10][97] = 42379;
		index[10][98] = 43332;
		index[10][99] = 44317;
		index[10][100] = 45292;
		index[10][101] = 46281;
		index[10][102] = 47221;
		index[10][103] = 48159;
		index[10][104] = 49169;
		index[10][105] = 50344;
		index[10][106] = 51680;
		index[10][107] = 52842;
		index[10][108] = 54173;
		index[10][109] = 55697;
		index[10][110] = 57408;
		index[10][111] = 59011;
		index[10][112] = 60459;
		index[10][113] = 62029;
		index[10][114] = 63589;
		index[10][115] = 65135;
		index[10][116] = 66680;
		index[10][117] = 68201;
		index[10][118] = 69633;
		index[10][119] = 71087;
		index[10][120] = 72962;
		index[10][121] = 74821;
		index[10][122] = 76782;
		index[10][123] = 79371;
		index[10][124] = 82054;
		index[10][125] = 84947;
		index[10][126] = 87407;
		index[10][127] = 89490;
		index[10][128] = 91477;
		index[10][129] = 93400;
		index[10][130] = 95112;
		index[10][131] = 96677;
		index[10][132] = 98129;
		index[10][133] = 99556;
		index[10][134] = 100867;
		index[10][135] = 102055;
		index[10][136] = 103286;
		index[10][137] = 104630;
		index[10][138] = 105966;
		index[10][139] = 107347;
		index[10][140] = 108660;
		index[10][141] = 109996;
		index[10][142] = 111213;
		index[10][143] = 112448;
		index[10][144] = 113677;
		index[10][145] = 114849;
		index[10][146] = 116082;
		index[10][147] = 117197;
		index[10][148] = 118225;
		index[10][149] = 119309;
		index[10][150] = 120477;
		index[10][151] = 121562;
		index[10][152] = 122589;
		index[10][153] = 123650;
		index[10][154] = 124811;
		index[10][155] = 126053;
		index[10][156] = 127223;
		index[10][157] = 128301;
		index[10][158] = 129379;
		index[10][159] = 130475;
		index[10][160] = 131572;
		index[10][161] = 132643;
		index[10][162] = 133714;
		index[10][163] = 134634;
		index[10][164] = 135559;
		index[10][165] = 136390;
		index[10][166] = 137189;
		index[10][167] = 138113;
		index[10][168] = 138850;
		index[10][169] = 139551;
		index[10][170] = 140280;
		index[10][171] = 141077;
		index[10][172] = 141933;
		index[10][173] = 142698;
		index[10][174] = 143421;
		index[10][175] = 144139;
		index[10][176] = 144936;
		index[10][177] = 145791;
		index[10][178] = 146606;
		index[10][179] = 147343;
		index[10][180] = 148117;
		index[10][181] = 148831;
		index[10][182] = 149606;
		index[10][183] = 150301;
		index[10][184] = 150944;
		index[10][185] = 151704;
		index[10][186] = 152460;
		index[10][187] = 153210;
		index[10][188] = 153969;
		index[10][189] = 154839;
		index[10][190] = 155613;
		index[10][191] = 156340;
		index[10][192] = 157076;
		index[10][193] = 157859;
		index[10][194] = 158725;
		index[10][195] = 159634;
		index[10][196] = 160434;
		index[10][197] = 161318;
		index[10][198] = 162218;
		index[10][199] = 163029;
		index[10][200] = 163870;
		index[10][201] = 164642;
		index[10][202] = 165596;
		index[10][203] = 166512;
		index[10][204] = 167486;
		index[10][205] = 168464;
		index[10][206] = 169464;
		index[10][207] = 170484;
		index[10][208] = 171554;
		index[10][209] = 172543;
		index[10][210] = 173573;
		index[10][211] = 174569;
		index[10][212] = 175601;
		index[10][213] = 176665;
		index[10][214] = 177748;
		index[10][215] = 178883;
		index[10][216] = 179976;
		index[10][217] = 181032;
		index[10][218] = 182153;
		index[10][219] = 183371;
		index[10][220] = 184512;
		index[10][221] = 185639;
		index[10][222] = 186697;
		index[10][223] = 187819;
		index[10][224] = 188964;
		index[10][225] = 190109;
		index[10][226] = 191354;
		index[10][227] = 192555;
		index[10][228] = 193583;
		index[10][229] = 194653;
		index[10][230] = 195609;
		index[10][231] = 196468;
		index[10][232] = 197500;
		index[10][233] = 198387;
		index[10][234] = 199271;
		index[10][235] = 200194;
		index[10][236] = 201147;
		index[10][237] = 202128;
		index[10][238] = 203029;
		index[10][239] = 203966;
		index[10][240] = 204997;
		index[10][241] = 206060;
		index[10][242] = 207049;
		index[10][243] = 207932;
		index[10][244] = 208645;
		index[10][245] = 209279;
		index[10][246] = 209910;
		index[10][247] = 210627;
		index[10][248] = 211627;
		index[10][249] = 212646;
		index[10][250] = 213771;
		index[10][251] = 214815;
		index[10][252] = 216031;
		index[10][253] = 217712;
		index[10][254] = 219398;
		index[10][255] = 221391;
		index[10][256] = 223670;
		index[10][257] = 225772;
		index[10][258] = 227786;
		index[10][259] = 229721;
		index[10][260] = 231565;
		index[10][261] = 233548;
		index[10][262] = 235824;
		index[10][263] = 238087;
		index[10][264] = 240486;
		index[10][265] = 242998;
		index[10][266] = 245695;
		index[10][267] = 249188;
		index[10][268] = 252928;
		index[10][269] = 256667;
		index[10][270] = 260419;
		index[10][271] = 264224;
		index[10][272] = 267816;
		index[10][273] = 271409;
		index[10][274] = 275058;
		index[10][275] = 278213;
		index[10][276] = 280916;
		index[10][277] = 283697;
		index[10][278] = 286210;
		index[10][279] = 288393;
		index[10][280] = 290410;
		index[10][281] = 292388;
		index[10][282] = 294089;
		index[10][283] = 295755;
		index[10][284] = 297367;
		index[10][285] = 298760;
		index[10][286] = 300088;
		index[10][287] = 301340;
		index[10][288] = 302563;
		index[10][289] = 303819;
		index[10][290] = 304948;
		index[10][291] = 306101;
		index[10][292] = 307386;
		index[10][293] = 308590;
		index[10][294] = 309532;
		index[10][295] = 310423;
		index[10][296] = 311375;
		index[10][297] = 312445;
		index[10][298] = 313412;
		index[10][299] = 314295;
		index[10][300] = 315168;
		index[10][301] = 316057;
		index[10][302] = 316969;
		index[10][303] = 317818;
		index[10][304] = 318610;
		index[10][305] = 319494;
		index[10][306] = 320395;
		index[10][307] = 321298;
		index[10][308] = 322135;
		index[10][309] = 323017;
		index[10][310] = 323806;
		index[10][311] = 324556;
		index[10][312] = 325273;
		index[10][313] = 325966;
		index[10][314] = 326655;
		index[10][315] = 327275;
		index[10][316] = 327862;
		index[10][317] = 328428;
		index[10][318] = 329050;
		index[10][319] = 329612;
		index[10][320] = 330190;
		index[10][321] = 330746;
		index[10][322] = 331335;
		index[10][323] = 331916;
		index[10][324] = 332525;
		index[10][325] = 333165;
		index[10][326] = 333744;
		index[10][327] = 334252;
		index[10][328] = 334768;
		index[10][329] = 335282;
		index[10][330] = 335821;
		index[10][331] = 336326;
		index[10][332] = 336799;
		index[10][333] = 337199;
		index[10][334] = 337668;
		index[10][335] = 338083;
		index[10][336] = 338483;
		index[10][337] = 338865;
		index[10][338] = 339286;
		index[10][339] = 339704;
		index[10][340] = 340127;
		index[10][341] = 340547;
		index[10][342] = 340978;
		index[10][343] = 341408;
		index[10][344] = 341821;
		index[10][345] = 342178;
		index[10][346] = 342562;
		index[10][347] = 342945;
		index[10][348] = 343303;
		index[10][349] = 343679;
		index[10][350] = 344030;
		index[10][351] = 344419;
		index[10][352] = 344761;
		index[10][353] = 345158;
		index[10][354] = 345527;
		index[10][355] = 345891;
		index[10][356] = 346276;
		index[10][357] = 346652;
		index[10][358] = 347013;
		index[10][359] = 347378;
	}

	/**
	 * Initializes the index in <tt>ac_n40.dat</tt>.
	 */
	private static void initialize_n40 ( ) {
		index[11][0] = 396;
		index[11][1] = 777;
		index[11][2] = 1099;
		index[11][3] = 1423;
		index[11][4] = 1787;
		index[11][5] = 2186;
		index[11][6] = 2595;
		index[11][7] = 2983;
		index[11][8] = 3354;
		index[11][9] = 3756;
		index[11][10] = 4205;
		index[11][11] = 4623;
		index[11][12] = 5114;
		index[11][13] = 5605;
		index[11][14] = 6048;
		index[11][15] = 6444;
		index[11][16] = 6847;
		index[11][17] = 7239;
		index[11][18] = 7576;
		index[11][19] = 7932;
		index[11][20] = 8322;
		index[11][21] = 8662;
		index[11][22] = 9047;
		index[11][23] = 9426;
		index[11][24] = 9800;
		index[11][25] = 10105;
		index[11][26] = 10456;
		index[11][27] = 10752;
		index[11][28] = 11154;
		index[11][29] = 11581;
		index[11][30] = 12026;
		index[11][31] = 12395;
		index[11][32] = 12762;
		index[11][33] = 13082;
		index[11][34] = 13411;
		index[11][35] = 13779;
		index[11][36] = 14069;
		index[11][37] = 14359;
		index[11][38] = 14649;
		index[11][39] = 14985;
		index[11][40] = 15327;
		index[11][41] = 15661;
		index[11][42] = 15992;
		index[11][43] = 16329;
		index[11][44] = 16676;
		index[11][45] = 17077;
		index[11][46] = 17422;
		index[11][47] = 17807;
		index[11][48] = 18129;
		index[11][49] = 18485;
		index[11][50] = 18862;
		index[11][51] = 19270;
		index[11][52] = 19675;
		index[11][53] = 19988;
		index[11][54] = 20330;
		index[11][55] = 20696;
		index[11][56] = 21068;
		index[11][57] = 21555;
		index[11][58] = 22007;
		index[11][59] = 22531;
		index[11][60] = 23065;
		index[11][61] = 23651;
		index[11][62] = 24209;
		index[11][63] = 24778;
		index[11][64] = 25360;
		index[11][65] = 25952;
		index[11][66] = 26634;
		index[11][67] = 27363;
		index[11][68] = 28037;
		index[11][69] = 28763;
		index[11][70] = 29483;
		index[11][71] = 30221;
		index[11][72] = 30995;
		index[11][73] = 31783;
		index[11][74] = 32561;
		index[11][75] = 33262;
		index[11][76] = 33941;
		index[11][77] = 34634;
		index[11][78] = 35229;
		index[11][79] = 35918;
		index[11][80] = 36864;
		index[11][81] = 37775;
		index[11][82] = 38663;
		index[11][83] = 39476;
		index[11][84] = 40346;
		index[11][85] = 41240;
		index[11][86] = 42228;
		index[11][87] = 43166;
		index[11][88] = 44147;
		index[11][89] = 45115;
		index[11][90] = 46145;
		index[11][91] = 47112;
		index[11][92] = 48122;
		index[11][93] = 49194;
		index[11][94] = 50389;
		index[11][95] = 51552;
		index[11][96] = 52719;
		index[11][97] = 54078;
		index[11][98] = 55458;
		index[11][99] = 57145;
		index[11][100] = 58764;
		index[11][101] = 60351;
		index[11][102] = 61936;
		index[11][103] = 63512;
		index[11][104] = 65134;
		index[11][105] = 66918;
		index[11][106] = 68730;
		index[11][107] = 70257;
		index[11][108] = 71976;
		index[11][109] = 73830;
		index[11][110] = 75677;
		index[11][111] = 77221;
		index[11][112] = 78719;
		index[11][113] = 80386;
		index[11][114] = 82321;
		index[11][115] = 84341;
		index[11][116] = 86607;
		index[11][117] = 89205;
		index[11][118] = 92195;
		index[11][119] = 95325;
		index[11][120] = 98119;
		index[11][121] = 100575;
		index[11][122] = 103058;
		index[11][123] = 105257;
		index[11][124] = 107464;
		index[11][125] = 109503;
		index[11][126] = 111717;
		index[11][127] = 113982;
		index[11][128] = 116390;
		index[11][129] = 119031;
		index[11][130] = 121518;
		index[11][131] = 124245;
		index[11][132] = 126859;
		index[11][133] = 128850;
		index[11][134] = 130471;
		index[11][135] = 131999;
		index[11][136] = 133588;
		index[11][137] = 135172;
		index[11][138] = 136836;
		index[11][139] = 138571;
		index[11][140] = 140278;
		index[11][141] = 141721;
		index[11][142] = 143040;
		index[11][143] = 144495;
		index[11][144] = 146090;
		index[11][145] = 147883;
		index[11][146] = 149967;
		index[11][147] = 151858;
		index[11][148] = 153506;
		index[11][149] = 155127;
		index[11][150] = 156960;
		index[11][151] = 159089;
		index[11][152] = 161192;
		index[11][153] = 163153;
		index[11][154] = 164944;
		index[11][155] = 166974;
		index[11][156] = 168750;
		index[11][157] = 170329;
		index[11][158] = 171954;
		index[11][159] = 173575;
		index[11][160] = 174933;
		index[11][161] = 176247;
		index[11][162] = 177416;
		index[11][163] = 178778;
		index[11][164] = 180333;
		index[11][165] = 181838;
		index[11][166] = 183123;
		index[11][167] = 184350;
		index[11][168] = 185568;
		index[11][169] = 186587;
		index[11][170] = 187663;
		index[11][171] = 189193;
		index[11][172] = 190746;
		index[11][173] = 192177;
		index[11][174] = 193607;
		index[11][175] = 195041;
		index[11][176] = 196170;
		index[11][177] = 197195;
		index[11][178] = 198059;
		index[11][179] = 199052;
		index[11][180] = 199931;
		index[11][181] = 200882;
		index[11][182] = 201848;
		index[11][183] = 202835;
		index[11][184] = 203719;
		index[11][185] = 204570;
		index[11][186] = 205378;
		index[11][187] = 206228;
		index[11][188] = 207119;
		index[11][189] = 207981;
		index[11][190] = 208960;
		index[11][191] = 209906;
		index[11][192] = 210885;
		index[11][193] = 211753;
		index[11][194] = 212667;
		index[11][195] = 213648;
		index[11][196] = 214533;
		index[11][197] = 215510;
		index[11][198] = 216549;
		index[11][199] = 217686;
		index[11][200] = 218704;
		index[11][201] = 220057;
		index[11][202] = 221337;
		index[11][203] = 222563;
		index[11][204] = 223653;
		index[11][205] = 224655;
		index[11][206] = 225843;
		index[11][207] = 227069;
		index[11][208] = 228286;
		index[11][209] = 229989;
		index[11][210] = 231678;
		index[11][211] = 233173;
		index[11][212] = 234552;
		index[11][213] = 236135;
		index[11][214] = 237662;
		index[11][215] = 238983;
		index[11][216] = 240144;
		index[11][217] = 241356;
		index[11][218] = 242777;
		index[11][219] = 244208;
		index[11][220] = 245638;
		index[11][221] = 247139;
		index[11][222] = 249116;
		index[11][223] = 251108;
		index[11][224] = 253070;
		index[11][225] = 255189;
		index[11][226] = 257644;
		index[11][227] = 259878;
		index[11][228] = 261946;
		index[11][229] = 263573;
		index[11][230] = 265215;
		index[11][231] = 266927;
		index[11][232] = 269012;
		index[11][233] = 271746;
		index[11][234] = 274590;
		index[11][235] = 277261;
		index[11][236] = 279457;
		index[11][237] = 281311;
		index[11][238] = 283005;
		index[11][239] = 284499;
		index[11][240] = 285737;
		index[11][241] = 287421;
		index[11][242] = 288909;
		index[11][243] = 290185;
		index[11][244] = 291840;
		index[11][245] = 294006;
		index[11][246] = 296731;
		index[11][247] = 299168;
		index[11][248] = 301909;
		index[11][249] = 304421;
		index[11][250] = 307251;
		index[11][251] = 310447;
		index[11][252] = 313242;
		index[11][253] = 316170;
		index[11][254] = 318358;
		index[11][255] = 320268;
		index[11][256] = 322039;
		index[11][257] = 324125;
		index[11][258] = 326537;
		index[11][259] = 329210;
		index[11][260] = 332138;
		index[11][261] = 335508;
		index[11][262] = 339053;
		index[11][263] = 342254;
		index[11][264] = 345573;
		index[11][265] = 349160;
		index[11][266] = 352749;
		index[11][267] = 355914;
		index[11][268] = 358440;
		index[11][269] = 361391;
		index[11][270] = 364519;
		index[11][271] = 368377;
		index[11][272] = 371933;
		index[11][273] = 375267;
		index[11][274] = 378687;
		index[11][275] = 381894;
		index[11][276] = 385109;
		index[11][277] = 388007;
		index[11][278] = 391082;
		index[11][279] = 394028;
		index[11][280] = 396613;
		index[11][281] = 399247;
		index[11][282] = 401413;
		index[11][283] = 403333;
		index[11][284] = 405566;
		index[11][285] = 407766;
		index[11][286] = 409835;
		index[11][287] = 411570;
		index[11][288] = 413177;
		index[11][289] = 414648;
		index[11][290] = 416005;
		index[11][291] = 417425;
		index[11][292] = 418535;
		index[11][293] = 419571;
		index[11][294] = 420651;
		index[11][295] = 421834;
		index[11][296] = 423143;
		index[11][297] = 424352;
		index[11][298] = 425549;
		index[11][299] = 426632;
		index[11][300] = 427699;
		index[11][301] = 428747;
		index[11][302] = 429791;
		index[11][303] = 430852;
		index[11][304] = 431866;
		index[11][305] = 432930;
		index[11][306] = 433984;
		index[11][307] = 434988;
		index[11][308] = 435982;
		index[11][309] = 436901;
		index[11][310] = 437804;
		index[11][311] = 438609;
		index[11][312] = 439351;
		index[11][313] = 440124;
		index[11][314] = 440844;
		index[11][315] = 441528;
		index[11][316] = 442211;
		index[11][317] = 442878;
		index[11][318] = 443532;
		index[11][319] = 444237;
		index[11][320] = 445004;
		index[11][321] = 445740;
		index[11][322] = 446482;
		index[11][323] = 447185;
		index[11][324] = 447830;
		index[11][325] = 448462;
		index[11][326] = 449130;
		index[11][327] = 449752;
		index[11][328] = 450359;
		index[11][329] = 450933;
		index[11][330] = 451525;
		index[11][331] = 452151;
		index[11][332] = 452700;
		index[11][333] = 453222;
		index[11][334] = 453674;
		index[11][335] = 454139;
		index[11][336] = 454664;
		index[11][337] = 455178;
		index[11][338] = 455694;
		index[11][339] = 456115;
		index[11][340] = 456559;
		index[11][341] = 456963;
		index[11][342] = 457412;
		index[11][343] = 457896;
		index[11][344] = 458376;
		index[11][345] = 458791;
		index[11][346] = 459175;
		index[11][347] = 459573;
		index[11][348] = 459968;
		index[11][349] = 460288;
		index[11][350] = 460617;
		index[11][351] = 460995;
		index[11][352] = 461332;
		index[11][353] = 461710;
		index[11][354] = 462052;
		index[11][355] = 462388;
		index[11][356] = 462770;
		index[11][357] = 463137;
		index[11][358] = 463469;
		index[11][359] = 463840;
	}

	/**
	 * Initializes the index in <tt>ac_n50.dat</tt>.
	 */
	private static void initialize_n50 ( ) {
		index[12][0] = 286;
		index[12][1] = 669;
		index[12][2] = 995;
		index[12][3] = 1314;
		index[12][4] = 1635;
		index[12][5] = 1949;
		index[12][6] = 2265;
		index[12][7] = 2621;
		index[12][8] = 2941;
		index[12][9] = 3261;
		index[12][10] = 3568;
		index[12][11] = 3835;
		index[12][12] = 4113;
		index[12][13] = 4413;
		index[12][14] = 4686;
		index[12][15] = 4986;
		index[12][16] = 5366;
		index[12][17] = 5722;
		index[12][18] = 6037;
		index[12][19] = 6304;
		index[12][20] = 6552;
		index[12][21] = 6787;
		index[12][22] = 7084;
		index[12][23] = 7391;
		index[12][24] = 7686;
		index[12][25] = 7959;
		index[12][26] = 8224;
		index[12][27] = 8462;
		index[12][28] = 8736;
		index[12][29] = 9046;
		index[12][30] = 9318;
		index[12][31] = 9630;
		index[12][32] = 9907;
		index[12][33] = 10222;
		index[12][34] = 10510;
		index[12][35] = 10772;
		index[12][36] = 11029;
		index[12][37] = 11270;
		index[12][38] = 11503;
		index[12][39] = 11779;
		index[12][40] = 12074;
		index[12][41] = 12395;
		index[12][42] = 12693;
		index[12][43] = 12984;
		index[12][44] = 13260;
		index[12][45] = 13559;
		index[12][46] = 13877;
		index[12][47] = 14200;
		index[12][48] = 14507;
		index[12][49] = 14818;
		index[12][50] = 15119;
		index[12][51] = 15429;
		index[12][52] = 15705;
		index[12][53] = 15988;
		index[12][54] = 16270;
		index[12][55] = 16576;
		index[12][56] = 16889;
		index[12][57] = 17237;
		index[12][58] = 17621;
		index[12][59] = 17987;
		index[12][60] = 18408;
		index[12][61] = 18760;
		index[12][62] = 19119;
		index[12][63] = 19526;
		index[12][64] = 19924;
		index[12][65] = 20251;
		index[12][66] = 20613;
		index[12][67] = 20909;
		index[12][68] = 21270;
		index[12][69] = 21654;
		index[12][70] = 22075;
		index[12][71] = 22438;
		index[12][72] = 22824;
		index[12][73] = 23151;
		index[12][74] = 23541;
		index[12][75] = 23915;
		index[12][76] = 24281;
		index[12][77] = 24703;
		index[12][78] = 25164;
		index[12][79] = 25652;
		index[12][80] = 26131;
		index[12][81] = 26658;
		index[12][82] = 27116;
		index[12][83] = 27550;
		index[12][84] = 28042;
		index[12][85] = 28542;
		index[12][86] = 29054;
		index[12][87] = 29556;
		index[12][88] = 30024;
		index[12][89] = 30553;
		index[12][90] = 31139;
		index[12][91] = 31714;
		index[12][92] = 32214;
		index[12][93] = 32766;
		index[12][94] = 33365;
		index[12][95] = 33868;
		index[12][96] = 34524;
		index[12][97] = 35100;
		index[12][98] = 35852;
		index[12][99] = 36610;
		index[12][100] = 37313;
		index[12][101] = 38008;
		index[12][102] = 38730;
		index[12][103] = 39517;
		index[12][104] = 40309;
		index[12][105] = 41154;
		index[12][106] = 42050;
		index[12][107] = 42905;
		index[12][108] = 43801;
		index[12][109] = 44631;
		index[12][110] = 45493;
		index[12][111] = 46355;
		index[12][112] = 47259;
		index[12][113] = 48161;
		index[12][114] = 49099;
		index[12][115] = 50048;
		index[12][116] = 50948;
		index[12][117] = 51828;
		index[12][118] = 52745;
		index[12][119] = 53503;
		index[12][120] = 54273;
		index[12][121] = 54975;
		index[12][122] = 55721;
		index[12][123] = 56587;
		index[12][124] = 57337;
		index[12][125] = 58258;
		index[12][126] = 59214;
		index[12][127] = 60198;
		index[12][128] = 61433;
		index[12][129] = 62696;
		index[12][130] = 64174;
		index[12][131] = 65716;
		index[12][132] = 67246;
		index[12][133] = 68991;
		index[12][134] = 70677;
		index[12][135] = 72356;
		index[12][136] = 74122;
		index[12][137] = 76344;
		index[12][138] = 78733;
		index[12][139] = 81049;
		index[12][140] = 83142;
		index[12][141] = 85335;
		index[12][142] = 87179;
		index[12][143] = 89086;
		index[12][144] = 91448;
		index[12][145] = 93862;
		index[12][146] = 96324;
		index[12][147] = 99182;
		index[12][148] = 102327;
		index[12][149] = 105477;
		index[12][150] = 108663;
		index[12][151] = 111657;
		index[12][152] = 114258;
		index[12][153] = 116964;
		index[12][154] = 119563;
		index[12][155] = 122083;
		index[12][156] = 124727;
		index[12][157] = 127389;
		index[12][158] = 130119;
		index[12][159] = 133088;
		index[12][160] = 135914;
		index[12][161] = 138524;
		index[12][162] = 141137;
		index[12][163] = 144014;
		index[12][164] = 146757;
		index[12][165] = 149509;
		index[12][166] = 152043;
		index[12][167] = 153964;
		index[12][168] = 155987;
		index[12][169] = 157984;
		index[12][170] = 159977;
		index[12][171] = 162290;
		index[12][172] = 164467;
		index[12][173] = 166266;
		index[12][174] = 167815;
		index[12][175] = 169432;
		index[12][176] = 171013;
		index[12][177] = 172655;
		index[12][178] = 174477;
		index[12][179] = 176181;
		index[12][180] = 177817;
		index[12][181] = 179402;
		index[12][182] = 180976;
		index[12][183] = 182580;
		index[12][184] = 184168;
		index[12][185] = 185649;
		index[12][186] = 186968;
		index[12][187] = 188211;
		index[12][188] = 189548;
		index[12][189] = 190956;
		index[12][190] = 192224;
		index[12][191] = 193414;
		index[12][192] = 194579;
		index[12][193] = 195663;
		index[12][194] = 196658;
		index[12][195] = 197725;
		index[12][196] = 198832;
		index[12][197] = 200053;
		index[12][198] = 201424;
		index[12][199] = 202761;
		index[12][200] = 204068;
		index[12][201] = 205158;
		index[12][202] = 206180;
		index[12][203] = 207152;
		index[12][204] = 208255;
		index[12][205] = 209417;
		index[12][206] = 210553;
		index[12][207] = 211649;
		index[12][208] = 212921;
		index[12][209] = 214125;
		index[12][210] = 215454;
		index[12][211] = 216870;
		index[12][212] = 218255;
		index[12][213] = 219516;
		index[12][214] = 220720;
		index[12][215] = 221797;
		index[12][216] = 222884;
		index[12][217] = 224016;
		index[12][218] = 225248;
		index[12][219] = 226470;
		index[12][220] = 227920;
		index[12][221] = 229218;
		index[12][222] = 230355;
		index[12][223] = 231423;
		index[12][224] = 232506;
		index[12][225] = 233753;
		index[12][226] = 235065;
		index[12][227] = 236411;
		index[12][228] = 237798;
		index[12][229] = 239186;
		index[12][230] = 240643;
		index[12][231] = 242300;
		index[12][232] = 243972;
		index[12][233] = 245787;
		index[12][234] = 248032;
		index[12][235] = 250577;
		index[12][236] = 252897;
		index[12][237] = 254695;
		index[12][238] = 256319;
		index[12][239] = 257925;
		index[12][240] = 259117;
		index[12][241] = 260227;
		index[12][242] = 261689;
		index[12][243] = 264012;
		index[12][244] = 266559;
		index[12][245] = 269084;
		index[12][246] = 271450;
		index[12][247] = 273837;
		index[12][248] = 275807;
		index[12][249] = 277389;
		index[12][250] = 278756;
		index[12][251] = 279901;
		index[12][252] = 281261;
		index[12][253] = 282773;
		index[12][254] = 284501;
		index[12][255] = 285998;
		index[12][256] = 287376;
		index[12][257] = 288742;
		index[12][258] = 290152;
		index[12][259] = 291664;
		index[12][260] = 293522;
		index[12][261] = 295592;
		index[12][262] = 297567;
		index[12][263] = 298964;
		index[12][264] = 300364;
		index[12][265] = 302092;
		index[12][266] = 304120;
		index[12][267] = 306129;
		index[12][268] = 308184;
		index[12][269] = 309819;
		index[12][270] = 311185;
		index[12][271] = 312667;
		index[12][272] = 314082;
		index[12][273] = 315368;
		index[12][274] = 316398;
		index[12][275] = 317439;
		index[12][276] = 318708;
		index[12][277] = 319921;
		index[12][278] = 320923;
		index[12][279] = 321705;
		index[12][280] = 322505;
		index[12][281] = 323566;
		index[12][282] = 324746;
		index[12][283] = 325907;
		index[12][284] = 327059;
		index[12][285] = 327970;
		index[12][286] = 328797;
		index[12][287] = 329673;
		index[12][288] = 330646;
		index[12][289] = 331634;
		index[12][290] = 332519;
		index[12][291] = 333307;
		index[12][292] = 334023;
		index[12][293] = 334809;
		index[12][294] = 335633;
		index[12][295] = 336495;
		index[12][296] = 337307;
		index[12][297] = 337948;
		index[12][298] = 338628;
		index[12][299] = 339307;
		index[12][300] = 339897;
		index[12][301] = 340445;
		index[12][302] = 340978;
		index[12][303] = 341443;
		index[12][304] = 341934;
		index[12][305] = 342458;
		index[12][306] = 343016;
		index[12][307] = 343453;
		index[12][308] = 343875;
		index[12][309] = 344361;
		index[12][310] = 344726;
		index[12][311] = 345120;
		index[12][312] = 345534;
		index[12][313] = 345963;
		index[12][314] = 346424;
		index[12][315] = 346843;
		index[12][316] = 347240;
		index[12][317] = 347595;
		index[12][318] = 347994;
		index[12][319] = 348445;
		index[12][320] = 348974;
		index[12][321] = 349420;
		index[12][322] = 349902;
		index[12][323] = 350372;
		index[12][324] = 350778;
		index[12][325] = 351280;
		index[12][326] = 351805;
		index[12][327] = 352293;
		index[12][328] = 352747;
		index[12][329] = 353275;
		index[12][330] = 353695;
		index[12][331] = 354108;
		index[12][332] = 354458;
		index[12][333] = 354824;
		index[12][334] = 355202;
		index[12][335] = 355530;
		index[12][336] = 355876;
		index[12][337] = 356227;
		index[12][338] = 356590;
		index[12][339] = 356956;
		index[12][340] = 357310;
		index[12][341] = 357706;
		index[12][342] = 358111;
		index[12][343] = 358493;
		index[12][344] = 358832;
		index[12][345] = 359258;
		index[12][346] = 359713;
		index[12][347] = 360098;
		index[12][348] = 360454;
		index[12][349] = 360791;
		index[12][350] = 361102;
		index[12][351] = 361424;
		index[12][352] = 361786;
		index[12][353] = 362144;
		index[12][354] = 362449;
		index[12][355] = 362779;
		index[12][356] = 363076;
		index[12][357] = 363320;
		index[12][358] = 363545;
		index[12][359] = 363805;
	}

	/**
	 * Initializes the index in <tt>ac_n60.dat</tt>.
	 */
	private static void initialize_n60 ( ) {
		index[13][0] = 463;
		index[13][1] = 879;
		index[13][2] = 1304;
		index[13][3] = 1741;
		index[13][4] = 2160;
		index[13][5] = 2621;
		index[13][6] = 3062;
		index[13][7] = 3478;
		index[13][8] = 3925;
		index[13][9] = 4377;
		index[13][10] = 4803;
		index[13][11] = 5289;
		index[13][12] = 5762;
		index[13][13] = 6258;
		index[13][14] = 6752;
		index[13][15] = 7265;
		index[13][16] = 7703;
		index[13][17] = 8149;
		index[13][18] = 8578;
		index[13][19] = 9061;
		index[13][20] = 9528;
		index[13][21] = 9976;
		index[13][22] = 10434;
		index[13][23] = 10902;
		index[13][24] = 11376;
		index[13][25] = 11882;
		index[13][26] = 12405;
		index[13][27] = 12960;
		index[13][28] = 13461;
		index[13][29] = 13981;
		index[13][30] = 14510;
		index[13][31] = 15004;
		index[13][32] = 15520;
		index[13][33] = 15981;
		index[13][34] = 16386;
		index[13][35] = 16788;
		index[13][36] = 17187;
		index[13][37] = 17628;
		index[13][38] = 18046;
		index[13][39] = 18445;
		index[13][40] = 18832;
		index[13][41] = 19240;
		index[13][42] = 19643;
		index[13][43] = 20044;
		index[13][44] = 20485;
		index[13][45] = 20926;
		index[13][46] = 21397;
		index[13][47] = 21888;
		index[13][48] = 22368;
		index[13][49] = 22825;
		index[13][50] = 23268;
		index[13][51] = 23707;
		index[13][52] = 24131;
		index[13][53] = 24562;
		index[13][54] = 24950;
		index[13][55] = 25359;
		index[13][56] = 25756;
		index[13][57] = 26146;
		index[13][58] = 26554;
		index[13][59] = 27018;
		index[13][60] = 27426;
		index[13][61] = 27916;
		index[13][62] = 28462;
		index[13][63] = 29011;
		index[13][64] = 29490;
		index[13][65] = 29949;
		index[13][66] = 30377;
		index[13][67] = 30853;
		index[13][68] = 31312;
		index[13][69] = 31832;
		index[13][70] = 32371;
		index[13][71] = 32870;
		index[13][72] = 33353;
		index[13][73] = 33883;
		index[13][74] = 34491;
		index[13][75] = 35053;
		index[13][76] = 35735;
		index[13][77] = 36318;
		index[13][78] = 36869;
		index[13][79] = 37480;
		index[13][80] = 38059;
		index[13][81] = 38719;
		index[13][82] = 39626;
		index[13][83] = 40575;
		index[13][84] = 41398;
		index[13][85] = 42178;
		index[13][86] = 42819;
		index[13][87] = 43448;
		index[13][88] = 44090;
		index[13][89] = 44646;
		index[13][90] = 45244;
		index[13][91] = 45867;
		index[13][92] = 46474;
		index[13][93] = 47048;
		index[13][94] = 47647;
		index[13][95] = 48237;
		index[13][96] = 48873;
		index[13][97] = 49474;
		index[13][98] = 50095;
		index[13][99] = 50779;
		index[13][100] = 51433;
		index[13][101] = 52095;
		index[13][102] = 52753;
		index[13][103] = 53391;
		index[13][104] = 54044;
		index[13][105] = 54763;
		index[13][106] = 55416;
		index[13][107] = 56263;
		index[13][108] = 57094;
		index[13][109] = 57890;
		index[13][110] = 58693;
		index[13][111] = 59533;
		index[13][112] = 60339;
		index[13][113] = 61023;
		index[13][114] = 61702;
		index[13][115] = 62396;
		index[13][116] = 63132;
		index[13][117] = 64072;
		index[13][118] = 65109;
		index[13][119] = 66147;
		index[13][120] = 67223;
		index[13][121] = 68230;
		index[13][122] = 69205;
		index[13][123] = 70299;
		index[13][124] = 71476;
		index[13][125] = 72670;
		index[13][126] = 73836;
		index[13][127] = 75003;
		index[13][128] = 75970;
		index[13][129] = 76929;
		index[13][130] = 77847;
		index[13][131] = 78754;
		index[13][132] = 79842;
		index[13][133] = 80888;
		index[13][134] = 81918;
		index[13][135] = 83130;
		index[13][136] = 84335;
		index[13][137] = 85554;
		index[13][138] = 86830;
		index[13][139] = 88232;
		index[13][140] = 89685;
		index[13][141] = 91167;
		index[13][142] = 92555;
		index[13][143] = 93904;
		index[13][144] = 95247;
		index[13][145] = 96732;
		index[13][146] = 98229;
		index[13][147] = 100007;
		index[13][148] = 102074;
		index[13][149] = 104319;
		index[13][150] = 106723;
		index[13][151] = 109196;
		index[13][152] = 111623;
		index[13][153] = 113922;
		index[13][154] = 116292;
		index[13][155] = 118674;
		index[13][156] = 120647;
		index[13][157] = 122658;
		index[13][158] = 124993;
		index[13][159] = 127323;
		index[13][160] = 129508;
		index[13][161] = 131457;
		index[13][162] = 133508;
		index[13][163] = 135730;
		index[13][164] = 137988;
		index[13][165] = 140182;
		index[13][166] = 142216;
		index[13][167] = 144233;
		index[13][168] = 146106;
		index[13][169] = 147737;
		index[13][170] = 149262;
		index[13][171] = 150905;
		index[13][172] = 152559;
		index[13][173] = 154608;
		index[13][174] = 157030;
		index[13][175] = 159769;
		index[13][176] = 162713;
		index[13][177] = 165568;
		index[13][178] = 168583;
		index[13][179] = 171167;
		index[13][180] = 173727;
		index[13][181] = 175837;
		index[13][182] = 177829;
		index[13][183] = 179647;
		index[13][184] = 181492;
		index[13][185] = 183064;
		index[13][186] = 184449;
		index[13][187] = 185679;
		index[13][188] = 186975;
		index[13][189] = 188248;
		index[13][190] = 189632;
		index[13][191] = 191057;
		index[13][192] = 192689;
		index[13][193] = 194208;
		index[13][194] = 195406;
		index[13][195] = 196684;
		index[13][196] = 198037;
		index[13][197] = 199251;
		index[13][198] = 200585;
		index[13][199] = 201920;
		index[13][200] = 203367;
		index[13][201] = 204942;
		index[13][202] = 206589;
		index[13][203] = 208158;
		index[13][204] = 209595;
		index[13][205] = 211006;
		index[13][206] = 212584;
		index[13][207] = 214440;
		index[13][208] = 216455;
		index[13][209] = 218271;
		index[13][210] = 219800;
		index[13][211] = 221122;
		index[13][212] = 222326;
		index[13][213] = 223498;
		index[13][214] = 224683;
		index[13][215] = 225971;
		index[13][216] = 227373;
		index[13][217] = 228821;
		index[13][218] = 230209;
		index[13][219] = 231661;
		index[13][220] = 232826;
		index[13][221] = 233896;
		index[13][222] = 234968;
		index[13][223] = 235987;
		index[13][224] = 237029;
		index[13][225] = 238170;
		index[13][226] = 239362;
		index[13][227] = 240471;
		index[13][228] = 241669;
		index[13][229] = 243003;
		index[13][230] = 244414;
		index[13][231] = 245945;
		index[13][232] = 247513;
		index[13][233] = 249026;
		index[13][234] = 250525;
		index[13][235] = 252002;
		index[13][236] = 253631;
		index[13][237] = 255380;
		index[13][238] = 257062;
		index[13][239] = 258721;
		index[13][240] = 260464;
		index[13][241] = 262268;
		index[13][242] = 264202;
		index[13][243] = 266033;
		index[13][244] = 267801;
		index[13][245] = 269333;
		index[13][246] = 270669;
		index[13][247] = 271906;
		index[13][248] = 273013;
		index[13][249] = 274081;
		index[13][250] = 275089;
		index[13][251] = 276090;
		index[13][252] = 277192;
		index[13][253] = 278285;
		index[13][254] = 279452;
		index[13][255] = 280563;
		index[13][256] = 281659;
		index[13][257] = 282706;
		index[13][258] = 283773;
		index[13][259] = 284792;
		index[13][260] = 285827;
		index[13][261] = 286880;
		index[13][262] = 287886;
		index[13][263] = 288840;
		index[13][264] = 289816;
		index[13][265] = 290783;
		index[13][266] = 291646;
		index[13][267] = 292480;
		index[13][268] = 293280;
		index[13][269] = 294095;
		index[13][270] = 294902;
		index[13][271] = 295674;
		index[13][272] = 296491;
		index[13][273] = 297352;
		index[13][274] = 298099;
		index[13][275] = 298899;
		index[13][276] = 299725;
		index[13][277] = 300512;
		index[13][278] = 301383;
		index[13][279] = 302088;
		index[13][280] = 302818;
		index[13][281] = 303498;
		index[13][282] = 304264;
		index[13][283] = 304947;
		index[13][284] = 305659;
		index[13][285] = 306390;
		index[13][286] = 307149;
		index[13][287] = 307912;
		index[13][288] = 308656;
		index[13][289] = 309376;
		index[13][290] = 310087;
		index[13][291] = 310803;
		index[13][292] = 311412;
		index[13][293] = 312052;
		index[13][294] = 312691;
		index[13][295] = 313349;
		index[13][296] = 314000;
		index[13][297] = 314638;
		index[13][298] = 315270;
		index[13][299] = 315904;
		index[13][300] = 316586;
		index[13][301] = 317272;
		index[13][302] = 318013;
		index[13][303] = 318704;
		index[13][304] = 319359;
		index[13][305] = 320050;
		index[13][306] = 320637;
		index[13][307] = 321164;
		index[13][308] = 321747;
		index[13][309] = 322252;
		index[13][310] = 322708;
		index[13][311] = 323152;
		index[13][312] = 323652;
		index[13][313] = 324125;
		index[13][314] = 324633;
		index[13][315] = 325179;
		index[13][316] = 325678;
		index[13][317] = 326184;
		index[13][318] = 326642;
		index[13][319] = 327131;
		index[13][320] = 327583;
		index[13][321] = 328058;
		index[13][322] = 328525;
		index[13][323] = 329035;
		index[13][324] = 329441;
		index[13][325] = 329868;
		index[13][326] = 330243;
		index[13][327] = 330617;
		index[13][328] = 331004;
		index[13][329] = 331357;
		index[13][330] = 331721;
		index[13][331] = 332098;
		index[13][332] = 332518;
		index[13][333] = 332975;
		index[13][334] = 333408;
		index[13][335] = 333803;
		index[13][336] = 334150;
		index[13][337] = 334549;
		index[13][338] = 334934;
		index[13][339] = 335321;
		index[13][340] = 335731;
		index[13][341] = 336131;
		index[13][342] = 336498;
		index[13][343] = 336910;
		index[13][344] = 337315;
		index[13][345] = 337720;
		index[13][346] = 338078;
		index[13][347] = 338457;
		index[13][348] = 338877;
		index[13][349] = 339227;
		index[13][350] = 339624;
		index[13][351] = 340023;
		index[13][352] = 340383;
		index[13][353] = 340726;
		index[13][354] = 341149;
		index[13][355] = 341589;
		index[13][356] = 342007;
		index[13][357] = 342398;
		index[13][358] = 342840;
		index[13][359] = 343293;
	}

	/**
	 * Sets the R.A. range of each block.
	 */
	static {
		index = new int[file_N][];
		for (int i = 0 ; i < file_N ; i++)
			index[i] = new int[360];

		initialize_p60();
		initialize_p50();
		initialize_p40();
		initialize_p30();
		initialize_p20();
		initialize_p10();
		initialize_p00();
		initialize_n00();
		initialize_n10();
		initialize_n20();
		initialize_n30();
		initialize_n40();
		initialize_n50();
		initialize_n60();
	}
}
