/*
 * @(#)TicReader.java
 *
 * Copyright (C) 1997-2007 Seiichi Yoshida
 * All rights reserved.
 */

package net.aerith.misao.catalog.io;
import java.io.*;
import java.net.*;
import java.util.Vector;
import net.aerith.misao.util.*;
import net.aerith.misao.util.star.*;
import net.aerith.misao.catalog.*;
import net.aerith.misao.catalog.star.*;
import net.aerith.misao.io.Decoder;
import net.aerith.misao.io.CdromNotFoundException;

/**
 * The <code>TicReader</code> is a class to read the Tycho Input 
 * Catalogue.
 * <p>
 * The (x,y) position is also set properly so that (0,0) represents
 * the specified R.A. and Decl. to <tt>open</tt> method and (1,1) 
 * represents the position 1 deg to the west and 1 deg to the north.
 *
 * @author  Seiichi Yoshida (comet@aerith.net)
 * @version 2004 May 8
 */

public class TicReader extends CatalogReader {
	/**
	 * The number of files.
	 */
	protected final static int file_N = 4;

	/**
	 * The file names.
	 */
	protected final static String[] filenames = {
		"tic1", "tic2", "tic3", "tic4"
	};

	/**
	 * The start declinations of each files.
	 */
	protected final static double[] file_decl_start = {
		0, 37.5, -37.5, -90
	};

	/**
	 * The end declinations of each files.
	 */
	protected final static double[] file_decl_end = {
		37.5, 90, 0, -37.5
	};

	/**
	 * The number of blocks of each files.
	 */
	protected final static int[] file_block_count = {
		392, 288, 460, 439
	};

	/**
	 * The smallest R.A. in degree of the block.
	 */
	protected static int[][] ra_start;

	/**
	 * The largest R.A. in degree of the block.
	 */
	protected static int[][] ra_end;

	/**
	 * The smallest Decl. in degree of the block.
	 */
	protected static int[][] decl_start;

	/**
	 * The largest Decl. in degree of the block.
	 */
	protected static int[][] decl_end;

	/**
	 * The current_input stream reader.
	 */
	protected BufferedReader current_reader;

	/**
	 * The index of current file in <code>file_list</code>.
	 */
	protected int current_file_index_in_list;

	/**
	 * The index of current file (0-3).
	 */
	protected int current_file_index;

	/**
	 * The index of the current block.
	 */
	protected long current_block_index = 0;

	/**
	 * The index of the star in the current block.
	 */
	protected long current_star_index = 0;

	/**
	 * The circum area to read stars.
	 */
	protected CircumArea circum_area;

	/**
	 * The list of file index to be read.
	 */
	protected Vector file_list;

	/**
	 * Constructs an empty <code>TicReader</code>.
	 */
	public TicReader ( ) {
		url_set.ignoreCases(true);
		url_set.acceptGzipped(true);

		setDefaultURL();
	}

	/**
	 * Constructs a <code>TicReader</code> with URL of directory 
	 * containing the AC 2000.2 Catalogue.
	 * @param url the URL of directory containing CD-ROMs data.
	 */
	public TicReader ( URL url ) {
		this();

		addURL(url);
	}

	/**
	 * Gets the catalog name. It must be unique among all subclasses.
	 * @return the catalog name.
	 */
	public String getName ( ) {
		return "Tycho Input Catalogue";
	}

	/**
	 * Checks if the catalog data is in a directory.
	 * @return true if the catalog data is in a directory.
	 */
	public boolean isInDirectory ( ) {
		return true;
	}

	/**
	 * Checks if the catalog supports the use in PIXY examination.
	 * @return true if the catalog can be used in PIXY examination.
	 */
	public boolean supportsExamination ( ) {
		return true;
	}

	/**
	 * Gets the maximum error of position in arcsec. It is the search
	 * area size to identify with other stars.
	 * @return the maximum error of position in arcsec.
	 */
	public double getMaximumPositionErrorInArcsec ( ) {
		return 5.0;
	}

	/**
	 * Gets the help message.
	 * @return the help message.
	 */
	public String getHelpMessage ( ) {
		String html = "<html><body>";
		html += "<p>";
		html += "Tycho Input Catalogue, Revised version<br>";
		html += "Astronomical Data Center catalog No. 1197A<br>";
		html += "ADC CD-ROM Vol. 2<br>";
		html += "</p><p>";
		html += "Download:";
		html += "<blockquote>";
		html += "<u><font color=\"#0000ff\">ftp://dbc.nao.ac.jp/DBC/NASAADC/catalogs/1/1197A/</font></u>";
		html += "</blockquote>";
		html += "</p>";
		html += "</body></html>";
		return html;
	}

	/**
	 * Opens a catalog. This method must be invoked at first.
	 * @param coor the R.A. and Decl. of the center.
	 * @param fov  the field of view to read in degree.
	 * @exception IOException if a file cannot be accessed.
	 * @exception FileNotFoundException if a file does not exists in
	 * any URL, and the file is in any CD-ROMs.
	 * @exception CdromNotFoundException if a file does not exists in
	 * any URL, and the file is in a CD-ROM.
	 */
	public void open ( Coor coor, double fov )
		throws IOException, FileNotFoundException, CdromNotFoundException
	{
		center_coor = coor;
		circum_area = null;
		if (coor != null)
			circum_area = new CircumArea(coor, fov / 2.0);
		file_list = new Vector();

		// Select files to read.
		for (int i = 0 ; i < file_N ; i++) {
			Coor start_coor = new Coor(0, file_decl_start[i]);
			Coor end_coor = new Coor(180, file_decl_end[i]);

			if (circum_area == null  ||  circum_area.overlapsArea(start_coor, end_coor)) {
				file_list.addElement(new Integer(i));
			} else {
				start_coor = new Coor(180, file_decl_start[i]);
				end_coor = new Coor(0, file_decl_end[i]);

				if (circum_area == null  ||  circum_area.overlapsArea(start_coor, end_coor))
					file_list.addElement(new Integer(i));
			}
		}

		// Initializes to read data one by one.
		current_reader = null;
	}

	/**
	 * Reads one data from the opened catalog. After this method is 
	 * invoked, the cursor is promoted to tne next star. When every 
	 * data is read, it returns null. Note that data with the same ID
	 * are also returned one by one.
	 * @return a star data.
	 * @exception IOException if a file cannot be accessed.
	 * @exception FileNotFoundException if a file does not exists in
	 * any URL, and the file is in any CD-ROMs.
	 * @exception CdromNotFoundException if a file does not exists in
	 * any URL, and the file is in a CD-ROM.
	 * @exception QueryFailException if the query to the server is 
	 * failed.
	 */
	public CatalogStar readNext ( )
		throws IOException, FileNotFoundException, CdromNotFoundException, QueryFailException
	{
		while (true) {
			try {
				// In the case of the very first reading.
				if (current_reader == null) {
					current_file_index_in_list = -1;
					throw new EOFException();
				}

				current_star_index++;

				// In the case it reaches to the end of a block.
				if (current_star_index == 2000) {
					current_block_index++;

					// In the case it reaches to the last block.
					if (current_block_index == file_block_count[current_file_index])
						throw new EOFException();

					Coor start_coor = new Coor((double)ra_start[current_file_index][(int)current_block_index], (double)decl_start[current_file_index][(int)current_block_index]);
					Coor end_coor = new Coor((double)ra_end[current_file_index][(int)current_block_index], (double)decl_end[current_file_index][(int)current_block_index]);
					if (circum_area == null  ||  circum_area.overlapsArea(start_coor, end_coor)) {
						// In the case within the field.
						current_star_index = 0;
					} else {
						// In the case not within the field.

						// In the case it reaches to the last block.
						if (current_block_index + 1 == file_block_count[current_file_index])
							throw new EOFException();

						// Skips to the next block.
						StreamSkipTimer skip_timer = new StreamSkipTimer(current_reader, 10000);
						skip_timer.skip(2000 * 81);

						current_star_index = 2000 - 1;

						continue;
					}
				}

				String record = current_reader.readLine();
				if (record == null)
					throw new EOFException();

				if (record.substring(0, 10).trim().length() > 0) {
					double ra = Format.doubleValueOf(record.substring(10, 20)) / 100.0 / 3600.0;
					double decl = Format.doubleValueOf(record.substring(20, 29)) / 100.0 / 3600.0 - 90.0;

					Coor coor = new Coor(ra, decl);

					if (circum_area == null  ||  circum_area.inArea(coor)) {
						short region = Short.parseShort(record.substring(0, 4).trim());
						short number = Short.parseShort(record.substring(5, 10).trim());

						TicStar star = new TicStar(region, number, coor);

						if (record.substring(38, 43).trim().length() > 0) {
							int b_mag = Integer.parseInt(record.substring(38, 43).trim());
							if (b_mag > 0)
								star.setBMagnitude((double)b_mag / 100.0);
						}

						if (record.substring(47, 52).trim().length() > 0) {
							int v_mag = Integer.parseInt(record.substring(47, 52).trim());
							if (v_mag > 0)
								star.setVMagnitude((double)v_mag / 100.0);
						}

						if (star.getMag() <= limiting_mag) {
							if (center_coor != null) {
								ChartMapFunction cmf = new ChartMapFunction(center_coor, 1.0, 0.0);
								Position position = cmf.mapCoordinatesToXY(coor);
								star.setPosition(position);
							}

							return star;
						}
					}
				}
			} catch (EOFException e) {
				// All files have been read.
				if (current_file_index_in_list + 1 ==  file_list.size())
					return null;

				current_file_index = ((Integer)file_list.elementAt(current_file_index_in_list + 1)).intValue();

				URL url = null;
				try {
					url = url_set.existsFile(filenames[current_file_index]);
				} catch ( FileNotFoundException exception ) {
					url = url_set.existsFile(filenames[current_file_index] + ".dat");
				}
				BufferedReader reader = Decoder.newReader(url);

				// Closes the current file after confirming the next file can be open properly.
				if (current_reader != null)
					current_reader.close();
				current_file_index_in_list++;

				current_reader = reader;

				current_block_index = -1;
				current_star_index = 2000 - 1;
			}
		}
	}

	/**
	 * Closes a catalog. This method must be invoked finally.
	 * @exception IOException if a file cannot be accessed.
	 */
	public void close ( )
		throws IOException
	{
		current_reader.close();
		current_reader = null;
	}

	/**
	 * Initializes the R.A. and Decl. of each blocks of tic1.dat.
	 */
	private static void initialize_Tic1 ( ) {
		ra_start[0][0] = 0;
		ra_end[0][0] = 10;
		decl_start[0][0] = 0;
		decl_end[0][0] = 8;
		ra_start[0][1] = 7;
		ra_end[0][1] = 23;
		decl_start[0][1] = 0;
		decl_end[0][1] = 8;
		ra_start[0][2] = 15;
		ra_end[0][2] = 30;
		decl_start[0][2] = 0;
		decl_end[0][2] = 8;
		ra_start[0][3] = 22;
		ra_end[0][3] = 38;
		decl_start[0][3] = 0;
		decl_end[0][3] = 8;
		ra_start[0][4] = 30;
		ra_end[0][4] = 45;
		decl_start[0][4] = 0;
		decl_end[0][4] = 8;
		ra_start[0][5] = 37;
		ra_end[0][5] = 53;
		decl_start[0][5] = 0;
		decl_end[0][5] = 8;
		ra_start[0][6] = 47;
		ra_end[0][6] = 60;
		decl_start[0][6] = 0;
		decl_end[0][6] = 8;
		ra_start[0][7] = 57;
		ra_end[0][7] = 68;
		decl_start[0][7] = 0;
		decl_end[0][7] = 8;
		ra_start[0][8] = 65;
		ra_end[0][8] = 75;
		decl_start[0][8] = 0;
		decl_end[0][8] = 8;
		ra_start[0][9] = 67;
		ra_end[0][9] = 81;
		decl_start[0][9] = 0;
		decl_end[0][9] = 8;
		ra_start[0][10] = 75;
		ra_end[0][10] = 83;
		decl_start[0][10] = 0;
		decl_end[0][10] = 4;
		ra_start[0][11] = 75;
		ra_end[0][11] = 84;
		decl_start[0][11] = 0;
		decl_end[0][11] = 8;
		ra_start[0][12] = 82;
		ra_end[0][12] = 90;
		decl_start[0][12] = 0;
		decl_end[0][12] = 6;
		ra_start[0][13] = 82;
		ra_end[0][13] = 92;
		decl_start[0][13] = 0;
		decl_end[0][13] = 8;
		ra_start[0][14] = 90;
		ra_end[0][14] = 98;
		decl_start[0][14] = 0;
		decl_end[0][14] = 4;
		ra_start[0][15] = 90;
		ra_end[0][15] = 98;
		decl_start[0][15] = 1;
		decl_end[0][15] = 6;
		ra_start[0][16] = 90;
		ra_end[0][16] = 98;
		decl_start[0][16] = 3;
		decl_end[0][16] = 8;
		ra_start[0][17] = 95;
		ra_end[0][17] = 104;
		decl_start[0][17] = 0;
		decl_end[0][17] = 8;
		ra_start[0][18] = 97;
		ra_end[0][18] = 105;
		decl_start[0][18] = 0;
		decl_end[0][18] = 4;
		ra_start[0][19] = 97;
		ra_end[0][19] = 104;
		decl_start[0][19] = 1;
		decl_end[0][19] = 4;
		ra_start[0][20] = 97;
		ra_end[0][20] = 105;
		decl_start[0][20] = 1;
		decl_end[0][20] = 6;
		ra_start[0][21] = 97;
		ra_end[0][21] = 105;
		decl_start[0][21] = 3;
		decl_end[0][21] = 8;
		ra_start[0][22] = 97;
		ra_end[0][22] = 104;
		decl_start[0][22] = 5;
		decl_end[0][22] = 8;
		ra_start[0][23] = 101;
		ra_end[0][23] = 109;
		decl_start[0][23] = 0;
		decl_end[0][23] = 8;
		ra_start[0][24] = 105;
		ra_end[0][24] = 113;
		decl_start[0][24] = 0;
		decl_end[0][24] = 4;
		ra_start[0][25] = 105;
		ra_end[0][25] = 113;
		decl_start[0][25] = 1;
		decl_end[0][25] = 6;
		ra_start[0][26] = 105;
		ra_end[0][26] = 113;
		decl_start[0][26] = 3;
		decl_end[0][26] = 6;
		ra_start[0][27] = 105;
		ra_end[0][27] = 113;
		decl_start[0][27] = 3;
		decl_end[0][27] = 8;
		ra_start[0][28] = 110;
		ra_end[0][28] = 119;
		decl_start[0][28] = 0;
		decl_end[0][28] = 8;
		ra_start[0][29] = 112;
		ra_end[0][29] = 120;
		decl_start[0][29] = 0;
		decl_end[0][29] = 4;
		ra_start[0][30] = 112;
		ra_end[0][30] = 120;
		decl_start[0][30] = 1;
		decl_end[0][30] = 8;
		ra_start[0][31] = 112;
		ra_end[0][31] = 126;
		decl_start[0][31] = 0;
		decl_end[0][31] = 8;
		ra_start[0][32] = 120;
		ra_end[0][32] = 128;
		decl_start[0][32] = 0;
		decl_end[0][32] = 6;
		ra_start[0][33] = 120;
		ra_end[0][33] = 134;
		decl_start[0][33] = 0;
		decl_end[0][33] = 8;
		ra_start[0][34] = 127;
		ra_end[0][34] = 135;
		decl_start[0][34] = 0;
		decl_end[0][34] = 6;
		ra_start[0][35] = 127;
		ra_end[0][35] = 143;
		decl_start[0][35] = 0;
		decl_end[0][35] = 8;
		ra_start[0][36] = 135;
		ra_end[0][36] = 150;
		decl_start[0][36] = 0;
		decl_end[0][36] = 8;
		ra_start[0][37] = 142;
		ra_end[0][37] = 154;
		decl_start[0][37] = 0;
		decl_end[0][37] = 8;
		ra_start[0][38] = 150;
		ra_end[0][38] = 165;
		decl_start[0][38] = 0;
		decl_end[0][38] = 8;
		ra_start[0][39] = 157;
		ra_end[0][39] = 173;
		decl_start[0][39] = 0;
		decl_end[0][39] = 8;
		ra_start[0][40] = 165;
		ra_end[0][40] = 183;
		decl_start[0][40] = 0;
		decl_end[0][40] = 8;
		ra_start[0][41] = 180;
		ra_end[0][41] = 193;
		decl_start[0][41] = 0;
		decl_end[0][41] = 8;
		ra_start[0][42] = 187;
		ra_end[0][42] = 203;
		decl_start[0][42] = 0;
		decl_end[0][42] = 8;
		ra_start[0][43] = 195;
		ra_end[0][43] = 210;
		decl_start[0][43] = 0;
		decl_end[0][43] = 8;
		ra_start[0][44] = 205;
		ra_end[0][44] = 219;
		decl_start[0][44] = 0;
		decl_end[0][44] = 8;
		ra_start[0][45] = 217;
		ra_end[0][45] = 229;
		decl_start[0][45] = 0;
		decl_end[0][45] = 8;
		ra_start[0][46] = 225;
		ra_end[0][46] = 239;
		decl_start[0][46] = 0;
		decl_end[0][46] = 8;
		ra_start[0][47] = 232;
		ra_end[0][47] = 242;
		decl_start[0][47] = 0;
		decl_end[0][47] = 8;
		ra_start[0][48] = 240;
		ra_end[0][48] = 248;
		decl_start[0][48] = 0;
		decl_end[0][48] = 8;
		ra_start[0][49] = 241;
		ra_end[0][49] = 255;
		decl_start[0][49] = 0;
		decl_end[0][49] = 8;
		ra_start[0][50] = 247;
		ra_end[0][50] = 259;
		decl_start[0][50] = 0;
		decl_end[0][50] = 8;
		ra_start[0][51] = 255;
		ra_end[0][51] = 263;
		decl_start[0][51] = 0;
		decl_end[0][51] = 6;
		ra_start[0][52] = 255;
		ra_end[0][52] = 270;
		decl_start[0][52] = 0;
		decl_end[0][52] = 8;
		ra_start[0][53] = 262;
		ra_end[0][53] = 270;
		decl_start[0][53] = 0;
		decl_end[0][53] = 6;
		ra_start[0][54] = 262;
		ra_end[0][54] = 272;
		decl_start[0][54] = 0;
		decl_end[0][54] = 8;
		ra_start[0][55] = 270;
		ra_end[0][55] = 278;
		decl_start[0][55] = 0;
		decl_end[0][55] = 4;
		ra_start[0][56] = 270;
		ra_end[0][56] = 278;
		decl_start[0][56] = 1;
		decl_end[0][56] = 6;
		ra_start[0][57] = 270;
		ra_end[0][57] = 283;
		decl_start[0][57] = 0;
		decl_end[0][57] = 8;
		ra_start[0][58] = 277;
		ra_end[0][58] = 285;
		decl_start[0][58] = 0;
		decl_end[0][58] = 6;
		ra_start[0][59] = 277;
		ra_end[0][59] = 289;
		decl_start[0][59] = 0;
		decl_end[0][59] = 8;
		ra_start[0][60] = 285;
		ra_end[0][60] = 293;
		decl_start[0][60] = 0;
		decl_end[0][60] = 4;
		ra_start[0][61] = 285;
		ra_end[0][61] = 293;
		decl_start[0][61] = 1;
		decl_end[0][61] = 6;
		ra_start[0][62] = 285;
		ra_end[0][62] = 293;
		decl_start[0][62] = 3;
		decl_end[0][62] = 8;
		ra_start[0][63] = 285;
		ra_end[0][63] = 297;
		decl_start[0][63] = 0;
		decl_end[0][63] = 8;
		ra_start[0][64] = 292;
		ra_end[0][64] = 300;
		decl_start[0][64] = 0;
		decl_end[0][64] = 4;
		ra_start[0][65] = 292;
		ra_end[0][65] = 300;
		decl_start[0][65] = 1;
		decl_end[0][65] = 6;
		ra_start[0][66] = 292;
		ra_end[0][66] = 300;
		decl_start[0][66] = 3;
		decl_end[0][66] = 8;
		ra_start[0][67] = 294;
		ra_end[0][67] = 304;
		decl_start[0][67] = 0;
		decl_end[0][67] = 8;
		ra_start[0][68] = 300;
		ra_end[0][68] = 308;
		decl_start[0][68] = 0;
		decl_end[0][68] = 4;
		ra_start[0][69] = 300;
		ra_end[0][69] = 308;
		decl_start[0][69] = 1;
		decl_end[0][69] = 8;
		ra_start[0][70] = 300;
		ra_end[0][70] = 314;
		decl_start[0][70] = 0;
		decl_end[0][70] = 8;
		ra_start[0][71] = 307;
		ra_end[0][71] = 315;
		decl_start[0][71] = 0;
		decl_end[0][71] = 6;
		ra_start[0][72] = 307;
		ra_end[0][72] = 319;
		decl_start[0][72] = 0;
		decl_end[0][72] = 8;
		ra_start[0][73] = 315;
		ra_end[0][73] = 323;
		decl_start[0][73] = 0;
		decl_end[0][73] = 6;
		ra_start[0][74] = 315;
		ra_end[0][74] = 330;
		decl_start[0][74] = 0;
		decl_end[0][74] = 8;
		ra_start[0][75] = 322;
		ra_end[0][75] = 333;
		decl_start[0][75] = 0;
		decl_end[0][75] = 8;
		ra_start[0][76] = 330;
		ra_end[0][76] = 338;
		decl_start[0][76] = 0;
		decl_end[0][76] = 8;
		ra_start[0][77] = 335;
		ra_end[0][77] = 345;
		decl_start[0][77] = 0;
		decl_end[0][77] = 8;
		ra_start[0][78] = 342;
		ra_end[0][78] = 355;
		decl_start[0][78] = 0;
		decl_end[0][78] = 8;
		ra_start[0][79] = 0;
		ra_end[0][79] = 360;
		decl_start[0][79] = 0;
		decl_end[0][79] = 10;
		ra_start[0][80] = 0;
		ra_end[0][80] = 16;
		decl_start[0][80] = 7;
		decl_end[0][80] = 15;
		ra_start[0][81] = 7;
		ra_end[0][81] = 23;
		decl_start[0][81] = 7;
		decl_end[0][81] = 15;
		ra_start[0][82] = 15;
		ra_end[0][82] = 31;
		decl_start[0][82] = 7;
		decl_end[0][82] = 15;
		ra_start[0][83] = 22;
		ra_end[0][83] = 39;
		decl_start[0][83] = 7;
		decl_end[0][83] = 15;
		ra_start[0][84] = 33;
		ra_end[0][84] = 51;
		decl_start[0][84] = 7;
		decl_end[0][84] = 15;
		ra_start[0][85] = 45;
		ra_end[0][85] = 62;
		decl_start[0][85] = 7;
		decl_end[0][85] = 15;
		ra_start[0][86] = 53;
		ra_end[0][86] = 69;
		decl_start[0][86] = 7;
		decl_end[0][86] = 15;
		ra_start[0][87] = 61;
		ra_end[0][87] = 77;
		decl_start[0][87] = 7;
		decl_end[0][87] = 15;
		ra_start[0][88] = 68;
		ra_end[0][88] = 85;
		decl_start[0][88] = 7;
		decl_end[0][88] = 15;
		ra_start[0][89] = 76;
		ra_end[0][89] = 85;
		decl_start[0][89] = 9;
		decl_end[0][89] = 14;
		ra_start[0][90] = 76;
		ra_end[0][90] = 92;
		decl_start[0][90] = 7;
		decl_end[0][90] = 15;
		ra_start[0][91] = 84;
		ra_end[0][91] = 92;
		decl_start[0][91] = 7;
		decl_end[0][91] = 14;
		ra_start[0][92] = 84;
		ra_end[0][92] = 92;
		decl_start[0][92] = 11;
		decl_end[0][92] = 15;
		ra_start[0][93] = 86;
		ra_end[0][93] = 94;
		decl_start[0][93] = 7;
		decl_end[0][93] = 15;
		ra_start[0][94] = 91;
		ra_end[0][94] = 100;
		decl_start[0][94] = 7;
		decl_end[0][94] = 12;
		ra_start[0][95] = 91;
		ra_end[0][95] = 100;
		decl_start[0][95] = 9;
		decl_end[0][95] = 14;
		ra_start[0][96] = 91;
		ra_end[0][96] = 100;
		decl_start[0][96] = 11;
		decl_end[0][96] = 15;
		ra_start[0][97] = 93;
		ra_end[0][97] = 104;
		decl_start[0][97] = 7;
		decl_end[0][97] = 15;
		ra_start[0][98] = 99;
		ra_end[0][98] = 108;
		decl_start[0][98] = 7;
		decl_end[0][98] = 12;
		ra_start[0][99] = 99;
		ra_end[0][99] = 108;
		decl_start[0][99] = 9;
		decl_end[0][99] = 12;
		ra_start[0][100] = 99;
		ra_end[0][100] = 108;
		decl_start[0][100] = 9;
		decl_end[0][100] = 14;
		ra_start[0][101] = 99;
		ra_end[0][101] = 108;
		decl_start[0][101] = 11;
		decl_end[0][101] = 15;
		ra_start[0][102] = 105;
		ra_end[0][102] = 113;
		decl_start[0][102] = 7;
		decl_end[0][102] = 15;
		ra_start[0][103] = 107;
		ra_end[0][103] = 115;
		decl_start[0][103] = 7;
		decl_end[0][103] = 12;
		ra_start[0][104] = 107;
		ra_end[0][104] = 115;
		decl_start[0][104] = 9;
		decl_end[0][104] = 15;
		ra_start[0][105] = 109;
		ra_end[0][105] = 123;
		decl_start[0][105] = 7;
		decl_end[0][105] = 15;
		ra_start[0][106] = 114;
		ra_end[0][106] = 123;
		decl_start[0][106] = 9;
		decl_end[0][106] = 15;
		ra_start[0][107] = 114;
		ra_end[0][107] = 131;
		decl_start[0][107] = 7;
		decl_end[0][107] = 15;
		ra_start[0][108] = 122;
		ra_end[0][108] = 131;
		decl_start[0][108] = 9;
		decl_end[0][108] = 15;
		ra_start[0][109] = 124;
		ra_end[0][109] = 138;
		decl_start[0][109] = 7;
		decl_end[0][109] = 15;
		ra_start[0][110] = 130;
		ra_end[0][110] = 146;
		decl_start[0][110] = 7;
		decl_end[0][110] = 15;
		ra_start[0][111] = 137;
		ra_end[0][111] = 154;
		decl_start[0][111] = 7;
		decl_end[0][111] = 15;
		ra_start[0][112] = 145;
		ra_end[0][112] = 161;
		decl_start[0][112] = 7;
		decl_end[0][112] = 15;
		ra_start[0][113] = 158;
		ra_end[0][113] = 174;
		decl_start[0][113] = 7;
		decl_end[0][113] = 15;
		ra_start[0][114] = 168;
		ra_end[0][114] = 184;
		decl_start[0][114] = 7;
		decl_end[0][114] = 15;
		ra_start[0][115] = 176;
		ra_end[0][115] = 197;
		decl_start[0][115] = 7;
		decl_end[0][115] = 15;
		ra_start[0][116] = 191;
		ra_end[0][116] = 207;
		decl_start[0][116] = 7;
		decl_end[0][116] = 15;
		ra_start[0][117] = 201;
		ra_end[0][117] = 223;
		decl_start[0][117] = 7;
		decl_end[0][117] = 15;
		ra_start[0][118] = 214;
		ra_end[0][118] = 230;
		decl_start[0][118] = 7;
		decl_end[0][118] = 15;
		ra_start[0][119] = 222;
		ra_end[0][119] = 238;
		decl_start[0][119] = 7;
		decl_end[0][119] = 15;
		ra_start[0][120] = 231;
		ra_end[0][120] = 246;
		decl_start[0][120] = 7;
		decl_end[0][120] = 15;
		ra_start[0][121] = 237;
		ra_end[0][121] = 253;
		decl_start[0][121] = 7;
		decl_end[0][121] = 15;
		ra_start[0][122] = 245;
		ra_end[0][122] = 261;
		decl_start[0][122] = 7;
		decl_end[0][122] = 15;
		ra_start[0][123] = 252;
		ra_end[0][123] = 261;
		decl_start[0][123] = 9;
		decl_end[0][123] = 15;
		ra_start[0][124] = 258;
		ra_end[0][124] = 269;
		decl_start[0][124] = 7;
		decl_end[0][124] = 15;
		ra_start[0][125] = 260;
		ra_end[0][125] = 269;
		decl_start[0][125] = 9;
		decl_end[0][125] = 15;
		ra_start[0][126] = 264;
		ra_end[0][126] = 276;
		decl_start[0][126] = 7;
		decl_end[0][126] = 15;
		ra_start[0][127] = 268;
		ra_end[0][127] = 276;
		decl_start[0][127] = 9;
		decl_end[0][127] = 14;
		ra_start[0][128] = 268;
		ra_end[0][128] = 276;
		decl_start[0][128] = 11;
		decl_end[0][128] = 15;
		ra_start[0][129] = 273;
		ra_end[0][129] = 282;
		decl_start[0][129] = 7;
		decl_end[0][129] = 15;
		ra_start[0][130] = 275;
		ra_end[0][130] = 284;
		decl_start[0][130] = 7;
		decl_end[0][130] = 12;
		ra_start[0][131] = 275;
		ra_end[0][131] = 284;
		decl_start[0][131] = 9;
		decl_end[0][131] = 14;
		ra_start[0][132] = 275;
		ra_end[0][132] = 284;
		decl_start[0][132] = 11;
		decl_end[0][132] = 15;
		ra_start[0][133] = 281;
		ra_end[0][133] = 292;
		decl_start[0][133] = 7;
		decl_end[0][133] = 15;
		ra_start[0][134] = 283;
		ra_end[0][134] = 292;
		decl_start[0][134] = 9;
		decl_end[0][134] = 14;
		ra_start[0][135] = 283;
		ra_end[0][135] = 293;
		decl_start[0][135] = 7;
		decl_end[0][135] = 15;
		ra_start[0][136] = 291;
		ra_end[0][136] = 299;
		decl_start[0][136] = 7;
		decl_end[0][136] = 10;
		ra_start[0][137] = 291;
		ra_end[0][137] = 299;
		decl_start[0][137] = 7;
		decl_end[0][137] = 12;
		ra_start[0][138] = 291;
		ra_end[0][138] = 299;
		decl_start[0][138] = 9;
		decl_end[0][138] = 14;
		ra_start[0][139] = 291;
		ra_end[0][139] = 299;
		decl_start[0][139] = 11;
		decl_end[0][139] = 15;
		ra_start[0][140] = 294;
		ra_end[0][140] = 305;
		decl_start[0][140] = 7;
		decl_end[0][140] = 15;
		ra_start[0][141] = 298;
		ra_end[0][141] = 307;
		decl_start[0][141] = 7;
		decl_end[0][141] = 12;
		ra_start[0][142] = 298;
		ra_end[0][142] = 307;
		decl_start[0][142] = 9;
		decl_end[0][142] = 14;
		ra_start[0][143] = 298;
		ra_end[0][143] = 307;
		decl_start[0][143] = 11;
		decl_end[0][143] = 15;
		ra_start[0][144] = 300;
		ra_end[0][144] = 313;
		decl_start[0][144] = 7;
		decl_end[0][144] = 15;
		ra_start[0][145] = 306;
		ra_end[0][145] = 315;
		decl_start[0][145] = 7;
		decl_end[0][145] = 14;
		ra_start[0][146] = 306;
		ra_end[0][146] = 315;
		decl_start[0][146] = 11;
		decl_end[0][146] = 15;
		ra_start[0][147] = 308;
		ra_end[0][147] = 322;
		decl_start[0][147] = 7;
		decl_end[0][147] = 15;
		ra_start[0][148] = 314;
		ra_end[0][148] = 322;
		decl_start[0][148] = 9;
		decl_end[0][148] = 15;
		ra_start[0][149] = 314;
		ra_end[0][149] = 330;
		decl_start[0][149] = 7;
		decl_end[0][149] = 16;
		ra_start[0][150] = 321;
		ra_end[0][150] = 334;
		decl_start[0][150] = 7;
		decl_end[0][150] = 15;
		ra_start[0][151] = 329;
		ra_end[0][151] = 338;
		decl_start[0][151] = 7;
		decl_end[0][151] = 15;
		ra_start[0][152] = 329;
		ra_end[0][152] = 345;
		decl_start[0][152] = 7;
		decl_end[0][152] = 15;
		ra_start[0][153] = 337;
		ra_end[0][153] = 353;
		decl_start[0][153] = 7;
		decl_end[0][153] = 15;
		ra_start[0][154] = 344;
		ra_end[0][154] = 360;
		decl_start[0][154] = 7;
		decl_end[0][154] = 15;
		ra_start[0][155] = 0;
		ra_end[0][155] = 360;
		decl_start[0][155] = 10;
		decl_end[0][155] = 20;
		ra_start[0][156] = 0;
		ra_end[0][156] = 16;
		decl_start[0][156] = 15;
		decl_end[0][156] = 23;
		ra_start[0][157] = 8;
		ra_end[0][157] = 24;
		decl_start[0][157] = 15;
		decl_end[0][157] = 23;
		ra_start[0][158] = 16;
		ra_end[0][158] = 32;
		decl_start[0][158] = 15;
		decl_end[0][158] = 23;
		ra_start[0][159] = 24;
		ra_end[0][159] = 40;
		decl_start[0][159] = 15;
		decl_end[0][159] = 23;
		ra_start[0][160] = 32;
		ra_end[0][160] = 48;
		decl_start[0][160] = 15;
		decl_end[0][160] = 23;
		ra_start[0][161] = 42;
		ra_end[0][161] = 56;
		decl_start[0][161] = 15;
		decl_end[0][161] = 23;
		ra_start[0][162] = 48;
		ra_end[0][162] = 64;
		decl_start[0][162] = 15;
		decl_end[0][162] = 23;
		ra_start[0][163] = 56;
		ra_end[0][163] = 72;
		decl_start[0][163] = 15;
		decl_end[0][163] = 23;
		ra_start[0][164] = 64;
		ra_end[0][164] = 80;
		decl_start[0][164] = 15;
		decl_end[0][164] = 23;
		ra_start[0][165] = 72;
		ra_end[0][165] = 80;
		decl_start[0][165] = 16;
		decl_end[0][165] = 23;
		ra_start[0][166] = 78;
		ra_end[0][166] = 88;
		decl_start[0][166] = 15;
		decl_end[0][166] = 23;
		ra_start[0][167] = 80;
		ra_end[0][167] = 88;
		decl_start[0][167] = 16;
		decl_end[0][167] = 21;
		ra_start[0][168] = 80;
		ra_end[0][168] = 90;
		decl_start[0][168] = 15;
		decl_end[0][168] = 23;
		ra_start[0][169] = 88;
		ra_end[0][169] = 96;
		decl_start[0][169] = 15;
		decl_end[0][169] = 17;
		ra_start[0][170] = 88;
		ra_end[0][170] = 96;
		decl_start[0][170] = 15;
		decl_end[0][170] = 19;
		ra_start[0][171] = 88;
		ra_end[0][171] = 96;
		decl_start[0][171] = 16;
		decl_end[0][171] = 21;
		ra_start[0][172] = 88;
		ra_end[0][172] = 96;
		decl_start[0][172] = 18;
		decl_end[0][172] = 23;
		ra_start[0][173] = 94;
		ra_end[0][173] = 102;
		decl_start[0][173] = 15;
		decl_end[0][173] = 23;
		ra_start[0][174] = 96;
		ra_end[0][174] = 104;
		decl_start[0][174] = 15;
		decl_end[0][174] = 19;
		ra_start[0][175] = 96;
		ra_end[0][175] = 104;
		decl_start[0][175] = 16;
		decl_end[0][175] = 21;
		ra_start[0][176] = 96;
		ra_end[0][176] = 104;
		decl_start[0][176] = 18;
		decl_end[0][176] = 23;
		ra_start[0][177] = 96;
		ra_end[0][177] = 104;
		decl_start[0][177] = 20;
		decl_end[0][177] = 23;
		ra_start[0][178] = 102;
		ra_end[0][178] = 110;
		decl_start[0][178] = 15;
		decl_end[0][178] = 23;
		ra_start[0][179] = 104;
		ra_end[0][179] = 112;
		decl_start[0][179] = 15;
		decl_end[0][179] = 19;
		ra_start[0][180] = 104;
		ra_end[0][180] = 112;
		decl_start[0][180] = 16;
		decl_end[0][180] = 21;
		ra_start[0][181] = 104;
		ra_end[0][181] = 112;
		decl_start[0][181] = 18;
		decl_end[0][181] = 23;
		ra_start[0][182] = 108;
		ra_end[0][182] = 120;
		decl_start[0][182] = 15;
		decl_end[0][182] = 23;
		ra_start[0][183] = 112;
		ra_end[0][183] = 120;
		decl_start[0][183] = 15;
		decl_end[0][183] = 21;
		ra_start[0][184] = 112;
		ra_end[0][184] = 122;
		decl_start[0][184] = 15;
		decl_end[0][184] = 23;
		ra_start[0][185] = 120;
		ra_end[0][185] = 128;
		decl_start[0][185] = 15;
		decl_end[0][185] = 21;
		ra_start[0][186] = 120;
		ra_end[0][186] = 134;
		decl_start[0][186] = 15;
		decl_end[0][186] = 23;
		ra_start[0][187] = 128;
		ra_end[0][187] = 136;
		decl_start[0][187] = 15;
		decl_end[0][187] = 23;
		ra_start[0][188] = 133;
		ra_end[0][188] = 144;
		decl_start[0][188] = 15;
		decl_end[0][188] = 23;
		ra_start[0][189] = 141;
		ra_end[0][189] = 160;
		decl_start[0][189] = 15;
		decl_end[0][189] = 23;
		ra_start[0][190] = 152;
		ra_end[0][190] = 168;
		decl_start[0][190] = 15;
		decl_end[0][190] = 23;
		ra_start[0][191] = 160;
		ra_end[0][191] = 180;
		decl_start[0][191] = 15;
		decl_end[0][191] = 23;
		ra_start[0][192] = 176;
		ra_end[0][192] = 192;
		decl_start[0][192] = 15;
		decl_end[0][192] = 23;
		ra_start[0][193] = 184;
		ra_end[0][193] = 206;
		decl_start[0][193] = 15;
		decl_end[0][193] = 23;
		ra_start[0][194] = 200;
		ra_end[0][194] = 216;
		decl_start[0][194] = 15;
		decl_end[0][194] = 23;
		ra_start[0][195] = 208;
		ra_end[0][195] = 224;
		decl_start[0][195] = 15;
		decl_end[0][195] = 23;
		ra_start[0][196] = 222;
		ra_end[0][196] = 240;
		decl_start[0][196] = 15;
		decl_end[0][196] = 23;
		ra_start[0][197] = 232;
		ra_end[0][197] = 248;
		decl_start[0][197] = 15;
		decl_end[0][197] = 23;
		ra_start[0][198] = 240;
		ra_end[0][198] = 256;
		decl_start[0][198] = 15;
		decl_end[0][198] = 23;
		ra_start[0][199] = 248;
		ra_end[0][199] = 256;
		decl_start[0][199] = 15;
		decl_end[0][199] = 23;
		ra_start[0][200] = 254;
		ra_end[0][200] = 264;
		decl_start[0][200] = 15;
		decl_end[0][200] = 23;
		ra_start[0][201] = 256;
		ra_end[0][201] = 266;
		decl_start[0][201] = 15;
		decl_end[0][201] = 23;
		ra_start[0][202] = 264;
		ra_end[0][202] = 272;
		decl_start[0][202] = 15;
		decl_end[0][202] = 19;
		ra_start[0][203] = 264;
		ra_end[0][203] = 272;
		decl_start[0][203] = 16;
		decl_end[0][203] = 23;
		ra_start[0][204] = 266;
		ra_end[0][204] = 275;
		decl_start[0][204] = 15;
		decl_end[0][204] = 23;
		ra_start[0][205] = 272;
		ra_end[0][205] = 280;
		decl_start[0][205] = 15;
		decl_end[0][205] = 19;
		ra_start[0][206] = 272;
		ra_end[0][206] = 280;
		decl_start[0][206] = 16;
		decl_end[0][206] = 21;
		ra_start[0][207] = 272;
		ra_end[0][207] = 280;
		decl_start[0][207] = 18;
		decl_end[0][207] = 23;
		ra_start[0][208] = 278;
		ra_end[0][208] = 288;
		decl_start[0][208] = 15;
		decl_end[0][208] = 23;
		ra_start[0][209] = 280;
		ra_end[0][209] = 288;
		decl_start[0][209] = 16;
		decl_end[0][209] = 21;
		ra_start[0][210] = 280;
		ra_end[0][210] = 288;
		decl_start[0][210] = 18;
		decl_end[0][210] = 23;
		ra_start[0][211] = 282;
		ra_end[0][211] = 292;
		decl_start[0][211] = 15;
		decl_end[0][211] = 23;
		ra_start[0][212] = 288;
		ra_end[0][212] = 296;
		decl_start[0][212] = 15;
		decl_end[0][212] = 19;
		ra_start[0][213] = 288;
		ra_end[0][213] = 296;
		decl_start[0][213] = 16;
		decl_end[0][213] = 22;
		ra_start[0][214] = 288;
		ra_end[0][214] = 298;
		decl_start[0][214] = 15;
		decl_end[0][214] = 23;
		ra_start[0][215] = 296;
		ra_end[0][215] = 304;
		decl_start[0][215] = 15;
		decl_end[0][215] = 17;
		ra_start[0][216] = 296;
		ra_end[0][216] = 304;
		decl_start[0][216] = 15;
		decl_end[0][216] = 19;
		ra_start[0][217] = 296;
		ra_end[0][217] = 304;
		decl_start[0][217] = 16;
		decl_end[0][217] = 21;
		ra_start[0][218] = 296;
		ra_end[0][218] = 304;
		decl_start[0][218] = 18;
		decl_end[0][218] = 23;
		ra_start[0][219] = 296;
		ra_end[0][219] = 304;
		decl_start[0][219] = 20;
		decl_end[0][219] = 23;
		ra_start[0][220] = 302;
		ra_end[0][220] = 310;
		decl_start[0][220] = 15;
		decl_end[0][220] = 23;
		ra_start[0][221] = 304;
		ra_end[0][221] = 312;
		decl_start[0][221] = 15;
		decl_end[0][221] = 19;
		ra_start[0][222] = 304;
		ra_end[0][222] = 312;
		decl_start[0][222] = 16;
		decl_end[0][222] = 21;
		ra_start[0][223] = 304;
		ra_end[0][223] = 312;
		decl_start[0][223] = 18;
		decl_end[0][223] = 23;
		ra_start[0][224] = 304;
		ra_end[0][224] = 314;
		decl_start[0][224] = 15;
		decl_end[0][224] = 23;
		ra_start[0][225] = 312;
		ra_end[0][225] = 320;
		decl_start[0][225] = 15;
		decl_end[0][225] = 19;
		ra_start[0][226] = 312;
		ra_end[0][226] = 320;
		decl_start[0][226] = 16;
		decl_end[0][226] = 21;
		ra_start[0][227] = 312;
		ra_end[0][227] = 324;
		decl_start[0][227] = 15;
		decl_end[0][227] = 23;
		ra_start[0][228] = 320;
		ra_end[0][228] = 328;
		decl_start[0][228] = 15;
		decl_end[0][228] = 21;
		ra_start[0][229] = 320;
		ra_end[0][229] = 332;
		decl_start[0][229] = 15;
		decl_end[0][229] = 23;
		ra_start[0][230] = 328;
		ra_end[0][230] = 336;
		decl_start[0][230] = 15;
		decl_end[0][230] = 21;
		ra_start[0][231] = 328;
		ra_end[0][231] = 344;
		decl_start[0][231] = 15;
		decl_end[0][231] = 23;
		ra_start[0][232] = 336;
		ra_end[0][232] = 344;
		decl_start[0][232] = 16;
		decl_end[0][232] = 23;
		ra_start[0][233] = 342;
		ra_end[0][233] = 352;
		decl_start[0][233] = 15;
		decl_end[0][233] = 23;
		ra_start[0][234] = 349;
		ra_end[0][234] = 360;
		decl_start[0][234] = 15;
		decl_end[0][234] = 23;
		ra_start[0][235] = 0;
		ra_end[0][235] = 360;
		decl_start[0][235] = 20;
		decl_end[0][235] = 30;
		ra_start[0][236] = 5;
		ra_end[0][236] = 17;
		decl_start[0][236] = 22;
		decl_end[0][236] = 30;
		ra_start[0][237] = 8;
		ra_end[0][237] = 26;
		decl_start[0][237] = 22;
		decl_end[0][237] = 30;
		ra_start[0][238] = 16;
		ra_end[0][238] = 34;
		decl_start[0][238] = 22;
		decl_end[0][238] = 30;
		ra_start[0][239] = 25;
		ra_end[0][239] = 42;
		decl_start[0][239] = 22;
		decl_end[0][239] = 30;
		ra_start[0][240] = 33;
		ra_end[0][240] = 49;
		decl_start[0][240] = 22;
		decl_end[0][240] = 30;
		ra_start[0][241] = 41;
		ra_end[0][241] = 51;
		decl_start[0][241] = 22;
		decl_end[0][241] = 30;
		ra_start[0][242] = 46;
		ra_end[0][242] = 59;
		decl_start[0][242] = 22;
		decl_end[0][242] = 30;
		ra_start[0][243] = 50;
		ra_end[0][243] = 61;
		decl_start[0][243] = 22;
		decl_end[0][243] = 30;
		ra_start[0][244] = 58;
		ra_end[0][244] = 67;
		decl_start[0][244] = 22;
		decl_end[0][244] = 30;
		ra_start[0][245] = 58;
		ra_end[0][245] = 76;
		decl_start[0][245] = 22;
		decl_end[0][245] = 30;
		ra_start[0][246] = 69;
		ra_end[0][246] = 84;
		decl_start[0][246] = 22;
		decl_end[0][246] = 30;
		ra_start[0][247] = 75;
		ra_end[0][247] = 84;
		decl_start[0][247] = 24;
		decl_end[0][247] = 30;
		ra_start[0][248] = 79;
		ra_end[0][248] = 90;
		decl_start[0][248] = 22;
		decl_end[0][248] = 30;
		ra_start[0][249] = 83;
		ra_end[0][249] = 93;
		decl_start[0][249] = 22;
		decl_end[0][249] = 27;
		ra_start[0][250] = 83;
		ra_end[0][250] = 93;
		decl_start[0][250] = 24;
		decl_end[0][250] = 29;
		ra_start[0][251] = 83;
		ra_end[0][251] = 93;
		decl_start[0][251] = 26;
		decl_end[0][251] = 30;
		ra_start[0][252] = 87;
		ra_end[0][252] = 97;
		decl_start[0][252] = 22;
		decl_end[0][252] = 30;
		ra_start[0][253] = 92;
		ra_end[0][253] = 101;
		decl_start[0][253] = 22;
		decl_end[0][253] = 27;
		ra_start[0][254] = 92;
		ra_end[0][254] = 101;
		decl_start[0][254] = 24;
		decl_end[0][254] = 29;
		ra_start[0][255] = 92;
		ra_end[0][255] = 101;
		decl_start[0][255] = 26;
		decl_end[0][255] = 30;
		ra_start[0][256] = 94;
		ra_end[0][256] = 105;
		decl_start[0][256] = 22;
		decl_end[0][256] = 30;
		ra_start[0][257] = 100;
		ra_end[0][257] = 109;
		decl_start[0][257] = 22;
		decl_end[0][257] = 27;
		ra_start[0][258] = 100;
		ra_end[0][258] = 109;
		decl_start[0][258] = 24;
		decl_end[0][258] = 27;
		ra_start[0][259] = 100;
		ra_end[0][259] = 109;
		decl_start[0][259] = 24;
		decl_end[0][259] = 30;
		ra_start[0][260] = 100;
		ra_end[0][260] = 116;
		decl_start[0][260] = 22;
		decl_end[0][260] = 30;
		ra_start[0][261] = 108;
		ra_end[0][261] = 118;
		decl_start[0][261] = 22;
		decl_end[0][261] = 29;
		ra_start[0][262] = 108;
		ra_end[0][262] = 123;
		decl_start[0][262] = 22;
		decl_end[0][262] = 30;
		ra_start[0][263] = 117;
		ra_end[0][263] = 126;
		decl_start[0][263] = 22;
		decl_end[0][263] = 30;
		ra_start[0][264] = 117;
		ra_end[0][264] = 134;
		decl_start[0][264] = 22;
		decl_end[0][264] = 30;
		ra_start[0][265] = 125;
		ra_end[0][265] = 143;
		decl_start[0][265] = 22;
		decl_end[0][265] = 30;
		ra_start[0][266] = 133;
		ra_end[0][266] = 151;
		decl_start[0][266] = 22;
		decl_end[0][266] = 30;
		ra_start[0][267] = 144;
		ra_end[0][267] = 162;
		decl_start[0][267] = 22;
		decl_end[0][267] = 30;
		ra_start[0][268] = 159;
		ra_end[0][268] = 176;
		decl_start[0][268] = 22;
		decl_end[0][268] = 30;
		ra_start[0][269] = 171;
		ra_end[0][269] = 193;
		decl_start[0][269] = 22;
		decl_end[0][269] = 30;
		ra_start[0][270] = 184;
		ra_end[0][270] = 201;
		decl_start[0][270] = 22;
		decl_end[0][270] = 30;
		ra_start[0][271] = 196;
		ra_end[0][271] = 218;
		decl_start[0][271] = 22;
		decl_end[0][271] = 30;
		ra_start[0][272] = 209;
		ra_end[0][272] = 227;
		decl_start[0][272] = 22;
		decl_end[0][272] = 30;
		ra_start[0][273] = 223;
		ra_end[0][273] = 243;
		decl_start[0][273] = 22;
		decl_end[0][273] = 30;
		ra_start[0][274] = 234;
		ra_end[0][274] = 252;
		decl_start[0][274] = 22;
		decl_end[0][274] = 30;
		ra_start[0][275] = 242;
		ra_end[0][275] = 260;
		decl_start[0][275] = 22;
		decl_end[0][275] = 30;
		ra_start[0][276] = 251;
		ra_end[0][276] = 260;
		decl_start[0][276] = 24;
		decl_end[0][276] = 30;
		ra_start[0][277] = 255;
		ra_end[0][277] = 268;
		decl_start[0][277] = 22;
		decl_end[0][277] = 30;
		ra_start[0][278] = 259;
		ra_end[0][278] = 268;
		decl_start[0][278] = 24;
		decl_end[0][278] = 30;
		ra_start[0][279] = 259;
		ra_end[0][279] = 275;
		decl_start[0][279] = 22;
		decl_end[0][279] = 30;
		ra_start[0][280] = 267;
		ra_end[0][280] = 277;
		decl_start[0][280] = 22;
		decl_end[0][280] = 27;
		ra_start[0][281] = 267;
		ra_end[0][281] = 277;
		decl_start[0][281] = 24;
		decl_end[0][281] = 30;
		ra_start[0][282] = 270;
		ra_end[0][282] = 283;
		decl_start[0][282] = 22;
		decl_end[0][282] = 30;
		ra_start[0][283] = 276;
		ra_end[0][283] = 285;
		decl_start[0][283] = 22;
		decl_end[0][283] = 27;
		ra_start[0][284] = 276;
		ra_end[0][284] = 285;
		decl_start[0][284] = 24;
		decl_end[0][284] = 29;
		ra_start[0][285] = 276;
		ra_end[0][285] = 285;
		decl_start[0][285] = 26;
		decl_end[0][285] = 31;
		ra_start[0][286] = 278;
		ra_end[0][286] = 291;
		decl_start[0][286] = 22;
		decl_end[0][286] = 30;
		ra_start[0][287] = 284;
		ra_end[0][287] = 293;
		decl_start[0][287] = 22;
		decl_end[0][287] = 27;
		ra_start[0][288] = 284;
		ra_end[0][288] = 294;
		decl_start[0][288] = 24;
		decl_end[0][288] = 29;
		ra_start[0][289] = 284;
		ra_end[0][289] = 294;
		decl_start[0][289] = 26;
		decl_end[0][289] = 30;
		ra_start[0][290] = 286;
		ra_end[0][290] = 294;
		decl_start[0][290] = 28;
		decl_end[0][290] = 30;
		ra_start[0][291] = 291;
		ra_end[0][291] = 302;
		decl_start[0][291] = 22;
		decl_end[0][291] = 30;
		ra_start[0][292] = 293;
		ra_end[0][292] = 302;
		decl_start[0][292] = 22;
		decl_end[0][292] = 29;
		ra_start[0][293] = 293;
		ra_end[0][293] = 302;
		decl_start[0][293] = 26;
		decl_end[0][293] = 30;
		ra_start[0][294] = 293;
		ra_end[0][294] = 298;
		decl_start[0][294] = 28;
		decl_end[0][294] = 30;
		ra_start[0][295] = 295;
		ra_end[0][295] = 304;
		decl_start[0][295] = 22;
		decl_end[0][295] = 30;
		ra_start[0][296] = 301;
		ra_end[0][296] = 310;
		decl_start[0][296] = 22;
		decl_end[0][296] = 27;
		ra_start[0][297] = 301;
		ra_end[0][297] = 310;
		decl_start[0][297] = 24;
		decl_end[0][297] = 28;
		ra_start[0][298] = 301;
		ra_end[0][298] = 310;
		decl_start[0][298] = 26;
		decl_end[0][298] = 30;
		ra_start[0][299] = 303;
		ra_end[0][299] = 314;
		decl_start[0][299] = 22;
		decl_end[0][299] = 30;
		ra_start[0][300] = 309;
		ra_end[0][300] = 319;
		decl_start[0][300] = 22;
		decl_end[0][300] = 27;
		ra_start[0][301] = 309;
		ra_end[0][301] = 319;
		decl_start[0][301] = 24;
		decl_end[0][301] = 29;
		ra_start[0][302] = 309;
		ra_end[0][302] = 319;
		decl_start[0][302] = 26;
		decl_end[0][302] = 30;
		ra_start[0][303] = 316;
		ra_end[0][303] = 327;
		decl_start[0][303] = 22;
		decl_end[0][303] = 30;
		ra_start[0][304] = 318;
		ra_end[0][304] = 327;
		decl_start[0][304] = 24;
		decl_end[0][304] = 29;
		ra_start[0][305] = 318;
		ra_end[0][305] = 329;
		decl_start[0][305] = 23;
		decl_end[0][305] = 31;
		ra_start[0][306] = 326;
		ra_end[0][306] = 335;
		decl_start[0][306] = 22;
		decl_end[0][306] = 29;
		ra_start[0][307] = 326;
		ra_end[0][307] = 335;
		decl_start[0][307] = 26;
		decl_end[0][307] = 30;
		ra_start[0][308] = 332;
		ra_end[0][308] = 344;
		decl_start[0][308] = 22;
		decl_end[0][308] = 30;
		ra_start[0][309] = 334;
		ra_end[0][309] = 348;
		decl_start[0][309] = 22;
		decl_end[0][309] = 30;
		ra_start[0][310] = 343;
		ra_end[0][310] = 352;
		decl_start[0][310] = 22;
		decl_end[0][310] = 30;
		ra_start[0][311] = 345;
		ra_end[0][311] = 360;
		decl_start[0][311] = 22;
		decl_end[0][311] = 30;
		ra_start[0][312] = 0;
		ra_end[0][312] = 360;
		decl_start[0][312] = 25;
		decl_end[0][312] = 34;
		ra_start[0][313] = 0;
		ra_end[0][313] = 16;
		decl_start[0][313] = 30;
		decl_end[0][313] = 38;
		ra_start[0][314] = 9;
		ra_end[0][314] = 18;
		decl_start[0][314] = 30;
		decl_end[0][314] = 38;
		ra_start[0][315] = 11;
		ra_end[0][315] = 27;
		decl_start[0][315] = 30;
		decl_end[0][315] = 38;
		ra_start[0][316] = 18;
		ra_end[0][316] = 36;
		decl_start[0][316] = 30;
		decl_end[0][316] = 38;
		ra_start[0][317] = 27;
		ra_end[0][317] = 36;
		decl_start[0][317] = 30;
		decl_end[0][317] = 38;
		ra_start[0][318] = 29;
		ra_end[0][318] = 45;
		decl_start[0][318] = 30;
		decl_end[0][318] = 38;
		ra_start[0][319] = 36;
		ra_end[0][319] = 45;
		decl_start[0][319] = 33;
		decl_end[0][319] = 38;
		ra_start[0][320] = 42;
		ra_end[0][320] = 54;
		decl_start[0][320] = 30;
		decl_end[0][320] = 38;
		ra_start[0][321] = 45;
		ra_end[0][321] = 63;
		decl_start[0][321] = 30;
		decl_end[0][321] = 38;
		ra_start[0][322] = 54;
		ra_end[0][322] = 63;
		decl_start[0][322] = 31;
		decl_end[0][322] = 38;
		ra_start[0][323] = 60;
		ra_end[0][323] = 72;
		decl_start[0][323] = 30;
		decl_end[0][323] = 38;
		ra_start[0][324] = 63;
		ra_end[0][324] = 81;
		decl_start[0][324] = 30;
		decl_end[0][324] = 38;
		ra_start[0][325] = 72;
		ra_end[0][325] = 81;
		decl_start[0][325] = 30;
		decl_end[0][325] = 36;
		ra_start[0][326] = 72;
		ra_end[0][326] = 81;
		decl_start[0][326] = 33;
		decl_end[0][326] = 38;
		ra_start[0][327] = 76;
		ra_end[0][327] = 90;
		decl_start[0][327] = 30;
		decl_end[0][327] = 38;
		ra_start[0][328] = 81;
		ra_end[0][328] = 90;
		decl_start[0][328] = 30;
		decl_end[0][328] = 34;
		ra_start[0][329] = 81;
		ra_end[0][329] = 90;
		decl_start[0][329] = 31;
		decl_end[0][329] = 36;
		ra_start[0][330] = 81;
		ra_end[0][330] = 90;
		decl_start[0][330] = 33;
		decl_end[0][330] = 38;
		ra_start[0][331] = 87;
		ra_end[0][331] = 99;
		decl_start[0][331] = 30;
		decl_end[0][331] = 38;
		ra_start[0][332] = 90;
		ra_end[0][332] = 99;
		decl_start[0][332] = 30;
		decl_end[0][332] = 34;
		ra_start[0][333] = 90;
		ra_end[0][333] = 99;
		decl_start[0][333] = 31;
		decl_end[0][333] = 36;
		ra_start[0][334] = 90;
		ra_end[0][334] = 99;
		decl_start[0][334] = 33;
		decl_end[0][334] = 38;
		ra_start[0][335] = 90;
		ra_end[0][335] = 101;
		decl_start[0][335] = 30;
		decl_end[0][335] = 38;
		ra_start[0][336] = 99;
		ra_end[0][336] = 108;
		decl_start[0][336] = 30;
		decl_end[0][336] = 34;
		ra_start[0][337] = 99;
		ra_end[0][337] = 108;
		decl_start[0][337] = 31;
		decl_end[0][337] = 38;
		ra_start[0][338] = 101;
		ra_end[0][338] = 117;
		decl_start[0][338] = 30;
		decl_end[0][338] = 38;
		ra_start[0][339] = 108;
		ra_end[0][339] = 117;
		decl_start[0][339] = 31;
		decl_end[0][339] = 38;
		ra_start[0][340] = 112;
		ra_end[0][340] = 126;
		decl_start[0][340] = 30;
		decl_end[0][340] = 38;
		ra_start[0][341] = 117;
		ra_end[0][341] = 135;
		decl_start[0][341] = 30;
		decl_end[0][341] = 38;
		ra_start[0][342] = 126;
		ra_end[0][342] = 144;
		decl_start[0][342] = 30;
		decl_end[0][342] = 38;
		ra_start[0][343] = 135;
		ra_end[0][343] = 153;
		decl_start[0][343] = 30;
		decl_end[0][343] = 38;
		ra_start[0][344] = 144;
		ra_end[0][344] = 162;
		decl_start[0][344] = 30;
		decl_end[0][344] = 38;
		ra_start[0][345] = 159;
		ra_end[0][345] = 180;
		decl_start[0][345] = 30;
		decl_end[0][345] = 38;
		ra_start[0][346] = 171;
		ra_end[0][346] = 198;
		decl_start[0][346] = 30;
		decl_end[0][346] = 38;
		ra_start[0][347] = 189;
		ra_end[0][347] = 207;
		decl_start[0][347] = 30;
		decl_end[0][347] = 38;
		ra_start[0][348] = 204;
		ra_end[0][348] = 225;
		decl_start[0][348] = 30;
		decl_end[0][348] = 38;
		ra_start[0][349] = 216;
		ra_end[0][349] = 234;
		decl_start[0][349] = 30;
		decl_end[0][349] = 38;
		ra_start[0][350] = 225;
		ra_end[0][350] = 243;
		decl_start[0][350] = 30;
		decl_end[0][350] = 38;
		ra_start[0][351] = 240;
		ra_end[0][351] = 252;
		decl_start[0][351] = 30;
		decl_end[0][351] = 38;
		ra_start[0][352] = 246;
		ra_end[0][352] = 261;
		decl_start[0][352] = 30;
		decl_end[0][352] = 38;
		ra_start[0][353] = 252;
		ra_end[0][353] = 270;
		decl_start[0][353] = 30;
		decl_end[0][353] = 38;
		ra_start[0][354] = 261;
		ra_end[0][354] = 270;
		decl_start[0][354] = 30;
		decl_end[0][354] = 38;
		ra_start[0][355] = 261;
		ra_end[0][355] = 279;
		decl_start[0][355] = 30;
		decl_end[0][355] = 38;
		ra_start[0][356] = 270;
		ra_end[0][356] = 279;
		decl_start[0][356] = 31;
		decl_end[0][356] = 36;
		ra_start[0][357] = 270;
		ra_end[0][357] = 282;
		decl_start[0][357] = 30;
		decl_end[0][357] = 38;
		ra_start[0][358] = 279;
		ra_end[0][358] = 288;
		decl_start[0][358] = 30;
		decl_end[0][358] = 34;
		ra_start[0][359] = 279;
		ra_end[0][359] = 288;
		decl_start[0][359] = 31;
		decl_end[0][359] = 35;
		ra_start[0][360] = 279;
		ra_end[0][360] = 288;
		decl_start[0][360] = 33;
		decl_end[0][360] = 38;
		ra_start[0][361] = 279;
		ra_end[0][361] = 291;
		decl_start[0][361] = 30;
		decl_end[0][361] = 38;
		ra_start[0][362] = 288;
		ra_end[0][362] = 297;
		decl_start[0][362] = 30;
		decl_end[0][362] = 32;
		ra_start[0][363] = 288;
		ra_end[0][363] = 297;
		decl_start[0][363] = 30;
		decl_end[0][363] = 34;
		ra_start[0][364] = 290;
		ra_end[0][364] = 297;
		decl_start[0][364] = 31;
		decl_end[0][364] = 34;
		ra_start[0][365] = 288;
		ra_end[0][365] = 297;
		decl_start[0][365] = 31;
		decl_end[0][365] = 36;
		ra_start[0][366] = 288;
		ra_end[0][366] = 297;
		decl_start[0][366] = 33;
		decl_end[0][366] = 38;
		ra_start[0][367] = 288;
		ra_end[0][367] = 297;
		decl_start[0][367] = 35;
		decl_end[0][367] = 38;
		ra_start[0][368] = 294;
		ra_end[0][368] = 304;
		decl_start[0][368] = 30;
		decl_end[0][368] = 38;
		ra_start[0][369] = 297;
		ra_end[0][369] = 306;
		decl_start[0][369] = 30;
		decl_end[0][369] = 34;
		ra_start[0][370] = 297;
		ra_end[0][370] = 306;
		decl_start[0][370] = 31;
		decl_end[0][370] = 36;
		ra_start[0][371] = 297;
		ra_end[0][371] = 304;
		decl_start[0][371] = 33;
		decl_end[0][371] = 36;
		ra_start[0][372] = 297;
		ra_end[0][372] = 306;
		decl_start[0][372] = 33;
		decl_end[0][372] = 38;
		ra_start[0][373] = 299;
		ra_end[0][373] = 306;
		decl_start[0][373] = 35;
		decl_end[0][373] = 38;
		ra_start[0][374] = 303;
		ra_end[0][374] = 315;
		decl_start[0][374] = 30;
		decl_end[0][374] = 38;
		ra_start[0][375] = 306;
		ra_end[0][375] = 315;
		decl_start[0][375] = 30;
		decl_end[0][375] = 34;
		ra_start[0][376] = 306;
		ra_end[0][376] = 315;
		decl_start[0][376] = 31;
		decl_end[0][376] = 36;
		ra_start[0][377] = 306;
		ra_end[0][377] = 315;
		decl_start[0][377] = 33;
		decl_end[0][377] = 38;
		ra_start[0][378] = 310;
		ra_end[0][378] = 318;
		decl_start[0][378] = 30;
		decl_end[0][378] = 38;
		ra_start[0][379] = 315;
		ra_end[0][379] = 324;
		decl_start[0][379] = 30;
		decl_end[0][379] = 34;
		ra_start[0][380] = 315;
		ra_end[0][380] = 324;
		decl_start[0][380] = 31;
		decl_end[0][380] = 34;
		ra_start[0][381] = 315;
		ra_end[0][381] = 324;
		decl_start[0][381] = 31;
		decl_end[0][381] = 36;
		ra_start[0][382] = 315;
		ra_end[0][382] = 324;
		decl_start[0][382] = 33;
		decl_end[0][382] = 38;
		ra_start[0][383] = 321;
		ra_end[0][383] = 333;
		decl_start[0][383] = 30;
		decl_end[0][383] = 38;
		ra_start[0][384] = 324;
		ra_end[0][384] = 333;
		decl_start[0][384] = 31;
		decl_end[0][384] = 36;
		ra_start[0][385] = 324;
		ra_end[0][385] = 337;
		decl_start[0][385] = 30;
		decl_end[0][385] = 38;
		ra_start[0][386] = 333;
		ra_end[0][386] = 342;
		decl_start[0][386] = 30;
		decl_end[0][386] = 34;
		ra_start[0][387] = 333;
		ra_end[0][387] = 342;
		decl_start[0][387] = 31;
		decl_end[0][387] = 38;
		ra_start[0][388] = 335;
		ra_end[0][388] = 351;
		decl_start[0][388] = 30;
		decl_end[0][388] = 38;
		ra_start[0][389] = 342;
		ra_end[0][389] = 351;
		decl_start[0][389] = 31;
		decl_end[0][389] = 38;
		ra_start[0][390] = 342;
		ra_end[0][390] = 360;
		decl_start[0][390] = 30;
		decl_end[0][390] = 38;
		ra_start[0][391] = 351;
		ra_end[0][391] = 360;
		decl_start[0][391] = 31;
		decl_end[0][391] = 38;
	}

	/**
	 * Initializes the R.A. and Decl. of each blocks of tic2.dat.
	 */
	private static void initialize_Tic2 ( ) {
		ra_start[1][0] = 0;
		ra_end[1][0] = 10;
		decl_start[1][0] = 37;
		decl_end[1][0] = 44;
		ra_start[1][1] = 0;
		ra_end[1][1] = 10;
		decl_start[1][1] = 41;
		decl_end[1][1] = 45;
		ra_start[1][2] = 7;
		ra_end[1][2] = 20;
		decl_start[1][2] = 37;
		decl_end[1][2] = 45;
		ra_start[1][3] = 10;
		ra_end[1][3] = 20;
		decl_start[1][3] = 39;
		decl_end[1][3] = 45;
		ra_start[1][4] = 17;
		ra_end[1][4] = 30;
		decl_start[1][4] = 37;
		decl_end[1][4] = 45;
		ra_start[1][5] = 20;
		ra_end[1][5] = 30;
		decl_start[1][5] = 39;
		decl_end[1][5] = 45;
		ra_start[1][6] = 22;
		ra_end[1][6] = 40;
		decl_start[1][6] = 37;
		decl_end[1][6] = 45;
		ra_start[1][7] = 30;
		ra_end[1][7] = 40;
		decl_start[1][7] = 37;
		decl_end[1][7] = 44;
		ra_start[1][8] = 30;
		ra_end[1][8] = 40;
		decl_start[1][8] = 41;
		decl_end[1][8] = 45;
		ra_start[1][9] = 35;
		ra_end[1][9] = 50;
		decl_start[1][9] = 37;
		decl_end[1][9] = 45;
		ra_start[1][10] = 40;
		ra_end[1][10] = 50;
		decl_start[1][10] = 39;
		decl_end[1][10] = 44;
		ra_start[1][11] = 40;
		ra_end[1][11] = 50;
		decl_start[1][11] = 41;
		decl_end[1][11] = 45;
		ra_start[1][12] = 45;
		ra_end[1][12] = 60;
		decl_start[1][12] = 37;
		decl_end[1][12] = 45;
		ra_start[1][13] = 50;
		ra_end[1][13] = 60;
		decl_start[1][13] = 39;
		decl_end[1][13] = 45;
		ra_start[1][14] = 50;
		ra_end[1][14] = 70;
		decl_start[1][14] = 37;
		decl_end[1][14] = 45;
		ra_start[1][15] = 60;
		ra_end[1][15] = 70;
		decl_start[1][15] = 39;
		decl_end[1][15] = 44;
		ra_start[1][16] = 60;
		ra_end[1][16] = 73;
		decl_start[1][16] = 37;
		decl_end[1][16] = 45;
		ra_start[1][17] = 70;
		ra_end[1][17] = 80;
		decl_start[1][17] = 37;
		decl_end[1][17] = 42;
		ra_start[1][18] = 70;
		ra_end[1][18] = 80;
		decl_start[1][18] = 39;
		decl_end[1][18] = 42;
		ra_start[1][19] = 70;
		ra_end[1][19] = 80;
		decl_start[1][19] = 39;
		decl_end[1][19] = 44;
		ra_start[1][20] = 70;
		ra_end[1][20] = 80;
		decl_start[1][20] = 41;
		decl_end[1][20] = 45;
		ra_start[1][21] = 75;
		ra_end[1][21] = 88;
		decl_start[1][21] = 37;
		decl_end[1][21] = 45;
		ra_start[1][22] = 80;
		ra_end[1][22] = 90;
		decl_start[1][22] = 37;
		decl_end[1][22] = 42;
		ra_start[1][23] = 80;
		ra_end[1][23] = 90;
		decl_start[1][23] = 39;
		decl_end[1][23] = 44;
		ra_start[1][24] = 80;
		ra_end[1][24] = 90;
		decl_start[1][24] = 41;
		decl_end[1][24] = 45;
		ra_start[1][25] = 90;
		ra_end[1][25] = 98;
		decl_start[1][25] = 37;
		decl_end[1][25] = 40;
		ra_start[1][26] = 90;
		ra_end[1][26] = 100;
		decl_start[1][26] = 37;
		decl_end[1][26] = 44;
		ra_start[1][27] = 90;
		ra_end[1][27] = 100;
		decl_start[1][27] = 41;
		decl_end[1][27] = 45;
		ra_start[1][28] = 95;
		ra_end[1][28] = 110;
		decl_start[1][28] = 37;
		decl_end[1][28] = 45;
		ra_start[1][29] = 100;
		ra_end[1][29] = 110;
		decl_start[1][29] = 39;
		decl_end[1][29] = 44;
		ra_start[1][30] = 100;
		ra_end[1][30] = 113;
		decl_start[1][30] = 37;
		decl_end[1][30] = 45;
		ra_start[1][31] = 110;
		ra_end[1][31] = 120;
		decl_start[1][31] = 37;
		decl_end[1][31] = 44;
		ra_start[1][32] = 110;
		ra_end[1][32] = 127;
		decl_start[1][32] = 37;
		decl_end[1][32] = 45;
		ra_start[1][33] = 120;
		ra_end[1][33] = 130;
		decl_start[1][33] = 37;
		decl_end[1][33] = 45;
		ra_start[1][34] = 126;
		ra_end[1][34] = 140;
		decl_start[1][34] = 37;
		decl_end[1][34] = 45;
		ra_start[1][35] = 140;
		ra_end[1][35] = 157;
		decl_start[1][35] = 37;
		decl_end[1][35] = 45;
		ra_start[1][36] = 150;
		ra_end[1][36] = 170;
		decl_start[1][36] = 37;
		decl_end[1][36] = 45;
		ra_start[1][37] = 160;
		ra_end[1][37] = 185;
		decl_start[1][37] = 37;
		decl_end[1][37] = 45;
		ra_start[1][38] = 180;
		ra_end[1][38] = 200;
		decl_start[1][38] = 37;
		decl_end[1][38] = 45;
		ra_start[1][39] = 190;
		ra_end[1][39] = 214;
		decl_start[1][39] = 37;
		decl_end[1][39] = 45;
		ra_start[1][40] = 210;
		ra_end[1][40] = 230;
		decl_start[1][40] = 37;
		decl_end[1][40] = 45;
		ra_start[1][41] = 220;
		ra_end[1][41] = 240;
		decl_start[1][41] = 37;
		decl_end[1][41] = 45;
		ra_start[1][42] = 230;
		ra_end[1][42] = 250;
		decl_start[1][42] = 37;
		decl_end[1][42] = 45;
		ra_start[1][43] = 240;
		ra_end[1][43] = 260;
		decl_start[1][43] = 37;
		decl_end[1][43] = 45;
		ra_start[1][44] = 250;
		ra_end[1][44] = 270;
		decl_start[1][44] = 37;
		decl_end[1][44] = 45;
		ra_start[1][45] = 260;
		ra_end[1][45] = 270;
		decl_start[1][45] = 37;
		decl_end[1][45] = 45;
		ra_start[1][46] = 262;
		ra_end[1][46] = 280;
		decl_start[1][46] = 37;
		decl_end[1][46] = 45;
		ra_start[1][47] = 270;
		ra_end[1][47] = 280;
		decl_start[1][47] = 39;
		decl_end[1][47] = 44;
		ra_start[1][48] = 270;
		ra_end[1][48] = 288;
		decl_start[1][48] = 37;
		decl_end[1][48] = 45;
		ra_start[1][49] = 280;
		ra_end[1][49] = 290;
		decl_start[1][49] = 37;
		decl_end[1][49] = 42;
		ra_start[1][50] = 280;
		ra_end[1][50] = 290;
		decl_start[1][50] = 39;
		decl_end[1][50] = 44;
		ra_start[1][51] = 280;
		ra_end[1][51] = 293;
		decl_start[1][51] = 37;
		decl_end[1][51] = 45;
		ra_start[1][52] = 290;
		ra_end[1][52] = 300;
		decl_start[1][52] = 37;
		decl_end[1][52] = 40;
		ra_start[1][53] = 290;
		ra_end[1][53] = 300;
		decl_start[1][53] = 37;
		decl_end[1][53] = 42;
		ra_start[1][54] = 290;
		ra_end[1][54] = 300;
		decl_start[1][54] = 39;
		decl_end[1][54] = 44;
		ra_start[1][55] = 290;
		ra_end[1][55] = 300;
		decl_start[1][55] = 41;
		decl_end[1][55] = 44;
		ra_start[1][56] = 290;
		ra_end[1][56] = 300;
		decl_start[1][56] = 41;
		decl_end[1][56] = 45;
		ra_start[1][57] = 297;
		ra_end[1][57] = 305;
		decl_start[1][57] = 37;
		decl_end[1][57] = 45;
		ra_start[1][58] = 300;
		ra_end[1][58] = 310;
		decl_start[1][58] = 37;
		decl_end[1][58] = 42;
		ra_start[1][59] = 300;
		ra_end[1][59] = 310;
		decl_start[1][59] = 39;
		decl_end[1][59] = 44;
		ra_start[1][60] = 300;
		ra_end[1][60] = 310;
		decl_start[1][60] = 41;
		decl_end[1][60] = 44;
		ra_start[1][61] = 300;
		ra_end[1][61] = 310;
		decl_start[1][61] = 41;
		decl_end[1][61] = 45;
		ra_start[1][62] = 307;
		ra_end[1][62] = 320;
		decl_start[1][62] = 37;
		decl_end[1][62] = 45;
		ra_start[1][63] = 310;
		ra_end[1][63] = 320;
		decl_start[1][63] = 37;
		decl_end[1][63] = 42;
		ra_start[1][64] = 310;
		ra_end[1][64] = 320;
		decl_start[1][64] = 39;
		decl_end[1][64] = 45;
		ra_start[1][65] = 312;
		ra_end[1][65] = 323;
		decl_start[1][65] = 37;
		decl_end[1][65] = 45;
		ra_start[1][66] = 320;
		ra_end[1][66] = 330;
		decl_start[1][66] = 37;
		decl_end[1][66] = 42;
		ra_start[1][67] = 320;
		ra_end[1][67] = 330;
		decl_start[1][67] = 39;
		decl_end[1][67] = 44;
		ra_start[1][68] = 320;
		ra_end[1][68] = 330;
		decl_start[1][68] = 41;
		decl_end[1][68] = 45;
		ra_start[1][69] = 320;
		ra_end[1][69] = 333;
		decl_start[1][69] = 37;
		decl_end[1][69] = 45;
		ra_start[1][70] = 330;
		ra_end[1][70] = 340;
		decl_start[1][70] = 37;
		decl_end[1][70] = 42;
		ra_start[1][71] = 330;
		ra_end[1][71] = 340;
		decl_start[1][71] = 39;
		decl_end[1][71] = 44;
		ra_start[1][72] = 330;
		ra_end[1][72] = 340;
		decl_start[1][72] = 41;
		decl_end[1][72] = 45;
		ra_start[1][73] = 337;
		ra_end[1][73] = 350;
		decl_start[1][73] = 37;
		decl_end[1][73] = 45;
		ra_start[1][74] = 340;
		ra_end[1][74] = 350;
		decl_start[1][74] = 39;
		decl_end[1][74] = 45;
		ra_start[1][75] = 340;
		ra_end[1][75] = 360;
		decl_start[1][75] = 37;
		decl_end[1][75] = 45;
		ra_start[1][76] = 350;
		ra_end[1][76] = 360;
		decl_start[1][76] = 39;
		decl_end[1][76] = 45;
		ra_start[1][77] = 0;
		ra_end[1][77] = 360;
		decl_start[1][77] = 43;
		decl_end[1][77] = 47;
		ra_start[1][78] = 0;
		ra_end[1][78] = 12;
		decl_start[1][78] = 45;
		decl_end[1][78] = 49;
		ra_start[1][79] = 0;
		ra_end[1][79] = 12;
		decl_start[1][79] = 46;
		decl_end[1][79] = 52;
		ra_start[1][80] = 0;
		ra_end[1][80] = 17;
		decl_start[1][80] = 45;
		decl_end[1][80] = 53;
		ra_start[1][81] = 11;
		ra_end[1][81] = 23;
		decl_start[1][81] = 45;
		decl_end[1][81] = 49;
		ra_start[1][82] = 11;
		ra_end[1][82] = 23;
		decl_start[1][82] = 47;
		decl_end[1][82] = 53;
		ra_start[1][83] = 11;
		ra_end[1][83] = 31;
		decl_start[1][83] = 45;
		decl_end[1][83] = 53;
		ra_start[1][84] = 22;
		ra_end[1][84] = 34;
		decl_start[1][84] = 45;
		decl_end[1][84] = 49;
		ra_start[1][85] = 22;
		ra_end[1][85] = 34;
		decl_start[1][85] = 46;
		decl_end[1][85] = 53;
		ra_start[1][86] = 22;
		ra_end[1][86] = 37;
		decl_start[1][86] = 45;
		decl_end[1][86] = 53;
		ra_start[1][87] = 33;
		ra_end[1][87] = 45;
		decl_start[1][87] = 45;
		decl_end[1][87] = 49;
		ra_start[1][88] = 33;
		ra_end[1][88] = 45;
		decl_start[1][88] = 46;
		decl_end[1][88] = 51;
		ra_start[1][89] = 33;
		ra_end[1][89] = 45;
		decl_start[1][89] = 48;
		decl_end[1][89] = 53;
		ra_start[1][90] = 36;
		ra_end[1][90] = 51;
		decl_start[1][90] = 45;
		decl_end[1][90] = 53;
		ra_start[1][91] = 45;
		ra_end[1][91] = 57;
		decl_start[1][91] = 45;
		decl_end[1][91] = 49;
		ra_start[1][92] = 45;
		ra_end[1][92] = 57;
		decl_start[1][92] = 47;
		decl_end[1][92] = 53;
		ra_start[1][93] = 50;
		ra_end[1][93] = 68;
		decl_start[1][93] = 45;
		decl_end[1][93] = 53;
		ra_start[1][94] = 56;
		ra_end[1][94] = 68;
		decl_start[1][94] = 46;
		decl_end[1][94] = 51;
		ra_start[1][95] = 56;
		ra_end[1][95] = 74;
		decl_start[1][95] = 45;
		decl_end[1][95] = 53;
		ra_start[1][96] = 67;
		ra_end[1][96] = 79;
		decl_start[1][96] = 45;
		decl_end[1][96] = 49;
		ra_start[1][97] = 67;
		ra_end[1][97] = 79;
		decl_start[1][97] = 46;
		decl_end[1][97] = 51;
		ra_start[1][98] = 67;
		ra_end[1][98] = 82;
		decl_start[1][98] = 45;
		decl_end[1][98] = 53;
		ra_start[1][99] = 78;
		ra_end[1][99] = 90;
		decl_start[1][99] = 45;
		decl_end[1][99] = 49;
		ra_start[1][100] = 78;
		ra_end[1][100] = 90;
		decl_start[1][100] = 46;
		decl_end[1][100] = 51;
		ra_start[1][101] = 78;
		ra_end[1][101] = 90;
		decl_start[1][101] = 48;
		decl_end[1][101] = 53;
		ra_start[1][102] = 87;
		ra_end[1][102] = 102;
		decl_start[1][102] = 45;
		decl_end[1][102] = 53;
		ra_start[1][103] = 90;
		ra_end[1][103] = 102;
		decl_start[1][103] = 46;
		decl_end[1][103] = 53;
		ra_start[1][104] = 92;
		ra_end[1][104] = 113;
		decl_start[1][104] = 45;
		decl_end[1][104] = 53;
		ra_start[1][105] = 101;
		ra_end[1][105] = 117;
		decl_start[1][105] = 45;
		decl_end[1][105] = 53;
		ra_start[1][106] = 112;
		ra_end[1][106] = 124;
		decl_start[1][106] = 45;
		decl_end[1][106] = 53;
		ra_start[1][107] = 116;
		ra_end[1][107] = 135;
		decl_start[1][107] = 45;
		decl_end[1][107] = 53;
		ra_start[1][108] = 123;
		ra_end[1][108] = 147;
		decl_start[1][108] = 45;
		decl_end[1][108] = 53;
		ra_start[1][109] = 135;
		ra_end[1][109] = 158;
		decl_start[1][109] = 45;
		decl_end[1][109] = 53;
		ra_start[1][110] = 153;
		ra_end[1][110] = 180;
		decl_start[1][110] = 45;
		decl_end[1][110] = 53;
		ra_start[1][111] = 168;
		ra_end[1][111] = 192;
		decl_start[1][111] = 45;
		decl_end[1][111] = 53;
		ra_start[1][112] = 186;
		ra_end[1][112] = 214;
		decl_start[1][112] = 45;
		decl_end[1][112] = 53;
		ra_start[1][113] = 202;
		ra_end[1][113] = 225;
		decl_start[1][113] = 45;
		decl_end[1][113] = 53;
		ra_start[1][114] = 217;
		ra_end[1][114] = 237;
		decl_start[1][114] = 45;
		decl_end[1][114] = 53;
		ra_start[1][115] = 234;
		ra_end[1][115] = 248;
		decl_start[1][115] = 45;
		decl_end[1][115] = 53;
		ra_start[1][116] = 243;
		ra_end[1][116] = 259;
		decl_start[1][116] = 45;
		decl_end[1][116] = 53;
		ra_start[1][117] = 251;
		ra_end[1][117] = 270;
		decl_start[1][117] = 45;
		decl_end[1][117] = 53;
		ra_start[1][118] = 258;
		ra_end[1][118] = 276;
		decl_start[1][118] = 45;
		decl_end[1][118] = 53;
		ra_start[1][119] = 270;
		ra_end[1][119] = 282;
		decl_start[1][119] = 45;
		decl_end[1][119] = 51;
		ra_start[1][120] = 270;
		ra_end[1][120] = 283;
		decl_start[1][120] = 45;
		decl_end[1][120] = 53;
		ra_start[1][121] = 281;
		ra_end[1][121] = 293;
		decl_start[1][121] = 45;
		decl_end[1][121] = 49;
		ra_start[1][122] = 281;
		ra_end[1][122] = 293;
		decl_start[1][122] = 46;
		decl_end[1][122] = 51;
		ra_start[1][123] = 281;
		ra_end[1][123] = 295;
		decl_start[1][123] = 45;
		decl_end[1][123] = 53;
		ra_start[1][124] = 292;
		ra_end[1][124] = 304;
		decl_start[1][124] = 45;
		decl_end[1][124] = 49;
		ra_start[1][125] = 292;
		ra_end[1][125] = 304;
		decl_start[1][125] = 46;
		decl_end[1][125] = 51;
		ra_start[1][126] = 292;
		ra_end[1][126] = 304;
		decl_start[1][126] = 48;
		decl_end[1][126] = 53;
		ra_start[1][127] = 300;
		ra_end[1][127] = 313;
		decl_start[1][127] = 45;
		decl_end[1][127] = 53;
		ra_start[1][128] = 303;
		ra_end[1][128] = 315;
		decl_start[1][128] = 45;
		decl_end[1][128] = 49;
		ra_start[1][129] = 303;
		ra_end[1][129] = 315;
		decl_start[1][129] = 46;
		decl_end[1][129] = 51;
		ra_start[1][130] = 303;
		ra_end[1][130] = 315;
		decl_start[1][130] = 48;
		decl_end[1][130] = 53;
		ra_start[1][131] = 304;
		ra_end[1][131] = 318;
		decl_start[1][131] = 45;
		decl_end[1][131] = 53;
		ra_start[1][132] = 315;
		ra_end[1][132] = 321;
		decl_start[1][132] = 45;
		decl_end[1][132] = 47;
		ra_start[1][133] = 315;
		ra_end[1][133] = 327;
		decl_start[1][133] = 45;
		decl_end[1][133] = 49;
		ra_start[1][134] = 315;
		ra_end[1][134] = 324;
		decl_start[1][134] = 46;
		decl_end[1][134] = 49;
		ra_start[1][135] = 315;
		ra_end[1][135] = 327;
		decl_start[1][135] = 46;
		decl_end[1][135] = 51;
		ra_start[1][136] = 315;
		ra_end[1][136] = 327;
		decl_start[1][136] = 48;
		decl_end[1][136] = 53;
		ra_start[1][137] = 320;
		ra_end[1][137] = 332;
		decl_start[1][137] = 45;
		decl_end[1][137] = 53;
		ra_start[1][138] = 326;
		ra_end[1][138] = 338;
		decl_start[1][138] = 45;
		decl_end[1][138] = 49;
		ra_start[1][139] = 326;
		ra_end[1][139] = 338;
		decl_start[1][139] = 46;
		decl_end[1][139] = 51;
		ra_start[1][140] = 326;
		ra_end[1][140] = 338;
		decl_start[1][140] = 48;
		decl_end[1][140] = 51;
		ra_start[1][141] = 326;
		ra_end[1][141] = 338;
		decl_start[1][141] = 48;
		decl_end[1][141] = 53;
		ra_start[1][142] = 331;
		ra_end[1][142] = 341;
		decl_start[1][142] = 45;
		decl_end[1][142] = 53;
		ra_start[1][143] = 337;
		ra_end[1][143] = 349;
		decl_start[1][143] = 45;
		decl_end[1][143] = 49;
		ra_start[1][144] = 337;
		ra_end[1][144] = 349;
		decl_start[1][144] = 46;
		decl_end[1][144] = 51;
		ra_start[1][145] = 337;
		ra_end[1][145] = 349;
		decl_start[1][145] = 48;
		decl_end[1][145] = 53;
		ra_start[1][146] = 337;
		ra_end[1][146] = 346;
		decl_start[1][146] = 50;
		decl_end[1][146] = 53;
		ra_start[1][147] = 343;
		ra_end[1][147] = 360;
		decl_start[1][147] = 45;
		decl_end[1][147] = 53;
		ra_start[1][148] = 348;
		ra_end[1][148] = 360;
		decl_start[1][148] = 45;
		decl_end[1][148] = 51;
		ra_start[1][149] = 348;
		ra_end[1][149] = 360;
		decl_start[1][149] = 48;
		decl_end[1][149] = 53;
		ra_start[1][150] = 0;
		ra_end[1][150] = 360;
		decl_start[1][150] = 50;
		decl_end[1][150] = 55;
		ra_start[1][151] = 0;
		ra_end[1][151] = 14;
		decl_start[1][151] = 52;
		decl_end[1][151] = 57;
		ra_start[1][152] = 0;
		ra_end[1][152] = 14;
		decl_start[1][152] = 54;
		decl_end[1][152] = 59;
		ra_start[1][153] = 0;
		ra_end[1][153] = 14;
		decl_start[1][153] = 56;
		decl_end[1][153] = 60;
		ra_start[1][154] = 7;
		ra_end[1][154] = 27;
		decl_start[1][154] = 52;
		decl_end[1][154] = 60;
		ra_start[1][155] = 13;
		ra_end[1][155] = 27;
		decl_start[1][155] = 52;
		decl_end[1][155] = 57;
		ra_start[1][156] = 13;
		ra_end[1][156] = 27;
		decl_start[1][156] = 54;
		decl_end[1][156] = 59;
		ra_start[1][157] = 13;
		ra_end[1][157] = 27;
		decl_start[1][157] = 56;
		decl_end[1][157] = 60;
		ra_start[1][158] = 23;
		ra_end[1][158] = 37;
		decl_start[1][158] = 52;
		decl_end[1][158] = 60;
		ra_start[1][159] = 26;
		ra_end[1][159] = 40;
		decl_start[1][159] = 52;
		decl_end[1][159] = 57;
		ra_start[1][160] = 26;
		ra_end[1][160] = 40;
		decl_start[1][160] = 54;
		decl_end[1][160] = 59;
		ra_start[1][161] = 27;
		ra_end[1][161] = 40;
		decl_start[1][161] = 56;
		decl_end[1][161] = 59;
		ra_start[1][162] = 26;
		ra_end[1][162] = 40;
		decl_start[1][162] = 56;
		decl_end[1][162] = 60;
		ra_start[1][163] = 33;
		ra_end[1][163] = 54;
		decl_start[1][163] = 52;
		decl_end[1][163] = 60;
		ra_start[1][164] = 40;
		ra_end[1][164] = 54;
		decl_start[1][164] = 54;
		decl_end[1][164] = 60;
		ra_start[1][165] = 46;
		ra_end[1][165] = 67;
		decl_start[1][165] = 52;
		decl_end[1][165] = 60;
		ra_start[1][166] = 53;
		ra_end[1][166] = 67;
		decl_start[1][166] = 54;
		decl_end[1][166] = 60;
		ra_start[1][167] = 60;
		ra_end[1][167] = 80;
		decl_start[1][167] = 52;
		decl_end[1][167] = 60;
		ra_start[1][168] = 66;
		ra_end[1][168] = 84;
		decl_start[1][168] = 52;
		decl_end[1][168] = 60;
		ra_start[1][169] = 80;
		ra_end[1][169] = 94;
		decl_start[1][169] = 52;
		decl_end[1][169] = 57;
		ra_start[1][170] = 80;
		ra_end[1][170] = 94;
		decl_start[1][170] = 54;
		decl_end[1][170] = 60;
		ra_start[1][171] = 90;
		ra_end[1][171] = 107;
		decl_start[1][171] = 52;
		decl_end[1][171] = 60;
		ra_start[1][172] = 93;
		ra_end[1][172] = 107;
		decl_start[1][172] = 54;
		decl_end[1][172] = 60;
		ra_start[1][173] = 103;
		ra_end[1][173] = 120;
		decl_start[1][173] = 52;
		decl_end[1][173] = 60;
		ra_start[1][174] = 106;
		ra_end[1][174] = 120;
		decl_start[1][174] = 56;
		decl_end[1][174] = 60;
		ra_start[1][175] = 113;
		ra_end[1][175] = 134;
		decl_start[1][175] = 52;
		decl_end[1][175] = 60;
		ra_start[1][176] = 120;
		ra_end[1][176] = 147;
		decl_start[1][176] = 52;
		decl_end[1][176] = 60;
		ra_start[1][177] = 137;
		ra_end[1][177] = 160;
		decl_start[1][177] = 52;
		decl_end[1][177] = 60;
		ra_start[1][178] = 155;
		ra_end[1][178] = 187;
		decl_start[1][178] = 52;
		decl_end[1][178] = 60;
		ra_start[1][179] = 173;
		ra_end[1][179] = 200;
		decl_start[1][179] = 52;
		decl_end[1][179] = 60;
		ra_start[1][180] = 186;
		ra_end[1][180] = 216;
		decl_start[1][180] = 52;
		decl_end[1][180] = 60;
		ra_start[1][181] = 213;
		ra_end[1][181] = 240;
		decl_start[1][181] = 52;
		decl_end[1][181] = 60;
		ra_start[1][182] = 226;
		ra_end[1][182] = 254;
		decl_start[1][182] = 52;
		decl_end[1][182] = 60;
		ra_start[1][183] = 240;
		ra_end[1][183] = 267;
		decl_start[1][183] = 52;
		decl_end[1][183] = 60;
		ra_start[1][184] = 253;
		ra_end[1][184] = 270;
		decl_start[1][184] = 52;
		decl_end[1][184] = 60;
		ra_start[1][185] = 266;
		ra_end[1][185] = 280;
		decl_start[1][185] = 52;
		decl_end[1][185] = 58;
		ra_start[1][186] = 266;
		ra_end[1][186] = 287;
		decl_start[1][186] = 52;
		decl_end[1][186] = 60;
		ra_start[1][187] = 280;
		ra_end[1][187] = 294;
		decl_start[1][187] = 52;
		decl_end[1][187] = 59;
		ra_start[1][188] = 280;
		ra_end[1][188] = 294;
		decl_start[1][188] = 56;
		decl_end[1][188] = 60;
		ra_start[1][189] = 286;
		ra_end[1][189] = 307;
		decl_start[1][189] = 52;
		decl_end[1][189] = 60;
		ra_start[1][190] = 293;
		ra_end[1][190] = 307;
		decl_start[1][190] = 52;
		decl_end[1][190] = 59;
		ra_start[1][191] = 293;
		ra_end[1][191] = 307;
		decl_start[1][191] = 56;
		decl_end[1][191] = 60;
		ra_start[1][192] = 300;
		ra_end[1][192] = 320;
		decl_start[1][192] = 52;
		decl_end[1][192] = 60;
		ra_start[1][193] = 306;
		ra_end[1][193] = 320;
		decl_start[1][193] = 54;
		decl_end[1][193] = 59;
		ra_start[1][194] = 306;
		ra_end[1][194] = 327;
		decl_start[1][194] = 52;
		decl_end[1][194] = 60;
		ra_start[1][195] = 323;
		ra_end[1][195] = 334;
		decl_start[1][195] = 52;
		decl_end[1][195] = 55;
		ra_start[1][196] = 320;
		ra_end[1][196] = 334;
		decl_start[1][196] = 52;
		decl_end[1][196] = 57;
		ra_start[1][197] = 320;
		ra_end[1][197] = 334;
		decl_start[1][197] = 54;
		decl_end[1][197] = 60;
		ra_start[1][198] = 323;
		ra_end[1][198] = 340;
		decl_start[1][198] = 52;
		decl_end[1][198] = 60;
		ra_start[1][199] = 336;
		ra_end[1][199] = 347;
		decl_start[1][199] = 52;
		decl_end[1][199] = 55;
		ra_start[1][200] = 333;
		ra_end[1][200] = 347;
		decl_start[1][200] = 52;
		decl_end[1][200] = 57;
		ra_start[1][201] = 333;
		ra_end[1][201] = 347;
		decl_start[1][201] = 54;
		decl_end[1][201] = 59;
		ra_start[1][202] = 336;
		ra_end[1][202] = 347;
		decl_start[1][202] = 56;
		decl_end[1][202] = 59;
		ra_start[1][203] = 333;
		ra_end[1][203] = 347;
		decl_start[1][203] = 56;
		decl_end[1][203] = 60;
		ra_start[1][204] = 343;
		ra_end[1][204] = 360;
		decl_start[1][204] = 52;
		decl_end[1][204] = 60;
		ra_start[1][205] = 346;
		ra_end[1][205] = 360;
		decl_start[1][205] = 52;
		decl_end[1][205] = 57;
		ra_start[1][206] = 346;
		ra_end[1][206] = 360;
		decl_start[1][206] = 54;
		decl_end[1][206] = 59;
		ra_start[1][207] = 346;
		ra_end[1][207] = 360;
		decl_start[1][207] = 56;
		decl_end[1][207] = 60;
		ra_start[1][208] = 0;
		ra_end[1][208] = 360;
		decl_start[1][208] = 58;
		decl_end[1][208] = 62;
		ra_start[1][209] = 0;
		ra_end[1][209] = 18;
		decl_start[1][209] = 60;
		decl_end[1][209] = 64;
		ra_start[1][210] = 0;
		ra_end[1][210] = 18;
		decl_start[1][210] = 61;
		decl_end[1][210] = 65;
		ra_start[1][211] = 0;
		ra_end[1][211] = 18;
		decl_start[1][211] = 63;
		decl_end[1][211] = 68;
		ra_start[1][212] = 12;
		ra_end[1][212] = 30;
		decl_start[1][212] = 60;
		decl_end[1][212] = 68;
		ra_start[1][213] = 17;
		ra_end[1][213] = 35;
		decl_start[1][213] = 60;
		decl_end[1][213] = 64;
		ra_start[1][214] = 17;
		ra_end[1][214] = 35;
		decl_start[1][214] = 61;
		decl_end[1][214] = 66;
		ra_start[1][215] = 17;
		ra_end[1][215] = 48;
		decl_start[1][215] = 60;
		decl_end[1][215] = 68;
		ra_start[1][216] = 34;
		ra_end[1][216] = 52;
		decl_start[1][216] = 60;
		decl_end[1][216] = 66;
		ra_start[1][217] = 34;
		ra_end[1][217] = 65;
		decl_start[1][217] = 60;
		decl_end[1][217] = 68;
		ra_start[1][218] = 51;
		ra_end[1][218] = 69;
		decl_start[1][218] = 60;
		decl_end[1][218] = 66;
		ra_start[1][219] = 51;
		ra_end[1][219] = 86;
		decl_start[1][219] = 60;
		decl_end[1][219] = 68;
		ra_start[1][220] = 68;
		ra_end[1][220] = 90;
		decl_start[1][220] = 60;
		decl_end[1][220] = 68;
		ra_start[1][221] = 85;
		ra_end[1][221] = 103;
		decl_start[1][221] = 60;
		decl_end[1][221] = 66;
		ra_start[1][222] = 85;
		ra_end[1][222] = 111;
		decl_start[1][222] = 60;
		decl_end[1][222] = 68;
		ra_start[1][223] = 102;
		ra_end[1][223] = 120;
		decl_start[1][223] = 60;
		decl_end[1][223] = 66;
		ra_start[1][224] = 102;
		ra_end[1][224] = 138;
		decl_start[1][224] = 60;
		decl_end[1][224] = 68;
		ra_start[1][225] = 120;
		ra_end[1][225] = 155;
		decl_start[1][225] = 60;
		decl_end[1][225] = 68;
		ra_start[1][226] = 137;
		ra_end[1][226] = 172;
		decl_start[1][226] = 60;
		decl_end[1][226] = 68;
		ra_start[1][227] = 154;
		ra_end[1][227] = 189;
		decl_start[1][227] = 60;
		decl_end[1][227] = 68;
		ra_start[1][228] = 171;
		ra_end[1][228] = 223;
		decl_start[1][228] = 60;
		decl_end[1][228] = 68;
		ra_start[1][229] = 205;
		ra_end[1][229] = 240;
		decl_start[1][229] = 60;
		decl_end[1][229] = 68;
		ra_start[1][230] = 222;
		ra_end[1][230] = 258;
		decl_start[1][230] = 60;
		decl_end[1][230] = 68;
		ra_start[1][231] = 240;
		ra_end[1][231] = 258;
		decl_start[1][231] = 60;
		decl_end[1][231] = 68;
		ra_start[1][232] = 245;
		ra_end[1][232] = 275;
		decl_start[1][232] = 60;
		decl_end[1][232] = 68;
		ra_start[1][233] = 257;
		ra_end[1][233] = 283;
		decl_start[1][233] = 60;
		decl_end[1][233] = 68;
		ra_start[1][234] = 274;
		ra_end[1][234] = 292;
		decl_start[1][234] = 60;
		decl_end[1][234] = 66;
		ra_start[1][235] = 274;
		ra_end[1][235] = 296;
		decl_start[1][235] = 60;
		decl_end[1][235] = 68;
		ra_start[1][236] = 291;
		ra_end[1][236] = 309;
		decl_start[1][236] = 60;
		decl_end[1][236] = 64;
		ra_start[1][237] = 291;
		ra_end[1][237] = 309;
		decl_start[1][237] = 61;
		decl_end[1][237] = 68;
		ra_start[1][238] = 293;
		ra_end[1][238] = 326;
		decl_start[1][238] = 60;
		decl_end[1][238] = 68;
		ra_start[1][239] = 308;
		ra_end[1][239] = 326;
		decl_start[1][239] = 61;
		decl_end[1][239] = 66;
		ra_start[1][240] = 308;
		ra_end[1][240] = 338;
		decl_start[1][240] = 60;
		decl_end[1][240] = 68;
		ra_start[1][241] = 325;
		ra_end[1][241] = 343;
		decl_start[1][241] = 60;
		decl_end[1][241] = 66;
		ra_start[1][242] = 325;
		ra_end[1][242] = 343;
		decl_start[1][242] = 63;
		decl_end[1][242] = 68;
		ra_start[1][243] = 334;
		ra_end[1][243] = 360;
		decl_start[1][243] = 60;
		decl_end[1][243] = 68;
		ra_start[1][244] = 342;
		ra_end[1][244] = 360;
		decl_start[1][244] = 60;
		decl_end[1][244] = 64;
		ra_start[1][245] = 342;
		ra_end[1][245] = 360;
		decl_start[1][245] = 61;
		decl_end[1][245] = 68;
		ra_start[1][246] = 0;
		ra_end[1][246] = 360;
		decl_start[1][246] = 65;
		decl_end[1][246] = 72;
		ra_start[1][247] = 0;
		ra_end[1][247] = 24;
		decl_start[1][247] = 69;
		decl_end[1][247] = 75;
		ra_start[1][248] = 13;
		ra_end[1][248] = 48;
		decl_start[1][248] = 67;
		decl_end[1][248] = 75;
		ra_start[1][249] = 24;
		ra_end[1][249] = 60;
		decl_start[1][249] = 67;
		decl_end[1][249] = 75;
		ra_start[1][250] = 48;
		ra_end[1][250] = 72;
		decl_start[1][250] = 67;
		decl_end[1][250] = 74;
		ra_start[1][251] = 48;
		ra_end[1][251] = 90;
		decl_start[1][251] = 67;
		decl_end[1][251] = 75;
		ra_start[1][252] = 72;
		ra_end[1][252] = 96;
		decl_start[1][252] = 67;
		decl_end[1][252] = 74;
		ra_start[1][253] = 72;
		ra_end[1][253] = 117;
		decl_start[1][253] = 67;
		decl_end[1][253] = 75;
		ra_start[1][254] = 96;
		ra_end[1][254] = 120;
		decl_start[1][254] = 67;
		decl_end[1][254] = 75;
		ra_start[1][255] = 96;
		ra_end[1][255] = 144;
		decl_start[1][255] = 67;
		decl_end[1][255] = 75;
		ra_start[1][256] = 120;
		ra_end[1][256] = 168;
		decl_start[1][256] = 67;
		decl_end[1][256] = 75;
		ra_start[1][257] = 144;
		ra_end[1][257] = 192;
		decl_start[1][257] = 67;
		decl_end[1][257] = 75;
		ra_start[1][258] = 168;
		ra_end[1][258] = 216;
		decl_start[1][258] = 67;
		decl_end[1][258] = 75;
		ra_start[1][259] = 192;
		ra_end[1][259] = 240;
		decl_start[1][259] = 67;
		decl_end[1][259] = 75;
		ra_start[1][260] = 216;
		ra_end[1][260] = 256;
		decl_start[1][260] = 67;
		decl_end[1][260] = 75;
		ra_start[1][261] = 240;
		ra_end[1][261] = 264;
		decl_start[1][261] = 67;
		decl_end[1][261] = 75;
		ra_start[1][262] = 248;
		ra_end[1][262] = 288;
		decl_start[1][262] = 67;
		decl_end[1][262] = 75;
		ra_start[1][263] = 264;
		ra_end[1][263] = 300;
		decl_start[1][263] = 67;
		decl_end[1][263] = 75;
		ra_start[1][264] = 288;
		ra_end[1][264] = 312;
		decl_start[1][264] = 67;
		decl_end[1][264] = 74;
		ra_start[1][265] = 288;
		ra_end[1][265] = 324;
		decl_start[1][265] = 67;
		decl_end[1][265] = 75;
		ra_start[1][266] = 312;
		ra_end[1][266] = 336;
		decl_start[1][266] = 67;
		decl_end[1][266] = 74;
		ra_start[1][267] = 312;
		ra_end[1][267] = 360;
		decl_start[1][267] = 67;
		decl_end[1][267] = 75;
		ra_start[1][268] = 336;
		ra_end[1][268] = 360;
		decl_start[1][268] = 69;
		decl_end[1][268] = 74;
		ra_start[1][269] = 0;
		ra_end[1][269] = 360;
		decl_start[1][269] = 71;
		decl_end[1][269] = 79;
		ra_start[1][270] = 0;
		ra_end[1][270] = 40;
		decl_start[1][270] = 76;
		decl_end[1][270] = 83;
		ra_start[1][271] = 0;
		ra_end[1][271] = 80;
		decl_start[1][271] = 75;
		decl_end[1][271] = 83;
		ra_start[1][272] = 40;
		ra_end[1][272] = 90;
		decl_start[1][272] = 75;
		decl_end[1][272] = 83;
		ra_start[1][273] = 80;
		ra_end[1][273] = 120;
		decl_start[1][273] = 75;
		decl_end[1][273] = 81;
		ra_start[1][274] = 80;
		ra_end[1][274] = 147;
		decl_start[1][274] = 75;
		decl_end[1][274] = 83;
		ra_start[1][275] = 120;
		ra_end[1][275] = 174;
		decl_start[1][275] = 75;
		decl_end[1][275] = 83;
		ra_start[1][276] = 160;
		ra_end[1][276] = 213;
		decl_start[1][276] = 75;
		decl_end[1][276] = 83;
		ra_start[1][277] = 200;
		ra_end[1][277] = 240;
		decl_start[1][277] = 75;
		decl_end[1][277] = 83;
		ra_start[1][278] = 213;
		ra_end[1][278] = 280;
		decl_start[1][278] = 75;
		decl_end[1][278] = 83;
		ra_start[1][279] = 240;
		ra_end[1][279] = 310;
		decl_start[1][279] = 75;
		decl_end[1][279] = 83;
		ra_start[1][280] = 280;
		ra_end[1][280] = 320;
		decl_start[1][280] = 74;
		decl_end[1][280] = 81;
		ra_start[1][281] = 280;
		ra_end[1][281] = 360;
		decl_start[1][281] = 75;
		decl_end[1][281] = 83;
		ra_start[1][282] = 320;
		ra_end[1][282] = 360;
		decl_start[1][282] = 76;
		decl_end[1][282] = 83;
		ra_start[1][283] = 0;
		ra_end[1][283] = 360;
		decl_start[1][283] = 80;
		decl_end[1][283] = 87;
		ra_start[1][284] = 0;
		ra_end[1][284] = 180;
		decl_start[1][284] = 82;
		decl_end[1][284] = 90;
		ra_start[1][285] = 120;
		ra_end[1][285] = 270;
		decl_start[1][285] = 82;
		decl_end[1][285] = 90;
		ra_start[1][286] = 240;
		ra_end[1][286] = 360;
		decl_start[1][286] = 82;
		decl_end[1][286] = 87;
		ra_start[1][287] = 240;
		ra_end[1][287] = 360;
		decl_start[1][287] = 84;
		decl_end[1][287] = 90;
	}

	/**
	 * Initializes the R.A. and Decl. of each blocks of tic3.dat.
	 */
	private static void initialize_Tic3 ( ) {
		ra_start[2][0] = 0;
		ra_end[2][0] = 10;
		decl_start[2][0] = -8;
		decl_end[2][0] = 0;
		ra_start[2][1] = 7;
		ra_end[2][1] = 21;
		decl_start[2][1] = -8;
		decl_end[2][1] = 0;
		ra_start[2][2] = 15;
		ra_end[2][2] = 30;
		decl_start[2][2] = -8;
		decl_end[2][2] = 0;
		ra_start[2][3] = 22;
		ra_end[2][3] = 38;
		decl_start[2][3] = -8;
		decl_end[2][3] = 0;
		ra_start[2][4] = 30;
		ra_end[2][4] = 45;
		decl_start[2][4] = -8;
		decl_end[2][4] = 0;
		ra_start[2][5] = 37;
		ra_end[2][5] = 53;
		decl_start[2][5] = -8;
		decl_end[2][5] = 0;
		ra_start[2][6] = 45;
		ra_end[2][6] = 60;
		decl_start[2][6] = -8;
		decl_end[2][6] = 0;
		ra_start[2][7] = 52;
		ra_end[2][7] = 65;
		decl_start[2][7] = -8;
		decl_end[2][7] = 0;
		ra_start[2][8] = 60;
		ra_end[2][8] = 68;
		decl_start[2][8] = -8;
		decl_end[2][8] = 1;
		ra_start[2][9] = 65;
		ra_end[2][9] = 75;
		decl_start[2][9] = -8;
		decl_end[2][9] = 0;
		ra_start[2][10] = 67;
		ra_end[2][10] = 77;
		decl_start[2][10] = -8;
		decl_end[2][10] = 0;
		ra_start[2][11] = 75;
		ra_end[2][11] = 83;
		decl_start[2][11] = -4;
		decl_end[2][11] = 0;
		ra_start[2][12] = 75;
		ra_end[2][12] = 83;
		decl_start[2][12] = -8;
		decl_end[2][12] = -1;
		ra_start[2][13] = 82;
		ra_end[2][13] = 90;
		decl_start[2][13] = -6;
		decl_end[2][13] = 0;
		ra_start[2][14] = 82;
		ra_end[2][14] = 94;
		decl_start[2][14] = -8;
		decl_end[2][14] = 0;
		ra_start[2][15] = 90;
		ra_end[2][15] = 98;
		decl_start[2][15] = -4;
		decl_end[2][15] = 0;
		ra_start[2][16] = 90;
		ra_end[2][16] = 98;
		decl_start[2][16] = -6;
		decl_end[2][16] = -1;
		ra_start[2][17] = 90;
		ra_end[2][17] = 98;
		decl_start[2][17] = -8;
		decl_end[2][17] = -3;
		ra_start[2][18] = 95;
		ra_end[2][18] = 104;
		decl_start[2][18] = -8;
		decl_end[2][18] = 0;
		ra_start[2][19] = 101;
		ra_end[2][19] = 105;
		decl_start[2][19] = -2;
		decl_end[2][19] = 0;
		ra_start[2][20] = 97;
		ra_end[2][20] = 105;
		decl_start[2][20] = -4;
		decl_end[2][20] = 0;
		ra_start[2][21] = 97;
		ra_end[2][21] = 105;
		decl_start[2][21] = -6;
		decl_end[2][21] = -1;
		ra_start[2][22] = 98;
		ra_end[2][22] = 105;
		decl_start[2][22] = -6;
		decl_end[2][22] = -3;
		ra_start[2][23] = 97;
		ra_end[2][23] = 105;
		decl_start[2][23] = -8;
		decl_end[2][23] = -3;
		ra_start[2][24] = 101;
		ra_end[2][24] = 107;
		decl_start[2][24] = -8;
		decl_end[2][24] = 0;
		ra_start[2][25] = 105;
		ra_end[2][25] = 111;
		decl_start[2][25] = -2;
		decl_end[2][25] = 0;
		ra_start[2][26] = 105;
		ra_end[2][26] = 113;
		decl_start[2][26] = -4;
		decl_end[2][26] = 0;
		ra_start[2][27] = 106;
		ra_end[2][27] = 113;
		decl_start[2][27] = -4;
		decl_end[2][27] = -1;
		ra_start[2][28] = 105;
		ra_end[2][28] = 113;
		decl_start[2][28] = -6;
		decl_end[2][28] = -1;
		ra_start[2][29] = 106;
		ra_end[2][29] = 111;
		decl_start[2][29] = -6;
		decl_end[2][29] = -3;
		ra_start[2][30] = 105;
		ra_end[2][30] = 113;
		decl_start[2][30] = -8;
		decl_end[2][30] = -3;
		ra_start[2][31] = 106;
		ra_end[2][31] = 111;
		decl_start[2][31] = -8;
		decl_end[2][31] = -5;
		ra_start[2][32] = 108;
		ra_end[2][32] = 117;
		decl_start[2][32] = -8;
		decl_end[2][32] = 0;
		ra_start[2][33] = 112;
		ra_end[2][33] = 120;
		decl_start[2][33] = -4;
		decl_end[2][33] = 0;
		ra_start[2][34] = 112;
		ra_end[2][34] = 120;
		decl_start[2][34] = -4;
		decl_end[2][34] = -1;
		ra_start[2][35] = 112;
		ra_end[2][35] = 120;
		decl_start[2][35] = -6;
		decl_end[2][35] = -1;
		ra_start[2][36] = 112;
		ra_end[2][36] = 120;
		decl_start[2][36] = -8;
		decl_end[2][36] = -3;
		ra_start[2][37] = 114;
		ra_end[2][37] = 124;
		decl_start[2][37] = -8;
		decl_end[2][37] = 0;
		ra_start[2][38] = 120;
		ra_end[2][38] = 128;
		decl_start[2][38] = -4;
		decl_end[2][38] = 0;
		ra_start[2][39] = 120;
		ra_end[2][39] = 128;
		decl_start[2][39] = -6;
		decl_end[2][39] = -1;
		ra_start[2][40] = 120;
		ra_end[2][40] = 129;
		decl_start[2][40] = -8;
		decl_end[2][40] = -1;
		ra_start[2][41] = 127;
		ra_end[2][41] = 135;
		decl_start[2][41] = -4;
		decl_end[2][41] = 0;
		ra_start[2][42] = 127;
		ra_end[2][42] = 135;
		decl_start[2][42] = -8;
		decl_end[2][42] = -1;
		ra_start[2][43] = 131;
		ra_end[2][43] = 143;
		decl_start[2][43] = -8;
		decl_end[2][43] = 0;
		ra_start[2][44] = 135;
		ra_end[2][44] = 145;
		decl_start[2][44] = -8;
		decl_end[2][44] = 0;
		ra_start[2][45] = 142;
		ra_end[2][45] = 150;
		decl_start[2][45] = -8;
		decl_end[2][45] = 0;
		ra_start[2][46] = 145;
		ra_end[2][46] = 158;
		decl_start[2][46] = -8;
		decl_end[2][46] = 0;
		ra_start[2][47] = 150;
		ra_end[2][47] = 165;
		decl_start[2][47] = -8;
		decl_end[2][47] = 0;
		ra_start[2][48] = 157;
		ra_end[2][48] = 173;
		decl_start[2][48] = -8;
		decl_end[2][48] = 0;
		ra_start[2][49] = 165;
		ra_end[2][49] = 180;
		decl_start[2][49] = -8;
		decl_end[2][49] = 0;
		ra_start[2][50] = 172;
		ra_end[2][50] = 188;
		decl_start[2][50] = -8;
		decl_end[2][50] = 0;
		ra_start[2][51] = 180;
		ra_end[2][51] = 195;
		decl_start[2][51] = -8;
		decl_end[2][51] = 0;
		ra_start[2][52] = 187;
		ra_end[2][52] = 203;
		decl_start[2][52] = -8;
		decl_end[2][52] = 0;
		ra_start[2][53] = 195;
		ra_end[2][53] = 210;
		decl_start[2][53] = -8;
		decl_end[2][53] = 0;
		ra_start[2][54] = 205;
		ra_end[2][54] = 218;
		decl_start[2][54] = -8;
		decl_end[2][54] = 0;
		ra_start[2][55] = 212;
		ra_end[2][55] = 225;
		decl_start[2][55] = -8;
		decl_end[2][55] = 0;
		ra_start[2][56] = 223;
		ra_end[2][56] = 233;
		decl_start[2][56] = -8;
		decl_end[2][56] = 0;
		ra_start[2][57] = 230;
		ra_end[2][57] = 242;
		decl_start[2][57] = -8;
		decl_end[2][57] = 0;
		ra_start[2][58] = 240;
		ra_end[2][58] = 250;
		decl_start[2][58] = -8;
		decl_end[2][58] = 0;
		ra_start[2][59] = 247;
		ra_end[2][59] = 255;
		decl_start[2][59] = -8;
		decl_end[2][59] = 0;
		ra_start[2][60] = 253;
		ra_end[2][60] = 267;
		decl_start[2][60] = -8;
		decl_end[2][60] = 0;
		ra_start[2][61] = 262;
		ra_end[2][61] = 272;
		decl_start[2][61] = -8;
		decl_end[2][61] = 0;
		ra_start[2][62] = 270;
		ra_end[2][62] = 285;
		decl_start[2][62] = -8;
		decl_end[2][62] = 0;
		ra_start[2][63] = 277;
		ra_end[2][63] = 285;
		decl_start[2][63] = -6;
		decl_end[2][63] = 0;
		ra_start[2][64] = 277;
		ra_end[2][64] = 285;
		decl_start[2][64] = -8;
		decl_end[2][64] = -3;
		ra_start[2][65] = 281;
		ra_end[2][65] = 289;
		decl_start[2][65] = -8;
		decl_end[2][65] = 1;
		ra_start[2][66] = 285;
		ra_end[2][66] = 293;
		decl_start[2][66] = -4;
		decl_end[2][66] = 0;
		ra_start[2][67] = 285;
		ra_end[2][67] = 293;
		decl_start[2][67] = -8;
		decl_end[2][67] = -2;
		ra_start[2][68] = 286;
		ra_end[2][68] = 299;
		decl_start[2][68] = -8;
		decl_end[2][68] = 0;
		ra_start[2][69] = 292;
		ra_end[2][69] = 300;
		decl_start[2][69] = -6;
		decl_end[2][69] = 0;
		ra_start[2][70] = 292;
		ra_end[2][70] = 300;
		decl_start[2][70] = -8;
		decl_end[2][70] = -3;
		ra_start[2][71] = 298;
		ra_end[2][71] = 308;
		decl_start[2][71] = -8;
		decl_end[2][71] = 0;
		ra_start[2][72] = 300;
		ra_end[2][72] = 308;
		decl_start[2][72] = -8;
		decl_end[2][72] = -1;
		ra_start[2][73] = 301;
		ra_end[2][73] = 315;
		decl_start[2][73] = -8;
		decl_end[2][73] = 0;
		ra_start[2][74] = 307;
		ra_end[2][74] = 315;
		decl_start[2][74] = -6;
		decl_end[2][74] = -1;
		ra_start[2][75] = 307;
		ra_end[2][75] = 323;
		decl_start[2][75] = -8;
		decl_end[2][75] = 0;
		ra_start[2][76] = 315;
		ra_end[2][76] = 323;
		decl_start[2][76] = -8;
		decl_end[2][76] = 0;
		ra_start[2][77] = 315;
		ra_end[2][77] = 330;
		decl_start[2][77] = -8;
		decl_end[2][77] = 0;
		ra_start[2][78] = 322;
		ra_end[2][78] = 335;
		decl_start[2][78] = -8;
		decl_end[2][78] = 0;
		ra_start[2][79] = 330;
		ra_end[2][79] = 340;
		decl_start[2][79] = -8;
		decl_end[2][79] = 0;
		ra_start[2][80] = 337;
		ra_end[2][80] = 345;
		decl_start[2][80] = -8;
		decl_end[2][80] = 0;
		ra_start[2][81] = 342;
		ra_end[2][81] = 355;
		decl_start[2][81] = -8;
		decl_end[2][81] = 0;
		ra_start[2][82] = 0;
		ra_end[2][82] = 360;
		decl_start[2][82] = -10;
		decl_end[2][82] = 0;
		ra_start[2][83] = 0;
		ra_end[2][83] = 16;
		decl_start[2][83] = -15;
		decl_end[2][83] = -7;
		ra_start[2][84] = 7;
		ra_end[2][84] = 23;
		decl_start[2][84] = -15;
		decl_end[2][84] = -7;
		ra_start[2][85] = 15;
		ra_end[2][85] = 31;
		decl_start[2][85] = -15;
		decl_end[2][85] = -7;
		ra_start[2][86] = 22;
		ra_end[2][86] = 39;
		decl_start[2][86] = -15;
		decl_end[2][86] = -7;
		ra_start[2][87] = 30;
		ra_end[2][87] = 46;
		decl_start[2][87] = -15;
		decl_end[2][87] = -7;
		ra_start[2][88] = 38;
		ra_end[2][88] = 54;
		decl_start[2][88] = -15;
		decl_end[2][88] = -7;
		ra_start[2][89] = 45;
		ra_end[2][89] = 62;
		decl_start[2][89] = -15;
		decl_end[2][89] = -7;
		ra_start[2][90] = 53;
		ra_end[2][90] = 69;
		decl_start[2][90] = -15;
		decl_end[2][90] = -7;
		ra_start[2][91] = 61;
		ra_end[2][91] = 72;
		decl_start[2][91] = -15;
		decl_end[2][91] = -7;
		ra_start[2][92] = 68;
		ra_end[2][92] = 77;
		decl_start[2][92] = -13;
		decl_end[2][92] = -7;
		ra_start[2][93] = 68;
		ra_end[2][93] = 85;
		decl_start[2][93] = -15;
		decl_end[2][93] = -7;
		ra_start[2][94] = 76;
		ra_end[2][94] = 85;
		decl_start[2][94] = -15;
		decl_end[2][94] = -7;
		ra_start[2][95] = 76;
		ra_end[2][95] = 92;
		decl_start[2][95] = -15;
		decl_end[2][95] = -7;
		ra_start[2][96] = 84;
		ra_end[2][96] = 92;
		decl_start[2][96] = -15;
		decl_end[2][96] = -9;
		ra_start[2][97] = 88;
		ra_end[2][97] = 100;
		decl_start[2][97] = -15;
		decl_end[2][97] = -7;
		ra_start[2][98] = 91;
		ra_end[2][98] = 100;
		decl_start[2][98] = -14;
		decl_end[2][98] = -9;
		ra_start[2][99] = 91;
		ra_end[2][99] = 100;
		decl_start[2][99] = -15;
		decl_end[2][99] = -11;
		ra_start[2][100] = 98;
		ra_end[2][100] = 106;
		decl_start[2][100] = -15;
		decl_end[2][100] = -7;
		ra_start[2][101] = 99;
		ra_end[2][101] = 108;
		decl_start[2][101] = -12;
		decl_end[2][101] = -7;
		ra_start[2][102] = 99;
		ra_end[2][102] = 108;
		decl_start[2][102] = -14;
		decl_end[2][102] = -9;
		ra_start[2][103] = 100;
		ra_end[2][103] = 108;
		decl_start[2][103] = -15;
		decl_end[2][103] = -11;
		ra_start[2][104] = 99;
		ra_end[2][104] = 108;
		decl_start[2][104] = -15;
		decl_end[2][104] = -13;
		ra_start[2][105] = 105;
		ra_end[2][105] = 112;
		decl_start[2][105] = -15;
		decl_end[2][105] = -7;
		ra_start[2][106] = 109;
		ra_end[2][106] = 115;
		decl_start[2][106] = -10;
		decl_end[2][106] = -7;
		ra_start[2][107] = 107;
		ra_end[2][107] = 115;
		decl_start[2][107] = -12;
		decl_end[2][107] = -7;
		ra_start[2][108] = 107;
		ra_end[2][108] = 115;
		decl_start[2][108] = -14;
		decl_end[2][108] = -9;
		ra_start[2][109] = 107;
		ra_end[2][109] = 113;
		decl_start[2][109] = -14;
		decl_end[2][109] = -11;
		ra_start[2][110] = 107;
		ra_end[2][110] = 115;
		decl_start[2][110] = -15;
		decl_end[2][110] = -11;
		ra_start[2][111] = 109;
		ra_end[2][111] = 113;
		decl_start[2][111] = -15;
		decl_end[2][111] = -13;
		ra_start[2][112] = 111;
		ra_end[2][112] = 117;
		decl_start[2][112] = -15;
		decl_end[2][112] = -7;
		ra_start[2][113] = 114;
		ra_end[2][113] = 123;
		decl_start[2][113] = -12;
		decl_end[2][113] = -7;
		ra_start[2][114] = 114;
		ra_end[2][114] = 123;
		decl_start[2][114] = -12;
		decl_end[2][114] = -9;
		ra_start[2][115] = 114;
		ra_end[2][115] = 123;
		decl_start[2][115] = -14;
		decl_end[2][115] = -9;
		ra_start[2][116] = 114;
		ra_end[2][116] = 123;
		decl_start[2][116] = -15;
		decl_end[2][116] = -11;
		ra_start[2][117] = 114;
		ra_end[2][117] = 121;
		decl_start[2][117] = -15;
		decl_end[2][117] = -13;
		ra_start[2][118] = 118;
		ra_end[2][118] = 131;
		decl_start[2][118] = -15;
		decl_end[2][118] = -7;
		ra_start[2][119] = 122;
		ra_end[2][119] = 131;
		decl_start[2][119] = -12;
		decl_end[2][119] = -7;
		ra_start[2][120] = 122;
		ra_end[2][120] = 131;
		decl_start[2][120] = -15;
		decl_end[2][120] = -10;
		ra_start[2][121] = 122;
		ra_end[2][121] = 133;
		decl_start[2][121] = -15;
		decl_end[2][121] = -7;
		ra_start[2][122] = 130;
		ra_end[2][122] = 138;
		decl_start[2][122] = -14;
		decl_end[2][122] = -7;
		ra_start[2][123] = 130;
		ra_end[2][123] = 138;
		decl_start[2][123] = -15;
		decl_end[2][123] = -11;
		ra_start[2][124] = 134;
		ra_end[2][124] = 146;
		decl_start[2][124] = -15;
		decl_end[2][124] = -7;
		ra_start[2][125] = 137;
		ra_end[2][125] = 146;
		decl_start[2][125] = -15;
		decl_end[2][125] = -9;
		ra_start[2][126] = 143;
		ra_end[2][126] = 154;
		decl_start[2][126] = -15;
		decl_end[2][126] = -7;
		ra_start[2][127] = 145;
		ra_end[2][127] = 161;
		decl_start[2][127] = -15;
		decl_end[2][127] = -7;
		ra_start[2][128] = 153;
		ra_end[2][128] = 166;
		decl_start[2][128] = -15;
		decl_end[2][128] = -7;
		ra_start[2][129] = 160;
		ra_end[2][129] = 169;
		decl_start[2][129] = -15;
		decl_end[2][129] = -7;
		ra_start[2][130] = 165;
		ra_end[2][130] = 177;
		decl_start[2][130] = -15;
		decl_end[2][130] = -7;
		ra_start[2][131] = 171;
		ra_end[2][131] = 184;
		decl_start[2][131] = -15;
		decl_end[2][131] = -7;
		ra_start[2][132] = 178;
		ra_end[2][132] = 192;
		decl_start[2][132] = -15;
		decl_end[2][132] = -7;
		ra_start[2][133] = 186;
		ra_end[2][133] = 200;
		decl_start[2][133] = -15;
		decl_end[2][133] = -7;
		ra_start[2][134] = 194;
		ra_end[2][134] = 207;
		decl_start[2][134] = -15;
		decl_end[2][134] = -7;
		ra_start[2][135] = 201;
		ra_end[2][135] = 215;
		decl_start[2][135] = -15;
		decl_end[2][135] = -7;
		ra_start[2][136] = 206;
		ra_end[2][136] = 223;
		decl_start[2][136] = -15;
		decl_end[2][136] = -7;
		ra_start[2][137] = 214;
		ra_end[2][137] = 230;
		decl_start[2][137] = -15;
		decl_end[2][137] = -7;
		ra_start[2][138] = 222;
		ra_end[2][138] = 238;
		decl_start[2][138] = -15;
		decl_end[2][138] = -7;
		ra_start[2][139] = 229;
		ra_end[2][139] = 246;
		decl_start[2][139] = -15;
		decl_end[2][139] = -7;
		ra_start[2][140] = 237;
		ra_end[2][140] = 249;
		decl_start[2][140] = -15;
		decl_end[2][140] = -7;
		ra_start[2][141] = 245;
		ra_end[2][141] = 261;
		decl_start[2][141] = -15;
		decl_end[2][141] = -7;
		ra_start[2][142] = 252;
		ra_end[2][142] = 269;
		decl_start[2][142] = -15;
		decl_end[2][142] = -7;
		ra_start[2][143] = 260;
		ra_end[2][143] = 270;
		decl_start[2][143] = -15;
		decl_end[2][143] = -7;
		ra_start[2][144] = 268;
		ra_end[2][144] = 276;
		decl_start[2][144] = -15;
		decl_end[2][144] = -7;
		ra_start[2][145] = 268;
		ra_end[2][145] = 282;
		decl_start[2][145] = -15;
		decl_end[2][145] = -7;
		ra_start[2][146] = 275;
		ra_end[2][146] = 284;
		decl_start[2][146] = -12;
		decl_end[2][146] = -7;
		ra_start[2][147] = 275;
		ra_end[2][147] = 284;
		decl_start[2][147] = -14;
		decl_end[2][147] = -9;
		ra_start[2][148] = 275;
		ra_end[2][148] = 284;
		decl_start[2][148] = -15;
		decl_end[2][148] = -11;
		ra_start[2][149] = 276;
		ra_end[2][149] = 286;
		decl_start[2][149] = -15;
		decl_end[2][149] = -7;
		ra_start[2][150] = 283;
		ra_end[2][150] = 292;
		decl_start[2][150] = -12;
		decl_end[2][150] = -7;
		ra_start[2][151] = 283;
		ra_end[2][151] = 292;
		decl_start[2][151] = -13;
		decl_end[2][151] = -9;
		ra_start[2][152] = 283;
		ra_end[2][152] = 292;
		decl_start[2][152] = -15;
		decl_end[2][152] = -11;
		ra_start[2][153] = 284;
		ra_end[2][153] = 295;
		decl_start[2][153] = -15;
		decl_end[2][153] = -7;
		ra_start[2][154] = 291;
		ra_end[2][154] = 299;
		decl_start[2][154] = -12;
		decl_end[2][154] = -7;
		ra_start[2][155] = 291;
		ra_end[2][155] = 299;
		decl_start[2][155] = -14;
		decl_end[2][155] = -9;
		ra_start[2][156] = 291;
		ra_end[2][156] = 303;
		decl_start[2][156] = -15;
		decl_end[2][156] = -7;
		ra_start[2][157] = 298;
		ra_end[2][157] = 307;
		decl_start[2][157] = -14;
		decl_end[2][157] = -7;
		ra_start[2][158] = 298;
		ra_end[2][158] = 309;
		decl_start[2][158] = -15;
		decl_end[2][158] = -8;
		ra_start[2][159] = 306;
		ra_end[2][159] = 315;
		decl_start[2][159] = -12;
		decl_end[2][159] = -7;
		ra_start[2][160] = 306;
		ra_end[2][160] = 315;
		decl_start[2][160] = -15;
		decl_end[2][160] = -9;
		ra_start[2][161] = 310;
		ra_end[2][161] = 322;
		decl_start[2][161] = -15;
		decl_end[2][161] = -7;
		ra_start[2][162] = 314;
		ra_end[2][162] = 324;
		decl_start[2][162] = -15;
		decl_end[2][162] = -7;
		ra_start[2][163] = 321;
		ra_end[2][163] = 330;
		decl_start[2][163] = -14;
		decl_end[2][163] = -7;
		ra_start[2][164] = 321;
		ra_end[2][164] = 338;
		decl_start[2][164] = -15;
		decl_end[2][164] = -7;
		ra_start[2][165] = 329;
		ra_end[2][165] = 345;
		decl_start[2][165] = -15;
		decl_end[2][165] = -7;
		ra_start[2][166] = 337;
		ra_end[2][166] = 353;
		decl_start[2][166] = -15;
		decl_end[2][166] = -7;
		ra_start[2][167] = 344;
		ra_end[2][167] = 359;
		decl_start[2][167] = -15;
		decl_end[2][167] = -7;
		ra_start[2][168] = 0;
		ra_end[2][168] = 360;
		decl_start[2][168] = -20;
		decl_end[2][168] = -7;
		ra_start[2][169] = 0;
		ra_end[2][169] = 16;
		decl_start[2][169] = -23;
		decl_end[2][169] = -15;
		ra_start[2][170] = 8;
		ra_end[2][170] = 24;
		decl_start[2][170] = -23;
		decl_end[2][170] = -15;
		ra_start[2][171] = 16;
		ra_end[2][171] = 32;
		decl_start[2][171] = -23;
		decl_end[2][171] = -15;
		ra_start[2][172] = 28;
		ra_end[2][172] = 40;
		decl_start[2][172] = -23;
		decl_end[2][172] = -15;
		ra_start[2][173] = 36;
		ra_end[2][173] = 51;
		decl_start[2][173] = -23;
		decl_end[2][173] = -15;
		ra_start[2][174] = 48;
		ra_end[2][174] = 59;
		decl_start[2][174] = -23;
		decl_end[2][174] = -15;
		ra_start[2][175] = 56;
		ra_end[2][175] = 64;
		decl_start[2][175] = -23;
		decl_end[2][175] = -15;
		ra_start[2][176] = 58;
		ra_end[2][176] = 72;
		decl_start[2][176] = -23;
		decl_end[2][176] = -15;
		ra_start[2][177] = 64;
		ra_end[2][177] = 80;
		decl_start[2][177] = -23;
		decl_end[2][177] = -15;
		ra_start[2][178] = 72;
		ra_end[2][178] = 80;
		decl_start[2][178] = -23;
		decl_end[2][178] = -15;
		ra_start[2][179] = 74;
		ra_end[2][179] = 88;
		decl_start[2][179] = -23;
		decl_end[2][179] = -15;
		ra_start[2][180] = 80;
		ra_end[2][180] = 88;
		decl_start[2][180] = -21;
		decl_end[2][180] = -16;
		ra_start[2][181] = 80;
		ra_end[2][181] = 94;
		decl_start[2][181] = -23;
		decl_end[2][181] = -15;
		ra_start[2][182] = 88;
		ra_end[2][182] = 96;
		decl_start[2][182] = -19;
		decl_end[2][182] = -15;
		ra_start[2][183] = 88;
		ra_end[2][183] = 96;
		decl_start[2][183] = -23;
		decl_end[2][183] = -16;
		ra_start[2][184] = 90;
		ra_end[2][184] = 102;
		decl_start[2][184] = -23;
		decl_end[2][184] = -15;
		ra_start[2][185] = 96;
		ra_end[2][185] = 104;
		decl_start[2][185] = -19;
		decl_end[2][185] = -15;
		ra_start[2][186] = 96;
		ra_end[2][186] = 104;
		decl_start[2][186] = -21;
		decl_end[2][186] = -16;
		ra_start[2][187] = 96;
		ra_end[2][187] = 104;
		decl_start[2][187] = -23;
		decl_end[2][187] = -18;
		ra_start[2][188] = 102;
		ra_end[2][188] = 110;
		decl_start[2][188] = -23;
		decl_end[2][188] = -15;
		ra_start[2][189] = 104;
		ra_end[2][189] = 112;
		decl_start[2][189] = -19;
		decl_end[2][189] = -15;
		ra_start[2][190] = 106;
		ra_end[2][190] = 112;
		decl_start[2][190] = -19;
		decl_end[2][190] = -16;
		ra_start[2][191] = 104;
		ra_end[2][191] = 112;
		decl_start[2][191] = -21;
		decl_end[2][191] = -16;
		ra_start[2][192] = 106;
		ra_end[2][192] = 112;
		decl_start[2][192] = -21;
		decl_end[2][192] = -18;
		ra_start[2][193] = 104;
		ra_end[2][193] = 112;
		decl_start[2][193] = -23;
		decl_end[2][193] = -18;
		ra_start[2][194] = 106;
		ra_end[2][194] = 112;
		decl_start[2][194] = -23;
		decl_end[2][194] = -20;
		ra_start[2][195] = 110;
		ra_end[2][195] = 116;
		decl_start[2][195] = -22;
		decl_end[2][195] = -15;
		ra_start[2][196] = 114;
		ra_end[2][196] = 120;
		decl_start[2][196] = -17;
		decl_end[2][196] = -15;
		ra_start[2][197] = 112;
		ra_end[2][197] = 120;
		decl_start[2][197] = -19;
		decl_end[2][197] = -15;
		ra_start[2][198] = 112;
		ra_end[2][198] = 120;
		decl_start[2][198] = -21;
		decl_end[2][198] = -16;
		ra_start[2][199] = 112;
		ra_end[2][199] = 118;
		decl_start[2][199] = -21;
		decl_end[2][199] = -18;
		ra_start[2][200] = 112;
		ra_end[2][200] = 120;
		decl_start[2][200] = -23;
		decl_end[2][200] = -18;
		ra_start[2][201] = 112;
		ra_end[2][201] = 118;
		decl_start[2][201] = -23;
		decl_end[2][201] = -20;
		ra_start[2][202] = 116;
		ra_end[2][202] = 122;
		decl_start[2][202] = -23;
		decl_end[2][202] = -15;
		ra_start[2][203] = 120;
		ra_end[2][203] = 128;
		decl_start[2][203] = -17;
		decl_end[2][203] = -15;
		ra_start[2][204] = 120;
		ra_end[2][204] = 128;
		decl_start[2][204] = -19;
		decl_end[2][204] = -15;
		ra_start[2][205] = 120;
		ra_end[2][205] = 128;
		decl_start[2][205] = -21;
		decl_end[2][205] = -16;
		ra_start[2][206] = 120;
		ra_end[2][206] = 128;
		decl_start[2][206] = -23;
		decl_end[2][206] = -18;
		ra_start[2][207] = 120;
		ra_end[2][207] = 128;
		decl_start[2][207] = -23;
		decl_end[2][207] = -20;
		ra_start[2][208] = 126;
		ra_end[2][208] = 136;
		decl_start[2][208] = -23;
		decl_end[2][208] = -15;
		ra_start[2][209] = 128;
		ra_end[2][209] = 136;
		decl_start[2][209] = -21;
		decl_end[2][209] = -16;
		ra_start[2][210] = 128;
		ra_end[2][210] = 138;
		decl_start[2][210] = -23;
		decl_end[2][210] = -15;
		ra_start[2][211] = 136;
		ra_end[2][211] = 144;
		decl_start[2][211] = -19;
		decl_end[2][211] = -15;
		ra_start[2][212] = 136;
		ra_end[2][212] = 144;
		decl_start[2][212] = -23;
		decl_end[2][212] = -16;
		ra_start[2][213] = 142;
		ra_end[2][213] = 152;
		decl_start[2][213] = -23;
		decl_end[2][213] = -15;
		ra_start[2][214] = 144;
		ra_end[2][214] = 152;
		decl_start[2][214] = -23;
		decl_end[2][214] = -16;
		ra_start[2][215] = 148;
		ra_end[2][215] = 160;
		decl_start[2][215] = -23;
		decl_end[2][215] = -15;
		ra_start[2][216] = 152;
		ra_end[2][216] = 168;
		decl_start[2][216] = -23;
		decl_end[2][216] = -15;
		ra_start[2][217] = 160;
		ra_end[2][217] = 168;
		decl_start[2][217] = -23;
		decl_end[2][217] = -15;
		ra_start[2][218] = 168;
		ra_end[2][218] = 176;
		decl_start[2][218] = -23;
		decl_end[2][218] = -15;
		ra_start[2][219] = 168;
		ra_end[2][219] = 184;
		decl_start[2][219] = -23;
		decl_end[2][219] = -15;
		ra_start[2][220] = 176;
		ra_end[2][220] = 192;
		decl_start[2][220] = -23;
		decl_end[2][220] = -15;
		ra_start[2][221] = 184;
		ra_end[2][221] = 200;
		decl_start[2][221] = -23;
		decl_end[2][221] = -15;
		ra_start[2][222] = 192;
		ra_end[2][222] = 208;
		decl_start[2][222] = -23;
		decl_end[2][222] = -15;
		ra_start[2][223] = 200;
		ra_end[2][223] = 212;
		decl_start[2][223] = -23;
		decl_end[2][223] = -15;
		ra_start[2][224] = 208;
		ra_end[2][224] = 218;
		decl_start[2][224] = -23;
		decl_end[2][224] = -15;
		ra_start[2][225] = 216;
		ra_end[2][225] = 224;
		decl_start[2][225] = -23;
		decl_end[2][225] = -15;
		ra_start[2][226] = 218;
		ra_end[2][226] = 232;
		decl_start[2][226] = -23;
		decl_end[2][226] = -15;
		ra_start[2][227] = 224;
		ra_end[2][227] = 240;
		decl_start[2][227] = -23;
		decl_end[2][227] = -15;
		ra_start[2][228] = 232;
		ra_end[2][228] = 240;
		decl_start[2][228] = -23;
		decl_end[2][228] = -16;
		ra_start[2][229] = 236;
		ra_end[2][229] = 248;
		decl_start[2][229] = -23;
		decl_end[2][229] = -15;
		ra_start[2][230] = 240;
		ra_end[2][230] = 256;
		decl_start[2][230] = -23;
		decl_end[2][230] = -15;
		ra_start[2][231] = 248;
		ra_end[2][231] = 264;
		decl_start[2][231] = -23;
		decl_end[2][231] = -15;
		ra_start[2][232] = 256;
		ra_end[2][232] = 264;
		decl_start[2][232] = -19;
		decl_end[2][232] = -15;
		ra_start[2][233] = 256;
		ra_end[2][233] = 264;
		decl_start[2][233] = -23;
		decl_end[2][233] = -17;
		ra_start[2][234] = 260;
		ra_end[2][234] = 270;
		decl_start[2][234] = -23;
		decl_end[2][234] = -15;
		ra_start[2][235] = 264;
		ra_end[2][235] = 272;
		decl_start[2][235] = -19;
		decl_end[2][235] = -15;
		ra_start[2][236] = 264;
		ra_end[2][236] = 272;
		decl_start[2][236] = -21;
		decl_end[2][236] = -16;
		ra_start[2][237] = 264;
		ra_end[2][237] = 274;
		decl_start[2][237] = -23;
		decl_end[2][237] = -15;
		ra_start[2][238] = 272;
		ra_end[2][238] = 278;
		decl_start[2][238] = -17;
		decl_end[2][238] = -15;
		ra_start[2][239] = 272;
		ra_end[2][239] = 280;
		decl_start[2][239] = -19;
		decl_end[2][239] = -15;
		ra_start[2][240] = 274;
		ra_end[2][240] = 279;
		decl_start[2][240] = -19;
		decl_end[2][240] = -16;
		ra_start[2][241] = 272;
		ra_end[2][241] = 280;
		decl_start[2][241] = -21;
		decl_end[2][241] = -16;
		ra_start[2][242] = 272;
		ra_end[2][242] = 280;
		decl_start[2][242] = -23;
		decl_end[2][242] = -18;
		ra_start[2][243] = 272;
		ra_end[2][243] = 278;
		decl_start[2][243] = -23;
		decl_end[2][243] = -20;
		ra_start[2][244] = 276;
		ra_end[2][244] = 286;
		decl_start[2][244] = -23;
		decl_end[2][244] = -15;
		ra_start[2][245] = 280;
		ra_end[2][245] = 288;
		decl_start[2][245] = -19;
		decl_end[2][245] = -15;
		ra_start[2][246] = 280;
		ra_end[2][246] = 288;
		decl_start[2][246] = -21;
		decl_end[2][246] = -16;
		ra_start[2][247] = 280;
		ra_end[2][247] = 288;
		decl_start[2][247] = -23;
		decl_end[2][247] = -18;
		ra_start[2][248] = 280;
		ra_end[2][248] = 290;
		decl_start[2][248] = -23;
		decl_end[2][248] = -16;
		ra_start[2][249] = 288;
		ra_end[2][249] = 296;
		decl_start[2][249] = -19;
		decl_end[2][249] = -15;
		ra_start[2][250] = 288;
		ra_end[2][250] = 296;
		decl_start[2][250] = -21;
		decl_end[2][250] = -16;
		ra_start[2][251] = 288;
		ra_end[2][251] = 296;
		decl_start[2][251] = -23;
		decl_end[2][251] = -18;
		ra_start[2][252] = 288;
		ra_end[2][252] = 304;
		decl_start[2][252] = -23;
		decl_end[2][252] = -15;
		ra_start[2][253] = 296;
		ra_end[2][253] = 304;
		decl_start[2][253] = -21;
		decl_end[2][253] = -15;
		ra_start[2][254] = 296;
		ra_end[2][254] = 312;
		decl_start[2][254] = -23;
		decl_end[2][254] = -15;
		ra_start[2][255] = 304;
		ra_end[2][255] = 312;
		decl_start[2][255] = -21;
		decl_end[2][255] = -15;
		ra_start[2][256] = 304;
		ra_end[2][256] = 320;
		decl_start[2][256] = -23;
		decl_end[2][256] = -14;
		ra_start[2][257] = 312;
		ra_end[2][257] = 320;
		decl_start[2][257] = -23;
		decl_end[2][257] = -16;
		ra_start[2][258] = 316;
		ra_end[2][258] = 328;
		decl_start[2][258] = -23;
		decl_end[2][258] = -15;
		ra_start[2][259] = 320;
		ra_end[2][259] = 336;
		decl_start[2][259] = -23;
		decl_end[2][259] = -15;
		ra_start[2][260] = 328;
		ra_end[2][260] = 344;
		decl_start[2][260] = -23;
		decl_end[2][260] = -15;
		ra_start[2][261] = 336;
		ra_end[2][261] = 353;
		decl_start[2][261] = -23;
		decl_end[2][261] = -15;
		ra_start[2][262] = 344;
		ra_end[2][262] = 360;
		decl_start[2][262] = -23;
		decl_end[2][262] = -15;
		ra_start[2][263] = 0;
		ra_end[2][263] = 360;
		decl_start[2][263] = -28;
		decl_end[2][263] = -15;
		ra_start[2][264] = 0;
		ra_end[2][264] = 17;
		decl_start[2][264] = -30;
		decl_end[2][264] = -22;
		ra_start[2][265] = 8;
		ra_end[2][265] = 26;
		decl_start[2][265] = -30;
		decl_end[2][265] = -22;
		ra_start[2][266] = 21;
		ra_end[2][266] = 34;
		decl_start[2][266] = -30;
		decl_end[2][266] = -22;
		ra_start[2][267] = 29;
		ra_end[2][267] = 45;
		decl_start[2][267] = -30;
		decl_end[2][267] = -22;
		ra_start[2][268] = 41;
		ra_end[2][268] = 54;
		decl_start[2][268] = -30;
		decl_end[2][268] = -22;
		ra_start[2][269] = 50;
		ra_end[2][269] = 62;
		decl_start[2][269] = -30;
		decl_end[2][269] = -22;
		ra_start[2][270] = 58;
		ra_end[2][270] = 67;
		decl_start[2][270] = -30;
		decl_end[2][270] = -22;
		ra_start[2][271] = 61;
		ra_end[2][271] = 76;
		decl_start[2][271] = -30;
		decl_end[2][271] = -22;
		ra_start[2][272] = 66;
		ra_end[2][272] = 82;
		decl_start[2][272] = -30;
		decl_end[2][272] = -22;
		ra_start[2][273] = 75;
		ra_end[2][273] = 84;
		decl_start[2][273] = -29;
		decl_end[2][273] = -22;
		ra_start[2][274] = 75;
		ra_end[2][274] = 93;
		decl_start[2][274] = -30;
		decl_end[2][274] = -22;
		ra_start[2][275] = 83;
		ra_end[2][275] = 93;
		decl_start[2][275] = -29;
		decl_end[2][275] = -22;
		ra_start[2][276] = 83;
		ra_end[2][276] = 95;
		decl_start[2][276] = -30;
		decl_end[2][276] = -22;
		ra_start[2][277] = 92;
		ra_end[2][277] = 101;
		decl_start[2][277] = -27;
		decl_end[2][277] = -22;
		ra_start[2][278] = 92;
		ra_end[2][278] = 101;
		decl_start[2][278] = -29;
		decl_end[2][278] = -24;
		ra_start[2][279] = 92;
		ra_end[2][279] = 101;
		decl_start[2][279] = -30;
		decl_end[2][279] = -26;
		ra_start[2][280] = 96;
		ra_end[2][280] = 107;
		decl_start[2][280] = -30;
		decl_end[2][280] = -22;
		ra_start[2][281] = 104;
		ra_end[2][281] = 109;
		decl_start[2][281] = -25;
		decl_end[2][281] = -22;
		ra_start[2][282] = 100;
		ra_end[2][282] = 109;
		decl_start[2][282] = -27;
		decl_end[2][282] = -22;
		ra_start[2][283] = 100;
		ra_end[2][283] = 109;
		decl_start[2][283] = -29;
		decl_end[2][283] = -24;
		ra_start[2][284] = 102;
		ra_end[2][284] = 109;
		decl_start[2][284] = -29;
		decl_end[2][284] = -26;
		ra_start[2][285] = 100;
		ra_end[2][285] = 109;
		decl_start[2][285] = -30;
		decl_end[2][285] = -26;
		ra_start[2][286] = 104;
		ra_end[2][286] = 113;
		decl_start[2][286] = -30;
		decl_end[2][286] = -22;
		ra_start[2][287] = 110;
		ra_end[2][287] = 116;
		decl_start[2][287] = -25;
		decl_end[2][287] = -22;
		ra_start[2][288] = 108;
		ra_end[2][288] = 118;
		decl_start[2][288] = -27;
		decl_end[2][288] = -22;
		ra_start[2][289] = 108;
		ra_end[2][289] = 115;
		decl_start[2][289] = -27;
		decl_end[2][289] = -24;
		ra_start[2][290] = 113;
		ra_end[2][290] = 118;
		decl_start[2][290] = -27;
		decl_end[2][290] = -24;
		ra_start[2][291] = 108;
		ra_end[2][291] = 118;
		decl_start[2][291] = -29;
		decl_end[2][291] = -24;
		ra_start[2][292] = 111;
		ra_end[2][292] = 118;
		decl_start[2][292] = -29;
		decl_end[2][292] = -26;
		ra_start[2][293] = 108;
		ra_end[2][293] = 118;
		decl_start[2][293] = -30;
		decl_end[2][293] = -26;
		ra_start[2][294] = 110;
		ra_end[2][294] = 118;
		decl_start[2][294] = -30;
		decl_end[2][294] = -28;
		ra_start[2][295] = 115;
		ra_end[2][295] = 122;
		decl_start[2][295] = -30;
		decl_end[2][295] = -22;
		ra_start[2][296] = 118;
		ra_end[2][296] = 126;
		decl_start[2][296] = -27;
		decl_end[2][296] = -22;
		ra_start[2][297] = 117;
		ra_end[2][297] = 122;
		decl_start[2][297] = -27;
		decl_end[2][297] = -24;
		ra_start[2][298] = 119;
		ra_end[2][298] = 126;
		decl_start[2][298] = -27;
		decl_end[2][298] = -24;
		ra_start[2][299] = 117;
		ra_end[2][299] = 126;
		decl_start[2][299] = -29;
		decl_end[2][299] = -24;
		ra_start[2][300] = 119;
		ra_end[2][300] = 126;
		decl_start[2][300] = -29;
		decl_end[2][300] = -26;
		ra_start[2][301] = 117;
		ra_end[2][301] = 126;
		decl_start[2][301] = -30;
		decl_end[2][301] = -26;
		ra_start[2][302] = 119;
		ra_end[2][302] = 126;
		decl_start[2][302] = -30;
		decl_end[2][302] = -28;
		ra_start[2][303] = 123;
		ra_end[2][303] = 134;
		decl_start[2][303] = -31;
		decl_end[2][303] = -22;
		ra_start[2][304] = 125;
		ra_end[2][304] = 134;
		decl_start[2][304] = -27;
		decl_end[2][304] = -23;
		ra_start[2][305] = 125;
		ra_end[2][305] = 134;
		decl_start[2][305] = -29;
		decl_end[2][305] = -24;
		ra_start[2][306] = 125;
		ra_end[2][306] = 134;
		decl_start[2][306] = -30;
		decl_end[2][306] = -26;
		ra_start[2][307] = 127;
		ra_end[2][307] = 141;
		decl_start[2][307] = -30;
		decl_end[2][307] = -22;
		ra_start[2][308] = 133;
		ra_end[2][308] = 143;
		decl_start[2][308] = -29;
		decl_end[2][308] = -22;
		ra_start[2][309] = 133;
		ra_end[2][309] = 143;
		decl_start[2][309] = -30;
		decl_end[2][309] = -26;
		ra_start[2][310] = 138;
		ra_end[2][310] = 151;
		decl_start[2][310] = -30;
		decl_end[2][310] = -22;
		ra_start[2][311] = 142;
		ra_end[2][311] = 151;
		decl_start[2][311] = -30;
		decl_end[2][311] = -24;
		ra_start[2][312] = 144;
		ra_end[2][312] = 160;
		decl_start[2][312] = -30;
		decl_end[2][312] = -22;
		ra_start[2][313] = 150;
		ra_end[2][313] = 160;
		decl_start[2][313] = -30;
		decl_end[2][313] = -24;
		ra_start[2][314] = 150;
		ra_end[2][314] = 168;
		decl_start[2][314] = -30;
		decl_end[2][314] = -22;
		ra_start[2][315] = 159;
		ra_end[2][315] = 168;
		decl_start[2][315] = -30;
		decl_end[2][315] = -24;
		ra_start[2][316] = 165;
		ra_end[2][316] = 176;
		decl_start[2][316] = -30;
		decl_end[2][316] = -22;
		ra_start[2][317] = 167;
		ra_end[2][317] = 185;
		decl_start[2][317] = -30;
		decl_end[2][317] = -22;
		ra_start[2][318] = 175;
		ra_end[2][318] = 192;
		decl_start[2][318] = -30;
		decl_end[2][318] = -22;
		ra_start[2][319] = 184;
		ra_end[2][319] = 193;
		decl_start[2][319] = -30;
		decl_end[2][319] = -22;
		ra_start[2][320] = 190;
		ra_end[2][320] = 201;
		decl_start[2][320] = -30;
		decl_end[2][320] = -22;
		ra_start[2][321] = 192;
		ra_end[2][321] = 210;
		decl_start[2][321] = -30;
		decl_end[2][321] = -22;
		ra_start[2][322] = 200;
		ra_end[2][322] = 212;
		decl_start[2][322] = -30;
		decl_end[2][322] = -22;
		ra_start[2][323] = 209;
		ra_end[2][323] = 218;
		decl_start[2][323] = -29;
		decl_end[2][323] = -22;
		ra_start[2][324] = 209;
		ra_end[2][324] = 227;
		decl_start[2][324] = -30;
		decl_end[2][324] = -22;
		ra_start[2][325] = 217;
		ra_end[2][325] = 227;
		decl_start[2][325] = -30;
		decl_end[2][325] = -24;
		ra_start[2][326] = 223;
		ra_end[2][326] = 235;
		decl_start[2][326] = -30;
		decl_end[2][326] = -22;
		ra_start[2][327] = 226;
		ra_end[2][327] = 243;
		decl_start[2][327] = -30;
		decl_end[2][327] = -22;
		ra_start[2][328] = 234;
		ra_end[2][328] = 243;
		decl_start[2][328] = -30;
		decl_end[2][328] = -24;
		ra_start[2][329] = 234;
		ra_end[2][329] = 252;
		decl_start[2][329] = -30;
		decl_end[2][329] = -22;
		ra_start[2][330] = 242;
		ra_end[2][330] = 254;
		decl_start[2][330] = -30;
		decl_end[2][330] = -22;
		ra_start[2][331] = 251;
		ra_end[2][331] = 260;
		decl_start[2][331] = -27;
		decl_end[2][331] = -22;
		ra_start[2][332] = 251;
		ra_end[2][332] = 260;
		decl_start[2][332] = -29;
		decl_end[2][332] = -24;
		ra_start[2][333] = 251;
		ra_end[2][333] = 260;
		decl_start[2][333] = -30;
		decl_end[2][333] = -26;
		ra_start[2][334] = 255;
		ra_end[2][334] = 268;
		decl_start[2][334] = -30;
		decl_end[2][334] = -22;
		ra_start[2][335] = 259;
		ra_end[2][335] = 268;
		decl_start[2][335] = -29;
		decl_end[2][335] = -24;
		ra_start[2][336] = 259;
		ra_end[2][336] = 268;
		decl_start[2][336] = -30;
		decl_end[2][336] = -26;
		ra_start[2][337] = 265;
		ra_end[2][337] = 275;
		decl_start[2][337] = -30;
		decl_end[2][337] = -22;
		ra_start[2][338] = 267;
		ra_end[2][338] = 277;
		decl_start[2][338] = -27;
		decl_end[2][338] = -22;
		ra_start[2][339] = 267;
		ra_end[2][339] = 277;
		decl_start[2][339] = -29;
		decl_end[2][339] = -24;
		ra_start[2][340] = 267;
		ra_end[2][340] = 273;
		decl_start[2][340] = -29;
		decl_end[2][340] = -26;
		ra_start[2][341] = 270;
		ra_end[2][341] = 275;
		decl_start[2][341] = -29;
		decl_end[2][341] = -26;
		ra_start[2][342] = 267;
		ra_end[2][342] = 277;
		decl_start[2][342] = -30;
		decl_end[2][342] = -26;
		ra_start[2][343] = 267;
		ra_end[2][343] = 273;
		decl_start[2][343] = -30;
		decl_end[2][343] = -28;
		ra_start[2][344] = 270;
		ra_end[2][344] = 275;
		decl_start[2][344] = -30;
		decl_end[2][344] = -28;
		ra_start[2][345] = 272;
		ra_end[2][345] = 281;
		decl_start[2][345] = -30;
		decl_end[2][345] = -22;
		ra_start[2][346] = 276;
		ra_end[2][346] = 285;
		decl_start[2][346] = -27;
		decl_end[2][346] = -22;
		ra_start[2][347] = 276;
		ra_end[2][347] = 285;
		decl_start[2][347] = -29;
		decl_end[2][347] = -24;
		ra_start[2][348] = 276;
		ra_end[2][348] = 285;
		decl_start[2][348] = -30;
		decl_end[2][348] = -26;
		ra_start[2][349] = 276;
		ra_end[2][349] = 287;
		decl_start[2][349] = -30;
		decl_end[2][349] = -22;
		ra_start[2][350] = 284;
		ra_end[2][350] = 294;
		decl_start[2][350] = -27;
		decl_end[2][350] = -22;
		ra_start[2][351] = 284;
		ra_end[2][351] = 294;
		decl_start[2][351] = -29;
		decl_end[2][351] = -24;
		ra_start[2][352] = 284;
		ra_end[2][352] = 294;
		decl_start[2][352] = -30;
		decl_end[2][352] = -26;
		ra_start[2][353] = 284;
		ra_end[2][353] = 300;
		decl_start[2][353] = -30;
		decl_end[2][353] = -22;
		ra_start[2][354] = 293;
		ra_end[2][354] = 302;
		decl_start[2][354] = -29;
		decl_end[2][354] = -22;
		ra_start[2][355] = 293;
		ra_end[2][355] = 304;
		decl_start[2][355] = -30;
		decl_end[2][355] = -22;
		ra_start[2][356] = 301;
		ra_end[2][356] = 310;
		decl_start[2][356] = -29;
		decl_end[2][356] = -22;
		ra_start[2][357] = 301;
		ra_end[2][357] = 314;
		decl_start[2][357] = -30;
		decl_end[2][357] = -22;
		ra_start[2][358] = 309;
		ra_end[2][358] = 319;
		decl_start[2][358] = -29;
		decl_end[2][358] = -22;
		ra_start[2][359] = 309;
		ra_end[2][359] = 327;
		decl_start[2][359] = -31;
		decl_end[2][359] = -22;
		ra_start[2][360] = 318;
		ra_end[2][360] = 333;
		decl_start[2][360] = -30;
		decl_end[2][360] = -22;
		ra_start[2][361] = 326;
		ra_end[2][361] = 335;
		decl_start[2][361] = -30;
		decl_end[2][361] = -22;
		ra_start[2][362] = 332;
		ra_end[2][362] = 344;
		decl_start[2][362] = -30;
		decl_end[2][362] = -22;
		ra_start[2][363] = 337;
		ra_end[2][363] = 352;
		decl_start[2][363] = -30;
		decl_end[2][363] = -22;
		ra_start[2][364] = 346;
		ra_end[2][364] = 360;
		decl_start[2][364] = -30;
		decl_end[2][364] = -22;
		ra_start[2][365] = 0;
		ra_end[2][365] = 360;
		decl_start[2][365] = -38;
		decl_end[2][365] = -27;
		ra_start[2][366] = 9;
		ra_end[2][366] = 23;
		decl_start[2][366] = -38;
		decl_end[2][366] = -30;
		ra_start[2][367] = 18;
		ra_end[2][367] = 36;
		decl_start[2][367] = -38;
		decl_end[2][367] = -30;
		ra_start[2][368] = 27;
		ra_end[2][368] = 45;
		decl_start[2][368] = -38;
		decl_end[2][368] = -30;
		ra_start[2][369] = 36;
		ra_end[2][369] = 54;
		decl_start[2][369] = -38;
		decl_end[2][369] = -30;
		ra_start[2][370] = 45;
		ra_end[2][370] = 63;
		decl_start[2][370] = -38;
		decl_end[2][370] = -30;
		ra_start[2][371] = 54;
		ra_end[2][371] = 72;
		decl_start[2][371] = -38;
		decl_end[2][371] = -30;
		ra_start[2][372] = 63;
		ra_end[2][372] = 75;
		decl_start[2][372] = -38;
		decl_end[2][372] = -30;
		ra_start[2][373] = 72;
		ra_end[2][373] = 81;
		decl_start[2][373] = -38;
		decl_end[2][373] = -30;
		ra_start[2][374] = 72;
		ra_end[2][374] = 90;
		decl_start[2][374] = -38;
		decl_end[2][374] = -30;
		ra_start[2][375] = 81;
		ra_end[2][375] = 91;
		decl_start[2][375] = -38;
		decl_end[2][375] = -31;
		ra_start[2][376] = 83;
		ra_end[2][376] = 99;
		decl_start[2][376] = -38;
		decl_end[2][376] = -30;
		ra_start[2][377] = 90;
		ra_end[2][377] = 99;
		decl_start[2][377] = -36;
		decl_end[2][377] = -30;
		ra_start[2][378] = 90;
		ra_end[2][378] = 102;
		decl_start[2][378] = -38;
		decl_end[2][378] = -30;
		ra_start[2][379] = 99;
		ra_end[2][379] = 108;
		decl_start[2][379] = -34;
		decl_end[2][379] = -30;
		ra_start[2][380] = 99;
		ra_end[2][380] = 108;
		decl_start[2][380] = -36;
		decl_end[2][380] = -31;
		ra_start[2][381] = 99;
		ra_end[2][381] = 108;
		decl_start[2][381] = -38;
		decl_end[2][381] = -33;
		ra_start[2][382] = 105;
		ra_end[2][382] = 113;
		decl_start[2][382] = -38;
		decl_end[2][382] = -30;
		ra_start[2][383] = 110;
		ra_end[2][383] = 117;
		decl_start[2][383] = -32;
		decl_end[2][383] = -30;
		ra_start[2][384] = 108;
		ra_end[2][384] = 117;
		decl_start[2][384] = -34;
		decl_end[2][384] = -30;
		ra_start[2][385] = 108;
		ra_end[2][385] = 117;
		decl_start[2][385] = -36;
		decl_end[2][385] = -31;
		ra_start[2][386] = 108;
		ra_end[2][386] = 117;
		decl_start[2][386] = -38;
		decl_end[2][386] = -33;
		ra_start[2][387] = 110;
		ra_end[2][387] = 120;
		decl_start[2][387] = -38;
		decl_end[2][387] = -30;
		ra_start[2][388] = 118;
		ra_end[2][388] = 126;
		decl_start[2][388] = -32;
		decl_end[2][388] = -30;
		ra_start[2][389] = 117;
		ra_end[2][389] = 126;
		decl_start[2][389] = -34;
		decl_end[2][389] = -30;
		ra_start[2][390] = 119;
		ra_end[2][390] = 126;
		decl_start[2][390] = -34;
		decl_end[2][390] = -31;
		ra_start[2][391] = 117;
		ra_end[2][391] = 126;
		decl_start[2][391] = -36;
		decl_end[2][391] = -31;
		ra_start[2][392] = 117;
		ra_end[2][392] = 126;
		decl_start[2][392] = -38;
		decl_end[2][392] = -33;
		ra_start[2][393] = 119;
		ra_end[2][393] = 129;
		decl_start[2][393] = -38;
		decl_end[2][393] = -30;
		ra_start[2][394] = 126;
		ra_end[2][394] = 133;
		decl_start[2][394] = -32;
		decl_end[2][394] = -30;
		ra_start[2][395] = 126;
		ra_end[2][395] = 135;
		decl_start[2][395] = -34;
		decl_end[2][395] = -30;
		ra_start[2][396] = 126;
		ra_end[2][396] = 135;
		decl_start[2][396] = -36;
		decl_end[2][396] = -31;
		ra_start[2][397] = 126;
		ra_end[2][397] = 135;
		decl_start[2][397] = -38;
		decl_end[2][397] = -33;
		ra_start[2][398] = 128;
		ra_end[2][398] = 140;
		decl_start[2][398] = -38;
		decl_end[2][398] = -30;
		ra_start[2][399] = 135;
		ra_end[2][399] = 144;
		decl_start[2][399] = -34;
		decl_end[2][399] = -30;
		ra_start[2][400] = 135;
		ra_end[2][400] = 144;
		decl_start[2][400] = -36;
		decl_end[2][400] = -31;
		ra_start[2][401] = 135;
		ra_end[2][401] = 144;
		decl_start[2][401] = -38;
		decl_end[2][401] = -33;
		ra_start[2][402] = 141;
		ra_end[2][402] = 153;
		decl_start[2][402] = -38;
		decl_end[2][402] = -30;
		ra_start[2][403] = 144;
		ra_end[2][403] = 153;
		decl_start[2][403] = -36;
		decl_end[2][403] = -31;
		ra_start[2][404] = 144;
		ra_end[2][404] = 153;
		decl_start[2][404] = -38;
		decl_end[2][404] = -33;
		ra_start[2][405] = 148;
		ra_end[2][405] = 162;
		decl_start[2][405] = -38;
		decl_end[2][405] = -30;
		ra_start[2][406] = 153;
		ra_end[2][406] = 162;
		decl_start[2][406] = -36;
		decl_end[2][406] = -31;
		ra_start[2][407] = 153;
		ra_end[2][407] = 169;
		decl_start[2][407] = -38;
		decl_end[2][407] = -30;
		ra_start[2][408] = 162;
		ra_end[2][408] = 171;
		decl_start[2][408] = -36;
		decl_end[2][408] = -30;
		ra_start[2][409] = 162;
		ra_end[2][409] = 180;
		decl_start[2][409] = -38;
		decl_end[2][409] = -30;
		ra_start[2][410] = 171;
		ra_end[2][410] = 180;
		decl_start[2][410] = -38;
		decl_end[2][410] = -30;
		ra_start[2][411] = 171;
		ra_end[2][411] = 189;
		decl_start[2][411] = -38;
		decl_end[2][411] = -30;
		ra_start[2][412] = 180;
		ra_end[2][412] = 189;
		decl_start[2][412] = -38;
		decl_end[2][412] = -31;
		ra_start[2][413] = 184;
		ra_end[2][413] = 198;
		decl_start[2][413] = -38;
		decl_end[2][413] = -30;
		ra_start[2][414] = 189;
		ra_end[2][414] = 203;
		decl_start[2][414] = -38;
		decl_end[2][414] = -30;
		ra_start[2][415] = 198;
		ra_end[2][415] = 207;
		decl_start[2][415] = -34;
		decl_end[2][415] = -30;
		ra_start[2][416] = 198;
		ra_end[2][416] = 207;
		decl_start[2][416] = -38;
		decl_end[2][416] = -31;
		ra_start[2][417] = 204;
		ra_end[2][417] = 216;
		decl_start[2][417] = -38;
		decl_end[2][417] = -30;
		ra_start[2][418] = 207;
		ra_end[2][418] = 216;
		decl_start[2][418] = -38;
		decl_end[2][418] = -31;
		ra_start[2][419] = 209;
		ra_end[2][419] = 225;
		decl_start[2][419] = -38;
		decl_end[2][419] = -30;
		ra_start[2][420] = 216;
		ra_end[2][420] = 225;
		decl_start[2][420] = -36;
		decl_end[2][420] = -31;
		ra_start[2][421] = 216;
		ra_end[2][421] = 234;
		decl_start[2][421] = -38;
		decl_end[2][421] = -30;
		ra_start[2][422] = 225;
		ra_end[2][422] = 234;
		decl_start[2][422] = -38;
		decl_end[2][422] = -30;
		ra_start[2][423] = 225;
		ra_end[2][423] = 243;
		decl_start[2][423] = -38;
		decl_end[2][423] = -30;
		ra_start[2][424] = 234;
		ra_end[2][424] = 243;
		decl_start[2][424] = -38;
		decl_end[2][424] = -30;
		ra_start[2][425] = 234;
		ra_end[2][425] = 252;
		decl_start[2][425] = -38;
		decl_end[2][425] = -30;
		ra_start[2][426] = 243;
		ra_end[2][426] = 252;
		decl_start[2][426] = -36;
		decl_end[2][426] = -30;
		ra_start[2][427] = 243;
		ra_end[2][427] = 255;
		decl_start[2][427] = -38;
		decl_end[2][427] = -30;
		ra_start[2][428] = 252;
		ra_end[2][428] = 261;
		decl_start[2][428] = -34;
		decl_end[2][428] = -30;
		ra_start[2][429] = 252;
		ra_end[2][429] = 261;
		decl_start[2][429] = -36;
		decl_end[2][429] = -31;
		ra_start[2][430] = 252;
		ra_end[2][430] = 261;
		decl_start[2][430] = -38;
		decl_end[2][430] = -33;
		ra_start[2][431] = 254;
		ra_end[2][431] = 266;
		decl_start[2][431] = -38;
		decl_end[2][431] = -30;
		ra_start[2][432] = 263;
		ra_end[2][432] = 270;
		decl_start[2][432] = -32;
		decl_end[2][432] = -30;
		ra_start[2][433] = 261;
		ra_end[2][433] = 270;
		decl_start[2][433] = -34;
		decl_end[2][433] = -30;
		ra_start[2][434] = 261;
		ra_end[2][434] = 271;
		decl_start[2][434] = -36;
		decl_end[2][434] = -31;
		ra_start[2][435] = 261;
		ra_end[2][435] = 270;
		decl_start[2][435] = -38;
		decl_end[2][435] = -33;
		ra_start[2][436] = 261;
		ra_end[2][436] = 270;
		decl_start[2][436] = -38;
		decl_end[2][436] = -35;
		ra_start[2][437] = 267;
		ra_end[2][437] = 275;
		decl_start[2][437] = -38;
		decl_end[2][437] = -30;
		ra_start[2][438] = 272;
		ra_end[2][438] = 279;
		decl_start[2][438] = -32;
		decl_end[2][438] = -30;
		ra_start[2][439] = 270;
		ra_end[2][439] = 279;
		decl_start[2][439] = -34;
		decl_end[2][439] = -30;
		ra_start[2][440] = 270;
		ra_end[2][440] = 279;
		decl_start[2][440] = -36;
		decl_end[2][440] = -31;
		ra_start[2][441] = 270;
		ra_end[2][441] = 279;
		decl_start[2][441] = -36;
		decl_end[2][441] = -33;
		ra_start[2][442] = 270;
		ra_end[2][442] = 279;
		decl_start[2][442] = -38;
		decl_end[2][442] = -33;
		ra_start[2][443] = 274;
		ra_end[2][443] = 284;
		decl_start[2][443] = -38;
		decl_end[2][443] = -30;
		ra_start[2][444] = 279;
		ra_end[2][444] = 288;
		decl_start[2][444] = -34;
		decl_end[2][444] = -30;
		ra_start[2][445] = 279;
		ra_end[2][445] = 288;
		decl_start[2][445] = -36;
		decl_end[2][445] = -31;
		ra_start[2][446] = 279;
		ra_end[2][446] = 288;
		decl_start[2][446] = -38;
		decl_end[2][446] = -33;
		ra_start[2][447] = 279;
		ra_end[2][447] = 293;
		decl_start[2][447] = -38;
		decl_end[2][447] = -30;
		ra_start[2][448] = 288;
		ra_end[2][448] = 297;
		decl_start[2][448] = -34;
		decl_end[2][448] = -30;
		ra_start[2][449] = 288;
		ra_end[2][449] = 297;
		decl_start[2][449] = -38;
		decl_end[2][449] = -31;
		ra_start[2][450] = 294;
		ra_end[2][450] = 306;
		decl_start[2][450] = -38;
		decl_end[2][450] = -30;
		ra_start[2][451] = 297;
		ra_end[2][451] = 309;
		decl_start[2][451] = -38;
		decl_end[2][451] = -30;
		ra_start[2][452] = 306;
		ra_end[2][452] = 315;
		decl_start[2][452] = -36;
		decl_end[2][452] = -30;
		ra_start[2][453] = 306;
		ra_end[2][453] = 324;
		decl_start[2][453] = -38;
		decl_end[2][453] = -30;
		ra_start[2][454] = 315;
		ra_end[2][454] = 324;
		decl_start[2][454] = -38;
		decl_end[2][454] = -30;
		ra_start[2][455] = 317;
		ra_end[2][455] = 333;
		decl_start[2][455] = -38;
		decl_end[2][455] = -30;
		ra_start[2][456] = 324;
		ra_end[2][456] = 342;
		decl_start[2][456] = -38;
		decl_end[2][456] = -30;
		ra_start[2][457] = 333;
		ra_end[2][457] = 351;
		decl_start[2][457] = -38;
		decl_end[2][457] = -30;
		ra_start[2][458] = 342;
		ra_end[2][458] = 360;
		decl_start[2][458] = -38;
		decl_end[2][458] = -30;
		ra_start[2][459] = 351;
		ra_end[2][459] = 360;
		decl_start[2][459] = -38;
		decl_end[2][459] = -30;
	}

	/**
	 * Initializes the R.A. and Decl. of each blocks of tic4.dat.
	 */
	private static void initialize_Tic4 ( ) {
		ra_start[3][0] = 0;
		ra_end[3][0] = 14;
		decl_start[3][0] = -45;
		decl_end[3][0] = -37;
		ra_start[3][1] = 10;
		ra_end[3][1] = 27;
		decl_start[3][1] = -45;
		decl_end[3][1] = -37;
		ra_start[3][2] = 20;
		ra_end[3][2] = 37;
		decl_start[3][2] = -45;
		decl_end[3][2] = -37;
		ra_start[3][3] = 30;
		ra_end[3][3] = 50;
		decl_start[3][3] = -45;
		decl_end[3][3] = -37;
		ra_start[3][4] = 40;
		ra_end[3][4] = 60;
		decl_start[3][4] = -45;
		decl_end[3][4] = -37;
		ra_start[3][5] = 50;
		ra_end[3][5] = 64;
		decl_start[3][5] = -45;
		decl_end[3][5] = -37;
		ra_start[3][6] = 60;
		ra_end[3][6] = 70;
		decl_start[3][6] = -45;
		decl_end[3][6] = -37;
		ra_start[3][7] = 60;
		ra_end[3][7] = 80;
		decl_start[3][7] = -45;
		decl_end[3][7] = -37;
		ra_start[3][8] = 70;
		ra_end[3][8] = 83;
		decl_start[3][8] = -45;
		decl_end[3][8] = -37;
		ra_start[3][9] = 80;
		ra_end[3][9] = 90;
		decl_start[3][9] = -44;
		decl_end[3][9] = -37;
		ra_start[3][10] = 80;
		ra_end[3][10] = 93;
		decl_start[3][10] = -45;
		decl_end[3][10] = -37;
		ra_start[3][11] = 90;
		ra_end[3][11] = 100;
		decl_start[3][11] = -42;
		decl_end[3][11] = -37;
		ra_start[3][12] = 90;
		ra_end[3][12] = 100;
		decl_start[3][12] = -45;
		decl_end[3][12] = -39;
		ra_start[3][13] = 95;
		ra_end[3][13] = 109;
		decl_start[3][13] = -45;
		decl_end[3][13] = -37;
		ra_start[3][14] = 100;
		ra_end[3][14] = 110;
		decl_start[3][14] = -43;
		decl_end[3][14] = -37;
		ra_start[3][15] = 100;
		ra_end[3][15] = 110;
		decl_start[3][15] = -45;
		decl_end[3][15] = -41;
		ra_start[3][16] = 102;
		ra_end[3][16] = 115;
		decl_start[3][16] = -45;
		decl_end[3][16] = -37;
		ra_start[3][17] = 110;
		ra_end[3][17] = 120;
		decl_start[3][17] = -42;
		decl_end[3][17] = -37;
		ra_start[3][18] = 110;
		ra_end[3][18] = 120;
		decl_start[3][18] = -44;
		decl_end[3][18] = -39;
		ra_start[3][19] = 110;
		ra_end[3][19] = 120;
		decl_start[3][19] = -45;
		decl_end[3][19] = -41;
		ra_start[3][20] = 115;
		ra_end[3][20] = 125;
		decl_start[3][20] = -45;
		decl_end[3][20] = -37;
		ra_start[3][21] = 120;
		ra_end[3][21] = 131;
		decl_start[3][21] = -42;
		decl_end[3][21] = -37;
		ra_start[3][22] = 120;
		ra_end[3][22] = 130;
		decl_start[3][22] = -42;
		decl_end[3][22] = -39;
		ra_start[3][23] = 120;
		ra_end[3][23] = 130;
		decl_start[3][23] = -44;
		decl_end[3][23] = -39;
		ra_start[3][24] = 120;
		ra_end[3][24] = 130;
		decl_start[3][24] = -45;
		decl_end[3][24] = -41;
		ra_start[3][25] = 122;
		ra_end[3][25] = 133;
		decl_start[3][25] = -45;
		decl_end[3][25] = -37;
		ra_start[3][26] = 130;
		ra_end[3][26] = 140;
		decl_start[3][26] = -40;
		decl_end[3][26] = -37;
		ra_start[3][27] = 130;
		ra_end[3][27] = 140;
		decl_start[3][27] = -42;
		decl_end[3][27] = -37;
		ra_start[3][28] = 130;
		ra_end[3][28] = 140;
		decl_start[3][28] = -44;
		decl_end[3][28] = -39;
		ra_start[3][29] = 130;
		ra_end[3][29] = 140;
		decl_start[3][29] = -45;
		decl_end[3][29] = -41;
		ra_start[3][30] = 132;
		ra_end[3][30] = 143;
		decl_start[3][30] = -45;
		decl_end[3][30] = -37;
		ra_start[3][31] = 140;
		ra_end[3][31] = 150;
		decl_start[3][31] = -42;
		decl_end[3][31] = -37;
		ra_start[3][32] = 140;
		ra_end[3][32] = 150;
		decl_start[3][32] = -44;
		decl_end[3][32] = -39;
		ra_start[3][33] = 140;
		ra_end[3][33] = 150;
		decl_start[3][33] = -45;
		decl_end[3][33] = -41;
		ra_start[3][34] = 142;
		ra_end[3][34] = 155;
		decl_start[3][34] = -45;
		decl_end[3][34] = -37;
		ra_start[3][35] = 150;
		ra_end[3][35] = 160;
		decl_start[3][35] = -42;
		decl_end[3][35] = -37;
		ra_start[3][36] = 150;
		ra_end[3][36] = 160;
		decl_start[3][36] = -44;
		decl_end[3][36] = -39;
		ra_start[3][37] = 150;
		ra_end[3][37] = 160;
		decl_start[3][37] = -45;
		decl_end[3][37] = -41;
		ra_start[3][38] = 152;
		ra_end[3][38] = 168;
		decl_start[3][38] = -45;
		decl_end[3][38] = -37;
		ra_start[3][39] = 160;
		ra_end[3][39] = 170;
		decl_start[3][39] = -44;
		decl_end[3][39] = -37;
		ra_start[3][40] = 160;
		ra_end[3][40] = 170;
		decl_start[3][40] = -45;
		decl_end[3][40] = -41;
		ra_start[3][41] = 167;
		ra_end[3][41] = 180;
		decl_start[3][41] = -45;
		decl_end[3][41] = -37;
		ra_start[3][42] = 170;
		ra_end[3][42] = 180;
		decl_start[3][42] = -45;
		decl_end[3][42] = -39;
		ra_start[3][43] = 170;
		ra_end[3][43] = 190;
		decl_start[3][43] = -45;
		decl_end[3][43] = -37;
		ra_start[3][44] = 180;
		ra_end[3][44] = 190;
		decl_start[3][44] = -44;
		decl_end[3][44] = -39;
		ra_start[3][45] = 180;
		ra_end[3][45] = 198;
		decl_start[3][45] = -45;
		decl_end[3][45] = -37;
		ra_start[3][46] = 190;
		ra_end[3][46] = 200;
		decl_start[3][46] = -44;
		decl_end[3][46] = -37;
		ra_start[3][47] = 190;
		ra_end[3][47] = 203;
		decl_start[3][47] = -45;
		decl_end[3][47] = -37;
		ra_start[3][48] = 200;
		ra_end[3][48] = 210;
		decl_start[3][48] = -42;
		decl_end[3][48] = -37;
		ra_start[3][49] = 200;
		ra_end[3][49] = 210;
		decl_start[3][49] = -44;
		decl_end[3][49] = -39;
		ra_start[3][50] = 200;
		ra_end[3][50] = 215;
		decl_start[3][50] = -45;
		decl_end[3][50] = -37;
		ra_start[3][51] = 210;
		ra_end[3][51] = 220;
		decl_start[3][51] = -43;
		decl_end[3][51] = -37;
		ra_start[3][52] = 210;
		ra_end[3][52] = 220;
		decl_start[3][52] = -45;
		decl_end[3][52] = -41;
		ra_start[3][53] = 212;
		ra_end[3][53] = 230;
		decl_start[3][53] = -45;
		decl_end[3][53] = -37;
		ra_start[3][54] = 220;
		ra_end[3][54] = 230;
		decl_start[3][54] = -42;
		decl_end[3][54] = -37;
		ra_start[3][55] = 220;
		ra_end[3][55] = 230;
		decl_start[3][55] = -44;
		decl_end[3][55] = -39;
		ra_start[3][56] = 220;
		ra_end[3][56] = 230;
		decl_start[3][56] = -45;
		decl_end[3][56] = -41;
		ra_start[3][57] = 227;
		ra_end[3][57] = 240;
		decl_start[3][57] = -45;
		decl_end[3][57] = -37;
		ra_start[3][58] = 230;
		ra_end[3][58] = 240;
		decl_start[3][58] = -44;
		decl_end[3][58] = -39;
		ra_start[3][59] = 230;
		ra_end[3][59] = 240;
		decl_start[3][59] = -45;
		decl_end[3][59] = -41;
		ra_start[3][60] = 230;
		ra_end[3][60] = 243;
		decl_start[3][60] = -45;
		decl_end[3][60] = -37;
		ra_start[3][61] = 240;
		ra_end[3][61] = 250;
		decl_start[3][61] = -44;
		decl_end[3][61] = -37;
		ra_start[3][62] = 240;
		ra_end[3][62] = 250;
		decl_start[3][62] = -45;
		decl_end[3][62] = -41;
		ra_start[3][63] = 247;
		ra_end[3][63] = 258;
		decl_start[3][63] = -45;
		decl_end[3][63] = -37;
		ra_start[3][64] = 250;
		ra_end[3][64] = 260;
		decl_start[3][64] = -42;
		decl_end[3][64] = -37;
		ra_start[3][65] = 250;
		ra_end[3][65] = 260;
		decl_start[3][65] = -44;
		decl_end[3][65] = -39;
		ra_start[3][66] = 251;
		ra_end[3][66] = 260;
		decl_start[3][66] = -44;
		decl_end[3][66] = -41;
		ra_start[3][67] = 250;
		ra_end[3][67] = 260;
		decl_start[3][67] = -45;
		decl_end[3][67] = -41;
		ra_start[3][68] = 255;
		ra_end[3][68] = 265;
		decl_start[3][68] = -45;
		decl_end[3][68] = -37;
		ra_start[3][69] = 262;
		ra_end[3][69] = 270;
		decl_start[3][69] = -40;
		decl_end[3][69] = -37;
		ra_start[3][70] = 260;
		ra_end[3][70] = 270;
		decl_start[3][70] = -42;
		decl_end[3][70] = -37;
		ra_start[3][71] = 263;
		ra_end[3][71] = 270;
		decl_start[3][71] = -42;
		decl_end[3][71] = -39;
		ra_start[3][72] = 260;
		ra_end[3][72] = 270;
		decl_start[3][72] = -44;
		decl_end[3][72] = -39;
		ra_start[3][73] = 262;
		ra_end[3][73] = 270;
		decl_start[3][73] = -44;
		decl_end[3][73] = -41;
		ra_start[3][74] = 260;
		ra_end[3][74] = 270;
		decl_start[3][74] = -45;
		decl_end[3][74] = -41;
		ra_start[3][75] = 262;
		ra_end[3][75] = 273;
		decl_start[3][75] = -45;
		decl_end[3][75] = -37;
		ra_start[3][76] = 270;
		ra_end[3][76] = 278;
		decl_start[3][76] = -40;
		decl_end[3][76] = -37;
		ra_start[3][77] = 270;
		ra_end[3][77] = 280;
		decl_start[3][77] = -42;
		decl_end[3][77] = -37;
		ra_start[3][78] = 270;
		ra_end[3][78] = 280;
		decl_start[3][78] = -44;
		decl_end[3][78] = -39;
		ra_start[3][79] = 270;
		ra_end[3][79] = 278;
		decl_start[3][79] = -44;
		decl_end[3][79] = -41;
		ra_start[3][80] = 270;
		ra_end[3][80] = 280;
		decl_start[3][80] = -45;
		decl_end[3][80] = -41;
		ra_start[3][81] = 272;
		ra_end[3][81] = 285;
		decl_start[3][81] = -45;
		decl_end[3][81] = -37;
		ra_start[3][82] = 280;
		ra_end[3][82] = 290;
		decl_start[3][82] = -42;
		decl_end[3][82] = -37;
		ra_start[3][83] = 280;
		ra_end[3][83] = 290;
		decl_start[3][83] = -44;
		decl_end[3][83] = -39;
		ra_start[3][84] = 280;
		ra_end[3][84] = 293;
		decl_start[3][84] = -45;
		decl_end[3][84] = -37;
		ra_start[3][85] = 290;
		ra_end[3][85] = 300;
		decl_start[3][85] = -44;
		decl_end[3][85] = -37;
		ra_start[3][86] = 290;
		ra_end[3][86] = 303;
		decl_start[3][86] = -45;
		decl_end[3][86] = -37;
		ra_start[3][87] = 300;
		ra_end[3][87] = 310;
		decl_start[3][87] = -43;
		decl_end[3][87] = -37;
		ra_start[3][88] = 300;
		ra_end[3][88] = 313;
		decl_start[3][88] = -45;
		decl_end[3][88] = -37;
		ra_start[3][89] = 310;
		ra_end[3][89] = 320;
		decl_start[3][89] = -44;
		decl_end[3][89] = -37;
		ra_start[3][90] = 310;
		ra_end[3][90] = 330;
		decl_start[3][90] = -45;
		decl_end[3][90] = -37;
		ra_start[3][91] = 320;
		ra_end[3][91] = 336;
		decl_start[3][91] = -45;
		decl_end[3][91] = -37;
		ra_start[3][92] = 330;
		ra_end[3][92] = 343;
		decl_start[3][92] = -45;
		decl_end[3][92] = -37;
		ra_start[3][93] = 340;
		ra_end[3][93] = 350;
		decl_start[3][93] = -45;
		decl_end[3][93] = -37;
		ra_start[3][94] = 346;
		ra_end[3][94] = 360;
		decl_start[3][94] = -45;
		decl_end[3][94] = -37;
		ra_start[3][95] = 0;
		ra_end[3][95] = 360;
		decl_start[3][95] = -53;
		decl_end[3][95] = -42;
		ra_start[3][96] = 3;
		ra_end[3][96] = 23;
		decl_start[3][96] = -53;
		decl_end[3][96] = -45;
		ra_start[3][97] = 15;
		ra_end[3][97] = 34;
		decl_start[3][97] = -53;
		decl_end[3][97] = -45;
		ra_start[3][98] = 26;
		ra_end[3][98] = 45;
		decl_start[3][98] = -53;
		decl_end[3][98] = -45;
		ra_start[3][99] = 40;
		ra_end[3][99] = 57;
		decl_start[3][99] = -53;
		decl_end[3][99] = -45;
		ra_start[3][100] = 45;
		ra_end[3][100] = 68;
		decl_start[3][100] = -53;
		decl_end[3][100] = -45;
		ra_start[3][101] = 56;
		ra_end[3][101] = 79;
		decl_start[3][101] = -53;
		decl_end[3][101] = -45;
		ra_start[3][102] = 67;
		ra_end[3][102] = 79;
		decl_start[3][102] = -53;
		decl_end[3][102] = -45;
		ra_start[3][103] = 75;
		ra_end[3][103] = 90;
		decl_start[3][103] = -53;
		decl_end[3][103] = -45;
		ra_start[3][104] = 78;
		ra_end[3][104] = 96;
		decl_start[3][104] = -53;
		decl_end[3][104] = -45;
		ra_start[3][105] = 90;
		ra_end[3][105] = 102;
		decl_start[3][105] = -49;
		decl_end[3][105] = -45;
		ra_start[3][106] = 90;
		ra_end[3][106] = 102;
		decl_start[3][106] = -53;
		decl_end[3][106] = -46;
		ra_start[3][107] = 95;
		ra_end[3][107] = 113;
		decl_start[3][107] = -53;
		decl_end[3][107] = -45;
		ra_start[3][108] = 101;
		ra_end[3][108] = 113;
		decl_start[3][108] = -49;
		decl_end[3][108] = -45;
		ra_start[3][109] = 101;
		ra_end[3][109] = 113;
		decl_start[3][109] = -53;
		decl_end[3][109] = -46;
		ra_start[3][110] = 101;
		ra_end[3][110] = 119;
		decl_start[3][110] = -53;
		decl_end[3][110] = -45;
		ra_start[3][111] = 112;
		ra_end[3][111] = 124;
		decl_start[3][111] = -49;
		decl_end[3][111] = -45;
		ra_start[3][112] = 112;
		ra_end[3][112] = 124;
		decl_start[3][112] = -49;
		decl_end[3][112] = -46;
		ra_start[3][113] = 112;
		ra_end[3][113] = 124;
		decl_start[3][113] = -51;
		decl_end[3][113] = -46;
		ra_start[3][114] = 112;
		ra_end[3][114] = 127;
		decl_start[3][114] = -53;
		decl_end[3][114] = -45;
		ra_start[3][115] = 123;
		ra_end[3][115] = 133;
		decl_start[3][115] = -47;
		decl_end[3][115] = -45;
		ra_start[3][116] = 123;
		ra_end[3][116] = 135;
		decl_start[3][116] = -49;
		decl_end[3][116] = -45;
		ra_start[3][117] = 126;
		ra_end[3][117] = 133;
		decl_start[3][117] = -49;
		decl_end[3][117] = -46;
		ra_start[3][118] = 123;
		ra_end[3][118] = 135;
		decl_start[3][118] = -51;
		decl_end[3][118] = -46;
		ra_start[3][119] = 123;
		ra_end[3][119] = 135;
		decl_start[3][119] = -53;
		decl_end[3][119] = -48;
		ra_start[3][120] = 129;
		ra_end[3][120] = 141;
		decl_start[3][120] = -53;
		decl_end[3][120] = -45;
		ra_start[3][121] = 135;
		ra_end[3][121] = 147;
		decl_start[3][121] = -49;
		decl_end[3][121] = -45;
		ra_start[3][122] = 135;
		ra_end[3][122] = 147;
		decl_start[3][122] = -51;
		decl_end[3][122] = -46;
		ra_start[3][123] = 135;
		ra_end[3][123] = 147;
		decl_start[3][123] = -53;
		decl_end[3][123] = -48;
		ra_start[3][124] = 137;
		ra_end[3][124] = 150;
		decl_start[3][124] = -53;
		decl_end[3][124] = -45;
		ra_start[3][125] = 146;
		ra_end[3][125] = 158;
		decl_start[3][125] = -49;
		decl_end[3][125] = -45;
		ra_start[3][126] = 146;
		ra_end[3][126] = 158;
		decl_start[3][126] = -49;
		decl_end[3][126] = -46;
		ra_start[3][127] = 146;
		ra_end[3][127] = 158;
		decl_start[3][127] = -51;
		decl_end[3][127] = -46;
		ra_start[3][128] = 146;
		ra_end[3][128] = 158;
		decl_start[3][128] = -53;
		decl_end[3][128] = -48;
		ra_start[3][129] = 151;
		ra_end[3][129] = 164;
		decl_start[3][129] = -53;
		decl_end[3][129] = -45;
		ra_start[3][130] = 157;
		ra_end[3][130] = 169;
		decl_start[3][130] = -49;
		decl_end[3][130] = -45;
		ra_start[3][131] = 157;
		ra_end[3][131] = 169;
		decl_start[3][131] = -51;
		decl_end[3][131] = -46;
		ra_start[3][132] = 157;
		ra_end[3][132] = 169;
		decl_start[3][132] = -53;
		decl_end[3][132] = -48;
		ra_start[3][133] = 163;
		ra_end[3][133] = 178;
		decl_start[3][133] = -53;
		decl_end[3][133] = -45;
		ra_start[3][134] = 168;
		ra_end[3][134] = 180;
		decl_start[3][134] = -49;
		decl_end[3][134] = -45;
		ra_start[3][135] = 168;
		ra_end[3][135] = 180;
		decl_start[3][135] = -51;
		decl_end[3][135] = -46;
		ra_start[3][136] = 168;
		ra_end[3][136] = 180;
		decl_start[3][136] = -53;
		decl_end[3][136] = -48;
		ra_start[3][137] = 174;
		ra_end[3][137] = 192;
		decl_start[3][137] = -53;
		decl_end[3][137] = -44;
		ra_start[3][138] = 180;
		ra_end[3][138] = 192;
		decl_start[3][138] = -51;
		decl_end[3][138] = -45;
		ra_start[3][139] = 180;
		ra_end[3][139] = 192;
		decl_start[3][139] = -53;
		decl_end[3][139] = -48;
		ra_start[3][140] = 180;
		ra_end[3][140] = 195;
		decl_start[3][140] = -53;
		decl_end[3][140] = -45;
		ra_start[3][141] = 191;
		ra_end[3][141] = 203;
		decl_start[3][141] = -49;
		decl_end[3][141] = -45;
		ra_start[3][142] = 191;
		ra_end[3][142] = 203;
		decl_start[3][142] = -51;
		decl_end[3][142] = -46;
		ra_start[3][143] = 191;
		ra_end[3][143] = 203;
		decl_start[3][143] = -53;
		decl_end[3][143] = -48;
		ra_start[3][144] = 194;
		ra_end[3][144] = 211;
		decl_start[3][144] = -53;
		decl_end[3][144] = -45;
		ra_start[3][145] = 202;
		ra_end[3][145] = 214;
		decl_start[3][145] = -49;
		decl_end[3][145] = -45;
		ra_start[3][146] = 202;
		ra_end[3][146] = 214;
		decl_start[3][146] = -51;
		decl_end[3][146] = -46;
		ra_start[3][147] = 202;
		ra_end[3][147] = 216;
		decl_start[3][147] = -53;
		decl_end[3][147] = -45;
		ra_start[3][148] = 213;
		ra_end[3][148] = 225;
		decl_start[3][148] = -49;
		decl_end[3][148] = -45;
		ra_start[3][149] = 213;
		ra_end[3][149] = 225;
		decl_start[3][149] = -51;
		decl_end[3][149] = -46;
		ra_start[3][150] = 213;
		ra_end[3][150] = 225;
		decl_start[3][150] = -53;
		decl_end[3][150] = -48;
		ra_start[3][151] = 213;
		ra_end[3][151] = 228;
		decl_start[3][151] = -53;
		decl_end[3][151] = -45;
		ra_start[3][152] = 225;
		ra_end[3][152] = 237;
		decl_start[3][152] = -47;
		decl_end[3][152] = -45;
		ra_start[3][153] = 225;
		ra_end[3][153] = 237;
		decl_start[3][153] = -49;
		decl_end[3][153] = -45;
		ra_start[3][154] = 225;
		ra_end[3][154] = 237;
		decl_start[3][154] = -51;
		decl_end[3][154] = -46;
		ra_start[3][155] = 225;
		ra_end[3][155] = 237;
		decl_start[3][155] = -53;
		decl_end[3][155] = -48;
		ra_start[3][156] = 225;
		ra_end[3][156] = 234;
		decl_start[3][156] = -53;
		decl_end[3][156] = -50;
		ra_start[3][157] = 230;
		ra_end[3][157] = 245;
		decl_start[3][157] = -53;
		decl_end[3][157] = -45;
		ra_start[3][158] = 236;
		ra_end[3][158] = 248;
		decl_start[3][158] = -49;
		decl_end[3][158] = -45;
		ra_start[3][159] = 236;
		ra_end[3][159] = 248;
		decl_start[3][159] = -51;
		decl_end[3][159] = -46;
		ra_start[3][160] = 236;
		ra_end[3][160] = 248;
		decl_start[3][160] = -53;
		decl_end[3][160] = -48;
		ra_start[3][161] = 239;
		ra_end[3][161] = 248;
		decl_start[3][161] = -53;
		decl_end[3][161] = -50;
		ra_start[3][162] = 244;
		ra_end[3][162] = 254;
		decl_start[3][162] = -53;
		decl_end[3][162] = -45;
		ra_start[3][163] = 247;
		ra_end[3][163] = 259;
		decl_start[3][163] = -49;
		decl_end[3][163] = -45;
		ra_start[3][164] = 247;
		ra_end[3][164] = 259;
		decl_start[3][164] = -49;
		decl_end[3][164] = -46;
		ra_start[3][165] = 247;
		ra_end[3][165] = 259;
		decl_start[3][165] = -51;
		decl_end[3][165] = -46;
		ra_start[3][166] = 247;
		ra_end[3][166] = 259;
		decl_start[3][166] = -53;
		decl_end[3][166] = -48;
		ra_start[3][167] = 253;
		ra_end[3][167] = 265;
		decl_start[3][167] = -53;
		decl_end[3][167] = -45;
		ra_start[3][168] = 261;
		ra_end[3][168] = 270;
		decl_start[3][168] = -47;
		decl_end[3][168] = -45;
		ra_start[3][169] = 258;
		ra_end[3][169] = 270;
		decl_start[3][169] = -49;
		decl_end[3][169] = -45;
		ra_start[3][170] = 258;
		ra_end[3][170] = 270;
		decl_start[3][170] = -51;
		decl_end[3][170] = -46;
		ra_start[3][171] = 258;
		ra_end[3][171] = 268;
		decl_start[3][171] = -51;
		decl_end[3][171] = -48;
		ra_start[3][172] = 258;
		ra_end[3][172] = 270;
		decl_start[3][172] = -53;
		decl_end[3][172] = -48;
		ra_start[3][173] = 261;
		ra_end[3][173] = 273;
		decl_start[3][173] = -53;
		decl_end[3][173] = -45;
		ra_start[3][174] = 270;
		ra_end[3][174] = 282;
		decl_start[3][174] = -47;
		decl_end[3][174] = -45;
		ra_start[3][175] = 270;
		ra_end[3][175] = 282;
		decl_start[3][175] = -49;
		decl_end[3][175] = -45;
		ra_start[3][176] = 270;
		ra_end[3][176] = 282;
		decl_start[3][176] = -51;
		decl_end[3][176] = -46;
		ra_start[3][177] = 270;
		ra_end[3][177] = 282;
		decl_start[3][177] = -53;
		decl_end[3][177] = -48;
		ra_start[3][178] = 272;
		ra_end[3][178] = 285;
		decl_start[3][178] = -53;
		decl_end[3][178] = -45;
		ra_start[3][179] = 281;
		ra_end[3][179] = 293;
		decl_start[3][179] = -49;
		decl_end[3][179] = -45;
		ra_start[3][180] = 281;
		ra_end[3][180] = 293;
		decl_start[3][180] = -51;
		decl_end[3][180] = -46;
		ra_start[3][181] = 281;
		ra_end[3][181] = 299;
		decl_start[3][181] = -53;
		decl_end[3][181] = -45;
		ra_start[3][182] = 292;
		ra_end[3][182] = 304;
		decl_start[3][182] = -49;
		decl_end[3][182] = -45;
		ra_start[3][183] = 292;
		ra_end[3][183] = 304;
		decl_start[3][183] = -53;
		decl_end[3][183] = -47;
		ra_start[3][184] = 298;
		ra_end[3][184] = 315;
		decl_start[3][184] = -53;
		decl_end[3][184] = -45;
		ra_start[3][185] = 303;
		ra_end[3][185] = 315;
		decl_start[3][185] = -53;
		decl_end[3][185] = -46;
		ra_start[3][186] = 309;
		ra_end[3][186] = 327;
		decl_start[3][186] = -53;
		decl_end[3][186] = -45;
		ra_start[3][187] = 315;
		ra_end[3][187] = 334;
		decl_start[3][187] = -53;
		decl_end[3][187] = -45;
		ra_start[3][188] = 326;
		ra_end[3][188] = 338;
		decl_start[3][188] = -53;
		decl_end[3][188] = -45;
		ra_start[3][189] = 334;
		ra_end[3][189] = 349;
		decl_start[3][189] = -53;
		decl_end[3][189] = -45;
		ra_start[3][190] = 342;
		ra_end[3][190] = 360;
		decl_start[3][190] = -53;
		decl_end[3][190] = -45;
		ra_start[3][191] = 0;
		ra_end[3][191] = 360;
		decl_start[3][191] = -58;
		decl_end[3][191] = -50;
		ra_start[3][192] = 0;
		ra_end[3][192] = 27;
		decl_start[3][192] = -60;
		decl_end[3][192] = -52;
		ra_start[3][193] = 13;
		ra_end[3][193] = 40;
		decl_start[3][193] = -60;
		decl_end[3][193] = -52;
		ra_start[3][194] = 26;
		ra_end[3][194] = 54;
		decl_start[3][194] = -60;
		decl_end[3][194] = -52;
		ra_start[3][195] = 40;
		ra_end[3][195] = 67;
		decl_start[3][195] = -60;
		decl_end[3][195] = -52;
		ra_start[3][196] = 53;
		ra_end[3][196] = 67;
		decl_start[3][196] = -60;
		decl_end[3][196] = -52;
		ra_start[3][197] = 62;
		ra_end[3][197] = 80;
		decl_start[3][197] = -60;
		decl_end[3][197] = -52;
		ra_start[3][198] = 66;
		ra_end[3][198] = 94;
		decl_start[3][198] = -60;
		decl_end[3][198] = -52;
		ra_start[3][199] = 80;
		ra_end[3][199] = 94;
		decl_start[3][199] = -60;
		decl_end[3][199] = -52;
		ra_start[3][200] = 80;
		ra_end[3][200] = 107;
		decl_start[3][200] = -60;
		decl_end[3][200] = -52;
		ra_start[3][201] = 93;
		ra_end[3][201] = 107;
		decl_start[3][201] = -59;
		decl_end[3][201] = -52;
		ra_start[3][202] = 93;
		ra_end[3][202] = 110;
		decl_start[3][202] = -60;
		decl_end[3][202] = -52;
		ra_start[3][203] = 106;
		ra_end[3][203] = 120;
		decl_start[3][203] = -57;
		decl_end[3][203] = -52;
		ra_start[3][204] = 106;
		ra_end[3][204] = 120;
		decl_start[3][204] = -59;
		decl_end[3][204] = -54;
		ra_start[3][205] = 106;
		ra_end[3][205] = 127;
		decl_start[3][205] = -60;
		decl_end[3][205] = -52;
		ra_start[3][206] = 120;
		ra_end[3][206] = 134;
		decl_start[3][206] = -57;
		decl_end[3][206] = -52;
		ra_start[3][207] = 120;
		ra_end[3][207] = 134;
		decl_start[3][207] = -59;
		decl_end[3][207] = -54;
		ra_start[3][208] = 120;
		ra_end[3][208] = 134;
		decl_start[3][208] = -60;
		decl_end[3][208] = -56;
		ra_start[3][209] = 123;
		ra_end[3][209] = 137;
		decl_start[3][209] = -60;
		decl_end[3][209] = -52;
		ra_start[3][210] = 133;
		ra_end[3][210] = 144;
		decl_start[3][210] = -55;
		decl_end[3][210] = -52;
		ra_start[3][211] = 140;
		ra_end[3][211] = 147;
		decl_start[3][211] = -55;
		decl_end[3][211] = -52;
		ra_start[3][212] = 133;
		ra_end[3][212] = 147;
		decl_start[3][212] = -57;
		decl_end[3][212] = -52;
		ra_start[3][213] = 136;
		ra_end[3][213] = 147;
		decl_start[3][213] = -57;
		decl_end[3][213] = -54;
		ra_start[3][214] = 133;
		ra_end[3][214] = 147;
		decl_start[3][214] = -59;
		decl_end[3][214] = -54;
		ra_start[3][215] = 133;
		ra_end[3][215] = 147;
		decl_start[3][215] = -60;
		decl_end[3][215] = -56;
		ra_start[3][216] = 133;
		ra_end[3][216] = 144;
		decl_start[3][216] = -60;
		decl_end[3][216] = -58;
		ra_start[3][217] = 140;
		ra_end[3][217] = 150;
		decl_start[3][217] = -60;
		decl_end[3][217] = -52;
		ra_start[3][218] = 146;
		ra_end[3][218] = 160;
		decl_start[3][218] = -55;
		decl_end[3][218] = -52;
		ra_start[3][219] = 146;
		ra_end[3][219] = 160;
		decl_start[3][219] = -57;
		decl_end[3][219] = -52;
		ra_start[3][220] = 150;
		ra_end[3][220] = 160;
		decl_start[3][220] = -57;
		decl_end[3][220] = -54;
		ra_start[3][221] = 146;
		ra_end[3][221] = 160;
		decl_start[3][221] = -59;
		decl_end[3][221] = -54;
		ra_start[3][222] = 150;
		ra_end[3][222] = 160;
		decl_start[3][222] = -59;
		decl_end[3][222] = -56;
		ra_start[3][223] = 146;
		ra_end[3][223] = 160;
		decl_start[3][223] = -60;
		decl_end[3][223] = -56;
		ra_start[3][224] = 146;
		ra_end[3][224] = 157;
		decl_start[3][224] = -60;
		decl_end[3][224] = -58;
		ra_start[3][225] = 153;
		ra_end[3][225] = 164;
		decl_start[3][225] = -60;
		decl_end[3][225] = -52;
		ra_start[3][226] = 160;
		ra_end[3][226] = 174;
		decl_start[3][226] = -55;
		decl_end[3][226] = -52;
		ra_start[3][227] = 160;
		ra_end[3][227] = 174;
		decl_start[3][227] = -57;
		decl_end[3][227] = -52;
		ra_start[3][228] = 160;
		ra_end[3][228] = 174;
		decl_start[3][228] = -59;
		decl_end[3][228] = -54;
		ra_start[3][229] = 163;
		ra_end[3][229] = 174;
		decl_start[3][229] = -59;
		decl_end[3][229] = -56;
		ra_start[3][230] = 160;
		ra_end[3][230] = 174;
		decl_start[3][230] = -60;
		decl_end[3][230] = -56;
		ra_start[3][231] = 163;
		ra_end[3][231] = 170;
		decl_start[3][231] = -60;
		decl_end[3][231] = -58;
		ra_start[3][232] = 166;
		ra_end[3][232] = 180;
		decl_start[3][232] = -60;
		decl_end[3][232] = -52;
		ra_start[3][233] = 176;
		ra_end[3][233] = 187;
		decl_start[3][233] = -55;
		decl_end[3][233] = -52;
		ra_start[3][234] = 173;
		ra_end[3][234] = 186;
		decl_start[3][234] = -57;
		decl_end[3][234] = -52;
		ra_start[3][235] = 173;
		ra_end[3][235] = 187;
		decl_start[3][235] = -59;
		decl_end[3][235] = -54;
		ra_start[3][236] = 176;
		ra_end[3][236] = 187;
		decl_start[3][236] = -59;
		decl_end[3][236] = -56;
		ra_start[3][237] = 173;
		ra_end[3][237] = 187;
		decl_start[3][237] = -60;
		decl_end[3][237] = -56;
		ra_start[3][238] = 177;
		ra_end[3][238] = 190;
		decl_start[3][238] = -60;
		decl_end[3][238] = -52;
		ra_start[3][239] = 186;
		ra_end[3][239] = 200;
		decl_start[3][239] = -55;
		decl_end[3][239] = -52;
		ra_start[3][240] = 186;
		ra_end[3][240] = 200;
		decl_start[3][240] = -57;
		decl_end[3][240] = -52;
		ra_start[3][241] = 186;
		ra_end[3][241] = 200;
		decl_start[3][241] = -59;
		decl_end[3][241] = -54;
		ra_start[3][242] = 186;
		ra_end[3][242] = 200;
		decl_start[3][242] = -60;
		decl_end[3][242] = -56;
		ra_start[3][243] = 186;
		ra_end[3][243] = 197;
		decl_start[3][243] = -60;
		decl_end[3][243] = -58;
		ra_start[3][244] = 193;
		ra_end[3][244] = 207;
		decl_start[3][244] = -60;
		decl_end[3][244] = -52;
		ra_start[3][245] = 200;
		ra_end[3][245] = 214;
		decl_start[3][245] = -57;
		decl_end[3][245] = -52;
		ra_start[3][246] = 200;
		ra_end[3][246] = 214;
		decl_start[3][246] = -59;
		decl_end[3][246] = -54;
		ra_start[3][247] = 200;
		ra_end[3][247] = 214;
		decl_start[3][247] = -59;
		decl_end[3][247] = -56;
		ra_start[3][248] = 200;
		ra_end[3][248] = 214;
		decl_start[3][248] = -60;
		decl_end[3][248] = -56;
		ra_start[3][249] = 203;
		ra_end[3][249] = 210;
		decl_start[3][249] = -60;
		decl_end[3][249] = -58;
		ra_start[3][250] = 206;
		ra_end[3][250] = 220;
		decl_start[3][250] = -60;
		decl_end[3][250] = -52;
		ra_start[3][251] = 213;
		ra_end[3][251] = 227;
		decl_start[3][251] = -57;
		decl_end[3][251] = -52;
		ra_start[3][252] = 213;
		ra_end[3][252] = 227;
		decl_start[3][252] = -59;
		decl_end[3][252] = -54;
		ra_start[3][253] = 213;
		ra_end[3][253] = 227;
		decl_start[3][253] = -59;
		decl_end[3][253] = -56;
		ra_start[3][254] = 213;
		ra_end[3][254] = 227;
		decl_start[3][254] = -60;
		decl_end[3][254] = -56;
		ra_start[3][255] = 216;
		ra_end[3][255] = 234;
		decl_start[3][255] = -61;
		decl_end[3][255] = -52;
		ra_start[3][256] = 230;
		ra_end[3][256] = 237;
		decl_start[3][256] = -55;
		decl_end[3][256] = -52;
		ra_start[3][257] = 226;
		ra_end[3][257] = 240;
		decl_start[3][257] = -57;
		decl_end[3][257] = -52;
		ra_start[3][258] = 226;
		ra_end[3][258] = 240;
		decl_start[3][258] = -59;
		decl_end[3][258] = -54;
		ra_start[3][259] = 226;
		ra_end[3][259] = 240;
		decl_start[3][259] = -59;
		decl_end[3][259] = -56;
		ra_start[3][260] = 226;
		ra_end[3][260] = 240;
		decl_start[3][260] = -60;
		decl_end[3][260] = -56;
		ra_start[3][261] = 233;
		ra_end[3][261] = 244;
		decl_start[3][261] = -60;
		decl_end[3][261] = -52;
		ra_start[3][262] = 240;
		ra_end[3][262] = 250;
		decl_start[3][262] = -55;
		decl_end[3][262] = -52;
		ra_start[3][263] = 246;
		ra_end[3][263] = 254;
		decl_start[3][263] = -55;
		decl_end[3][263] = -52;
		ra_start[3][264] = 240;
		ra_end[3][264] = 254;
		decl_start[3][264] = -57;
		decl_end[3][264] = -52;
		ra_start[3][265] = 243;
		ra_end[3][265] = 254;
		decl_start[3][265] = -57;
		decl_end[3][265] = -54;
		ra_start[3][266] = 240;
		ra_end[3][266] = 254;
		decl_start[3][266] = -59;
		decl_end[3][266] = -54;
		ra_start[3][267] = 240;
		ra_end[3][267] = 254;
		decl_start[3][267] = -60;
		decl_end[3][267] = -56;
		ra_start[3][268] = 240;
		ra_end[3][268] = 254;
		decl_start[3][268] = -60;
		decl_end[3][268] = -58;
		ra_start[3][269] = 250;
		ra_end[3][269] = 260;
		decl_start[3][269] = -60;
		decl_end[3][269] = -52;
		ra_start[3][270] = 253;
		ra_end[3][270] = 267;
		decl_start[3][270] = -57;
		decl_end[3][270] = -52;
		ra_start[3][271] = 253;
		ra_end[3][271] = 267;
		decl_start[3][271] = -59;
		decl_end[3][271] = -54;
		ra_start[3][272] = 253;
		ra_end[3][272] = 267;
		decl_start[3][272] = -60;
		decl_end[3][272] = -56;
		ra_start[3][273] = 253;
		ra_end[3][273] = 267;
		decl_start[3][273] = -60;
		decl_end[3][273] = -58;
		ra_start[3][274] = 263;
		ra_end[3][274] = 279;
		decl_start[3][274] = -60;
		decl_end[3][274] = -52;
		ra_start[3][275] = 266;
		ra_end[3][275] = 280;
		decl_start[3][275] = -57;
		decl_end[3][275] = -52;
		ra_start[3][276] = 266;
		ra_end[3][276] = 280;
		decl_start[3][276] = -60;
		decl_end[3][276] = -54;
		ra_start[3][277] = 266;
		ra_end[3][277] = 287;
		decl_start[3][277] = -60;
		decl_end[3][277] = -52;
		ra_start[3][278] = 280;
		ra_end[3][278] = 294;
		decl_start[3][278] = -57;
		decl_end[3][278] = -52;
		ra_start[3][279] = 280;
		ra_end[3][279] = 294;
		decl_start[3][279] = -60;
		decl_end[3][279] = -54;
		ra_start[3][280] = 283;
		ra_end[3][280] = 307;
		decl_start[3][280] = -60;
		decl_end[3][280] = -52;
		ra_start[3][281] = 293;
		ra_end[3][281] = 307;
		decl_start[3][281] = -59;
		decl_end[3][281] = -52;
		ra_start[3][282] = 293;
		ra_end[3][282] = 320;
		decl_start[3][282] = -60;
		decl_end[3][282] = -52;
		ra_start[3][283] = 306;
		ra_end[3][283] = 320;
		decl_start[3][283] = -60;
		decl_end[3][283] = -52;
		ra_start[3][284] = 310;
		ra_end[3][284] = 334;
		decl_start[3][284] = -60;
		decl_end[3][284] = -52;
		ra_start[3][285] = 320;
		ra_end[3][285] = 347;
		decl_start[3][285] = -60;
		decl_end[3][285] = -52;
		ra_start[3][286] = 333;
		ra_end[3][286] = 352;
		decl_start[3][286] = -60;
		decl_end[3][286] = -52;
		ra_start[3][287] = 0;
		ra_end[3][287] = 360;
		decl_start[3][287] = -62;
		decl_end[3][287] = -52;
		ra_start[3][288] = 0;
		ra_end[3][288] = 18;
		decl_start[3][288] = -68;
		decl_end[3][288] = -60;
		ra_start[3][289] = 5;
		ra_end[3][289] = 35;
		decl_start[3][289] = -68;
		decl_end[3][289] = -60;
		ra_start[3][290] = 17;
		ra_end[3][290] = 52;
		decl_start[3][290] = -68;
		decl_end[3][290] = -60;
		ra_start[3][291] = 34;
		ra_end[3][291] = 63;
		decl_start[3][291] = -68;
		decl_end[3][291] = -60;
		ra_start[3][292] = 51;
		ra_end[3][292] = 69;
		decl_start[3][292] = -68;
		decl_end[3][292] = -60;
		ra_start[3][293] = 62;
		ra_end[3][293] = 86;
		decl_start[3][293] = -68;
		decl_end[3][293] = -60;
		ra_start[3][294] = 68;
		ra_end[3][294] = 86;
		decl_start[3][294] = -68;
		decl_end[3][294] = -61;
		ra_start[3][295] = 81;
		ra_end[3][295] = 103;
		decl_start[3][295] = -68;
		decl_end[3][295] = -60;
		ra_start[3][296] = 85;
		ra_end[3][296] = 103;
		decl_start[3][296] = -68;
		decl_end[3][296] = -61;
		ra_start[3][297] = 90;
		ra_end[3][297] = 120;
		decl_start[3][297] = -68;
		decl_end[3][297] = -60;
		ra_start[3][298] = 102;
		ra_end[3][298] = 120;
		decl_start[3][298] = -66;
		decl_end[3][298] = -60;
		ra_start[3][299] = 102;
		ra_end[3][299] = 125;
		decl_start[3][299] = -68;
		decl_end[3][299] = -60;
		ra_start[3][300] = 120;
		ra_end[3][300] = 138;
		decl_start[3][300] = -64;
		decl_end[3][300] = -60;
		ra_start[3][301] = 120;
		ra_end[3][301] = 138;
		decl_start[3][301] = -66;
		decl_end[3][301] = -61;
		ra_start[3][302] = 120;
		ra_end[3][302] = 138;
		decl_start[3][302] = -68;
		decl_end[3][302] = -63;
		ra_start[3][303] = 124;
		ra_end[3][303] = 146;
		decl_start[3][303] = -68;
		decl_end[3][303] = -60;
		ra_start[3][304] = 141;
		ra_end[3][304] = 155;
		decl_start[3][304] = -62;
		decl_end[3][304] = -60;
		ra_start[3][305] = 137;
		ra_end[3][305] = 155;
		decl_start[3][305] = -64;
		decl_end[3][305] = -60;
		ra_start[3][306] = 137;
		ra_end[3][306] = 150;
		decl_start[3][306] = -64;
		decl_end[3][306] = -61;
		ra_start[3][307] = 137;
		ra_end[3][307] = 155;
		decl_start[3][307] = -66;
		decl_end[3][307] = -61;
		ra_start[3][308] = 137;
		ra_end[3][308] = 155;
		decl_start[3][308] = -68;
		decl_end[3][308] = -63;
		ra_start[3][309] = 141;
		ra_end[3][309] = 159;
		decl_start[3][309] = -68;
		decl_end[3][309] = -60;
		ra_start[3][310] = 154;
		ra_end[3][310] = 163;
		decl_start[3][310] = -62;
		decl_end[3][310] = -60;
		ra_start[3][311] = 158;
		ra_end[3][311] = 168;
		decl_start[3][311] = -62;
		decl_end[3][311] = -60;
		ra_start[3][312] = 154;
		ra_end[3][312] = 172;
		decl_start[3][312] = -64;
		decl_end[3][312] = -60;
		ra_start[3][313] = 154;
		ra_end[3][313] = 168;
		decl_start[3][313] = -64;
		decl_end[3][313] = -61;
		ra_start[3][314] = 154;
		ra_end[3][314] = 172;
		decl_start[3][314] = -66;
		decl_end[3][314] = -61;
		ra_start[3][315] = 154;
		ra_end[3][315] = 172;
		decl_start[3][315] = -68;
		decl_end[3][315] = -63;
		ra_start[3][316] = 158;
		ra_end[3][316] = 176;
		decl_start[3][316] = -68;
		decl_end[3][316] = -60;
		ra_start[3][317] = 171;
		ra_end[3][317] = 179;
		decl_start[3][317] = -62;
		decl_end[3][317] = -60;
		ra_start[3][318] = 175;
		ra_end[3][318] = 185;
		decl_start[3][318] = -62;
		decl_end[3][318] = -60;
		ra_start[3][319] = 171;
		ra_end[3][319] = 189;
		decl_start[3][319] = -64;
		decl_end[3][319] = -60;
		ra_start[3][320] = 171;
		ra_end[3][320] = 180;
		decl_start[3][320] = -64;
		decl_end[3][320] = -61;
		ra_start[3][321] = 175;
		ra_end[3][321] = 189;
		decl_start[3][321] = -64;
		decl_end[3][321] = -61;
		ra_start[3][322] = 171;
		ra_end[3][322] = 189;
		decl_start[3][322] = -66;
		decl_end[3][322] = -61;
		ra_start[3][323] = 175;
		ra_end[3][323] = 189;
		decl_start[3][323] = -66;
		decl_end[3][323] = -63;
		ra_start[3][324] = 171;
		ra_end[3][324] = 189;
		decl_start[3][324] = -68;
		decl_end[3][324] = -63;
		ra_start[3][325] = 175;
		ra_end[3][325] = 193;
		decl_start[3][325] = -68;
		decl_end[3][325] = -60;
		ra_start[3][326] = 188;
		ra_end[3][326] = 202;
		decl_start[3][326] = -62;
		decl_end[3][326] = -60;
		ra_start[3][327] = 197;
		ra_end[3][327] = 206;
		decl_start[3][327] = -62;
		decl_end[3][327] = -60;
		ra_start[3][328] = 188;
		ra_end[3][328] = 206;
		decl_start[3][328] = -64;
		decl_end[3][328] = -60;
		ra_start[3][329] = 192;
		ra_end[3][329] = 206;
		decl_start[3][329] = -64;
		decl_end[3][329] = -61;
		ra_start[3][330] = 188;
		ra_end[3][330] = 206;
		decl_start[3][330] = -66;
		decl_end[3][330] = -61;
		ra_start[3][331] = 188;
		ra_end[3][331] = 206;
		decl_start[3][331] = -66;
		decl_end[3][331] = -63;
		ra_start[3][332] = 188;
		ra_end[3][332] = 206;
		decl_start[3][332] = -68;
		decl_end[3][332] = -63;
		ra_start[3][333] = 192;
		ra_end[3][333] = 210;
		decl_start[3][333] = -68;
		decl_end[3][333] = -60;
		ra_start[3][334] = 205;
		ra_end[3][334] = 215;
		decl_start[3][334] = -62;
		decl_end[3][334] = -60;
		ra_start[3][335] = 210;
		ra_end[3][335] = 219;
		decl_start[3][335] = -62;
		decl_end[3][335] = -60;
		ra_start[3][336] = 205;
		ra_end[3][336] = 223;
		decl_start[3][336] = -64;
		decl_end[3][336] = -60;
		ra_start[3][337] = 205;
		ra_end[3][337] = 215;
		decl_start[3][337] = -64;
		decl_end[3][337] = -61;
		ra_start[3][338] = 210;
		ra_end[3][338] = 219;
		decl_start[3][338] = -64;
		decl_end[3][338] = -61;
		ra_start[3][339] = 205;
		ra_end[3][339] = 223;
		decl_start[3][339] = -66;
		decl_end[3][339] = -61;
		ra_start[3][340] = 205;
		ra_end[3][340] = 223;
		decl_start[3][340] = -66;
		decl_end[3][340] = -63;
		ra_start[3][341] = 205;
		ra_end[3][341] = 223;
		decl_start[3][341] = -68;
		decl_end[3][341] = -63;
		ra_start[3][342] = 214;
		ra_end[3][342] = 232;
		decl_start[3][342] = -68;
		decl_end[3][342] = -60;
		ra_start[3][343] = 222;
		ra_end[3][343] = 240;
		decl_start[3][343] = -64;
		decl_end[3][343] = -60;
		ra_start[3][344] = 222;
		ra_end[3][344] = 236;
		decl_start[3][344] = -64;
		decl_end[3][344] = -61;
		ra_start[3][345] = 231;
		ra_end[3][345] = 240;
		decl_start[3][345] = -64;
		decl_end[3][345] = -61;
		ra_start[3][346] = 222;
		ra_end[3][346] = 240;
		decl_start[3][346] = -66;
		decl_end[3][346] = -61;
		ra_start[3][347] = 227;
		ra_end[3][347] = 236;
		decl_start[3][347] = -66;
		decl_end[3][347] = -63;
		ra_start[3][348] = 223;
		ra_end[3][348] = 240;
		decl_start[3][348] = -68;
		decl_end[3][348] = -63;
		ra_start[3][349] = 222;
		ra_end[3][349] = 232;
		decl_start[3][349] = -68;
		decl_end[3][349] = -65;
		ra_start[3][350] = 227;
		ra_end[3][350] = 240;
		decl_start[3][350] = -68;
		decl_end[3][350] = -65;
		ra_start[3][351] = 235;
		ra_end[3][351] = 249;
		decl_start[3][351] = -68;
		decl_end[3][351] = -60;
		ra_start[3][352] = 244;
		ra_end[3][352] = 258;
		decl_start[3][352] = -62;
		decl_end[3][352] = -60;
		ra_start[3][353] = 240;
		ra_end[3][353] = 258;
		decl_start[3][353] = -64;
		decl_end[3][353] = -60;
		ra_start[3][354] = 244;
		ra_end[3][354] = 258;
		decl_start[3][354] = -64;
		decl_end[3][354] = -61;
		ra_start[3][355] = 240;
		ra_end[3][355] = 254;
		decl_start[3][355] = -66;
		decl_end[3][355] = -62;
		ra_start[3][356] = 240;
		ra_end[3][356] = 258;
		decl_start[3][356] = -68;
		decl_end[3][356] = -63;
		ra_start[3][357] = 244;
		ra_end[3][357] = 265;
		decl_start[3][357] = -68;
		decl_end[3][357] = -60;
		ra_start[3][358] = 257;
		ra_end[3][358] = 275;
		decl_start[3][358] = -64;
		decl_end[3][358] = -60;
		ra_start[3][359] = 257;
		ra_end[3][359] = 275;
		decl_start[3][359] = -66;
		decl_end[3][359] = -61;
		ra_start[3][360] = 257;
		ra_end[3][360] = 275;
		decl_start[3][360] = -68;
		decl_end[3][360] = -63;
		ra_start[3][361] = 257;
		ra_end[3][361] = 283;
		decl_start[3][361] = -68;
		decl_end[3][361] = -60;
		ra_start[3][362] = 274;
		ra_end[3][362] = 292;
		decl_start[3][362] = -64;
		decl_end[3][362] = -60;
		ra_start[3][363] = 274;
		ra_end[3][363] = 292;
		decl_start[3][363] = -68;
		decl_end[3][363] = -61;
		ra_start[3][364] = 278;
		ra_end[3][364] = 309;
		decl_start[3][364] = -68;
		decl_end[3][364] = -60;
		ra_start[3][365] = 291;
		ra_end[3][365] = 309;
		decl_start[3][365] = -68;
		decl_end[3][365] = -61;
		ra_start[3][366] = 292;
		ra_end[3][366] = 326;
		decl_start[3][366] = -68;
		decl_end[3][366] = -60;
		ra_start[3][367] = 308;
		ra_end[3][367] = 331;
		decl_start[3][367] = -68;
		decl_end[3][367] = -61;
		ra_start[3][368] = 325;
		ra_end[3][368] = 343;
		decl_start[3][368] = -68;
		decl_end[3][368] = -60;
		ra_start[3][369] = 325;
		ra_end[3][369] = 360;
		decl_start[3][369] = -68;
		decl_end[3][369] = -60;
		ra_start[3][370] = 0;
		ra_end[3][370] = 360;
		decl_start[3][370] = -70;
		decl_end[3][370] = -62;
		ra_start[3][371] = 0;
		ra_end[3][371] = 24;
		decl_start[3][371] = -75;
		decl_end[3][371] = -67;
		ra_start[3][372] = 8;
		ra_end[3][372] = 48;
		decl_start[3][372] = -75;
		decl_end[3][372] = -67;
		ra_start[3][373] = 24;
		ra_end[3][373] = 56;
		decl_start[3][373] = -75;
		decl_end[3][373] = -67;
		ra_start[3][374] = 48;
		ra_end[3][374] = 72;
		decl_start[3][374] = -75;
		decl_end[3][374] = -67;
		ra_start[3][375] = 48;
		ra_end[3][375] = 96;
		decl_start[3][375] = -75;
		decl_end[3][375] = -67;
		ra_start[3][376] = 72;
		ra_end[3][376] = 96;
		decl_start[3][376] = -74;
		decl_end[3][376] = -67;
		ra_start[3][377] = 72;
		ra_end[3][377] = 108;
		decl_start[3][377] = -75;
		decl_end[3][377] = -67;
		ra_start[3][378] = 96;
		ra_end[3][378] = 120;
		decl_start[3][378] = -74;
		decl_end[3][378] = -67;
		ra_start[3][379] = 96;
		ra_end[3][379] = 126;
		decl_start[3][379] = -75;
		decl_end[3][379] = -67;
		ra_start[3][380] = 120;
		ra_end[3][380] = 144;
		decl_start[3][380] = -72;
		decl_end[3][380] = -67;
		ra_start[3][381] = 120;
		ra_end[3][381] = 144;
		decl_start[3][381] = -74;
		decl_end[3][381] = -69;
		ra_start[3][382] = 120;
		ra_end[3][382] = 144;
		decl_start[3][382] = -75;
		decl_end[3][382] = -71;
		ra_start[3][383] = 132;
		ra_end[3][383] = 156;
		decl_start[3][383] = -75;
		decl_end[3][383] = -67;
		ra_start[3][384] = 144;
		ra_end[3][384] = 168;
		decl_start[3][384] = -72;
		decl_end[3][384] = -67;
		ra_start[3][385] = 144;
		ra_end[3][385] = 166;
		decl_start[3][385] = -72;
		decl_end[3][385] = -69;
		ra_start[3][386] = 144;
		ra_end[3][386] = 168;
		decl_start[3][386] = -74;
		decl_end[3][386] = -69;
		ra_start[3][387] = 144;
		ra_end[3][387] = 174;
		decl_start[3][387] = -75;
		decl_end[3][387] = -67;
		ra_start[3][388] = 168;
		ra_end[3][388] = 186;
		decl_start[3][388] = -70;
		decl_end[3][388] = -67;
		ra_start[3][389] = 168;
		ra_end[3][389] = 192;
		decl_start[3][389] = -72;
		decl_end[3][389] = -67;
		ra_start[3][390] = 168;
		ra_end[3][390] = 192;
		decl_start[3][390] = -74;
		decl_end[3][390] = -69;
		ra_start[3][391] = 168;
		ra_end[3][391] = 192;
		decl_start[3][391] = -75;
		decl_end[3][391] = -71;
		ra_start[3][392] = 175;
		ra_end[3][392] = 204;
		decl_start[3][392] = -75;
		decl_end[3][392] = -67;
		ra_start[3][393] = 192;
		ra_end[3][393] = 216;
		decl_start[3][393] = -72;
		decl_end[3][393] = -67;
		ra_start[3][394] = 192;
		ra_end[3][394] = 216;
		decl_start[3][394] = -72;
		decl_end[3][394] = -69;
		ra_start[3][395] = 192;
		ra_end[3][395] = 216;
		decl_start[3][395] = -74;
		decl_end[3][395] = -69;
		ra_start[3][396] = 192;
		ra_end[3][396] = 216;
		decl_start[3][396] = -75;
		decl_end[3][396] = -71;
		ra_start[3][397] = 204;
		ra_end[3][397] = 228;
		decl_start[3][397] = -75;
		decl_end[3][397] = -67;
		ra_start[3][398] = 222;
		ra_end[3][398] = 240;
		decl_start[3][398] = -70;
		decl_end[3][398] = -67;
		ra_start[3][399] = 216;
		ra_end[3][399] = 240;
		decl_start[3][399] = -72;
		decl_end[3][399] = -67;
		ra_start[3][400] = 216;
		ra_end[3][400] = 240;
		decl_start[3][400] = -74;
		decl_end[3][400] = -69;
		ra_start[3][401] = 216;
		ra_end[3][401] = 240;
		decl_start[3][401] = -75;
		decl_end[3][401] = -71;
		ra_start[3][402] = 216;
		ra_end[3][402] = 246;
		decl_start[3][402] = -75;
		decl_end[3][402] = -67;
		ra_start[3][403] = 240;
		ra_end[3][403] = 264;
		decl_start[3][403] = -70;
		decl_end[3][403] = -67;
		ra_start[3][404] = 240;
		ra_end[3][404] = 264;
		decl_start[3][404] = -72;
		decl_end[3][404] = -67;
		ra_start[3][405] = 240;
		ra_end[3][405] = 264;
		decl_start[3][405] = -74;
		decl_end[3][405] = -69;
		ra_start[3][406] = 240;
		ra_end[3][406] = 264;
		decl_start[3][406] = -75;
		decl_end[3][406] = -71;
		ra_start[3][407] = 258;
		ra_end[3][407] = 288;
		decl_start[3][407] = -75;
		decl_end[3][407] = -67;
		ra_start[3][408] = 264;
		ra_end[3][408] = 288;
		decl_start[3][408] = -74;
		decl_end[3][408] = -69;
		ra_start[3][409] = 264;
		ra_end[3][409] = 300;
		decl_start[3][409] = -75;
		decl_end[3][409] = -67;
		ra_start[3][410] = 288;
		ra_end[3][410] = 312;
		decl_start[3][410] = -74;
		decl_end[3][410] = -67;
		ra_start[3][411] = 288;
		ra_end[3][411] = 324;
		decl_start[3][411] = -75;
		decl_end[3][411] = -67;
		ra_start[3][412] = 312;
		ra_end[3][412] = 336;
		decl_start[3][412] = -74;
		decl_end[3][412] = -67;
		ra_start[3][413] = 312;
		ra_end[3][413] = 360;
		decl_start[3][413] = -75;
		decl_end[3][413] = -67;
		ra_start[3][414] = 336;
		ra_end[3][414] = 360;
		decl_start[3][414] = -75;
		decl_end[3][414] = -67;
		ra_start[3][415] = 0;
		ra_end[3][415] = 360;
		decl_start[3][415] = -79;
		decl_end[3][415] = -72;
		ra_start[3][416] = 0;
		ra_end[3][416] = 70;
		decl_start[3][416] = -83;
		decl_end[3][416] = -75;
		ra_start[3][417] = 40;
		ra_end[3][417] = 80;
		decl_start[3][417] = -83;
		decl_end[3][417] = -75;
		ra_start[3][418] = 40;
		ra_end[3][418] = 120;
		decl_start[3][418] = -83;
		decl_end[3][418] = -75;
		ra_start[3][419] = 80;
		ra_end[3][419] = 120;
		decl_start[3][419] = -83;
		decl_end[3][419] = -76;
		ra_start[3][420] = 110;
		ra_end[3][420] = 160;
		decl_start[3][420] = -82;
		decl_end[3][420] = -75;
		ra_start[3][421] = 120;
		ra_end[3][421] = 160;
		decl_start[3][421] = -83;
		decl_end[3][421] = -76;
		ra_start[3][422] = 140;
		ra_end[3][422] = 200;
		decl_start[3][422] = -83;
		decl_end[3][422] = -75;
		ra_start[3][423] = 160;
		ra_end[3][423] = 200;
		decl_start[3][423] = -83;
		decl_end[3][423] = -76;
		ra_start[3][424] = 190;
		ra_end[3][424] = 240;
		decl_start[3][424] = -83;
		decl_end[3][424] = -75;
		ra_start[3][425] = 200;
		ra_end[3][425] = 240;
		decl_start[3][425] = -79;
		decl_end[3][425] = -75;
		ra_start[3][426] = 200;
		ra_end[3][426] = 240;
		decl_start[3][426] = -83;
		decl_end[3][426] = -76;
		ra_start[3][427] = 201;
		ra_end[3][427] = 270;
		decl_start[3][427] = -83;
		decl_end[3][427] = -75;
		ra_start[3][428] = 240;
		ra_end[3][428] = 280;
		decl_start[3][428] = -79;
		decl_end[3][428] = -75;
		ra_start[3][429] = 240;
		ra_end[3][429] = 290;
		decl_start[3][429] = -83;
		decl_end[3][429] = -75;
		ra_start[3][430] = 280;
		ra_end[3][430] = 320;
		decl_start[3][430] = -81;
		decl_end[3][430] = -75;
		ra_start[3][431] = 280;
		ra_end[3][431] = 360;
		decl_start[3][431] = -83;
		decl_end[3][431] = -75;
		ra_start[3][432] = 0;
		ra_end[3][432] = 360;
		decl_start[3][432] = -85;
		decl_end[3][432] = -76;
		ra_start[3][433] = 0;
		ra_end[3][433] = 120;
		decl_start[3][433] = -87;
		decl_end[3][433] = -82;
		ra_start[3][434] = 0;
		ra_end[3][434] = 210;
		decl_start[3][434] = -90;
		decl_end[3][434] = -82;
		ra_start[3][435] = 120;
		ra_end[3][435] = 240;
		decl_start[3][435] = -89;
		decl_end[3][435] = -82;
		ra_start[3][436] = 120;
		ra_end[3][436] = 330;
		decl_start[3][436] = -90;
		decl_end[3][436] = -82;
		ra_start[3][437] = 240;
		ra_end[3][437] = 360;
		decl_start[3][437] = -89;
		decl_end[3][437] = -82;
		ra_start[3][438] = 240;
		ra_end[3][438] = 360;
		decl_start[3][438] = -90;
		decl_end[3][438] = -86;
	}

	/**
	 * Sets the R.A. range of each block.
	 */
	static {
		ra_start = new int[file_N][];
		ra_end = new int[file_N][];
		decl_start = new int[file_N][];
		decl_end = new int[file_N][];

		for (int i = 0 ; i < file_N ; i++) {
			ra_start[i] = new int[file_block_count[i]];
			ra_end[i] = new int[file_block_count[i]];
			decl_start[i] = new int[file_block_count[i]];
			decl_end[i] = new int[file_block_count[i]];
		}

		initialize_Tic1();
		initialize_Tic2();
		initialize_Tic3();
		initialize_Tic4();
	}
}
